package com.mxnavi.tspv2.authservice;

import com.mxnavi.tspv2.authservice.model.ActInfo;
import com.mxnavi.tspv2.authservice.model.DataAuthActiveRequest;
import com.mxnavi.tspv2.authservice.model.RegIdAndVer;
import com.mxnavi.tspv2.authservice.model.SoftActiveRequest;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.authservice.model.FirstActiveRequest;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/7/21.
 */
public class AuthService extends BaseTsp {
    // 请求服务类型为鉴权服务
    final int thisServiceType = Config.MxNGTPServiceType.MX_AUTH.getServiceType();
    // 鉴权服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_AUTH.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int AUTH_FIRSTACTIVATED = 0;//数据鉴权：首次激活
    final int AUTH_DATAAUTHACTIVE = 1;//数据鉴权：获取数据激活码
    final int AUTH_SOFTACTIVE = 2;//获取软件激活码

    public interface OnFirstActiveListener extends CallBackListener {
        /**
         * 数据鉴权：首次激活回调函数
         * @param actCode 软件激活码
         */
        public void onFirstActivate(String actCode);
    }

    public interface OnDataAuthActiveListener extends CallBackListener{
        /**
         * 获取数据激活码回调函数
         * @param actInfos 激活信息集合
         */
        public void onDataAuthActive(List<ActInfo> actInfos);
    }

    public interface OnSoftActiveListener extends CallBackListener{
        /**
         * 获取软件激活码回调函数
         * @param actCode 软件激活码
         */
        public void onSoftActive(String actCode);
    }

    OnFirstActiveListener mOnFirstActivateListener;
    OnDataAuthActiveListener mOnDataAuthActiveListener;
    OnSoftActiveListener mOnSoftActiveListener;

    /**
     * 数据鉴权：首次激活
     * @param firstActiveRequest 首次激活请求参数
     * @param onFirstActivateListener 回调接口
     * @return 请求ID，0:参数错误
     */
    public int firstActive(FirstActiveRequest firstActiveRequest,OnFirstActiveListener onFirstActivateListener){
        return firstActive(firstActiveRequest, onFirstActivateListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *数据鉴权：首次激活
     * @param firstActiveRequest 首次激活请求参数
     * @param onFirstActivateListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int firstActive(FirstActiveRequest firstActiveRequest,OnFirstActiveListener onFirstActivateListener,int timeOutSecs){
        return firstActive(firstActiveRequest, onFirstActivateListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *数据鉴权：首次激活
     * @param firstActiveRequest 首次激活请求参数
     * @param onFirstActivateListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int firstActive(FirstActiveRequest firstActiveRequest,OnFirstActiveListener onFirstActivateListener,int timeOutSecs, int priorityLevel){
        if (firstActiveRequest ==null|| firstActiveRequest.getProductId()==null || firstActiveRequest.getDeviceNo()==null
                || firstActiveRequest.getSoftVer()==null|| firstActiveRequest.getDataVer()==null){
            if (onFirstActivateListener!=null){
                onFirstActivateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnFirstActivateListener = onFirstActivateListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "std/auth/firstActiveHasNum";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", firstActiveRequest.getProductId());
        params.put("device_no", firstActiveRequest.getDeviceNo());
        params.put("soft_ver", firstActiveRequest.getSoftVer());
        params.put("data_ver", firstActiveRequest.getDataVer());
        if (firstActiveRequest.isEncryption()){
            params.put("encryption",1);
        }else {
            params.put("encryption",0);
        }
        if (firstActiveRequest.getSchemeId()!=null){
            params.put("scheme_id", firstActiveRequest.getSchemeId());
        }
        if (firstActiveRequest.isTestFlag()){
            params.put("test_flag",1);
        }else {
            params.put("test_flag",0);
        }
        if (firstActiveRequest.getVin()!=null){
            params.put("vin", firstActiveRequest.getVin());
        }
        if (firstActiveRequest.getUserName()!=null){
            params.put("user_name", firstActiveRequest.getUserName());
        }
        if (firstActiveRequest.getCardNo()!=null){
            params.put("card_no", firstActiveRequest.getCardNo());
        }
        if (firstActiveRequest.getMobilePhone()!=null){
            params.put("mobile_phone", firstActiveRequest.getMobilePhone());
        }
        if (firstActiveRequest.getFixedPhone()!=null){
            params.put("fixed_phone", firstActiveRequest.getFixedPhone());
        }
        params.put("device_type", firstActiveRequest.getDeviceType());
        params.put("gender", firstActiveRequest.getGender());
        if (firstActiveRequest.getAddress()!=null){
            params.put("address", firstActiveRequest.getAddress());
        }
        if (firstActiveRequest.getMail()!=null){
            params.put("mail", firstActiveRequest.getMail());
        }
        if (firstActiveRequest.getHandlers()!=null){
            params.put("handlers", firstActiveRequest.getHandlers());
        }
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, AUTH_FIRSTACTIVATED);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取数据激活码
     * @param dataAuthActiveRequest 获取数据激活码请求信息
     * @param onDataAuthActiveListener 获取数据激活码回调接口
     * @return 请求ID，0:参数错误
     */
    public int dataAuthActive(DataAuthActiveRequest dataAuthActiveRequest,OnDataAuthActiveListener onDataAuthActiveListener){
        return dataAuthActive(dataAuthActiveRequest, onDataAuthActiveListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取数据激活码
     * @param dataAuthActiveRequest 获取数据激活码请求信息
     * @param onDataAuthActiveListener 获取数据激活码回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int dataAuthActive(DataAuthActiveRequest dataAuthActiveRequest,OnDataAuthActiveListener onDataAuthActiveListener,int timeOutSecs){
        return dataAuthActive(dataAuthActiveRequest, onDataAuthActiveListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取数据激活码
     * @param dataAuthActiveRequest 获取数据激活码请求信息
     * @param onDataAuthActiveListener 获取数据激活码回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int dataAuthActive(DataAuthActiveRequest dataAuthActiveRequest,OnDataAuthActiveListener onDataAuthActiveListener,int timeOutSecs, int priorityLevel){
        if (dataAuthActiveRequest==null||dataAuthActiveRequest.getProductId()==null||dataAuthActiveRequest.getDeviceNo()==null||dataAuthActiveRequest.getRegIdAndVers()==null
                ||dataAuthActiveRequest.getSoftVersion()==null){
            if (onDataAuthActiveListener!=null){
                onDataAuthActiveListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnDataAuthActiveListener = onDataAuthActiveListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "std/auth/dataAuthActive";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id",dataAuthActiveRequest.getProductId());
        params.put("device_no",dataAuthActiveRequest.getDeviceNo());
        List<RegIdAndVer> regIdAndVers = dataAuthActiveRequest.getRegIdAndVers();
        StringBuffer dataVersion = new StringBuffer();
        StringBuffer regionId = new StringBuffer();
        for (int i=0;i<regIdAndVers.size();i++){
            RegIdAndVer regIdAndVer = regIdAndVers.get(i);
            if(i == regIdAndVers.size()-1){
                dataVersion.append(regIdAndVer.getDataVersion());
                regionId.append(regIdAndVer.getRegionId());
            }else {
                dataVersion.append(regIdAndVer.getDataVersion()).append(",");
                regionId.append(regIdAndVer.getRegionId()).append(",");
            }
        }
        params.put("region_ids",regionId.toString());
        params.put("data_versions",dataVersion.toString());
        params.put("soft_version",dataAuthActiveRequest.getSoftVersion());
        if (dataAuthActiveRequest.isEncryption()){
            params.put("encryption",1);
        }else {
            params.put("encryption",0);
        }
        if (dataAuthActiveRequest.getActiveMode()!=null){
            params.put("active_mode",dataAuthActiveRequest.getActiveMode());
        }
        if (dataAuthActiveRequest.getActiveToolSoftver()!=null){
            params.put("active_tool_softver",dataAuthActiveRequest.getActiveToolSoftver());
        }
        if (dataAuthActiveRequest.getHandlers()!=null){
            params.put("handlers",dataAuthActiveRequest.getHandlers());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, AUTH_DATAAUTHACTIVE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取软件激活码
     * @param softActiveRequest 获取软件激活码请求信息
     * @param onSoftActiveListener 回调监听
     * @return 请求ID，0:参数错误
     */
    public int softActive(SoftActiveRequest softActiveRequest,OnSoftActiveListener onSoftActiveListener){
        return softActive(softActiveRequest,onSoftActiveListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取软件激活码
     * @param softActiveRequest 获取软件激活码请求信息
     * @param onSoftActiveListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int softActive(SoftActiveRequest softActiveRequest,OnSoftActiveListener onSoftActiveListener,int timeOutSecs){
        return softActive(softActiveRequest,onSoftActiveListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取软件激活码
     * @param softActiveRequest 获取软件激活码请求信息
     * @param onSoftActiveListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int softActive(SoftActiveRequest softActiveRequest,OnSoftActiveListener onSoftActiveListener,int timeOutSecs, int priorityLevel){
        if (softActiveRequest==null||softActiveRequest.getProductId()==null||softActiveRequest.getDeviceNo()==null
                ||softActiveRequest.getSoftVer()==null||softActiveRequest.getDataVer()==null){
            if (onSoftActiveListener!=null){
                onSoftActiveListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnSoftActiveListener = onSoftActiveListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "std/auth/softActive";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", softActiveRequest.getProductId());
        params.put("device_no", softActiveRequest.getDeviceNo());
        params.put("soft_ver", softActiveRequest.getSoftVer());
        params.put("data_ver", softActiveRequest.getDataVer());
        if (softActiveRequest.isEncryption()){
            params.put("encryption",1);
        }else {
            params.put("encryption",0);
        }
        params.put("active_mode", softActiveRequest.getActiveMode());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, AUTH_SOFTACTIVE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case AUTH_FIRSTACTIVATED:
                if (mOnFirstActivateListener != null) {
                    mOnFirstActivateListener.onTimeOut();
                }
                break;
            case AUTH_DATAAUTHACTIVE:
                if (mOnDataAuthActiveListener != null) {
                    mOnDataAuthActiveListener.onTimeOut();
                }
                break;
            case AUTH_SOFTACTIVE:
                if (mOnSoftActiveListener != null) {
                    mOnSoftActiveListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, AuthService.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case AUTH_FIRSTACTIVATED:
                if (mOnFirstActivateListener != null){
                    if(hasDSPTError){
                        mOnFirstActivateListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            String actCode = jsonObject.getString("actCode");
                            mOnFirstActivateListener.onFirstActivate(actCode);
                        }else {
                            mOnFirstActivateListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnFirstActivateListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case AUTH_DATAAUTHACTIVE:
                if (mOnDataAuthActiveListener!=null){
                    if(hasDSPTError){
                        mOnDataAuthActiveListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            List<ActInfo> actInfos = new ArrayList<ActInfo>();
                            ActInfo actInfo;
                            JSONArray actInfoArray = jsonObject.getJSONArray("actInfo");
                            for (int i=0;i<actInfoArray.length();i++){
                                JSONObject actInfoObj = actInfoArray.getJSONObject(i);
                                actInfo = new ActInfo();
                                actInfo.setActCode(actInfoObj.getString("actCode"));
                                actInfo.setDataVersion(actInfoObj.getString("dataVersion"));
                                actInfo.setRegionId(actInfoObj.getInt("regionId"));
                                actInfos.add(actInfo);
                            }
                            mOnDataAuthActiveListener.onDataAuthActive(actInfos);
                        }else {
                            mOnDataAuthActiveListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDataAuthActiveListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case AUTH_SOFTACTIVE:
                if (mOnSoftActiveListener != null) {
                    if(hasDSPTError){
                        mOnSoftActiveListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            String actCode = jsonObject.getString("actCode");
                            mOnSoftActiveListener.onSoftActive(actCode);
                        }else {
                            mOnSoftActiveListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSoftActiveListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
