package com.mxnavi.tspv2.bigDataMining;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.mxnavi.tspv2.bigDataMining.model.ResidentCityInfo;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2018/3/14.
 */

public class BigDataService extends BaseTsp {
    private static final String TAG = "CarManager";
    // 请求服务类型为大数据服务
    final int thisServiceType = Config.MxNGTPServiceType.MX_BIG_DATA_MINING.getServiceType();
    //  大数据服务ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_BIG_DATA_MINING.getServiceVer();

    public BigDataService() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int GET_RESIDENT_CITIES = 1;//获取常驻城市
    GetResidentCitiesListner mGetResidentCitiesListner;
    /**
     * 获取常驻城市回调接口
     */
    public interface GetResidentCitiesListner extends CallBackListener{
        /**
         * 获取常驻城市回调
         * @param list 常用城市列表
         */
        public void onGetResidentCities(List<ResidentCityInfo> list);
    }
    /**
     * 获取常驻城市
     * @param getResidentCitiesListner
     * @return
     */
    public int getResidentCities(GetResidentCitiesListner getResidentCitiesListner){
    	return getResidentCities(getResidentCitiesListner,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 
     * @param getResidentCitiesListner
     * @param timeOutSecs
     * @return
     */
    public int getResidentCities(GetResidentCitiesListner getResidentCitiesListner,int timeOutSecs){
    	return getResidentCities(getResidentCitiesListner,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取常驻城市
     * @param getResidentCitiesListner
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getResidentCities(GetResidentCitiesListner getResidentCitiesListner,int timeOutSecs, int priorityLevel){
        mGetResidentCitiesListner = getResidentCitiesListner;
        int eventId = RequestDispathUtil.createEventId();
        String url = "residentArea/getResidentCities";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("deviceNo", Config.VIN);

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_RESIDENT_CITIES);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }
    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, BigDataService.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }
    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GET_RESIDENT_CITIES://1
                if(mGetResidentCitiesListner != null){
                    mGetResidentCitiesListner.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GET_RESIDENT_CITIES://1
                if(mGetResidentCitiesListner != null){
                    if (hasDSPTError) {
                        mGetResidentCitiesListner.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<ResidentCityInfo> radiosList = null;
                            if(jsonObject.has("data")){
                                JSONObject dataObj = jsonObject.getJSONObject("data");
                                Gson gson = new Gson();
                                Type jsonType = new TypeToken<List<ResidentCityInfo>>() {}.getType();
                                radiosList = gson.fromJson(dataObj.getJSONArray("cityList").toString(), jsonType);
                            }

                            mGetResidentCitiesListner.onGetResidentCities(radiosList);
                        } else {
                            String msg = jsonObject.getString("message");
                            mGetResidentCitiesListner.onFail(respCode, msg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetResidentCitiesListner.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
