package com.mxnavi.tspv2.carauth;

import com.google.gson.Gson;
import com.mxnavi.tspv2.carauth.model.CarActivateResult;
import com.mxnavi.tspv2.carauth.model.CarRealTimeInfo;
import com.mxnavi.tspv2.carauth.model.ListVehicleInfoResult;
import com.mxnavi.tspv2.carauth.model.MapAuthResult;
import com.mxnavi.tspv2.carauth.model.QueryMyCarInfoResult;
import com.mxnavi.tspv2.carauth.model.BindingMyCarInfo;
import com.mxnavi.tspv2.carauth.model.CarInfo;
import com.mxnavi.tspv2.carauth.model.ServiceList;
import com.mxnavi.tspv2.carauth.model.UnBindingMyCar;
import com.mxnavi.tspv2.carauth.model.UnbindUserVehicleInfo;
import com.mxnavi.tspv2.carauth.model.UpdateMyCarInfo;
import com.mxnavi.tspv2.carauth.model.VehicleActivateQuery;
import com.mxnavi.tspv2.carauth.model.VehicleInfo;
import com.mxnavi.tspv2.carauth.model.VehiclePositionInfo;
import com.mxnavi.tspv2.carauth.model.VerifyUserVehicleQuery;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.security.auth.callback.Callback;

import static com.mxnavi.tspv2.core.Constants.MISSING_PARAMETER_ERROR_CODE;
import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/1/31.
 */

public class CarAuth extends BaseTsp {
    // 请求服务类型为鉴权服务
    final int thisServiceType = Config.MxNGTPServiceType.MX_AUTH.getServiceType();
    //  鉴权服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_AUTH.getServiceVer();
    // 请求服务类型为用户鉴权服务
    final int carAuthServiceType = Config.MxNGTPServiceType.USER_AUTH.getServiceType();
    // 用户鉴权服务的ServiceData协议版本
    final int carAuth_serviceDataVer = Config.MxNGTPServiceType.USER_AUTH.getServiceVer();
    // 请求服务类型为监控集成服务 对应mx_si_framework
    final int monitorIntegratorServiceType = Config.MxNGTPServiceType.MONITOR_INTEGRATOR.getServiceType();
    // 监控集成服务的ServiceData协议版本
    final int monitorIntegrator_serviceDataVer = Config.MxNGTPServiceType.MONITOR_INTEGRATOR.getServiceVer();
    // 车辆鉴权服务 (新增)
    final int vehicleServiceType = Config.MxNGTPServiceType.MX_VEHICLE.getServiceType();
    final int vehicleServiceDataVer = Config.MxNGTPServiceType.MX_VEHICLE.getServiceVer();
    // SI 鉴权服务
    final int siServiceType = Config.MxNGTPServiceType.MX_SI.getServiceType();
    final int siServiceDataVer = Config.MxNGTPServiceType.MX_SI.getServiceVer();

    public CarAuth() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int QUERYMYCAR = 1;//查询用户车辆关系
    final int UPDATEMYCAR = 2;//修改个人用户车辆关系信息
    final int BINDINGMYCAR = 3;//绑定个人用户车辆关系(单个绑定)
    final int UNBINDINGMYCAR = 4;//解绑我的爱车
    final int GETVEHICLEREALTIMEINFOFORAPP = 5;//APP端查询车辆实时信息
    final int UNBINDUSERVEHICLE = 6 ;//解绑用户车辆关系
    final int VERIFY_PIN = 7 ;//PIN码验证
    final int GETVEHICLEPOSITIONFORAPP = 8;//APP端查询车辆位置
    final int GET_LIST_VEHICLE_INFO = 9; // 查询车辆信息列表
    final int GET_VERIFY_USER_VEHICLE = 10; // 审核用户车辆关系
    final int IS_VEHICLE_ACTIVATE = 11 ; // 查询车辆是否激活
    final int ACTIVATE_VEHICLE = 12; // 整车激活
    final int GET_MAP_AUTH = 13; // 获取VIN服务鉴权
    final int APP_VEHICLE_ACTIVATE = 14; // app扫码激活车辆信息

    /**
     * 查询用户车辆关系
     * @param userId 用户ID
     * @return 请求ID
     */
    public int queryMyCar(String userId, String token) {
        return queryMyCar(userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询用户车辆关系
     * @param userId 用户ID
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int queryMyCar(String userId, String token, int timeOutSecs) {
        return queryMyCar(userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询用户车辆关系
     * @param userId 用户ID
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int queryMyCar(String userId, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)) {
            if (mQueryMyCarListener != null) {
                mQueryMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/vehicle/userVehicle/listUserVehicle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, this.monitorIntegratorServiceType, this.monitorIntegrator_serviceDataVer, 
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, QUERYMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改个人用户车辆关系信息
     * @param updateMyCarInfo
     * @return 请求ID
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo, String token) {
        return updateMyCar(updateMyCarInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户车辆关系信息
     * @param updateMyCarInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo, String token, int timeOutSecs) {
        return updateMyCar(updateMyCarInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户车辆关系信息
     * @param updateMyCarInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo, String token, int timeOutSecs, int priorityLevel) {
        if (updateMyCarInfo == null
                ||updateMyCarInfo.getEngineNum() == null
                || updateMyCarInfo.getInvoicePhoto() == null
                || updateMyCarInfo.getOldVin() == null
                || updateMyCarInfo.getUserId() == null
                || updateMyCarInfo.getVin() == null
                || Utils.strIsEmpty(token)) {
            if (mUpdateMyCarListener != null) {
                mUpdateMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/vehicle/userVehicle/updateBindUserVehicle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("userId", updateMyCarInfo.getUserId());
        params.put("vin", updateMyCarInfo.getVin());
        params.put("oldVin", updateMyCarInfo.getOldVin());
        params.put("invoicePhoto", updateMyCarInfo.getInvoicePhoto());
        params.put("engineNum", updateMyCarInfo.getEngineNum());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                updateMyCarInfo.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 绑定个人用户车辆关系(单个绑定)
     * @param bindingMyCarInfo
     * @return 请求ID
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo, String token) {
        return bindingMyCar(bindingMyCarInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定个人用户车辆关系(单个绑定)
     * @param bindingMyCarInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo, String token, int timeOutSecs) {
        return bindingMyCar(bindingMyCarInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定个人用户车辆关系(单个绑定)
     * @param bindingMyCarInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo, String token, int timeOutSecs, int priorityLevel) {
        if (bindingMyCarInfo == null
                || bindingMyCarInfo.getEngineNum() == null
                || bindingMyCarInfo.getInvoicePhoto() == null
                || bindingMyCarInfo.getUserId() == null
                || bindingMyCarInfo.getVin() == null
                || Utils.strIsEmpty(token)) {
            if (mBindingMyCarListener != null) {
                mBindingMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/vehicle/userVehicle/saveBindUserVehicle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if(bindingMyCarInfo.getProductId()!=null&&bindingMyCarInfo.getProductId().length()>0){
            params.put("productId", bindingMyCarInfo.getProductId());
        }else{
            params.put("productId", Config.CUSTOMER_KEY);
        }
        params.put("userId", bindingMyCarInfo.getUserId());
        params.put("vin", bindingMyCarInfo.getVin());
        params.put("invoicePhoto", bindingMyCarInfo.getInvoicePhoto());
        params.put("engineNum", bindingMyCarInfo.getEngineNum());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                bindingMyCarInfo.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, BINDINGMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 解绑我的爱车
     * @param unBindingMyCar
     * @return 请求ID
     */
    public int unBindingMyCar(UnBindingMyCar unBindingMyCar, String token) {
        return unBindingMyCar(unBindingMyCar, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 解绑我的爱车
     * @param unBindingMyCar
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int unBindingMyCar(UnBindingMyCar unBindingMyCar, String token, int timeOutSecs) {
        return unBindingMyCar(unBindingMyCar, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 解绑我的爱车
     * @param unBindingMyCar
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int unBindingMyCar(UnBindingMyCar unBindingMyCar, String token, int timeOutSecs, int priorityLevel) {
        if (unBindingMyCar == null
                || unBindingMyCar.getUserId() == null
                || unBindingMyCar.getValidateCode() == null
                || unBindingMyCar.getValidSessionId() == null
                || unBindingMyCar.getPhone() ==null
                || Utils.strIsEmpty(token)) {
            if (mUnBindingMyCarListener != null) {
                mUnBindingMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/userVehicle/vehicle/unbindUserVehicleByValidateCode";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if(unBindingMyCar.getProductId()!=null&&unBindingMyCar.getProductId().length()>0){
            params.put("productId", unBindingMyCar.getProductId());
        }else{
            params.put("productId", Config.CUSTOMER_KEY);
        }
        params.put("userId", unBindingMyCar.getUserId());
        params.put("validateCode", unBindingMyCar.getValidateCode());
        params.put("validSessionId", unBindingMyCar.getValidSessionId());
        if (unBindingMyCar.getVins() != null){
            params.put("vins", unBindingMyCar.getVins());
        }
        params.put("phone", unBindingMyCar.getPhone());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                unBindingMyCar.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UNBINDINGMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * APP端查询车辆实时信息
     * @param carInfo
     * @param token
     * @return 请求ID
     */
    public int getVehicleRealTimeInfo(CarInfo carInfo, String token) {
        return getVehicleRealTimeInfo(carInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP端查询车辆实时信息
     * @param carInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int getVehicleRealTimeInfo(CarInfo carInfo, String token, int timeOutSecs) {
        return getVehicleRealTimeInfo(carInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP端查询车辆实时信息
     * @param carInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int getVehicleRealTimeInfo(CarInfo carInfo, String token, int timeOutSecs, int priorityLevel) {
        if (carInfo == null
                || carInfo.getUserId() == null
                || carInfo.getTypeId() == null
                || carInfo.getVehicleBrandId() == null
                || carInfo.getVehicleModelId() == null
                || carInfo.getVehicleSeriesId() == null
                || carInfo.getVin() == null
                || Utils.strIsEmpty(token)) {
            if (mInquireVehicleRealTimeInfoListener != null) {
                mInquireVehicleRealTimeInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "vehicle/getVehicleRealTimeInfoForApp";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", carInfo.getVin());
        params.put("vehicleBrandId", carInfo.getVehicleBrandId());
        params.put("vehicleSeriesId", carInfo.getVehicleSeriesId());
        params.put("vehicleModelId", carInfo.getVehicleModelId());
        if(carInfo.getProductId()!=null&&carInfo.getProductId().length()>0){
            params.put("productId", carInfo.getProductId());
        }else{
            params.put("productId", Config.CUSTOMER_KEY);
        }

        params.put("userId", carInfo.getUserId());
        params.put("typeId", carInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                carInfo.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETVEHICLEREALTIMEINFOFORAPP);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 解绑用户车辆关系
     * @param unbindUserVehicleInfo
     * @return 请求ID
     */
    public int unbindUserVehicle(UnbindUserVehicleInfo unbindUserVehicleInfo) {
        return unbindUserVehicle(unbindUserVehicleInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 解绑用户车辆关系
     * @param unbindUserVehicleInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int unbindUserVehicle(UnbindUserVehicleInfo unbindUserVehicleInfo, int timeOutSecs) {
        return unbindUserVehicle(unbindUserVehicleInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 解绑用户车辆关系
     * @param unbindUserVehicleInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int unbindUserVehicle(UnbindUserVehicleInfo unbindUserVehicleInfo, int timeOutSecs, int priorityLevel) {
        if (unbindUserVehicleInfo == null
                || unbindUserVehicleInfo.getUserId() == null
                || unbindUserVehicleInfo.getVins() == null
                || unbindUserVehicleInfo.getToken() == null) {
            if (mUnbindUserVehicle != null) {
                mUnbindUserVehicle.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/vehicle/userVehicle/unbindUserVehicle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if(unbindUserVehicleInfo.getProductId()!=null&&unbindUserVehicleInfo.getProductId().length()>0){
            params.put("productId", unbindUserVehicleInfo.getProductId());
        }else{
            params.put("productId", Config.CUSTOMER_KEY);
        }
        params.put("userId", unbindUserVehicleInfo.getUserId());
        params.put("vins", unbindUserVehicleInfo.getVins());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                unbindUserVehicleInfo.getUserId(), unbindUserVehicleInfo.getToken(), requestData, vehicleServiceType, vehicleServiceDataVer, 
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UNBINDUSERVEHICLE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * PIN码验证
     * @param pin pin码
     * @param userId 用户id
     * @param token 用户登录成功后的token
     * @return
     */
    public int verifyPin(String pin,String userId, String token){
        return verifyPin(pin, userId,token,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * PIN码验证
     * @param pin pin码
     * @param userId 用户id
     * @param token 用户登录成功后的token
     * @param timeOutSecs 接口超时时间
     * @return
     */
    public int verifyPin(String pin,String userId, String token,int timeOutSecs) {
        return verifyPin(pin, userId,token,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * PIN码验证
     * @param pin pin码
     * @param userId 用户id
     * @param token 用户登录成功后的token
     * @param timeOutSecs 接口超时时间
     * @param priorityLevel 接口优先级
     * @return
     */
    public int verifyPin(String pin,String userId, String token, int timeOutSecs, int priorityLevel){
        if (pin == null
                || token == null
                || userId== null) {
            if (mVerifyPinListener != null) {
                mVerifyPinListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/userVehicle/user/verifyPin";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("pinCode", pin);
        params.put("userId", userId);
        params.put("token", token);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, VERIFY_PIN);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;

    }

    public int getVehiclePositionForApp(CarInfo carInfo, String token) {
        return getVehiclePositionForApp(carInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int getVehiclePositionForApp(CarInfo carInfo, String token, int timeOutSecs) {
        return getVehiclePositionForApp(carInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    public int getVehiclePositionForApp(CarInfo carInfo, String token, int timeOutSecs, int priorityLevel) {
        if (carInfo == null
                || carInfo.getUserId() == null
                || carInfo.getTypeId() == null
                || carInfo.getVehicleBrandId() == null
                || carInfo.getVehicleModelId() == null
                || carInfo.getVehicleSeriesId() == null
                || carInfo.getVin() == null
                || Utils.strIsEmpty(token)) {
            if (mGetVehiclePositionForAppListener != null) {
                mGetVehiclePositionForAppListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "vehicle/getVehiclePositionForApp";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", carInfo.getVin());
        params.put("vehicleBrandId", carInfo.getVehicleBrandId());
        params.put("vehicleSeriesId", carInfo.getVehicleSeriesId());
        params.put("vehicleModelId", carInfo.getVehicleModelId());
        if(carInfo.getProductId()!=null&&carInfo.getProductId().length()>0){
            params.put("productId", carInfo.getProductId());
        }else{
            params.put("productId", Config.CUSTOMER_KEY);
        }

        params.put("userId", carInfo.getUserId());
        params.put("typeId", carInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                carInfo.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETVEHICLEPOSITIONFORAPP);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /* 查询车辆信息列表 */
    public int getListVehicleInfo(VehicleInfo vehicleInfo, String token) {
        return getListVehicleInfo(vehicleInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /* 查询车辆信息列表 */
    public int getListVehicleInfo(VehicleInfo vehicleInfo, String token, int timeOutSecs) {
        return getListVehicleInfo(vehicleInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /* 查询车辆信息列表 */
    public int getListVehicleInfo(VehicleInfo vehicleInfo, String token, int timeOutSecs, int priorityLevel) {
        if (vehicleInfo == null
                || vehicleInfo.getUserId() == null
                || vehicleInfo.getProductId() == null
                || vehicleInfo.getTypeId() == null
                || Utils.strIsEmpty(token)) {
            if (mGetListVehicleInfoListener != null) {
                mGetListVehicleInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "vehicle/getVehicleStaticDataList";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if (vehicleInfo.getProductId() != null && vehicleInfo.getProductId().length() > 0) {
            params.put("productId", vehicleInfo.getProductId());
        } else {
            params.put("productId", Config.CUSTOMER_KEY);
        }
        params.put("typeId", vehicleInfo.getTypeId());
        if (!Utils.strIsEmpty(vehicleInfo.getUserId())) {
            params.put("userId", vehicleInfo.getUserId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getGroupId())) {
            params.put("groupId", vehicleInfo.getGroupId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getFleetId())) {
            params.put("fleetId", vehicleInfo.getFleetId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVin())) {
            params.put("vin", vehicleInfo.getVin());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVehicleBrandId())) {
            params.put("vehicleBrandId", vehicleInfo.getVehicleBrandId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVehicleSeriesId())) {
            params.put("vehicleSeriesId", vehicleInfo.getVehicleSeriesId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVehicleModelId())) {
            params.put("vehicleModelId", vehicleInfo.getVehicleModelId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getFleetIds())) {
            params.put("fleetIds", vehicleInfo.getFleetIds());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getManufactureDateStart())) {
            params.put("manufactureDateStart", vehicleInfo.getManufactureDateStart());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getManufactureDateEnd())) {
            params.put("manufactureDateEnd", vehicleInfo.getManufactureDateEnd());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getTotalMileage())) {
            params.put("totalMileage", vehicleInfo.getTotalMileage());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getDealerProvince())) {
            params.put("dealerProvince", vehicleInfo.getDealerProvince());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getDealerCity())) {
            params.put("dealerCity", vehicleInfo.getDealerCity());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getTboxId())) {
            params.put("tboxId", vehicleInfo.getTboxId());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getTboxSoftwareVer())) {
            params.put("tboxSoftwareVer", vehicleInfo.getTboxSoftwareVer());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVehicleCategory())) {
            params.put("vehicleCategory", vehicleInfo.getVehicleCategory());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getVehicleOwnerType())) {
            params.put("vehicleOwnerType", vehicleInfo.getVehicleOwnerType());
        }
        if (!Utils.strIsEmpty(vehicleInfo.getReportFlag())) {
            params.put("reportFlag", vehicleInfo.getReportFlag());
        }

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                vehicleInfo.getUserId(), token, requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_LIST_VEHICLE_INFO);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 审核用户车辆关系
     * @param verifyUserVehicleQuery
     * @param token
     * @return
     */
    public int getVerifyUserVehicle(VerifyUserVehicleQuery verifyUserVehicleQuery,String token) {
        return getVerifyUserVehicle(verifyUserVehicleQuery,token,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 审核车辆用户关系
     * @param verifyUserVehicleQuery
     * @param token
     * @param timeOutSecs
     * @return
     */
    public int getVerifyUserVehicle(VerifyUserVehicleQuery verifyUserVehicleQuery,String token,int timeOutSecs) {
        return getVerifyUserVehicle(verifyUserVehicleQuery,token,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 审核车辆用户关系
     * @param verifyUserVehicleQuery
     * @param token
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getVerifyUserVehicle(VerifyUserVehicleQuery verifyUserVehicleQuery,String token,int timeOutSecs,int priorityLevel) {
        if (verifyUserVehicleQuery == null || verifyUserVehicleQuery.getUserId() == null
                || verifyUserVehicleQuery.getProductId() == null
                || verifyUserVehicleQuery.getVin() == null
                || verifyUserVehicleQuery.getState() == null ){
            if (mGetListVerifyUserVehicleListener != null) {
                mGetListVerifyUserVehicleListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "vehicle/userVehicle/verifyUserVehicle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if (verifyUserVehicleQuery.getProductId() != null && verifyUserVehicleQuery.getProductId().length() > 0) {
            params.put("productId", verifyUserVehicleQuery.getProductId());
        } else {
            params.put("productId", Config.CUSTOMER_KEY);
        }
        if (!Utils.strIsEmpty(verifyUserVehicleQuery.getUserId())){
            params.put("userId", verifyUserVehicleQuery.getUserId());
        }
        if (!Utils.strIsEmpty(verifyUserVehicleQuery.getVin())){
            params.put("vin", verifyUserVehicleQuery.getVin());
        }
        if (!Utils.strIsEmpty(verifyUserVehicleQuery.getState())){
            params.put("state", verifyUserVehicleQuery.getState());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                verifyUserVehicleQuery.getUserId(), token, requestData, vehicleServiceType, vehicleServiceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_VERIFY_USER_VEHICLE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询车辆是否激活
     * @param
     * @param
     * @return
     */
    public int getVehicleActivate() {
        return getVehicleActivate(null, null, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车辆是否激活
     * @param productId
     * @param vin
     * @return
     */
    public int getVehicleActivate(String productId, String vin ) {
        return getVehicleActivate(productId, vin, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车辆是否激活
     * @param productId
     * @param vin
     * @param timeOutSecs
     * @return
     */
    public int getVehicleActivate(String productId, String vin, int timeOutSecs ) {
        return getVehicleActivate(productId, vin, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车辆是否激活
     * @param productId
     * @param vin
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getVehicleActivate(String productId, String vin, int timeOutSecs, int priorityLevel) {
        int eventId = RequestDispathUtil.createEventId();
        String url = "isVehicleActivate";
        LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
        if (!Utils.strIsEmpty(productId)){
            params.put("productId", productId);
        } else {
            params.put("productId", Config.CUSTOMER_KEY);
        }
        if (!Utils.strIsEmpty(vin)){
            params.put("vin", vin);
        } else {
            params.put("vin", Config.VIN);
        }
        String requestData = Utils.makeRestFulUrlSuffix(params, url);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, thisServiceType, serviceDataVer
                );
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, IS_VEHICLE_ACTIVATE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 整车激活
     * @return
     */
    public int activateVehicle() {
        return activateVehicle(null, null, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 整车激活
     * @param productId
     * @param vin
     * @param timeOutSecs
     * @return
     */
    public int activateVehicle(String productId, String vin, int timeOutSecs) {
        return activateVehicle(productId, vin, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 整车激活
     * @param productId
     * @param vin
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int activateVehicle(String productId, String vin, int timeOutSecs, int priorityLevel) {
        int eventId = RequestDispathUtil.createEventId();
        String url = "activateVehicleService";
        LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
        if(!Utils.strIsEmpty(productId)){
            params.put("productId", productId);
        } else {
            params.put("productId", Config.CUSTOMER_KEY);
        }
        if (!Utils.strIsEmpty(vin)){
            params.put("vin", vin);
        } else {
            params.put("vin", Config.VIN);
        }
        String requestData = Utils.makeRestFulUrlSuffix(params,url);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, ACTIVATE_VEHICLE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 获取VIN服务鉴权
     * @return
     */
    public int getMapAuth(){
        int eventId = RequestDispathUtil.createEventId();
        String url = "validateServiceAuth";
        LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("clientType", Config.DEVICE_TYPE);
        params.put("clientId", Config.VIN);
        params.put("serviceIds", "73");
        String requestData = Utils.makeRestFulUrlSuffix(params, url);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        //添加对应关系
        addCallbackMap(eventId, GET_MAP_AUTH);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = Config.DEFAULT_TIMEOUT_SECS;
        requestMessage.priority = Config.DEFAULT_PRIORITY;
        return eventId;
    }

    /**
     * app扫码激活车辆信息
     * @param vehicleActivateQuery
     * @return
     */
    public int appVehicleActivate(VehicleActivateQuery vehicleActivateQuery){
        return appVehicleActivate(vehicleActivateQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * app扫码激活车辆信息
     * @param vehicleActivateQuery
     * @param timeOutSecs
     * @return
     */
    public int appVehicleActivate(VehicleActivateQuery vehicleActivateQuery, int timeOutSecs){
        return appVehicleActivate(vehicleActivateQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * app扫码激活车辆信息
     * @param vehicleActivateQuery
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int appVehicleActivate(VehicleActivateQuery vehicleActivateQuery, int timeOutSecs, int priorityLevel){
        if (vehicleActivateQuery == null || vehicleActivateQuery.getProductId() == null
                || vehicleActivateQuery.getUserId() == null || vehicleActivateQuery.getVin() == null
                || vehicleActivateQuery.getEngineNum() == null || vehicleActivateQuery.getTypeId() == null ){
            if (mOnAppVehicleActivateListener != null ){
                mOnAppVehicleActivateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "vehicleActive/appVehicleActive";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", vehicleActivateQuery.getProductId());
        params.put("userId", vehicleActivateQuery.getUserId());
        params.put("vin", vehicleActivateQuery.getVin());
        params.put("engineNum", vehicleActivateQuery.getEngineNum());
        params.put("typeId", vehicleActivateQuery.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, monitorIntegratorServiceType, monitorIntegrator_serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, APP_VEHICLE_ACTIVATE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    public interface QueryMyCarListener extends CallBackListener {
        /**
         * 查询我的爱车
         *
         * @param list 爱车信息列表
         */
        public void onQueryMyCar(List<QueryMyCarInfoResult> list);
    }
    public interface UpdateMyCarListener extends CallBackListener {
        /**
         * 修改我的爱车成功
         */
        public void onUpdateMyCarSuccess(int respCode);
    }
    public interface BindingMyCarListener extends CallBackListener {
        /**
         * 绑定我的爱车成功
         */
        public void onBindingMyCarSuccess(int resCode);
    }
    public interface UnBindingMyCarListener extends CallBackListener {
        /**
         * 解绑我的爱车成功
         */
        public void onUnBindingMyCarSuccess();
    }
    public interface InquireVehicleRealTimeInfoListener extends CallBackListener {
        /**
         * APP端查询车辆实时信息
         */
        public void onInquireVehicleRealTimeInfoSuccess(CarRealTimeInfo carRealTimeInfo);
    }
    public interface UnbindUserVehicle extends CallBackListener {
        /**
         * 解绑用户车辆关系
         */
        public void onUnbindUserVehicleSuccess();
    }
    public  interface VerifyPinListener extends CallBackListener {
        /**
         * 解绑用户车辆关系
         */
        public void onVerifyPinSuccess();

        /**
         * 验证失败
         * @param errorCode -100: 失败，未知错误,-102: 缺失参数,10000用户不存在,10002用户ID与登录不匹配,30008 PIN密码不正确
         * 30009 PIN密码为空，请进行首次设置
         * @param errorMsg
         */
        public void onFail(int errorCode, String errorMsg);
    }
    public interface GetVehiclePositionForAppListener extends CallBackListener {
        /**
         * APP端查询车辆位置
         * @param vehiclePositionInfo
         */
        public void onGetVehiclePositionForApp(VehiclePositionInfo vehiclePositionInfo);
    }

    public interface GetListVehicleInfoListener extends CallBackListener {
        /**
         * 查询车辆信息列表
         * @param listVehicleInfoResult
         */
        public void onGetVehiclePositionForApp(List<ListVehicleInfoResult> listVehicleInfoResult);
    }

    public interface GetListVerifyUserVehicleListener extends CallBackListener {
        /**
         * 审核用户车辆关系
         * @param
         */
        public void onGetVerifyUserVehicle();
    }

    public interface GetIsVehicleActivateListener extends CallBackListener {
        /**
         * 查询车辆是否激活
         * @param carActivateResult
         */
        public void isVehicleActivate(CarActivateResult carActivateResult);
    }
    public interface ActivateVehicleListener extends CallBackListener {
        /**
         * 整车激活
         */
        public void onActivateVehicle();
    }

    public interface OnGetMapAuthListener extends CallBackListener {
        /**
         * 获取VIN服务鉴权信息
         * @param mapAuthResult
         */
        public void onGetMapAuth(MapAuthResult mapAuthResult);
    }

    public interface OnAppVehicleActivateListener extends CallBackListener {
        /**
         * 获取app扫描激活车辆信息
         * @param requestId
         */
        public void onAppVehicleActivate(int requestId);
    }

    QueryMyCarListener mQueryMyCarListener;//查询我的爱车信息
    UpdateMyCarListener mUpdateMyCarListener;//修改我的爱车信息
    BindingMyCarListener mBindingMyCarListener;//绑定我的爱车
    UnBindingMyCarListener mUnBindingMyCarListener;//解绑我的爱车
    InquireVehicleRealTimeInfoListener mInquireVehicleRealTimeInfoListener;//APP端查询车辆实时信息
    UnbindUserVehicle mUnbindUserVehicle;//解绑用户车辆关系
    VerifyPinListener mVerifyPinListener;
    GetVehiclePositionForAppListener mGetVehiclePositionForAppListener;
    GetListVehicleInfoListener mGetListVehicleInfoListener; // 查询车辆信息列表
    GetListVerifyUserVehicleListener mGetListVerifyUserVehicleListener; // 审核用户车辆关系
    GetIsVehicleActivateListener mGetIsVehicleActivateListener; // 查询车辆是否激活
    ActivateVehicleListener mActivateVehicleListener; // 整车激活
    OnGetMapAuthListener mOnGetMapAuthListener; // 获取VIN服务鉴权
    OnAppVehicleActivateListener mOnAppVehicleActivateListener; // app扫描激活车辆信息

    public void setQueryMyCarListener(QueryMyCarListener queryMyCarListener) {
        this.mQueryMyCarListener = queryMyCarListener;
    }
    public void setUpdateMyCarListener(UpdateMyCarListener updateMyCarListener) {
        this.mUpdateMyCarListener = updateMyCarListener;
    }
    public void setBindingMyCarListener(BindingMyCarListener bindingMyCarListener) {
        this.mBindingMyCarListener = bindingMyCarListener;
    }
    public void setUnBindingMyCarListener(UnBindingMyCarListener unBindingMyCarListener) {
        this.mUnBindingMyCarListener = unBindingMyCarListener;
    }
    public void setInquireVehicleRealTimeInfoListener(InquireVehicleRealTimeInfoListener inquireVehicleRealTimeInfoListener) {
        this.mInquireVehicleRealTimeInfoListener = inquireVehicleRealTimeInfoListener;
    }
    public void setUnbindUserVehicleListener(UnbindUserVehicle unbindUserVehicleListener) {
        this.mUnbindUserVehicle = unbindUserVehicleListener;
    }
    public void setVerifyPinListener(VerifyPinListener verifyPinListener){
        mVerifyPinListener = verifyPinListener;
    }
    public void setGetVehiclePositionForAppListener(GetVehiclePositionForAppListener getVehiclePositionForAppListener){
        mGetVehiclePositionForAppListener = getVehiclePositionForAppListener;
    }
    public void setGetListVehicleInfoListener(GetListVehicleInfoListener mGetListVehicleInfoListener) {
        this.mGetListVehicleInfoListener = mGetListVehicleInfoListener;
    }
    public void setGetListVerifyUserVehicleListener(GetListVerifyUserVehicleListener mGetListVerifyUserVehicleListener) {
        this.mGetListVerifyUserVehicleListener = mGetListVerifyUserVehicleListener;
    }
    public void setGetIsVehicleActivateListener(GetIsVehicleActivateListener mGetIsVehicleActivateListener) {
        this.mGetIsVehicleActivateListener = mGetIsVehicleActivateListener;
    }
    public void setActivateVehicleListener(ActivateVehicleListener mActivateVehicleListener) {
        this.mActivateVehicleListener = mActivateVehicleListener;
    }
    public void setOnGetMapAuthListener (OnGetMapAuthListener mOnGetMapAuthListener){
        this.mOnGetMapAuthListener= mOnGetMapAuthListener;
    }
    public void setOnAppVehicleActivateListener(OnAppVehicleActivateListener mOnAppVehicleActivateListener){
        this.mOnAppVehicleActivateListener = mOnAppVehicleActivateListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, CarAuth.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case QUERYMYCAR://1
                if (mQueryMyCarListener != null) {
                    mQueryMyCarListener.onTimeOut();
                }
                break;
            case UPDATEMYCAR://2
                if (mUpdateMyCarListener != null) {
                    mUpdateMyCarListener.onTimeOut();
                }
                break;
            case BINDINGMYCAR://3
                if (mBindingMyCarListener != null) {
                    mBindingMyCarListener.onTimeOut();
                }
                break;
            case UNBINDINGMYCAR://4
                if (mUnBindingMyCarListener != null) {
                    mUnBindingMyCarListener.onTimeOut();
                }
                break;
            case GETVEHICLEREALTIMEINFOFORAPP://5
                if (mInquireVehicleRealTimeInfoListener != null) {
                    mInquireVehicleRealTimeInfoListener.onTimeOut();
                }
                break;
            case UNBINDUSERVEHICLE://6
                if (mUnbindUserVehicle != null) {
                    mUnbindUserVehicle.onTimeOut();
                }
                break;
            case VERIFY_PIN://7
                if (mVerifyPinListener != null) {
                    mVerifyPinListener.onTimeOut();
                }
                break;
            case GETVEHICLEPOSITIONFORAPP://8
                if (mGetVehiclePositionForAppListener != null) {
                    mGetVehiclePositionForAppListener.onTimeOut();
                }
                break;
            case GET_LIST_VEHICLE_INFO:
                if (mGetListVehicleInfoListener != null) {
                    mGetListVehicleInfoListener.onTimeOut();
                }
                break;
            case GET_VERIFY_USER_VEHICLE:
                if (mGetListVerifyUserVehicleListener != null) {
                    mGetListVerifyUserVehicleListener.onTimeOut();
                }
                break;
            case IS_VEHICLE_ACTIVATE:
                if (mGetIsVehicleActivateListener != null) {
                    mGetIsVehicleActivateListener.onTimeOut();
                }
                break;
            case ACTIVATE_VEHICLE:
                if (mActivateVehicleListener != null ){
                    mActivateVehicleListener.onTimeOut();
                }
                break;
            case GET_MAP_AUTH :
                if (mOnGetMapAuthListener != null ){
                    mOnGetMapAuthListener.onTimeOut();
                }
                break;
            case APP_VEHICLE_ACTIVATE:
                if (mOnAppVehicleActivateListener != null){
                    mOnAppVehicleActivateListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case QUERYMYCAR://1
                if (mQueryMyCarListener != null) {
                    if (hasDSPTError) {
                        mQueryMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        List<QueryMyCarInfoResult> list = null;
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray jsonArray = jsonObject.getJSONArray("rows");
                            int size = jsonArray.length();
                            if (size>0){
                                list = new ArrayList<QueryMyCarInfoResult>(size);
                                for (int i= 0; i < size; i++) {
                                    JSONObject dataObject = jsonArray.getJSONObject(i);
                                    QueryMyCarInfoResult info = new QueryMyCarInfoResult();
                                    if (dataObject.has("userId  ")) {
                                        info.setUserId(dataObject.getString("userId"));
                                    }
                                    if (dataObject.has("productId")) {
                                        info.setProductId(dataObject.getString("productId"));
                                    }
                                    if (dataObject.has("vin")) {
                                        info.setVin(dataObject.getString("vin"));
                                    }
                                    if (dataObject.has("invoicePhoto")) {
                                        info.setInvoicePhoto(dataObject.getString("invoicePhoto"));
                                    }
                                    if (dataObject.has("state")) {
                                        info.setState(dataObject.getString("state"));
                                    }
                                    if (dataObject.has("createTime")) {
                                        info.setCreateTime(dataObject.getString("createTime"));
                                    }
                                    if (dataObject.has("engineNum")) {
                                        info.setEngineNum(dataObject.getString("engineNum"));
                                    }
                                    if (dataObject.has("motorNum")) {
                                        info.setMotorNum(dataObject.getString("motorNum"));
                                    }
                                    if (dataObject.has("licensePlate")) {
                                        info.setLicensePlate(dataObject.getString("licensePlate"));
                                    }
                                    if (dataObject.has("vehicleBrandName")) {
                                        info.setVehicleBrandName(dataObject.getString("vehicleBrandName"));
                                    }
                                    if (dataObject.has("vehicleSeriesName")) {
                                        info.setVehicleSeriesName(dataObject.getString("vehicleSeriesName"));
                                    }
                                    if (dataObject.has("vehicleModelName")) {
                                        info.setVehicleModelName(dataObject.getString("vehicleModelName"));
                                    }
                                    if (dataObject.has("reason")) {
                                        info.setReason(dataObject.getString("reason"));
                                    }
                                    if (dataObject.has("vehicleBrandId")) {
                                        info.setVehicleBrandId(dataObject.getString("vehicleBrandId"));
                                    }
                                    if (dataObject.has("vehicleSeriesId")) {
                                        info.setVehicleSeriesId(dataObject.getString("vehicleSeriesId"));
                                    }
                                    if (dataObject.has("vehicleModelId")) {
                                        info.setVehicleModelId(dataObject.getString("vehicleModelId"));
                                    }
                                    list.add(info);
                                }
                            }
                            mQueryMyCarListener.onQueryMyCar(list);
                        }else {
                            mQueryMyCarListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATEMYCAR:
                if (mUpdateMyCarListener != null) {
                    if (hasDSPTError) {
                        mUpdateMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateMyCarListener.onUpdateMyCarSuccess(respCode);
                        }else {
                            mUpdateMyCarListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mUpdateMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case BINDINGMYCAR://3
                if (mBindingMyCarListener != null) {
                    if (hasDSPTError) {
                        mBindingMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mBindingMyCarListener.onBindingMyCarSuccess(respCode);
                        }else {
                            mBindingMyCarListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UNBINDINGMYCAR://4
                if (mUnBindingMyCarListener != null) {
                    if (hasDSPTError) {
                        mUnBindingMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUnBindingMyCarListener.onUnBindingMyCarSuccess();
                        }else {
                            mUnBindingMyCarListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETVEHICLEREALTIMEINFOFORAPP://5
                if (mInquireVehicleRealTimeInfoListener != null) {
                    if (hasDSPTError) {
                        mInquireVehicleRealTimeInfoListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {

                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            CarRealTimeInfo carRealTimeInfo = null;
                            if (jsonObject.has("data")){
                                LatLng latLng = new LatLng();
                                carRealTimeInfo = new CarRealTimeInfo();
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                if (dataObject.has("vin")){
                                    carRealTimeInfo.setVin(dataObject.getString("vin"));
                                }
                                if (dataObject.has("licensePlate")){
                                    carRealTimeInfo.setLicensePlate(dataObject.getString("licensePlate"));
                                }
                                if (dataObject.has("positionStatus")){
                                    carRealTimeInfo.setPositionStatus(dataObject.getString("positionStatus"));
                                }
                                if (dataObject.has("longitude")){
                                    latLng.setLongitude(dataObject.getDouble("longitude"));
                                }
                                if (dataObject.has("latitude")){
                                    latLng.setLatitude(dataObject.getDouble("latitude"));
                                }
                                if (dataObject.has("longitude") || dataObject.has("latitude")){
                                    carRealTimeInfo.setLatLng(latLng);
                                }
                                if (dataObject.has("vehiclePosition")){
                                    carRealTimeInfo.setVehiclePosition(dataObject.getString("vehiclePosition"));
                                }
                                if (dataObject.has("provinceCode")){
                                    carRealTimeInfo.setProvinceCode(dataObject.getString("provinceCode"));
                                }
                                if (dataObject.has("cityCode")){
                                    carRealTimeInfo.setCityCode(dataObject.getString("cityCode"));
                                }
                                if (dataObject.has("totalMileage")){
                                    carRealTimeInfo.setTotalMileage(dataObject.getString("totalMileage"));
                                }
                                if (dataObject.has("rechargeMileage")){
                                    carRealTimeInfo.setRechargeMileage(dataObject.getString("rechargeMileage"));
                                }
                                if (dataObject.has("highestAlarmGrade")){
                                    carRealTimeInfo.setHighestAlarmGrade(dataObject.getString("highestAlarmGrade"));
                                }
                                if (dataObject.has("onlineStatus")){
                                    carRealTimeInfo.setOnlineStatus(dataObject.getInt("onlineStatus"));
                                }
                                if (dataObject.has("chargingStatus")){
                                    carRealTimeInfo.setChargingStatus(dataObject.getInt("chargingStatus"));
                                }
                                if (dataObject.has("soc")){
                                    carRealTimeInfo.setSoc(dataObject.getInt("soc"));
                                }
                            }
                            mInquireVehicleRealTimeInfoListener.onInquireVehicleRealTimeInfoSuccess(carRealTimeInfo);
                        }else {
                            mInquireVehicleRealTimeInfoListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mInquireVehicleRealTimeInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UNBINDUSERVEHICLE://6
                if (mUnbindUserVehicle != null) {
                    if (hasDSPTError) {
                        mUnbindUserVehicle.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUnbindUserVehicle.onUnbindUserVehicleSuccess();
                        }else {
                            mUnbindUserVehicle.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mUnbindUserVehicle.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case VERIFY_PIN://7
                if (mVerifyPinListener != null) {
                    if (hasDSPTError) {
                        mVerifyPinListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mVerifyPinListener.onVerifyPinSuccess();
                        }else {
                            mVerifyPinListener.onFail(respCode, "");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mVerifyPinListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
//            List<SearchUserListResult> list = new ArrayList<SearchUserListResult>();
//            SearchUserListResult result;
//            JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
//            int respCode = jsonObject.getInt("respCode");
//            if (respCode == Constants.RESPCODE_SUCCESS) {
//                JSONArray userInfoList = jsonObject.getJSONArray("user_info_list");
//                for (int i=0;i<userInfoList.length();i++){
//                    JSONObject userInfo = userInfoList.getJSONObject(i);
//                    result = new SearchUserListResult();
//                    result.setAppId(userInfo.getString("appId"));
//                    if (userInfo.has("groupid")){
//                        result.setGroupId(userInfo.getString("groupid"));
//                    }
            case GETVEHICLEPOSITIONFORAPP://8
                if (mGetVehiclePositionForAppListener != null) {
                    if (hasDSPTError) {
                        mGetVehiclePositionForAppListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            VehiclePositionInfo vehiclePositionInfo = null;
                            if (jsonObject.has("data")){
                                LatLng latLng = new LatLng();
                                vehiclePositionInfo = new VehiclePositionInfo();
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                if (dataObject.has("collectionTime")){
                                    vehiclePositionInfo.setCollectionTime(dataObject.getString("collectionTime"));
                                }
                                if (dataObject.has("direction")){
                                    vehiclePositionInfo.setDirection(dataObject.getInt("direction"));
                                }
                                if (dataObject.has("longitude")){
                                    latLng.setLongitude(dataObject.getDouble("longitude"));
                                }
                                if (dataObject.has("latitude")){
                                    latLng.setLatitude(dataObject.getDouble("latitude"));
                                }
                                if (dataObject.has("longitude") || dataObject.has("latitude")){
                                    vehiclePositionInfo.setLatLng(latLng);
                                }
                                if (dataObject.has("onlineStatus")){
                                    vehiclePositionInfo.setOnlineStatus(dataObject.getInt("onlineStatus"));
                                }
                            }
                            mGetVehiclePositionForAppListener.onGetVehiclePositionForApp(vehiclePositionInfo);
                        }else {
                            mGetVehiclePositionForAppListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetVehiclePositionForAppListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GET_LIST_VEHICLE_INFO:
                if (mGetListVehicleInfoListener != null) {
                    if (hasDSPTError) {
                        mGetListVehicleInfoListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<ListVehicleInfoResult> listVehicleInfoResult = decodeListVehicleInfoResult(jsonObject);
                            mGetListVehicleInfoListener.onGetVehiclePositionForApp(listVehicleInfoResult);
                        } else {
                            mGetListVehicleInfoListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mGetListVehicleInfoListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GET_VERIFY_USER_VEHICLE:
                if (mGetListVerifyUserVehicleListener != null) {
                    if (hasDSPTError){
                        mGetListVerifyUserVehicleListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject  = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            mGetListVerifyUserVehicleListener.onGetVerifyUserVehicle();
                        }else {
                            switch (respCode){
                                case -100:
                                    mGetListVerifyUserVehicleListener.onFail(respCode,errorMsg);
                                    break;
                                case -102:
                                    mGetListVerifyUserVehicleListener.onFail(respCode,errorMsg);
                                    break;
                                case Constants.OTHERS_USER_BINDED_FAIL_ERROR_CODE:
                                    mGetListVerifyUserVehicleListener.onFail(respCode, Constants.OTHERS_USER_BINDED_FAIL);
                                    break;
                                case Constants.OTHERS_GROUP_BINDED_FAIL_ERROR_CODE:
                                    mGetListVerifyUserVehicleListener.onFail(respCode,Constants.OTHERS_GROUP_BINDED_FAIL);
                                    break;
                                default:
                                    break;
                            }

                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetListVerifyUserVehicleListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,
                                Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            case IS_VEHICLE_ACTIVATE:
                if (mGetIsVehicleActivateListener != null ){
                    if (hasDSPTError){
                        mGetIsVehicleActivateListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject  = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        CarActivateResult carActivateResult;
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            JSONObject dataObject = jsonObject.getJSONObject("data");
                            carActivateResult = new CarActivateResult();
                            if (dataObject.has("isActivate")){
                                carActivateResult.setIsActivate(dataObject.getInt("isActivate"));
                            }
                            mGetIsVehicleActivateListener.isVehicleActivate(carActivateResult);
                        }else {
                            switch (respCode){
                                case -100:
                                    mGetIsVehicleActivateListener.onFail(respCode,errorMsg);
                                    break;
                                case MISSING_PARAMETER_ERROR_CODE:
                                    mGetIsVehicleActivateListener.onFail(respCode, Constants.MISSING_PARAMETER_FAIL);
                                    break;
                                case Constants.VEHICLE_NOT_EXIS_ERROR_CODE:
                                    mGetIsVehicleActivateListener.onFail(respCode, Constants.VEHICLE_NOT_EXIS_FAIL);
                                    break;
                                default:
                                    break;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetIsVehicleActivateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,
                                Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            case ACTIVATE_VEHICLE:
                if (mActivateVehicleListener != null ){
                    if (hasDSPTError) {
                        mActivateVehicleListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    JSONObject jsonObject  = null;
                    try {
                        jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mActivateVehicleListener.onActivateVehicle();
                        }else {
                            switch (respCode){
                                case -100:
                                    mActivateVehicleListener.onFail(respCode,errorMsg);
                                    break;
                                case MISSING_PARAMETER_ERROR_CODE:
                                    mActivateVehicleListener.onFail(respCode, Constants.MISSING_PARAMETER_FAIL);
                                    break;
                                case Constants.VEHICLE_NOT_EXIS_ERROR_CODE:
                                    mActivateVehicleListener.onFail(respCode, Constants.VEHICLE_NOT_EXIS_FAIL);
                                    break;
                                case Constants.BUSINESS_ORDERS_NOT_EXIST_ERROR_CODE:
                                    mActivateVehicleListener.onFail(respCode, Constants.BUSINESS_ORDERS_NOT_EXIST_FAIL);
                                    break;
                                case Constants.VEHICLE_NOT_APPROVED_ERROR_CODE:
                                    mActivateVehicleListener.onFail(respCode, Constants.VEHICLE_NOT_APPROVED_FAIL);
                                    break;
                                case Constants.VIN_ACTIVATE_ERROR_CODE:
                                    mActivateVehicleListener.onFail(respCode, Constants.VIN_ACTIVATE_FAIL);
                                    break;
                                default:
                                    break;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
                break;
            case GET_MAP_AUTH:
                if (mOnGetMapAuthListener != null ){
                    if (hasDSPTError){
                        mOnGetMapAuthListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    MapAuthResult mapAuthResult;
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            mapAuthResult = new MapAuthResult();
                            mapAuthResult.setServerTime(jsonObject.getString("serverTime"));
                            List<ServiceList> lists = new ArrayList<ServiceList>();
                            ServiceList serviceList;
                            JSONArray jsonArray = jsonObject.getJSONArray("serviceList");
                            for (int i = 0; i< jsonArray.length(); i++){
                                serviceList = new ServiceList();
                                JSONObject object = jsonArray.getJSONObject(i);
                                serviceList.setServiceID(object.getString("serviceID"));
                                serviceList.setAuthCode(object.getInt("authCode"));
                                serviceList.setStartTime(object.getString("startTime"));
                                serviceList.setExpireTime(object.getString("expireTime"));
                                serviceList.setRemainingTimes(object.getString("remainingTimes"));
                                lists.add(serviceList);
                            }
                            mapAuthResult.setList(lists);
//                            if (serviceList.getAuthCode() != 100){
//                                mOnGetMapAuthListener.onFail(errorCode, "服务权限验证未通过");
//                                return;
//                            }
//                            if (serviceList != null) {
//                                try {
//                                    String expireTime = serviceList.getExpireTime();
//                                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
//                                    if (!expireTime.equals(-1)) {
//                                        if (Utils.hasExpired(new Date(), format.parse(expireTime))) {
//                                            // 授权已过期
//                                            mOnGetMapAuthListener.onFail(errorCode, errorMsg);
//                                            return;
//                                        }
//                                    }
//                                } catch (ParseException e) {
//                                    e.printStackTrace();
//                                }
//                            }
                            mOnGetMapAuthListener.onGetMapAuth(mapAuthResult);
                        }else {
                            switch (respCode){
                                case -100:
                                    mOnGetMapAuthListener.onFail(respCode, errorMsg);
                                    break;
                                case Constants.MISSING_PARAMETER_ERROR_CODE:
                                    mOnGetMapAuthListener.onFail(respCode, Constants.MISSING_PARAMETER_FAIL);
                                    break;
                                case Constants.MISSING_CUSTOEMR_KEY_ERROR_CODE:
                                    mOnGetMapAuthListener.onFail(respCode, Constants.MISSING_CUSTOMER_KEY_FAIL);
                                    break;
                                case Constants.DEVICE_NOT_EXITS_ERROR_CODE:
                                    mOnGetMapAuthListener.onFail(respCode, Constants.DEVICE_NOT_EXITS_FAIL);
                                    break;
                                case Constants.DEVICE_NO_AVAILABLE_SERVICE_ERROR_CODE:
                                    mOnGetMapAuthListener.onFail(respCode, Constants.DEVICE_NO_AVAILABLE_SERVICE_FAIL);
                                    break;
                                case Constants.DEVICE_NO_SERVICE_PERMISSION_ERROR_CODE:
                                    mOnGetMapAuthListener.onFail(respCode, Constants.DEVICE_NO_SERVICE_PERMISSION_FAIL);
                                    break;
                                default:
                                    break;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetMapAuthListener.onFail(errorCode, errorMsg);
                    }
                }
                break;
            case APP_VEHICLE_ACTIVATE:
                if (mOnAppVehicleActivateListener != null ){
                    if (hasDSPTError){
                        mOnAppVehicleActivateListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            mOnAppVehicleActivateListener.onAppVehicleActivate(requestHandleId);
                        }else {
                            switch (respCode){
                                case -100:
                                    mOnAppVehicleActivateListener.onFail(respCode, errorMsg);
                                    break;
                                case -258102:
                                    mOnAppVehicleActivateListener.onFail(respCode, "参数错误");
                                    break;
                                case -258201:
                                    mOnAppVehicleActivateListener.onFail(respCode, "用户不存在");
                                    break;
                                case -258202:
                                    mOnAppVehicleActivateListener.onFail(respCode, "用户未实名认证");
                                    break;
                                case -258301:
                                    mOnAppVehicleActivateListener.onFail(respCode, "车辆不存在");
                                    break;
                                case -258205:
                                    mOnAppVehicleActivateListener.onFail(respCode, "车辆未绑定或绑定未审核通过");
                                    break;
                                case -258408:
                                    mOnAppVehicleActivateListener.onFail(respCode, "vin已经激活");
                                    break;
                                case -258405:
                                    mOnAppVehicleActivateListener.onFail(respCode, "商务订单不存在");
                                    break;
                                default:
                                    break;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnAppVehicleActivateListener.onFail(errorCode, errorMsg);
                    }
                }
                break;
            default:
                break;
        }
    }

    private List<ListVehicleInfoResult> decodeListVehicleInfoResult(JSONObject jsonObject) throws Exception {
        Gson gson = new Gson();
        List<ListVehicleInfoResult> listVehicleInfoResultList = new ArrayList<ListVehicleInfoResult>();
        if (jsonObject.has("rows")) {
            JSONArray dataArray = jsonObject.getJSONArray("rows");
            for (int i = 0; i < dataArray.length(); i++) {
                ListVehicleInfoResult listVehicleInfoResult = gson.fromJson(dataArray.get(i).toString(), ListVehicleInfoResult.class);
                listVehicleInfoResultList.add(listVehicleInfoResult);
            }
        }
        return listVehicleInfoResultList;
    }
}
