package com.mxnavi.tspv2.chargerStation;

import com.google.gson.Gson;
import com.mxnavi.tspv2.chargerStation.model.SearchAroundInfo;
import com.mxnavi.tspv2.chargerStation.model.SearchAroundResult;
import com.mxnavi.tspv2.chargerStation.model.SearchChargerInfoDate;
import com.mxnavi.tspv2.chargerStation.model.SearchChargerInfoResult;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/5/21.
 */

public class ChargerStation extends BaseTsp {
    // 请求服务类型为充电桩服务
    final int thisServiceType = Config.MxNGTPServiceType.CHARGER_STATION.getServiceType();
    // 充电桩服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.CHARGER_STATION.getServiceVer();

    public ChargerStation() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int SEARCHAROUND = 1;//取得点周边充电站
    final int SEARCHCHARGERINFO = 2;//查询充电站的充电桩信息

    /**
     * 取得点周边充电站
     * @param searchAroundInfo
     * @return
     */
    public int searchAround(SearchAroundInfo searchAroundInfo) {
        return searchAround(searchAroundInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 取得点周边充电站
     * @param searchAroundInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @return
     */
    public int searchAround(SearchAroundInfo searchAroundInfo, int timeOutSecs) {
        return searchAround(searchAroundInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 取得点周边充电站
     * @param searchAroundInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int searchAround(SearchAroundInfo searchAroundInfo, int timeOutSecs, int priorityLevel) {
        if (String.valueOf(searchAroundInfo.getLatLng().getLatitude()).equals("")
                ||String.valueOf(searchAroundInfo.getLatLng().getLongitude()).equals("")
                ||String .valueOf(searchAroundInfo.getRadius()).equals("")
                ||String .valueOf(searchAroundInfo.getStationType()).equals("")) {
            if (mSearchAroundListener != null) {
                mSearchAroundListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "i-ev/searchAround";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("lon", searchAroundInfo.getLatLng().getLongitude());
        params.put("lat", searchAroundInfo.getLatLng().getLatitude());
        params.put("radius", searchAroundInfo.getRadius());
        if (!String .valueOf(searchAroundInfo.getStationType()).equals("")){
            params.put("stationType", searchAroundInfo.getStationType());
        }
        if (!String.valueOf(searchAroundInfo.getSupportOrder()).equals("")){
            params.put("supportOrder",searchAroundInfo.getSupportOrder());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEARCHAROUND);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询充电站的充电桩信息
     * @param stationId
     * @return
     */
    public int searchChargerInfo(String stationId) {
        return searchChargerInfo(stationId, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询充电站的充电桩信息
     * @param stationId 充电站ID
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @return
     */
    public int searchChargerInfo(String stationId, int timeOutSecs) {
        return searchChargerInfo(stationId, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询充电站的充电桩信息
     * @param stationId 充电站ID
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int searchChargerInfo(String stationId, int timeOutSecs, int priorityLevel) {
        if (stationId==null) {
            if (mSearchChargerInfoListener != null) {
                mSearchChargerInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "i-ev/searchChargerInfo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("stationId", stationId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEARCHCHARGERINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }
    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, ChargerStation.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    public interface SearchAroundListener extends CallBackListener {
        public void onSearchAround(int requestId, SearchAroundResult searchAroundResult);
    }
    public interface SearchChargerInfoListener extends CallBackListener {
        public void onSearchChargerInfo(int requestId, SearchChargerInfoResult searchChargerInfoResult);
    }

    SearchAroundListener mSearchAroundListener;
    SearchChargerInfoListener mSearchChargerInfoListener;


    public void setSearchAroundListener(SearchAroundListener searchAroundListener) {
        this.mSearchAroundListener = searchAroundListener;
    }
    public void setSearchChargerInfoListener(SearchChargerInfoListener searchChargerInfoListener) {
        this.mSearchChargerInfoListener = searchChargerInfoListener;
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHAROUND://1
                if (mSearchAroundListener != null) {
                    mSearchAroundListener.onTimeOut();
                }
                break;
            case SEARCHCHARGERINFO://2
                if (mSearchChargerInfoListener != null) {
                    mSearchChargerInfoListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHAROUND:
                if (mSearchAroundListener != null) {
                    if (hasDSPTError) {
                        mSearchAroundListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            SearchAroundResult searchAroundResult = gson.fromJson(response, SearchAroundResult.class);
                            mSearchAroundListener.onSearchAround(requestHandleId, searchAroundResult);
                        } else {
                            mSearchAroundListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mSearchAroundListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCHCHARGERINFO:
                if (mSearchChargerInfoListener != null) {
                    if (hasDSPTError) {
                        mSearchChargerInfoListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            SearchChargerInfoResult searchChargerInfoResult = gson.fromJson(response, SearchChargerInfoResult.class);
                            mSearchChargerInfoListener.onSearchChargerInfo(requestHandleId, searchChargerInfoResult);


                        } else {
                            mSearchChargerInfoListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mSearchChargerInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
