package com.mxnavi.tspv2.collection;

import com.mxnavi.tspv2.collection.model.UploadMacAddrQuery;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by Administrator on 2018/9/12.
 * 收藏服务
 */
public class Collection extends BaseTsp{
    // 请求服务类型为云服务
    final int thisServiceType = Config.MxNGTPServiceType.MX_CLOUD_COLL.getServiceType();
    // 云服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_CLOUD_COLL.getServiceVer();

    public Collection() {}

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int UPLOAD_MAC_ADDR = 1; // 车厘子MAC,ADDR上报


    OnUploadMacAddrListener mOnUploadMacAddrListener; // 车厘子MAC,ADDR上报接口监听

    /**
     * 车厘子MAC,ADDR上报
     * @param uploadMacAddrQuery
     * @return
     */
    public int uploadMacAddr(UploadMacAddrQuery uploadMacAddrQuery){
        return uploadMacAddr(uploadMacAddrQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车厘子MAC,ADDR上报
     * @param uploadMacAddrQuery
     * @param timeOutSecs
     * @return
     */
    public int uploadMacAddr(UploadMacAddrQuery uploadMacAddrQuery, int timeOutSecs){
        return uploadMacAddr(uploadMacAddrQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

        /**
         * 车厘子MAC,ADDR上报
         * @param uploadMacAddrQuery
         * @param timeOutSecs
         * @param priorityLevel
         * @return
         */
        public int uploadMacAddr(UploadMacAddrQuery uploadMacAddrQuery, int timeOutSecs, int priorityLevel) {
            if (uploadMacAddrQuery == null || uploadMacAddrQuery.getSn() == null) {
                if (mOnUploadMacAddrListener != null) {
                    mOnUploadMacAddrListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                }
                return 0;
            }
            int eventId = RequestDispathUtil.createEventId();
            String url = "clz/uploadMacAddr";
            HashMap<String, Object> params = new HashMap<String, Object>();
            params.put("sn", uploadMacAddrQuery.getSn());
            if (uploadMacAddrQuery.getMac() != null ){
                params.put("mac", uploadMacAddrQuery.getMac());
            }
            if (uploadMacAddrQuery.getImei() != null){
                params.put("imei", uploadMacAddrQuery.getImei());
            }
            String requestData = Utils.makeUrl(url, params);
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                    requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, UPLOAD_MAC_ADDR);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
            return eventId;
        }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, Collection.this);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case UPLOAD_MAC_ADDR: // 车厘子MAC,ADDR上报
                if (mOnUploadMacAddrListener != null){
                    if (hasDSPTError){
                        mOnUploadMacAddrListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS ){
                            mOnUploadMacAddrListener.onUploadMacAddrSuccess(requestHandleId);
                        }else {
                            mOnUploadMacAddrListener.onFail(respCode, errorMsg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnUploadMacAddrListener.onFail(errorCode, errorMsg);
                    }
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case UPLOAD_MAC_ADDR:
                if (mOnUploadMacAddrListener != null){
                    mOnUploadMacAddrListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    public interface OnUploadMacAddrListener extends CallBackListener {
        public void onUploadMacAddrSuccess(int requestId);
    }

    public void setOnUploadMacAddrListener(OnUploadMacAddrListener onUploadMacAddrListener) {
        this.mOnUploadMacAddrListener = onUploadMacAddrListener;
    }
}
