package com.mxnavi.tspv2.core;

import android.content.res.AssetManager;

import com.mxnavi.tspv2.core.config.ConnectTypeInfo;
import com.mxnavi.tspv2.core.netty.DSPTClientInitializer;
import com.mxnavi.tspv2.dns.DnsService;
import com.mxnavi.tspv2.dns.OnGetIpListener;
import com.mxnavi.tspv2.dns.model.IpResult;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioSocketChannel;

/**
 * Created by Administrator on 2016/3/30.
 */
public class Connection {

    private Connection(){
    }
    private static Connection connection;
    public  static Connection getInstance(){
        if(connection==null){
        	synchronized(Connection.class){
        		 if (connection == null) {
        			 connection = new Connection();
        		 }
        	}
        }
        return connection;
    }

    DnsService dnsService = DnsService.getInstance();
    EventLoopGroup  group =   new NioEventLoopGroup(1);
    private  Channel chanel;
    private  ScheduledExecutorService executorService = Executors.newScheduledThreadPool(1);
    ExecutorService fixedThreadPool = Executors.newFixedThreadPool(1, new TspThreadFactory("startTcp"));

    public  volatile boolean   IS_LOGIN_SUCCESS = false;
    public  volatile boolean   READFILE_FINISH = false;
    protected  ChannelFuture channelFuture;
//    volatile boolean IS_STOP_RECONNECTING = false;
    // 网络连接状态，true-已经连接上,false-未连接
    volatile boolean NETWORK_STATE = true;
    volatile  boolean IS_SERVICE_STOP = false;
    public  Channel getChanel(){
        return chanel;
    }

    class Start implements Runnable{
        public String mHostConfigFilePath = "";
        public AssetManager assetManager = null;
        public void run() {
            readConfigFile(mHostConfigFilePath,assetManager);
            if(Config.hasTcpConnect>0){
                startupTcp();
            }

        }
    }
    Start start = new Start();
    Object object = new Object();
    Object reconnectLock = new Object();// 重连锁
    public  void start(String hostConfigFilePath,AssetManager assetManager) {
        if(IS_LOGIN_SUCCESS == true){
            return;
        }
        if(fixedThreadPool == null){
            fixedThreadPool = Executors.newFixedThreadPool(1,new TspThreadFactory("startTcp"));
        }

        start.mHostConfigFilePath = hostConfigFilePath;
        start.assetManager = assetManager;
        fixedThreadPool.execute(start);

//        MessageBuilder.instance().setInitializeInfo(initializeInfo);
//        businessThreadPool.init(5,10);
    }

    private  void startupTcp() {
        try {
            Utils.loge("tspsdk connection startup ======");
//            if(group.isShuttingDown()||group.isTerminated()||group.isShutdown()){
//                Utils.loge("group 重新new");
//                group = new NioEventLoopGroup(1);
//            }
            // Netty事件循环
            Bootstrap b = new Bootstrap();
            b.group(group)
                    .channel(NioSocketChannel.class)
                    .option(ChannelOption.TCP_NODELAY, true)
                    .option(ChannelOption.CONNECT_TIMEOUT_MILLIS, 30000)
                    .handler(new DSPTClientInitializer());


            // Make a group of connection.
//            for (int i = 0; i < CONNECTION_COUNT; ++i) {
            channelFuture= b.connect( dnsService.getIpByDomain(Config.TCPHOST).getIp(), Config.TCPPORT).sync();


            chanel =channelFuture.channel();

            chanel.closeFuture().sync();
            IS_LOGIN_SUCCESS = false;

        } catch (Exception e) {
//            e.printStackTrace();
            Utils.loge("star tcp exception " + e.getMessage());
//            Utils.loge(e.getMessage());
        } finally {
            Utils.loge("tspsdk connection has stoped ======");
            IS_LOGIN_SUCCESS = false;
            try{
                if(IS_SERVICE_STOP){// 如果是停止服务，咋不再重启
                  return;
                }

                if(executorService == null){
                    executorService = Executors.newScheduledThreadPool(1,new TspThreadFactory("antostartTcp"));
                }

                executorService.execute(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            synchronized (reconnectLock){
                                reconnectLock.wait(5000);
                            }
//                            TimeUnit.SECONDS.sleep(5);
                            if(!NETWORK_STATE){// 如果网络还未连接上,则不需要自动启动，等待网络连接成功后唤醒
                                synchronized (reconnectLock){
                                    reconnectLock.wait();
                                }
                            }

                            startupTcp();

                        } catch (Exception e) {

                            e.printStackTrace();

                        }
                    }
                });
            } catch (Exception e) {
                Utils.loge("++++" + e.getMessage());
            }

            //            group.shutdownGracefully();
        }
    }
   protected void restart(){
       if(group!=null){
           group.shutdownGracefully();
       }
   }
    public  void stop(){
        if(group!=null){
            group.shutdownGracefully();
        }
        IS_LOGIN_SUCCESS = false;
        READFILE_FINISH = false;
        IS_SERVICE_STOP = true;
        if(executorService != null){
            executorService.shutdownNow();
        }

        if(fixedThreadPool != null){
            fixedThreadPool.shutdownNow();
        }

        fixedThreadPool = null;
        executorService = null;
        connection = null;
    }
    public  void noticeNetWorkState(boolean isConnected){
        Utils.loge("noticeNetWorkState="+isConnected);
        if(isConnected){// 网络连接上
            NETWORK_STATE = true;
            synchronized (reconnectLock){
                reconnectLock.notify();
            }
            // 因域名解析失败后，会间隔1分钟再进行解析，所以网络连接成功之后，结束1分钟的wait，马上进行解析
            IpResult ipResult =  dnsService.getIpByDomain(Config.TCPHOST);
            if(Utils.strIsEmpty(ipResult.getIp())){
            	  dnsService.notifyParseDomain();
            }
        }else{// 网络断
            NETWORK_STATE = false;
          
        }
    }
    

    /**
     * 从SD卡中读取配置文件
     * @param filePath
     * @return
     */
    private  byte[] getConfigFileBytes(String filePath){
    	 Utils.loge("getConfigFileBytes");
        if(filePath==null||filePath.length()==0){
            return null;
        }
        File file = new File(filePath);

        FileInputStream inputStream = null;
        byte[] bytes = null;
        try {
            inputStream = new FileInputStream(file);
            bytes = new byte[inputStream.available()];
            inputStream.read(bytes);
            inputStream.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-not exists");
        } catch (IOException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-read fail");
        }
        return bytes;
    }

    /**
     * 从APP中的assets中读取配置文件
     * @param filePath
     * @param assetManager
     * @return
     */
    private  byte[] getConfigFileBytes(String filePath,AssetManager assetManager){
        byte[] buffer = null;
        InputStream is = null;
        try{
             is = assetManager.open(filePath);
            int size = is.available();

            // Read the entire asset into a local byte buffer.
            buffer = new byte[size];
            is.read(buffer);
            is.close();
        }catch (FileNotFoundException e) {
            e.printStackTrace();
            throw new IllegalArgumentException("assets/"+filePath+"-not exists");
        }catch (IOException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-read fail");
        }finally {
            try{
                if(is!=null){
                    is.close();
                }

            }catch (IOException e){
                e.printStackTrace();
            }
        }
        return buffer;
    }
    private void readConfigFile(String filePath,AssetManager assetManager){
        byte[] bytes = null;
        if(assetManager==null){
            bytes = this.getConfigFileBytes(filePath);
        }else{
            bytes = this.getConfigFileBytes(filePath,assetManager);
        }
        if(bytes == null){
            return;
        }
        Config.hasTcpConnect = 0;
        Config.hasHttpConnect = 0;
        String data = "";
        try {
            byte[] codeData = AESCodec.decrypt2Buf(bytes, "29a78d282d31c5f9c0a7f637da977eb9");
            data = new String(codeData,"UTF-8");
            Utils.loge(filePath+"read success");
            JSONTokener jsonParser = new JSONTokener(data);
            JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
            JSONObject TCPNGTPObject = jsonObject.getJSONObject("TCPNGTP");
//            Config.TCPHOST = TCPNGTPObject.getString("server_host");
//            Config.TCPPORT = TCPNGTPObject.getInt("port");
//
//            JSONObject HTTPNGTPObject = jsonObject.getJSONObject("HTTPNGTP");
//            Config.HTTP_URL = "http://"+HTTPNGTPObject.getString("server_host")+":"+HTTPNGTPObject.getInt("port")+Config.HTTP_PATH;


            JSONObject CONNCONFIGObject =  jsonObject.getJSONObject("CONNCONFIG");
            Iterator<String> iterator = CONNCONFIGObject.keys();
            ConnectTypeInfo connectTypeInfo;
            while (iterator.hasNext()){
                String key = iterator.next();
                JSONObject obj = CONNCONFIGObject.getJSONObject(key);
                String PROTO = obj.getString("PROTO");
                int purpose =   obj.getInt("purpose");
                String switchs = obj.getString("switch");
                connectTypeInfo = new ConnectTypeInfo();

                connectTypeInfo.setProto(PROTO);
                connectTypeInfo.setPurpose(purpose);
                if("on".equalsIgnoreCase(switchs)){
                    connectTypeInfo.setIsOn(true);
                }
                Config.conectInfomap.put(Config.serviceTypeNameMap.get(key),connectTypeInfo);
                if("TCP".equalsIgnoreCase(PROTO)&&"on".equalsIgnoreCase(switchs)){
                    Config.hasTcpConnect++;
                }
                if("HTTP".equalsIgnoreCase(PROTO)&&"on".equalsIgnoreCase(switchs)){
                    Config.hasHttpConnect++;
                }
            }
        } catch (JSONException e){
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-parse json failed，please check json");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-parse to UTF-8 fail");
        } catch (Exception e) {
            e.printStackTrace();
            throw new IllegalArgumentException(filePath+"-decode fail");
        }
        if(Config.hasTcpConnect<=0){// 只有HTTP连接
            RequestDispathUtil.httpRequestHandler = new OkHttpRequest();

            // 提交确定长短连接之前的请求
            RequestDispathUtil.submitCacheMessage(0);
            fixedThreadPool.shutdownNow();
            READFILE_FINISH = true;
            return;
        }
        RequestDispathUtil.tcpRequestHandler = new TcpRequest();

        if( Config.hasHttpConnect>0){// 存在HTTP请求
            RequestDispathUtil.httpRequestHandler = new OkHttpRequest();

            // 提交确定长短连接之前的请求
            RequestDispathUtil.submitCacheMessage(0);
        }

        READFILE_FINISH = true;
        Utils.loge("Connection:dnsService.getIpByDomain");
        IpResult ipResult =  dnsService.getIpByDomain(Config.TCPHOST);
        if(Utils.strIsEmpty(ipResult.getIp())){
            OnGetIpListener onGetIpListener = new OnGetIpListener() {
                    public void onGetIp(String ip) {
                        synchronized (object) {
                            object.notify();
//                            Utils.loge("线程继续运行");
                        }
                    }
                };
            Utils.loge("Connection:dnsService.addDomainName");
            dnsService.addDomainName(Config.TCPHOST, 3,onGetIpListener );
            synchronized (object) {
                try {
//                    Utils.loge("线程wait");
                    object.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

    }


}
