package com.mxnavi.tspv2.core;


import com.mxnavi.tspv2.core.config.ConnectTypeInfo;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import java.util.concurrent.PriorityBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

import org.json.JSONObject;

/**
 * Created by Administrator on 2016/3/30.
 */
public class RequestDispathUtil {
//    private static Map<MxNGTPMessage.ServiceType,ResponseListener> listenerHashMap = new HashMap<MxNGTPMessage.ServiceType,ResponseListener>();
    private static  AtomicInteger EVENT_ATOMIC = new AtomicInteger(1);
    protected static PriorityBlockingQueue<PrioritizedRequestMessage> cacheQueue =new PriorityBlockingQueue<PrioritizedRequestMessage>();
//    public static PriorityBlockingQueue<PrioritizedRequestMessage> httpCacheQueue =new PriorityBlockingQueue<PrioritizedRequestMessage>();


    public static TcpRequest tcpRequestHandler;
    public static OkHttpRequest httpRequestHandler;
    private static RequestInterface getRequestHandler(String proto){
        if("TCP".equalsIgnoreCase(proto)){
            return tcpRequestHandler;
        }else{
            return httpRequestHandler;
        }
    }
    private static boolean isTcpRequest(String proto){
        if("TCP".equalsIgnoreCase(proto)){
            return true;
        }
        return false;
    }
    private static String getProto(int serviceType){
        ConnectTypeInfo connectTypeInfo = Config.conectInfomap.get(serviceType);
        String proto = "HTTP";//网络请求默认为http
        if(connectTypeInfo!=null) {
            proto = connectTypeInfo.getProto();
        }else{
            if(httpRequestHandler!=null){
                proto = "HTTP";
            }else{
                proto = "TCP";
            }
        }
        return proto;
    }

    public static void submitRequest(PrioritizedRequestMessage requestMessage){
        requestMessage.createTimeStamp = System.currentTimeMillis();
        TimeOut.startCheckTimeOut();

        if(!Connection.getInstance().READFILE_FINISH){// 配置文件还未读取成功
            Utils.loge("cacheQueue.offer=eventid="+requestMessage.evenId+"servicetype="+requestMessage.serviceType);
            cacheQueue.offer(requestMessage);

            return;
        }

        String proto = getProto(requestMessage.serviceType);

        if(!Connection.getInstance().IS_LOGIN_SUCCESS){// 如果TCP未登录成功，发HTTP请求正常
            if(isTcpRequest(proto)){
                Utils.loge("2tcpCacheQueue.offer=eventid="+requestMessage.evenId+"servicetype="+requestMessage.serviceType);
                cacheQueue.offer(requestMessage);
                return;
            }
        }

        getRequestHandler(proto).sendRequest(requestMessage);
    }
    public static void submitCacheMessage(int type){
        while (cacheQueue.size()>0) {
            PrioritizedRequestMessage msg = cacheQueue.poll();
            String proto = getProto(msg.serviceType);
            if(isTcpRequest(proto)){
                tcpRequestHandler.cacheQueue.offer(msg);
            }else{
                Utils.loge("httpRequestHandler.offer=eventid="+msg.evenId);
                httpRequestHandler.cacheQueue.offer(msg);
            }
        }
        if(type==0){
            httpRequestHandler.submitCacheMessage();
        }else{
        	// 主动获取未推送成功的发送到车的消息
        	getSendToCarMessage();
            tcpRequestHandler.submitCacheMessage();
        }
    }
    public static void clearStorage(){
        EVENT_ATOMIC.set(1);
        cacheQueue.clear();
        if(tcpRequestHandler!=null){
            tcpRequestHandler.clearStorage();
        }
        if(httpRequestHandler!=null){
            httpRequestHandler.clearStorage();
        }
        tcpRequestHandler = null;
        httpRequestHandler = null;
    }

    static public void onFinishRequest(int requestHandleId){
        if(tcpRequestHandler!=null){
            tcpRequestHandler.onFinishRequest(requestHandleId);
        }
        if(httpRequestHandler!=null){
            httpRequestHandler.onFinishRequest(requestHandleId);
        }

    }
    static public void cancelRequest(int requestId){
        if(tcpRequestHandler!=null){
            tcpRequestHandler.cancelRequest(requestId);
        }
        if(httpRequestHandler!=null){
            httpRequestHandler.cancelRequest(requestId);
        }
    }
    public   static int createEventId(){
        return EVENT_ATOMIC.getAndIncrement();

    }
    private static void getSendToCarMessage(){
    	   int eventId = RequestDispathUtil.createEventId();
           String url = "mx_wechat/sendtocar/NotifyInfoToVehicle";
           JSONObject params = new JSONObject();
           try{
               params.put("vin", Config.VIN);
               params.put("product_id", Config.CUSTOMER_KEY+"");
               params.put("wecar_id", Config.VIN);
//               params.put("weixin_openid",  weChatDeviceInfo.getWeChatOpenId());
               String requestData = url + "?param="+params.toString();
               MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, Config.MxNGTPServiceType.SEND_TO_CAR.getServiceType(), Config.MxNGTPServiceType.SEND_TO_CAR.getServiceVer());
               PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
               RequestDispathUtil.submitRequest(requestMessage);
               Utils.loge("getSendToCarMessage execute");
           }catch (Exception e) {
               e.printStackTrace();
           }
    }
//    static public void reConnection(){
//        requestHandler.reConnection();
//    }
}
