package com.mxnavi.tspv2.core.netty;

import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;

import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.os.PowerManager;

import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.tsp.TSP;
import com.mxnavi.tspv2.utils.Utils;

/**
 * Created by Administrator on 2016/3/29.
 */
public class HeartBeatReqHandler extends SimpleChannelInboundHandler {

    private volatile ScheduledFuture<?> heartBeat;
    private volatile ScheduledFuture<?> timeout;
    private volatile long lastReadTime = System.currentTimeMillis();
    // 网络状态闲置的秒数
//    private long noBusinessTimeoutSeconds = 100;
    // 发送心跳包的频率
    long heartBeatPeriodSeconds = 290;
//    private final long timeoutMillis = TimeUnit.SECONDS.toMillis(noBusinessTimeoutSeconds);

    SendReceiver mSendReceiver = new SendReceiver();
    ChannelHandlerContext mCtx;
    PendingIntent mPendingIntent;
    // 解决深度休眠的时候获取电源锁设置下次发心跳的时机
    private PowerManager mPowerManager;
    private PowerManager.WakeLock mWakeLock;

     final static String ACTION_SEND = "TSPSDK_HEARTBEAT_ACTION";
//    mPowerManager = (PowerManager) mContext.getSystemService(Service.POWER_SERVICE);
//    mWakeLock = mPowerManager.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK, "The screen is always on Lock");
//            mWakeLock.setReferenceCounted(false);
//
    @Override
    public void channelRead0(ChannelHandlerContext ctx, Object msg) throws Exception {
        MxNGTPMessage.MxMessage mxMessage = (MxNGTPMessage.MxMessage) msg;
        mCtx = ctx;
        if(mPowerManager == null){
            mPowerManager = (PowerManager) TSP.context.getSystemService(TSP.context.POWER_SERVICE);
            mWakeLock = mPowerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "TSPSDK-HEARTBEAT");
            mWakeLock.setReferenceCounted(false);
        }
//        System.out.println("channelRead ---> "+ msg);
        MxNGTPMessage.NGTPDispatcherData dispatcherData = mxMessage.getNgtpMessage().getDispatcherData();
        MxNGTPMessage.DSPTMessageType DSPTMessageType = dispatcherData.getDsptMessageType();
        MxNGTPMessage.MXMessageType messageType = mxMessage.getMessageType();
        if (DSPTMessageType.getNumber() == MxNGTPMessage.DSPTMessageType.LOGIN_ACK.getNumber()) {//登录成功

            IntentFilter filter=new IntentFilter(ACTION_SEND);
            TSP.context.registerReceiver(mSendReceiver,filter);
            setAlarm(TSP.context);

//            this.timeout = ctx.executor().schedule(new ReadTimeoutTask(ctx), this.timeoutMillis, TimeUnit.MILLISECONDS);
        }else if(messageType.getNumber() == MxNGTPMessage.MXMessageType.HEARTBEAT.getNumber()){// 心跳消息
            Utils.loge("Client receive server heart beat message : ---> " + msg);

        }else{// 业务信息
            // 有业务信息就停止发送心跳
//            if (heartBeat != null) {
//                heartBeat.cancel(true);
//                heartBeat = null;
//            }
            this.lastReadTime = System.currentTimeMillis();
            // 重新设置下次发送心跳的时间
            setAlarm(TSP.context);
//            if(this.timeout == null||this.timeout.isDone()||this.timeout.isCancelled()){
//                this.timeout = ctx.executor().schedule(new ReadTimeoutTask(ctx), this.timeoutMillis, TimeUnit.MILLISECONDS);
//            }

            ctx.fireChannelRead(msg);
        }
    }
    public void setAlarm(Context context){
        AlarmManager am = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
      
        if(mPendingIntent == null){
        	Intent i = new Intent(ACTION_SEND);
            mPendingIntent = PendingIntent.getBroadcast(context, 0, i, 0);
        }
        if (Build.VERSION.SDK_INT >= 19){
            Utils.loge("Build.VERSION.SDK_INT"+Build.VERSION.SDK_INT);
            am.setWindow(AlarmManager.RTC_WAKEUP, System.currentTimeMillis()+heartBeatPeriodSeconds*1000L, 0L, mPendingIntent);
        }else{
            am.setInexactRepeating(AlarmManager.RTC_WAKEUP, System.currentTimeMillis()+heartBeatPeriodSeconds*1000L,heartBeatPeriodSeconds*1000L, mPendingIntent);
        }


    }
    public class SendReceiver extends BroadcastReceiver {
     

        @Override
        public void onReceive(final Context context, Intent intent) {
        	 if (null != mWakeLock){  
        		 mWakeLock.acquire(2000);
             }  
            String action = intent.getAction();
            if (ACTION_SEND.equals(action)) {
                Utils.loge("AlarmManager send a message");
                MxNGTPMessage.MxMessage heartbeatMessage = MessageBuilder.instance().makeHeartbeatMessage();
                Utils.loge("Client send heart beat messsage to server 60stimer : ---> " + heartbeatMessage);
                mCtx.writeAndFlush(heartbeatMessage);
                if (Build.VERSION.SDK_INT >= 19){
                    AlarmManager am = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
                    am.setWindow(AlarmManager.RTC_WAKEUP, System.currentTimeMillis()+heartBeatPeriodSeconds*1000L, 0L, mPendingIntent);
                }
            }
            if (null != mWakeLock){ 
            	mWakeLock.release();
            }
            
        }
    }

//    private class HeartBeatTask implements Runnable {
//
//        private final ChannelHandlerContext ctx;
//
//        public HeartBeatTask(final ChannelHandlerContext ctx) {
//            this.ctx = ctx;
//        }
//
//        @Override
//        public void run() {
//            MxNGTPMessage.MxMessage heartbeatMessage = MessageBuilder.instance().makeHeartbeatMessage();
//            Utils.loge("Client send heart beat messsage to server 80stimer : ---> " + heartbeatMessage);
//            ctx.writeAndFlush(heartbeatMessage);
//        }
//
//    }
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        cause.printStackTrace();
        TSP.context.unregisterReceiver(mSendReceiver);
        AlarmManager am = (AlarmManager) TSP.context.getSystemService(Context.ALARM_SERVICE);
        am.cancel(mPendingIntent);
        if (heartBeat != null) {
            heartBeat.cancel(true);
            heartBeat = null;
        }
        ctx.fireExceptionCaught(cause);

    }
//    private final class ReadTimeoutTask implements Runnable {
//        private final ChannelHandlerContext ctx;
//
//        ReadTimeoutTask(ChannelHandlerContext ctx) {
//            this.ctx = ctx;
//        }
//
//        public void run() {
//            if(this.ctx.channel().isOpen()) {
//                long currentTime = System.currentTimeMillis();
//                long nextDelay = HeartBeatReqHandler.this.timeoutMillis - (currentTime - HeartBeatReqHandler.this.lastReadTime);
//
//                if(nextDelay <= 0L) {
////                    HeartBeatReqHandler.this.timeout = this.ctx.executor().schedule(this, HeartBeatReqHandler.this.timeoutMillis, TimeUnit.MILLISECONDS);
//                    // 开始发心跳
//                    if(heartBeat==null||heartBeat.isCancelled()){// 心跳停止时重新开启
//                        Utils.loge("start heart ---> ");
//                        heartBeat = ctx.executor().scheduleAtFixedRate(
//                                new HeartBeatTask(ctx), 0, heartBeatPeriodSeconds,
//                                TimeUnit.SECONDS);
//                    }
//                } else {
//                    HeartBeatReqHandler.this.timeout = this.ctx.executor().schedule(this, nextDelay, TimeUnit.MILLISECONDS);
//                }
//
//            }
//        }
//
//    }
    private void destroy() {
    	try{
    		   TSP.context.unregisterReceiver(mSendReceiver);
    	}catch(Exception e){
    		
    	}
     
//        if(this.timeout != null) {
//            this.timeout.cancel(false);
//            this.timeout = null;
//        }
//        if (heartBeat != null) {
//            heartBeat.cancel(true);
//            heartBeat = null;
//        }
    }

    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        this.destroy();
//        super.channelInactive(ctx);
    }
    @Override
    public void handlerRemoved(ChannelHandlerContext ctx) throws Exception {
        this.destroy();
    }

}
