package com.mxnavi.tspv2.datashare;

import com.mxnavi.tspv2.datashare.model.DeviceInfo;
import com.mxnavi.tspv2.datashare.model.FileInfo;
import com.mxnavi.tspv2.datashare.model.FileRequireInfo;
import com.mxnavi.tspv2.datashare.model.FolderInfo;
import com.mxnavi.tspv2.datashare.model.Message;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Created by Administrator on 2016/7/20.
 * desc
 */
public class DataShareClient {

    int MAXLENGTH = 1024*40;
    int MAGNO_BROADCAST = 1;
    int MAGNO_SENDDEVICENO = 2;
    int MAGNO_GETDATA = 3;
    int MAGNO_SENDFILE_TOTALINFO = 4;
    int MAGNO_SENDFILE_INFO = 5;
    int MAGNO_SENDFILE = 6;
    int MAGNO_NOTICE_FINISH = 7;

    int RESP_OK =200;
    int RESP_FILE_FINISH = 201;
    int RESP_MEMORY_NOT_ENOUGN = 300;
    int RESP_SERVER_ERROR = 500;
    int RESP_REFUSE = 304;
    int RESP_VALID_ERROR = 400;

    int FILE_NOT_FOUND = -1;
    int IO_EXCEPTION = -2;
    
    
    
    final String currVer="1.0";
    String[] supportVer={"1.0","1.1"};
    // 发广播
    final String broadcastMsg="{\"msgType\":0, \"msgNo\":"+MAGNO_BROADCAST+",\"version\":\"1.0\"," +
            " \"ptt\":[\"1.0\",\"1.1\"] ," +
            "\"deviceName\":\"wangqian\"" +
            "}";

    // 发送设备号
    String senddiviceNoMsg = "{\"msgType\":0,\"msgNo\":"+MAGNO_SENDDEVICENO+"," +
            "\"deviceNo\":\"12f3\"" +
            "}";
    // 获取车机软件信息
    String requestDataMsg = "{\"msgType\":0,\"msgNo\":"+MAGNO_GETDATA+"}";

    String sendFileTotalInfoMsg = "{\"msgType\":0,\"msgNo\":"+MAGNO_SENDFILE_TOTALINFO+",\"totalFileNum\":1,\"totalFileSize\":1,\"folderPath\":\"\",\"deviceNo\":\"\"}";

    String sendFileInfoMsg = "{\"msgType\":0,\"msgNo\":"+MAGNO_SENDFILE_INFO+"," +
                    "\"deviceNo\":\"12f3\"," +
                    "\"filePath\":\"lj\\02.dat\"," +
                    "\"fileSize\":10000," +
                    "\"filemd5\":\"sdf23wer\"" +
                    "}";
    String sendFileHead = "{\"msgType\":0,\"msgNo\":"+MAGNO_SENDFILE +","+
            "\"deviceNo\":\"12f3\"," +
            "\"filePath\":\"lj\\02.dat\",\"size\":4033," +
            "\"offset\":11" +
            "}";
    String noticeFileShareFinish = "{\"msgType\":0,\"msgNo\":"+MAGNO_NOTICE_FINISH +"}";


    int serverPort = 9999;

    int clientPort = 0;
    int receiveFilePort = 0;
    InetAddress serverAddress;
//    int receivePort;

    final int STATE_BROADCAST =1;
    final int STATE_RECEIVE_RESP_BROADCAST = 2;
    final int STATE_SEND_DEVICENO = 3;
    final int STATE_RECEIVE_RESP_DEVICENO = 4;

    final int STATE_SEND_REQUEST_DATA = 5;
    final int STATE_RECEIVE_RESP_DATA = 6;

    final int STATE_SEND_TOTALFILEINFO = 7;
    final int STATE_SEND_FILEINFO = 8;
    final int STATE_RECEIVE_RESP_FILEINFO = 9;
    final int STATE_RECEIVE_REQUIRE_FILEINFO = 10;
    final int STATE_SEND_FILE = 11;
    final int STATE_RECEIVE_RESP_FILE = 12;
    final int STATE_SEND_NITICE_FILESHAREFINISH = 13;




    String deviceNO ;
    String deviceName ;
    DatagramSocket clientsocket;
    DatagramSocket receiveSocket;

    ReceiveFileTask receiveFileTask;

    ThreadPoolExecutor  executor;
    ArrayBlockingQueue arrayBlockingQueue =   new ArrayBlockingQueue<Runnable>(2);
    Map<Integer,Message> msgMap = new HashMap<Integer,Message>();
    enum FileResult{
        COMPLETE,
        CONTINUE,
        RESEND,
        RECEIVER_ERROR
    }
    public DataShareClient(String imei, String deviceName){
        deviceNO = imei;
        this.deviceName = deviceName;
        executor = new ThreadPoolExecutor(1, 1, 30, TimeUnit.SECONDS,arrayBlockingQueue,new ThreadPoolExecutor.DiscardPolicy() );
        executor.allowCoreThreadTimeOut(true);

    }


//    Object lock = new Object();
    int timeout = 30000;//30秒
    int socketTimeOut = 5000;
    int broadcastInterval = 5;//广播重发间隔为5秒
    int msgInterval = 5;
    int loopMaxNum = timeout/socketTimeOut;



    GetDataFromCarDeviceListener mGetDataFromCarDeviceListener;
    Future getDatafuture = null;
    Future sendDatafuture = null;
    GetDataTask getDataTask = new GetDataTask();
    SendFileTask sendFileTask = new SendFileTask();
    /**
     * 开始发送数据
     * @param fileInfoList 要发送的文件列表
     * @param folderInfo 文件夹信息
     * @param sendDataListener 回调
     * @return 是否开始成功，false:上一个获取线程还未结束，不能重复添加获取线程；true:开始成功
     */
    public boolean startSendFiles(List<FileInfo> fileInfoList,FolderInfo folderInfo,SendDataListener sendDataListener){
        if(sendFileTask.isDone()){
            sendFileTask.init(fileInfoList, folderInfo,sendDataListener);
            sendFileTask.start();
            sendDatafuture = executor.submit(sendFileTask);
            return true;
        }
        return false;

    }

    /**
     * 开始获取数据
     * @param getDataFromCarDeviceListener
     * @return 是否开始成功，false:上一个获取线程还未结束，不能重复添加获取线程；true:开始成功
     */

    public boolean startGetData( GetDataFromCarDeviceListener getDataFromCarDeviceListener){
        mGetDataFromCarDeviceListener = getDataFromCarDeviceListener;
        if(getDataTask.isDone()){
            getDataTask.start();
            getDatafuture = executor.submit(getDataTask);
            return true;
        }
        return false;
    }

    /**
     * 停止获取数据
     */
    public void stopGetData(){
        getDataTask.stop();
        if(getDatafuture!=null){
            getDatafuture.cancel(true);
        }

    }

    /**
     * 停止发送文件
     */
    public void stopSendFiles(){
        sendFileTask.stop();
        if(sendDatafuture!=null){
        	sendDatafuture.cancel(true);
        }
    }

//    private void exit(){
//        if(receiveFileTask!=null){
//            receiveFileTask.stop();
//        }
//    }
    private class ConnectTask{
        boolean isPause = false;
        boolean isTimeout = false;
        public void stopConnect(){
            isPause = true;
        }

        public void start(){
            isPause = false;
            isTimeout = false;
        }
        public boolean isTimeout(){
        	return  isTimeout;
        }
        private void createConnect(ConnectListener connectListener){
            int currentState = 0;
            try {
                if (clientsocket == null||clientsocket.isClosed()) {
                    clientsocket = new DatagramSocket();
                    clientsocket.setSoTimeout(socketTimeOut);
                    clientsocket.setReceiveBufferSize(1024*64);
                    clientsocket.setSendBufferSize(1024*64);
                }
                if (receiveSocket == null||receiveSocket.isClosed()) {
                    receiveSocket = new DatagramSocket();
                }
                clientPort = clientsocket.getLocalPort();
                receiveFilePort = receiveSocket.getLocalPort();
                clientsocket.setBroadcast(true);
            }catch (SocketException e) {
                e.printStackTrace();
            }
            Utils.loge("socket.getLocalPort=" + clientPort);
            // 发送广播
            broadcastToServer(clientsocket);

            currentState = STATE_BROADCAST;

            byte[] buf = new byte[1024];
            DatagramPacket packet = new DatagramPacket(buf, buf.length,null,serverPort);

            while (!isPause){
            	  Utils.loge("createConnect 开始接收:");
                try {
                    clientsocket.receive(packet);
                } catch (IOException e) {
                    e.printStackTrace();
                    if(currentState == STATE_BROADCAST){
                        // 发送广播
                      int result  =  reSendMsg(STATE_BROADCAST, loopMaxNum, broadcastInterval);
//                        broadcastToServer(clientsocket);
                      if(result == -1){
                    	  isTimeout = true;
                    	   if(connectListener!=null){
                    		   connectListener.onConnectTimeOut();
                    	   }
                    	  isPause = true;
                    	  break;
                      }
                    }
                    if(currentState == STATE_SEND_DEVICENO){
                        int result = reSendMsg(STATE_SEND_DEVICENO,loopMaxNum,msgInterval);
                        if(result == -1){
                            // 重新发送广播
                            currentState = STATE_BROADCAST;
                            broadcastToServer(clientsocket);
                            continue;
                        }else{
                            continue;
                        }

                    }
                    continue;
                }
              
                String recvMsg = new String(buf, 0,
                        packet.getLength());
                recvMsg = Utils.decode(recvMsg);
                Utils.loge("createConnect recvMsg:" + recvMsg);
                if(currentState == STATE_BROADCAST){
                    try {

                        DeviceInfo deviceInfo = receiveRespOfBroadcast(packet, recvMsg);
                        if(deviceInfo!=null){

                            if(connectListener!=null){
                                connectListener.onReceiveRespForBroadcast(deviceInfo);
                            }
                            if(!deviceInfo.isVerMatchFlag()){
                                if(connectListener!=null){
                                    connectListener.onFunVerNotMatch();
                                }
                                clientsocket.close();

                                break;
                            }else{
                                currentState = STATE_RECEIVE_RESP_BROADCAST;
                                // 指定只接受此IP的信息
                                packet.setAddress(serverAddress);


                            }
                        }else{
                            reSendMsg(STATE_BROADCAST,-1,broadcastInterval);
                        }

                    }catch (JSONException e){
                        e.printStackTrace();
                        reSendMsg(STATE_BROADCAST,-1,broadcastInterval);
                        continue;
                    }
                }
                if(currentState == STATE_RECEIVE_RESP_BROADCAST){
                    try {
                        sendDeviceNO();
                    }catch (Exception e){
                    	e.printStackTrace();
                        continue;
                    }
                    currentState = STATE_SEND_DEVICENO;
                    Utils.loge("createConnect isPause=:" + isPause);
                    continue;

                }
                if(currentState == STATE_SEND_DEVICENO){
                    try {
                        if(receiveRespOfDevice(recvMsg)){
                            currentState = STATE_RECEIVE_RESP_DEVICENO;
                        }else{
                            int result = reSendMsg(STATE_SEND_DEVICENO,loopMaxNum,msgInterval);
                            if(result ==-1){
    //                            // 重发次数超长，则认为断开
    //                            if(connectListener!=null){
    //                                connectListener.onDisConnect();
    //                            }
    //                            clientsocket.close();
    //                            break;
                                // 重新发送广播
                                currentState = STATE_BROADCAST;
                                broadcastToServer(clientsocket);
                                continue;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        int result = reSendMsg(STATE_SEND_DEVICENO,loopMaxNum,msgInterval);
                        if(result ==-1){
                            // 重发次数超长，则认为断开
    //                        if(connectListener!=null){
    //                            connectListener.onDisConnect();
    //                        }
    //                        clientsocket.close();
                            // 重新发送广播
                            currentState = STATE_BROADCAST;
                            broadcastToServer(clientsocket);
    //                        break;
                            continue;
                        }else{
                            continue;
                        }

                    }

                }
                if(currentState == STATE_RECEIVE_RESP_DEVICENO){
                    if(connectListener!=null){
                    	  Utils.loge("connectListener.onConnectSuccess start");
                        connectListener.onConnectSuccess();
                        Utils.loge("connectListener.onConnectSuccess end");
                    }
                    if(receiveFileTask!=null){
                        receiveFileTask.setConectedAddress(serverAddress);
                    }else{
    //                    receiveFileTask = new ReceiveFileTask(receiveSocket,serverAddress,connectListener);
    //                    arrayBlockingQueue.offer(receiveFileTask);
                    }

                    Utils.loge("connectListener.break");
                    break;
                }


            }
            Utils.loge("connectListener.isPause="+isPause);
            if(isPause){
                clientsocket.close();
            }

        }

    }






    class GetDataTask  implements Runnable{
        public ConnectTask connectTask = new ConnectTask();
        private volatile boolean isStopGetData = false;
        private volatile boolean isDone = true;
        @Override
        public void run() {
            isDone = false;
            Utils.loge("GetDataTask线程开始");
            connectTask.createConnect(mGetDataFromCarDeviceListener);
            if(!connectTask.isTimeout()){
            	  Utils.loge("getDataFun start");
                  getDataFun();
                  if(mGetDataFromCarDeviceListener!=null){
                      mGetDataFromCarDeviceListener.onStopGetDataFromCarDevice();
                  }
            }
            isDone = true;
            Utils.loge("GetDataTask线程结束");
        }
        public void stop(){
            connectTask.stopConnect();
            isStopGetData = true;
        }
        public void start(){
            connectTask.start();
            isStopGetData = false;
            isDone = false;
        }
        public boolean isDone(){
            return isDone;
        }
        private void getDataFun() {
            int currentState = 0;
            if(clientsocket==null||clientsocket.isClosed()){
                return ;
            }

            try {
                sendRequestOfData();
            }catch (IOException e){
                e.printStackTrace();

            }
            currentState = STATE_SEND_REQUEST_DATA;


            byte[] buf  = null;
            DatagramPacket packet = null;
            while (!isStopGetData){
                try {
                    buf = new byte[5120];
                    packet = new DatagramPacket(buf, buf.length,serverAddress,serverPort);
                    clientsocket.receive(packet);
                } catch (IOException e) {//重发
                	e.printStackTrace();
                    int result = reSendMsg(STATE_SEND_REQUEST_DATA,loopMaxNum,msgInterval);
                    if(result == -1){
                        if(mGetDataFromCarDeviceListener!=null){
                            mGetDataFromCarDeviceListener.onDisConnect();
                        }
                        clientsocket.close();
                        break;
                    }
                    continue;
                }

          

                String recvMsg = new String(buf, 0,
                        packet.getLength());
                recvMsg = Utils.decode(recvMsg);
                Utils.loge("GetDataTask 接到信息：" + recvMsg);
                try {

                    JSONTokener jsonTokener = new JSONTokener(recvMsg);
                    Object obj = jsonTokener.nextValue();
                    if(!(obj  instanceof JSONObject)){
                        throw new JSONException(recvMsg+"is not json ");
                    }
                    JSONObject json = (JSONObject) obj;
                    int msgType = json.getInt("msgType");
                    int msgNo = json.getInt("msgNo");
                    if(msgType == 1 && msgNo == MAGNO_GETDATA){
                        JSONObject infoJson = json.getJSONObject("info");
                        String deviceNo = null;
                        String customerkey= null;
                        String secretkey= null;
                        String httpServerUrl= null;
                        if(json.has("device_no")){
                        	deviceNo = json.getString("device_no");
                        }
                        if(json.has("customerkey")){
                        	customerkey = json.getString("customerkey");
                        }
                        if(json.has("secretkey")){
                        	secretkey = json.getString("secretkey");
                        }
                        if(json.has("serverurl")){
                        	httpServerUrl = json.getString("serverurl");
                        }
                        if(mGetDataFromCarDeviceListener!=null){
                            mGetDataFromCarDeviceListener.onGetDataFromCarDevice(deviceNo,customerkey,
                            		secretkey,httpServerUrl,
                            		infoJson);
                        }
                        clientsocket.close();
                        break;
                    }else {// 重发消息
                        int result = reSendMsg(STATE_SEND_REQUEST_DATA,loopMaxNum,msgInterval);
                        if(result == -1){
                            if(mGetDataFromCarDeviceListener!=null){
                                mGetDataFromCarDeviceListener.onDisConnect();
                            }
                            clientsocket.close();
                            break;
                        }
                    }
                }catch (JSONException e){
                    e.printStackTrace();
                    int result = reSendMsg(STATE_SEND_REQUEST_DATA,loopMaxNum,msgInterval);
                    if(result == -1){
                        if(mGetDataFromCarDeviceListener!=null){
                            mGetDataFromCarDeviceListener.onDisConnect();
                        }
                        clientsocket.close();
                        break;
                    }
                }

            }

        }
    }

    class SendFileTask implements Runnable{
        List<FileInfo> fileInfoList ;
        FolderInfo folderInfo;
        SendDataListener sendDataListener;
        ConnectTask connectTask = new ConnectTask();
        int finishFlag = 0;
        int currentState = 0;
        private volatile boolean isStopSendFile = false;
        private volatile boolean isDone = true;
        // 已经传输完成的字节数，用于计算传输进度
        private long hasSendFileLength;
        // 文件列表的文件总大小
        private long totalFileLength;
        // 此次真正传输的字节数，不包换断点续传传过的字节，用于计算传输速度
        private long realSendFileLength;

        long startTime;

//        public SendFileTask(List<FileInfo> mfileInfoList,FolderInfo mfolderInfo,SendDataListener msendDataListener){
//            this.fileInfoList = mfileInfoList;
//            this.folderInfo = mfolderInfo;
//            this.sendDataListener = msendDataListener;
//        }
        public void init(List<FileInfo> mfileInfoList,FolderInfo mfolderInfo,SendDataListener msendDataListener){
            this.fileInfoList = mfileInfoList;
            this.folderInfo = mfolderInfo;
            this.sendDataListener = msendDataListener;
            totalFileLength = 0l;
            hasSendFileLength = 0l;
            realSendFileLength = 0l;
            if(mfileInfoList!=null){
                for(FileInfo fileInfo:mfileInfoList){
                    totalFileLength +=  fileInfo.getFileSize();
                }
            }
        }
        public void stop(){
            connectTask.stopConnect();
            isStopSendFile = true;
        }
        public void start(){
            connectTask.start();
            isStopSendFile = false;
            isDone = false;
        }
        public boolean isDone(){
            return isDone;
        }
        @Override
        public void run() {
            Utils.loge("发送线程开始");
            connectTask.createConnect(sendDataListener);
            if(!connectTask.isTimeout()){
                sendFileFun();
                if(sendDataListener!=null&&isStopSendFile){
                    sendDataListener.onStopSendFile();
                }
            }
            isDone = true;
            Utils.loge("发送线程结束");

        }
        private void sendFileFun(){
            if(clientsocket==null||clientsocket.isClosed()){
                return ;
            }
            try {// 发送文件总信息，用于校验完整性
                sendFileTotalInfo();
            } catch (Exception e) {
                // 连接失败
            }
            currentState = STATE_SEND_TOTALFILEINFO;
            while(!isStopSendFile) {
                Utils.loge("开始传文件总信息" );
                byte[] buf = new byte[1024];
                DatagramPacket packet = new DatagramPacket(buf, buf.length);
                try {
                    clientsocket.receive(packet);
                } catch (IOException e) {
                    e.printStackTrace();
                    if (currentState == STATE_SEND_TOTALFILEINFO) {// 重发文件夹总信息
                        int result = reSendMsg(STATE_SEND_TOTALFILEINFO, loopMaxNum, msgInterval);
                        if (result == -1) {
                            if (sendDataListener != null) {
                                sendDataListener.onDisConnect();
                            }
                            clientsocket.close();
                            msgMap.remove(STATE_SEND_TOTALFILEINFO);
                            isStopSendFile = true;
                            break;

                        }
                    }
                    continue;
                }
                String recvMsg = new String(buf, 0,
                        packet.getLength());
                recvMsg = Utils.decode(recvMsg);
                Utils.loge("接到文件总信息回复：" + recvMsg);
                if (currentState == STATE_SEND_TOTALFILEINFO) {
                    try {
                        int state = receiveRespOfTotalFileInfo(recvMsg);
                        if (state == RESP_MEMORY_NOT_ENOUGN) {// 车机空间不足
                            msgMap.remove(STATE_SEND_TOTALFILEINFO);
                            if(sendDataListener!=null){
                                sendDataListener.onReceiverNotEnoughMemory();
                            }
                            isStopSendFile = true;
                            break;
                        }else if(state == RESP_OK){
                            msgMap.remove(STATE_SEND_TOTALFILEINFO);
                            break;
                        }else if(state == -1){
                            int result = reSendMsg(STATE_SEND_TOTALFILEINFO,loopMaxNum,msgInterval);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                msgMap.remove(STATE_SEND_TOTALFILEINFO);
                                isStopSendFile = true;
                                break;
                            }
                            continue;
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        int result = reSendMsg(STATE_SEND_TOTALFILEINFO, loopMaxNum, msgInterval);
                        if (result == -1) {
                            if (sendDataListener != null) {
                                sendDataListener.onDisConnect();
                            }
                            clientsocket.close();
                            msgMap.remove(STATE_SEND_TOTALFILEINFO);
                            isStopSendFile = true;
                            break;
                        }
                        continue;
                    }


                }


            }

            startTime = System.currentTimeMillis();
            while(fileInfoList.size()>0&&!isStopSendFile){
                if(clientsocket==null){
                    return ;
                };
                sendFileInfo(fileInfoList);
                currentState = STATE_SEND_FILEINFO;
                byte[] buf = new byte[1024];
                DatagramPacket packet = new DatagramPacket(buf, buf.length);
                // 指定只接受此IP的信息
                packet.setAddress(serverAddress);

                while (!isStopSendFile) {
                    try {
                        clientsocket.receive(packet);
                    } catch (IOException e) {
                        e.printStackTrace();
                        if(currentState == STATE_SEND_FILEINFO){
                            int result = reSendMsg(STATE_SEND_FILEINFO,loopMaxNum,msgInterval);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                msgMap.remove(STATE_SEND_FILEINFO);
                                isStopSendFile = true;
                                break;
                            }
                        }
                        if(currentState == STATE_SEND_FILE){

                            int result = reSendMsg(STATE_SEND_FILE,loopMaxNum,msgInterval);
                            Utils.loge("重发文件result"+result);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                msgMap.remove(STATE_SEND_FILE);
                                isStopSendFile = true;
                                break;
                            }


                        }
                        continue;
                    }

                    String recvMsg = new String(buf, 0,
                            packet.getLength());
                    recvMsg = Utils.decode(recvMsg);
//                    Utils.loge("接到信息："+recvMsg);
                    Utils.loge("接到文件回复：" + recvMsg);

                    FileRequireInfo fileRequireInfo = null;
                    if(currentState == STATE_SEND_FILEINFO){
                        try {
                            fileRequireInfo = receiveRespOfFileInfo(recvMsg);
                        }catch (JSONException e){
                            e.printStackTrace();
                            int result = reSendMsg(STATE_SEND_FILEINFO,loopMaxNum,msgInterval);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                isStopSendFile = true;
                                break;
                            }
                            continue;
                        }
                        if(fileRequireInfo == null){
                            int result = reSendMsg(STATE_SEND_FILEINFO,loopMaxNum,msgInterval);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                stopSendFiles();
                                break;
                            }
                            continue;
                        }else{
                            msgMap.remove(STATE_SEND_FILEINFO);
                            if(fileRequireInfo.getResult() == RESP_FILE_FINISH){// 已传输完成
                                Utils.loge("此文件早已传输完成");
                                // 增加已传输完成的字节长度
                                hasSendFileLength+=fileInfoList.get(0).getFileSize();
                               fileInfoList.remove(0);
                                if(fileInfoList.size()==0){
                                    // 更新速度和进度
                                    callbackUpdateProcess();
                                    finishFlag = 1;// 所有文件传输完毕
                               
                                }
                                break;
                            }else if(fileRequireInfo.getResult() == RESP_MEMORY_NOT_ENOUGN){// 接收端空间不足
                                if(sendDataListener!=null){
                                    sendDataListener.onReceiverNotEnoughMemory();
                                   
                                }
                                stopSendFiles();
                                break;
                            }else if(fileRequireInfo.getResult() ==FILE_NOT_FOUND){// 文件路径不存在
                            	  if(sendDataListener!=null){
                                      sendDataListener.onSendFileFail(fileInfoList.get(0).getFileAllPath(), 3, "文件路径不存在");
                                     
                                  }
                            	  stopSendFiles();
                                  break;
                            }else if(fileRequireInfo.getResult() ==IO_EXCEPTION){// 读取文件发生的其他IO
                            	if(sendDataListener!=null){
                                    sendDataListener.onSendFileFail(fileInfoList.get(0).getFileAllPath(), 4, "读取文件发生的其他IO异常");
                                                     }
                            	stopSendFiles();
                                break;
                            }
                        }


                    }

                    if(currentState == STATE_RECEIVE_RESP_FILEINFO){//收到文件信息反馈，开始传文件

                        try{
                            sendFile(fileRequireInfo.getOffset()<0?0:fileRequireInfo.getOffset());
//                            sendFile();
                            currentState = STATE_SEND_FILE;
                        }catch (IOException e){
                            e.printStackTrace();
                        }

                    }
                    if(currentState == STATE_SEND_FILE){
                        try {
                            FileResult fileResult = receiveRespOfFile(recvMsg);
                            if(fileResult == null){
                                int result = reSendMsg(STATE_SEND_FILE,loopMaxNum,msgInterval);
                                if(result == -1){
                                    if(sendDataListener!=null){
                                        sendDataListener.onDisConnect();
                                    }
                                    clientsocket.close();
                                    msgMap.remove(STATE_SEND_FILEINFO);
                                    isStopSendFile = true;
                                    break;
                                }
                                continue;
                            }else if(fileResult == FileResult.COMPLETE){// 一个文件传输完毕
//                                finishFlag = 1;
                                if(fileInfoList.size()==0){// 所有文件传输完毕
                                    finishFlag = 1;
                                }

                                break;
                            }else if(fileResult == FileResult.CONTINUE){// 继续往下传
                                try{
                                    sendFile(-1);
                                    currentState = STATE_SEND_FILE;
                                }catch (IOException e){
                                    e.printStackTrace();
                                }
                            }else if(fileResult == FileResult.RECEIVER_ERROR){//传输失败
                                isStopSendFile = true;
                                break;
                            }else if(fileResult == FileResult.RESEND){// 文件校验不通过，需要重新传单个文件

                                FileInfo fileInfo = fileInfoList.get(0);
                                fileInfo.setOffset(0);
                                break;
//                                try{
//                                    sendFile(0);
//                                    currentState = STATE_SEND_FILE;
//                                }catch (IOException e){
//                                    e.printStackTrace();
//                                }
                            }

                        }catch (JSONException e){
                            e.printStackTrace();
                            int result = reSendMsg(STATE_SEND_FILE,loopMaxNum,msgInterval);
                            if(result == -1){
                                if(sendDataListener!=null){
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                msgMap.remove(STATE_SEND_FILEINFO);
                                isStopSendFile = true;
                                break;
                            }
                        }


                    }

                }
            }
            if(finishFlag == 1&&!isStopSendFile){// 所有文件传输完毕
                try {
                    Utils.loge("发送文件传完通知");
                    sendNoticeFileShareFinish();
                } catch (Exception e) {
                    // 连接失败
                }
                currentState = STATE_SEND_NITICE_FILESHAREFINISH;

                while(!isStopSendFile) {

                    byte[] buf = new byte[1024];
                    DatagramPacket packet = new DatagramPacket(buf, buf.length);
                    try {
                        clientsocket.receive(packet);
                    } catch (IOException e) {
                        e.printStackTrace();
                        if (currentState == STATE_SEND_NITICE_FILESHAREFINISH) {// 重发文件夹总信息
                            int result = reSendMsg(STATE_SEND_NITICE_FILESHAREFINISH, loopMaxNum, msgInterval);
                            if (result == -1) {
                                if (sendDataListener != null) {
                                    sendDataListener.onDisConnect();
                                }
                                clientsocket.close();
                                msgMap.remove(STATE_SEND_NITICE_FILESHAREFINISH);
                                isStopSendFile = true;
                                break;

                            }

                        }
                        continue;
                    }
                    String recvMsg = new String(buf, 0,
                            packet.getLength());
                    recvMsg = Utils.decode(recvMsg);
                    Utils.loge("接收文件传完通知"+recvMsg);
                    if (currentState == STATE_SEND_NITICE_FILESHAREFINISH) {
                        try {
                            int result =  receiveRespOfNoticeFileShareFinish(recvMsg);
                            msgMap.remove(STATE_SEND_NITICE_FILESHAREFINISH);
                            if(result == RESP_OK){
                                if(sendDataListener!=null){
                                    sendDataListener.onSendAllFileFinish();
                                }
                            }else if(result == RESP_VALID_ERROR){// 校验不通过，需要全部重传
                                if(sendDataListener!=null){
                                    sendDataListener.onSendFileFail("",2,"文件夹校验不通过，需要全部重新传");
                                }
                            }

                            break;
                        } catch (JSONException e) {
                            if (currentState == STATE_SEND_NITICE_FILESHAREFINISH) {// 重发文件夹总信息
                                int result = reSendMsg(STATE_SEND_NITICE_FILESHAREFINISH, loopMaxNum, msgInterval);
                                if (result == -1) {
                                    if (sendDataListener != null) {
                                        sendDataListener.onDisConnect();
                                    }
                                    clientsocket.close();
                                    msgMap.remove(STATE_SEND_NITICE_FILESHAREFINISH);
                                    isStopSendFile = true;
                                    break;

                                }
                            }
                            continue;
                        }


                    }


                }



            }


        }
        private void sendNoticeFileShareFinish()throws IOException{

            String msg = Utils.encode(noticeFileShareFinish);
            sendMsg(msg);
            Message currentMsg = new Message();
            currentMsg.setID(STATE_SEND_NITICE_FILESHAREFINISH);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(msg.getBytes());
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_NITICE_FILESHAREFINISH, currentMsg);
        }
        private int receiveRespOfNoticeFileShareFinish(String msg)throws JSONException{
            JSONTokener jsonTokener = new JSONTokener(msg);
            Object obj = jsonTokener.nextValue();
            if(obj instanceof JSONObject){
                JSONObject json = (JSONObject) obj;
                int msgType = json.getInt("msgType");
                int msgNo = json.getInt("msgNo");
                if(!(msgType==1&&msgNo==MAGNO_NOTICE_FINISH)){
                    throw new JSONException("");

                }else{
                    return json.getInt("result");
                }
            }else{
                throw new JSONException(msg+"is not json ");
            }


        }
        private void sendFileTotalInfo()throws IOException{

            String msg = getSendFileTotalInfoMsg(fileInfoList, folderInfo);
            sendMsg(msg);
            Message currentMsg = new Message();
            currentMsg.setID(STATE_SEND_TOTALFILEINFO);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(msg.getBytes());
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_TOTALFILEINFO, currentMsg);
        }
        private void sendFileInfo(List<FileInfo> fileInfoList){

            FileInfo fileInfo = fileInfoList.get(0);
            String msg = getSendFileInfoMsg(fileInfo);
            try {
                sendMsg(msg);
            }catch (Exception f){
                f.printStackTrace();
            }

            Message currentMsg = new Message();
            currentMsg.setID(STATE_SEND_FILEINFO);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(msg.getBytes());
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_FILEINFO, currentMsg);
        }
        private int receiveRespOfTotalFileInfo(String msg)throws JSONException{
            JSONTokener jsonTokener = new JSONTokener(msg);
            Object obj = jsonTokener.nextValue();
            if(!(obj instanceof JSONObject)){
                throw new JSONException(msg+"is not json ");
            }
            JSONObject json = (JSONObject) obj;
            int msgType = json.getInt("msgType");
            int msgNo = json.getInt("msgNo");
            if(msgType==1&&msgNo==MAGNO_SENDFILE_TOTALINFO){
                int result = json.getInt("result");
                if(result==RESP_OK){
                    currentState = STATE_SEND_FILEINFO;//进入发单个文件信息状态
                }
                return result;

            }
            return -1;
        }
        RandomAccessFile randomAccessFile;
        /**
         * 接受文件信息的确认信息
         * @param msg
         * @throws JSONException
         */
        private FileRequireInfo receiveRespOfFileInfo(String msg)throws JSONException{
            JSONTokener jsonTokener = new JSONTokener(msg);
            Object obj = jsonTokener.nextValue();
            if(!(obj instanceof JSONObject)){
                throw new JSONException(msg+"is not json ");
            }
            JSONObject json = (JSONObject) obj;
            int msgType = json.getInt("msgType");
            int msgNo = json.getInt("msgNo");
            if(msgType==1&&msgNo==MAGNO_SENDFILE_INFO){
                int result = json.getInt("result");
                long offset = json.getLong("offset");
                FileRequireInfo fileRequireInfo =  new FileRequireInfo();
                fileRequireInfo.setResult(result);
                fileRequireInfo.setOffset(offset);
                currentState = STATE_RECEIVE_RESP_FILEINFO;
                try{
                    randomAccessFile  = new RandomAccessFile(fileInfoList.get(0).getFileAllPath(), "rwd");
                }catch (FileNotFoundException e) {
                	  e.printStackTrace();
                	  fileRequireInfo.setResult(FILE_NOT_FOUND);
                	  
                }catch (Exception e){
                    e.printStackTrace();
                    fileRequireInfo.setResult(IO_EXCEPTION);
                    
                }
                return fileRequireInfo;
            }
            return null;
        }

        private FileResult receiveRespOfFile(String msg)throws JSONException{
            JSONTokener jsonTokener = new JSONTokener(msg);
            Object obj = jsonTokener.nextValue();
            if(!(obj instanceof JSONObject)){
                throw new JSONException(msg+"is not json ");
            }
            JSONObject json = (JSONObject) obj;
            int msgType = json.getInt("msgType");
            int msgNo = json.getInt("msgNo");
            if(msgType==1&&msgNo==MAGNO_SENDFILE){
                int result = json.getInt("result");
                String msgstr = json.getString("msg");
                FileInfo fileInfo = fileInfoList.get(0);
                if(result==RESP_OK){
                    realSendFileLength+=fileInfo.getSendBufferLength();
                    fileInfo.setOffset(fileInfo.getSendBufferLength() + fileInfo.getOffset());
                    hasSendFileLength+=fileInfo.getSendBufferLength();
                    // 更新传输进度和速度
                    callbackUpdateProcess();
                    fileInfo.setSendBufferLength(0);
                    File file = new File( fileInfo.getFileAllPath());
//                    Utils.loge("文件总大小："+file.length()+" 已发送文件大小："+fileInfo.getOffset());
                    if(file.length()<=fileInfo.getOffset()){// 此文件已经传完
                        fileInfoList.remove(0);//
                        if(sendDataListener!=null){
                            sendDataListener.onSendSingleFileFinish(fileInfo.getFileAllPath());
                        }
                        return FileResult.COMPLETE;
                    }else{
                        return FileResult.CONTINUE;
                    }

                }else if(result==RESP_VALID_ERROR){// 重新传输
                    if(sendDataListener!=null){
                        sendDataListener.onSendFileFail(fileInfo.getFileAllPath(),0,"文件传输不完整，需要重新传输");
                    }
                    // 回退发送进度
                    hasSendFileLength -= fileInfo.getOffset();
                    return FileResult.RESEND;

                }else{
                    if(sendDataListener!=null){
                        sendDataListener.onSendFileFail(fileInfo.getFileAllPath(),1,msgstr);
                    }
                    return FileResult.RECEIVER_ERROR;
                }

            }else{
                return null;
            }
        }
//        private void sendFile() throws IOException {
//            while(true){
//                FileInfo fileInfo = fileInfoList.get(0);
//                JSONObject  sendJson = new JSONObject();
//
//                try {
//                    sendJson.put("msgType", 0);
//                    sendJson.put("msgNo", MAGNO_SENDFILE);
//                    sendJson.put("deviceNo",deviceNO);
//                    sendJson.put("filePath", fileInfo.getFileRelativePath());
//
//                    sendJson.put("offset",fileInfo.getOffset());
//                } catch (JSONException e) {
//                    e.printStackTrace();
//                }
//
//                if(fileInfo.getOffset()==0){
//                    Utils.loge("开始传文件" );
//                }
//
//                byte[] strbyte = (sendJson.toString()+"&").getBytes();
//                int fileLength = MAXLENGTH-strbyte.length-20;
////                        while(){
//                byte[] filebytes = new byte[fileLength];
//
//
//
//                randomAccessFile.seek(fileInfo.getOffset());
//                int readSize = randomAccessFile.read(filebytes);
//                if(readSize==-1){
//                    Utils.loge("文件传输完毕" );
//                    break;
//                }
//                try {
//                    sendJson.put("size",readSize);
//                } catch (JSONException e) {
//                    e.printStackTrace();
//                }
//                strbyte = (sendJson.toString()+"&").getBytes();
//
////            randomAccessFile.close();
//                byte[] bytes = concat(strbyte,filebytes,readSize);
//
//
//
//                sendMsg(bytes);
//                fileInfo.setOffset(fileInfo.getOffset()+readSize);
//                fileInfo.setHasSendLength(readSize) ;
//            }
//        }
        private void sendFile(long offset) throws IOException {

            JSONObject  sendJson = new JSONObject();
            FileInfo fileInfo = fileInfoList.get(0);
            try {
                sendJson.put("msgType", 0);
                sendJson.put("msgNo", MAGNO_SENDFILE);
                sendJson.put("deviceNo",deviceNO);
                sendJson.put("filePath", fileInfo.getFileRelativePath());
                if(offset>=0){// 第一次传此文件
                    fileInfo.setOffset(offset);
                    hasSendFileLength+=offset;
                }
                sendJson.put("offset",fileInfo.getOffset());

            } catch (JSONException e) {
                e.printStackTrace();
            }

            if(fileInfo.getOffset()==0){
                Utils.loge("开始传文件" );
            }



            byte[] strbyte = (Utils.encode(sendJson.toString())+"&").getBytes();
//            byte[] strbyte = (sendJson.toString()+"&").getBytes();
            int fileLength = MAXLENGTH-strbyte.length-30;
//                        while(){
            byte[] filebytes = new byte[fileLength];



            randomAccessFile.seek(fileInfo.getOffset());
            int readSize = randomAccessFile.read(filebytes);

            Utils.loge("读取文件大小："+readSize);
            if(readSize==-1){
                readSize = 0;
            }
            try {
                sendJson.put("size",readSize);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            strbyte = (Utils.encode(sendJson.toString())+"&").getBytes();
//            strbyte = (sendJson.toString()+"&").getBytes();

//            randomAccessFile.close();
            byte[] bytes = concat(strbyte,filebytes,readSize);

            sendMsg(bytes);

            fileInfo.setSendBufferLength(readSize) ;

            Message currentMsg = msgMap.get(STATE_SEND_FILE);
            if(currentMsg == null){
                currentMsg = new Message();

            }
            currentMsg.setID(STATE_SEND_FILE);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(bytes);
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_FILE, currentMsg);


        }
        private void callbackUpdateProcess(){
            if(sendDataListener!=null){
                long gapTime = System.currentTimeMillis()  - startTime;
                float percent = 0f;
                if(totalFileLength>0){
                	  int temp = (int)(((float)hasSendFileLength)/totalFileLength*10000);
                	  percent = (float)temp/100;
                }
                sendDataListener.updateProcess(percent,((float) realSendFileLength) / gapTime * 1000);
            }
        }
    }





    private void broadcastToServer(DatagramSocket mBRDSocket) {
        DatagramPacket mBRDPacket = null;
        try {
//            String encodeStr = Utils.encode(getbroadcastMsg());
            String encodeStr = getbroadcastMsg();
            byte[] buf = encodeStr.getBytes();
            mBRDPacket = new DatagramPacket(buf, buf.length,null,serverPort);
            InetAddress broadcastAddress = Utils.getBroadcastAddress();
            if(broadcastAddress !=null){
                Utils.loge("Util.getBroadcastAddress()=" + broadcastAddress.toString());
                mBRDPacket.setAddress(broadcastAddress);

                mBRDSocket.send(mBRDPacket);
            }else{
                Utils.loge("Util.getBroadcastAddress()= null");
            }

            Message currentMsg = new Message();
            currentMsg.setID(STATE_BROADCAST);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(buf);
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_BROADCAST,currentMsg);
        } catch (Exception e) {
            e.printStackTrace();
        }


    }

    /**
     * 重发消息
     * @param msgId
     * @param maxSendNum
     * @param interval
     * @return -1-重发次数达到上限,0-已重发，1-还未到时间间隔，未重发
     */
    private int reSendMsg(int msgId,int maxSendNum,int interval){
        Message currentMsg = msgMap.get(msgId);
        Utils.loge("reSendMsg msgId="+msgId+"maxSendNum="+maxSendNum+"interval="+interval );
        if(currentMsg==null){
            return -2;
        }
        Utils.loge("reSendMsg currentMsg.getReSendNum()="+currentMsg.getReSendNum() );
      
        if(maxSendNum>0&&currentMsg.getReSendNum()>=maxSendNum-1){
            return -1;
        }else if(System.currentTimeMillis()-currentMsg.getSendTime()>=interval*1000*(currentMsg.getReSendNum()+1)){
      	  Utils.loge("reSendMsg currentMsg.getSendTime()="+currentMsg.getSendTime() +"System.currentTimeMillis()="+System.currentTimeMillis());
        	try {
                sendMsg(currentMsg.getContent()); 
            } catch (IOException e) {
                e.printStackTrace();
            }
        	currentMsg.setReSendNum(currentMsg.getReSendNum() + 1);
            return 0;
        }else{
            return 1;
        }

    }
    private DeviceInfo receiveRespOfBroadcast(DatagramPacket packet,String msg) throws JSONException {
        JSONTokener jsonTokener = new JSONTokener(msg);
        Object obj = jsonTokener.nextValue();
        if(!(obj instanceof JSONObject)){
            throw new JSONException(msg+"is not json ");
        }
        JSONObject json = (JSONObject) obj;
            int msgType = json.getInt("msgType");
            int msgNo = json.getInt("msgNo");
            if(msgType==1&&msgNo==MAGNO_BROADCAST){

                String deviceName = json.getString("deviceName");
                String pttVer = json.getString("ptt");
                int verMatch = json.getInt("verMatch");

                DeviceInfo deviceInfo = new DeviceInfo();
                deviceInfo.setDeviceName(deviceName);
                deviceInfo.setVerCode(pttVer);
                deviceInfo.setVerMatchFlag(verMatch==1?true:false);
                if(verMatch==1){
                    serverAddress = packet.getAddress();

                }
               return deviceInfo;
            }else{
                return null;// 消息步骤不对
            }


    }
    private void sendDeviceNO() throws Exception{
        Utils.loge("sendDeviceNO");
        try {
            String msg = getSenddiviceNoMsg();
            sendMsg(msg);
            Message currentMsg = new Message();
            currentMsg.setID(STATE_SEND_DEVICENO);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(msg.getBytes());
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_DEVICENO, currentMsg);
        }catch (Exception f){
            f.printStackTrace();
        }

    }

    private boolean  receiveRespOfDevice(String msg)throws JSONException{
        JSONTokener jsonTokener = new JSONTokener(msg);
        Object obj = jsonTokener.nextValue();
        if(!(obj instanceof JSONObject)){
            throw new JSONException(msg+"is not json ");
        }
        JSONObject json  = (JSONObject)obj;
        int msgType = json.getInt("msgType");
        int msgNo = json.getInt("msgNo");
        if(msgType==1&&msgNo==MAGNO_SENDDEVICENO){
            return true;
        }
        return false;
    }
    private void sendRequestOfData() throws IOException{
    	  Utils.loge("sendRequestOfData start");
        try {
            String encode =  Utils.encode(requestDataMsg);
            sendMsg(encode);
            Message currentMsg = new Message();
            currentMsg.setID(STATE_SEND_REQUEST_DATA);
            currentMsg.setReSendNum(0);
            currentMsg.setContent(encode.getBytes());
            currentMsg.setSendTime(System.currentTimeMillis());
            msgMap.put(STATE_SEND_REQUEST_DATA, currentMsg);
        }catch (IOException f){
            f.printStackTrace();
        }
        Utils.loge("sendRequestOfData end");
    }

    private void sendMsg(String msg) throws IOException{
        byte[] buf = msg.getBytes();
        DatagramPacket mBRDPacket = new DatagramPacket(buf, buf.length,serverAddress,serverPort);
        if(serverAddress==null){
            mBRDPacket = new DatagramPacket(buf, buf.length,Utils.getBroadcastAddress(),serverPort);
        }
        clientsocket.send(mBRDPacket);

    }
    private void sendMsg(byte[] msg) throws IOException{
        DatagramPacket mBRDPacket;
        if(serverAddress==null){
            if(Utils.getBroadcastAddress()!=null){
                mBRDPacket = new DatagramPacket(msg, msg.length,Utils.getBroadcastAddress(),serverPort);
                clientsocket.send(mBRDPacket);
            }

        }else{
            mBRDPacket = new DatagramPacket(msg, msg.length,serverAddress,serverPort);
            clientsocket.send(mBRDPacket);
        }


    }


    private static  byte[] concat(byte[] first, byte[] second) {
        byte[] result = Arrays.copyOf(first, first.length + second.length);
        System.arraycopy(second, 0, result, first.length, second.length);
        return result;
    }
    private static  byte[] concat(byte[] first, byte[] second,int sendcondlength) {
        byte[] result = Arrays.copyOf(first, first.length + second.length);
        System.arraycopy(second, 0, result, first.length, sendcondlength);
        return result;
    }




    private String getbroadcastMsg(){
        JSONTokener jsonTokener = new JSONTokener(broadcastMsg);

        JSONObject json = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            json.put("version",currVer);
            JSONArray pptArray = new JSONArray();
            for(int i =0;i<supportVer.length;i++){
                pptArray.put(supportVer[i]);
            }
            json.put("ptt",pptArray);
            json.put("deviceName",deviceName);
            json.put("port",clientPort);
            json.put("receivePort", receiveFilePort);
            return Utils.encode(json.toString());
        }catch (JSONException e){
            e.printStackTrace();
        }
        return null;
    }
    private String getSendFileTotalInfoMsg(List<FileInfo> fileInfoList,FolderInfo folderInfo){

        JSONTokener jsonTokener = new JSONTokener(sendFileTotalInfoMsg);
        JSONObject json = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            json.put("folderPath",folderInfo.getFolderPath());
            json.put("totalFileNum",folderInfo.getTotalFileNum());
            json.put("totalFileSize",folderInfo.getTotalFileSize());
            json.put("deviceNo",deviceNO);
            json.put("dataName",folderInfo.getDataName());
            int size = fileInfoList.size();
            JSONArray fileArray = new JSONArray();
            for(int i =0;i<size;i++){
                JSONObject jsonObject = new JSONObject();
                FileInfo fileInfo = fileInfoList.get(i);
                jsonObject.put("filePath",fileInfo.getFileRelativePath());
                jsonObject.put("fileSize",fileInfo.getFileSize());
                jsonObject.put("filemd5",fileInfo.getMd5());
                fileArray.put(jsonObject);
            }
            json.put("fileList", fileArray);
            return Utils.encode(json.toString());
        }catch (JSONException e){
            e.printStackTrace();
        }
        return null;
    }
    private String getSendFileInfoMsg(FileInfo fileInfo){
        JSONTokener jsonTokener = new JSONTokener(sendFileInfoMsg);
        JSONObject json = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            json.put("filePath",fileInfo.getFileRelativePath());
            json.put("fileSize",fileInfo.getFileSize());
            json.put("filemd5",fileInfo.getMd5());
            json.put("deviceNo",deviceNO);
             return Utils.encode(json.toString());
        }catch (JSONException e){
            e.printStackTrace();
        }
        return null;
    }
    private String getSenddiviceNoMsg(){
        JSONTokener jsonTokener = new JSONTokener(senddiviceNoMsg);
        JSONObject json = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            json.put("deviceNo",deviceNO);
            return Utils.encode(json.toString());
        }catch (JSONException e){
            e.printStackTrace();
        }
        return null;
    }
}
