package com.mxnavi.tspv2.download;

import android.util.Log;
import android.webkit.URLUtil;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Created by Administrator on 2016/5/24.
 * desc
 */
public class DownloadPool {
    final String TAG="TSPSDK-DownloadPool";
    static private DownloadPool downloadPool ;
    ThreadPoolExecutor executor;
//    private int mMaxTask = 2;
    /**
     *  获取单例对象
     * @param maxTask 最大线程数
     * @return
     */
      public static synchronized DownloadPool getInstance(int maxTask) {
          if(downloadPool==null){
              if(maxTask<=0){
                  return null;
              }
              downloadPool = new DownloadPool(maxTask);
          }

          return downloadPool;
      }

    private DownloadPool(int maxTask){

         executor = new ThreadPoolExecutor(maxTask, maxTask, 30, TimeUnit.SECONDS,linkedBlockingQueue);
         executor.allowCoreThreadTimeOut(true);
    }
    ConcurrentHashMap<String,DownloadTask> taskMap = new ConcurrentHashMap<String,DownloadTask>();


    LinkedBlockingQueue linkedBlockingQueue = new LinkedBlockingQueue<DownloadTask>();




    /**
     * 添加下载任务
     * @param downloadInfo 下载任务信息
     * @return true 添加成功 false 已经添加过
     */
    public boolean addDownloadTask(DownloadInfo downloadInfo){
        if(downloadInfo.getDownloadTaskId()==null){
            Log.e(TAG, "downloadTaskId is invalid. downloadTaskId : " + downloadInfo.getDownloadTaskId());
            throw new IllegalArgumentException("downloadTaskId is invalid");
        }
        if (downloadInfo.getFilePath() == null || downloadInfo.getFilePath().trim().length() == 0) {
            Log.e(TAG, "filePath is invalid. file name : " + downloadInfo.getFilePath());
            throw new IllegalArgumentException("filePath is invalid");
        }

        if (null == downloadInfo.getDownloadUrl() || !URLUtil.isHttpUrl(downloadInfo.getDownloadUrl())) {
            Log.e(TAG, "invalid http url:" + downloadInfo.getDownloadUrl());
            throw new IllegalArgumentException("invalid http url");
        }

        DownloadTask downloadTask = new DownloadTask(downloadInfo.getDownloadTaskId(),downloadInfo.getTotalFileSize(),
                downloadInfo.getDownloadUrl(),downloadInfo.getStartPos(),  downloadInfo.getFilePath(), downloadInfo.getDownloadListener(),DownloadPool.this);
        if(linkedBlockingQueue.contains(downloadTask)||taskMap.containsKey(downloadInfo.getDownloadTaskId())){
            Log.e(TAG, "task existed : ");
            return false;
        }

        taskMap.put(downloadInfo.getDownloadTaskId(), downloadTask);
        downloadTask.onStart();
        executor.execute(downloadTask);

        return true;
    }

    public void  setDownloadListener(String downloadTaskId,DownloadListener downloadListener){
        DownloadTask downloadTask = taskMap.get(downloadTaskId);
        if(downloadTask !=null) {
            downloadTask.setDownloadListener(downloadListener);
        }
    }
    /**
     * 继续下载
     * @param downloadInfo 下载任务信息
     */
    public void onContinue(DownloadInfo downloadInfo){
        DownloadTask downloadTask = taskMap.get(downloadInfo.getDownloadTaskId());
        if(downloadTask !=null){
            if(downloadTask.isDownLoading()){
                return ;
            }
            downloadTask.onStart();
            executor.execute(downloadTask);
        }else{
            addDownloadTask(downloadInfo);
        }
    }

    /**
     * 暂停下载
     * @param downloadTaskId 下载任务ID
     */
    public void onPause(String downloadTaskId){
        DownloadTask downloadTask = taskMap.get(downloadTaskId);
        if(downloadTask !=null){
            downloadTask.onPause();
        }
    }


    /**
     * 取消下载
     * @param downloadTaskId
     */
    public void deleteDownloadTask(String downloadTaskId){
        DownloadTask downloadTask = taskMap.get(downloadTaskId);
        if(downloadTask !=null){
            downloadTask.onPause();
            if(linkedBlockingQueue.contains(downloadTask)){
                linkedBlockingQueue.remove(downloadTask);
            }
        }

        taskMap.remove(downloadTaskId);

    }


    /**
     * 下载任务是否正在下载中
     * @param downloadTaskId 下载任务ID
     * @return true：下载中 false 未下载
     */
    public  boolean isDownLoading(String downloadTaskId){
        DownloadTask downloadTask = taskMap.get(downloadTaskId);
        if(downloadTask !=null){
            return downloadTask.isDownLoading();
        }
        return false;
    }

    /**
     * 停止所有下载任务
     */
    public void pauseAll(){
        Collection<DownloadTask> DownloadTasks = taskMap.values();
        Iterator<DownloadTask> iterator = DownloadTasks.iterator();
        while(iterator.hasNext()){
            iterator.next().onPause();
        }
    }

    /**
     * 删除所有下载任务
     */
    public void deleteAllDownloadTask(){
        pauseAll();
        taskMap.clear();
        linkedBlockingQueue.clear();

    }
    /**
     * 开始所有下载任务
     */
//    public void continueAll(){
//        Collection<DownloadTask> DownloadTasks = taskMap.values();
//        Iterator<DownloadTask> iterator = DownloadTasks.iterator();
//        while(iterator.hasNext()){
//            DownloadTask downloadTask = iterator.next();
//            if(!downloadTask.isDownLoading()){
//                iterator.next().onStart();
//                executor.execute(downloadTask);
//            }
//
//        }
//    }
    protected void finishDownloadTask(String downloadTaskId){
        taskMap.remove(downloadTaskId);
    }
    public interface DownloadListener {
        /**
         * 通知开始下载
         * @param taskId 下载任务ID
         */
        public void onDownloadStart(String taskId);
        /**
         * 下载失败
         * @param downloadTaskId 下载任务ID
         * @param errormsg 错误信息
         */
        public void onDownloadFail(String downloadTaskId, String errormsg);
        /**
         * 下载已停止接口
         * @param downloadTaskId 下载任务ID
         */
        public void onDownloadStop(String downloadTaskId);


        /**
         * 下载完成接口
         * @param downloadTaskId 下载任务ID
         */
        public void onDownloadCompleted(String downloadTaskId);

        /**
         * 存储空间不足
         * @param downloadTaskId 下载任务ID
         * @param fileSize 此次下载文件大小
         */
        public void noEnoughMemoryAvailable(String downloadTaskId, long fileSize);

        /**
         * 更新下载进度
         * @param downloadTaskId 下载任务ID
         * @param completeSize 已下载文件的大小，单位为字节
         * @param downloadSpeed 下载速度 单位是B/s
         */
        public void updateProcess(String downloadTaskId, long completeSize,float downloadSpeed);

        /**
         * 下载时文件读写错误
         * @param downloadTaskId 下载任务ID
         */
        public void onFileWriteFail(String downloadTaskId);

        /**
         * 第一次下载时，返回文件的总大小
         * @param downloadTaskId 下载任务ID
         * @param fileSize 文件的总大小 单位为字节
         */
        public void getFileSize(String downloadTaskId, long fileSize);

        /**
         * 连接超时
         * @param downloadTaskId 下载任务ID
         */
        public void onConnectTimeOut(String downloadTaskId);

        /**
         * 下载前创建接收下载文件失败
         * @param downloadTaskId 下载任务ID
         */
        public void onMakeFileFail(String downloadTaskId);

    }

}
