package com.mxnavi.tspv2.download;

import android.util.Log;

import com.mxnavi.tspv2.utils.Utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DecimalFormat;


public class DownloadTask implements Runnable{
    final String TAG ="TSP-SDK-DownloadTask";
	final int MESSAGE_CONNETCT_FAIL = 0;
	final int MESSAGE_CONNETCT_TIMEOUT = 1;
	final int MESSAGE_MAKEFILE_FAIL = 2;
	final int MESSAGE_FILEWRITE_ERROR = 3;
	final int MESSAGE_UPDATE_PROCESS = 4;
	final int MESSAGE_DOWNLOAD_COMPLETE = 5;
	final int MESSAGE_GET_FILESIZE = 6;
	final int MESSAGE_NOEnoughSPACE = 7;
	final int MESSAGE_HAS_STOPPED_DOWNLOAD = 8;
	
	DownloadPool.DownloadListener mDownloadListener;

	private String download_url;
	private String mFilePath;
	private String taskId;

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        DownloadTask that = (DownloadTask) o;

        if (!taskId.equals(that.taskId)) return false;
        return mdownloadPool.equals(that.mdownloadPool);

    }

    @Override
    public int hashCode() {
        int result = taskId.hashCode();
        result = 31 * result + mdownloadPool.hashCode();
        return result;
    }

    //	private String logFilePath = Environment.getExternalStorageDirectory()
//			.getAbsolutePath() + "/" + "TEST";
//	String downLoadLogFilePath = logFilePath + "/download.txt";// {"status":"","totalSize":11,"completeSize":11,"version":""}
//	String logInitString = "{\"status\":\"\",\"totalSize\":0,\"completeSize\":0,\"version\":\"\"}";
//	File downLoadLogFile;
//	JSONObject logJson;


	private long fileLength;
    private long completeSize = 0;
    private DownloadPool mdownloadPool;
    private volatile boolean isDownLoad = false;


//	DownLoadThread downLoadThread;

	private Object mPauseLock = new Object();

    protected DownloadTask(String id,long totalFileSize, String downloadUrl, long startPos, String filePath, DownloadPool.DownloadListener downloadListener, DownloadPool downloadPool){
		taskId = id;
		fileLength = totalFileSize;
		mDownloadListener = downloadListener;
		download_url = downloadUrl;
		completeSize = startPos;
		mFilePath = filePath;
		mdownloadPool = downloadPool;
	}
    protected void  setDownloadListener(DownloadPool.DownloadListener downloadListener){
        mDownloadListener = downloadListener;
    }
    private void callback(int messageType,Object arg1,Object arg2){
        switch (messageType) {
            case	MESSAGE_CONNETCT_FAIL :
                if(mDownloadListener!=null){
                    mDownloadListener.onDownloadFail(taskId,"网络连接失败");
                }
                break;


            case MESSAGE_CONNETCT_TIMEOUT :
                if(mDownloadListener!=null){
                    mDownloadListener.onConnectTimeOut(taskId);
                }
                break;
            case MESSAGE_MAKEFILE_FAIL  :
                if(mDownloadListener!=null){
                    mDownloadListener.onMakeFileFail(taskId);
                }
                break;
            case MESSAGE_FILEWRITE_ERROR :
                if(mDownloadListener!=null){

                    mDownloadListener.onFileWriteFail(taskId);
                }
                break;

            case MESSAGE_UPDATE_PROCESS:
                if(mDownloadListener!=null){
                    mDownloadListener.updateProcess(taskId,(Long)arg1,(Float)arg2);
                }
                break;

            case MESSAGE_DOWNLOAD_COMPLETE :
				mdownloadPool.finishDownloadTask(taskId);
                if(mDownloadListener!=null){
                    mDownloadListener.onDownloadCompleted(taskId);
                }
                break;
            case MESSAGE_GET_FILESIZE :
                if(mDownloadListener!=null){
                    mDownloadListener.getFileSize(taskId,(Long) arg1);
                }
                break;
            case MESSAGE_NOEnoughSPACE :
                if(mDownloadListener!=null){
                    mDownloadListener.noEnoughMemoryAvailable(taskId,(Long)arg1);
                }
                break;
            case MESSAGE_HAS_STOPPED_DOWNLOAD:
                if(mDownloadListener!=null){
                    mDownloadListener.onDownloadStop(taskId);
                }
                break;
            
        }
    }

	protected void onPause() {
		synchronized (mPauseLock) {
			Log.e("==========", "onPause");
			isDownLoad = false;
		}

	}


	protected void onStart() {
		if(download_url==null||mFilePath==null){
			return ;
		}
		if(isDownLoad){
			return ;
		}
		isDownLoad = true;
//		downLoadThread = new DownLoadThread();
//		downLoadThread.start();

	}


	//	 CountDownLatch threadEndLatch  = new CountDownLatch(1);
//	private void onResume() {
//		synchronized (mPauseLock) {
//			Log.e("==========", "onResume");
//			isDownLoad = true;
//			mPauseLock.notifyAll();
//		}
//
//	}

	protected boolean isDownLoading() {
		return isDownLoad;
	}


    // 如果连接超时则自动连接5次
    private int connectCount =1;
    private void download(){
        if(mDownloadListener!=null){
            mDownloadListener.onDownloadStart(taskId);
        }
		HttpURLConnection connection = null;
		RandomAccessFile randomAccessFile = null;
		InputStream inputStream;
		long startSize = 0;
		try {
			URL url = new URL(download_url);
			connection = (HttpURLConnection) url.openConnection();
			connection.setConnectTimeout(15000);
			connection.setRequestMethod("GET");
			connection.setReadTimeout(15000);
			// int recode=connection.getResponseCode();
			Log.e(TAG, "completeSize="+completeSize+"fileLength="+fileLength);
			if (completeSize > 0) {
				connection.setRequestProperty("Range", "bytes="
						+ (completeSize) + "-" + fileLength);
			} else {
				fileLength = connection.getContentLength();
			}
			if (Utils.getAvailableExternalMemorySize() < fileLength) {// SD卡没有足够空间
				isDownLoad = false;
                callback(MESSAGE_NOEnoughSPACE, fileLength, 0);

				return;
//						threadEndLatch.countDown();

			}
		
			int status = connection.getResponseCode();
			Log.e(TAG, "getResponseCode="+status);
		
			inputStream = connection.getInputStream();
			Utils.loge("文件大小"+fileLength);
			if (completeSize ==0){
				callback(MESSAGE_GET_FILESIZE, fileLength, 0);
				Log.e("sendStartMessage", "fileLength=" + fileLength);
			}
		} catch (MalformedURLException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			isDownLoad = false;
			Log.e(TAG, "MalformedURLException");
            callback(MESSAGE_CONNETCT_FAIL,null,0);
			return;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			if (e.toString().startsWith("java.net.SocketTimeoutException")) {
				Log.e(TAG, "IOException");
				connectCount++;
				if(connectCount>5){
                    callback(MESSAGE_CONNETCT_TIMEOUT,null,0);
					connectCount =1;
				}else{
					download();
				}

			} else {
				Log.e(TAG, "otherException");
                callback(MESSAGE_CONNETCT_FAIL, null, 0);
				isDownLoad = false;
			}

			return;
		} catch (Exception e) {
			e.printStackTrace();
			Log.e(TAG, "Exception");
			isDownLoad = false;
            callback(MESSAGE_CONNETCT_FAIL, null, 0);
			return;
		}
		connectCount =1;
		File file = new File(mFilePath);
		if (!file.exists()) {

			try {
				if (!file.getParentFile().exists()) {
					file.getParentFile().mkdirs();
				}
				file.createNewFile();
			} catch (IOException e) {
				// TODO Auto-generated catch block
                callback(MESSAGE_MAKEFILE_FAIL, null, 0);
				e.printStackTrace();
				isDownLoad = false;
				return;

			}
		}
		int length = -2;
		if(file.length()==0){
			completeSize = 0;
			Utils.loge("file.length()=0");
		}
	   Utils.loge("file.length()===="+file.length());
		// 本地访问文件
		try {
			randomAccessFile = new RandomAccessFile(mFilePath, "rwd");

			randomAccessFile.seek(completeSize);
			byte[] buffer = new byte[4096];
			int i = 0;
			long startTime = System.currentTimeMillis();
			while (isDownLoad && (length = inputStream.read(buffer)) != -1) {
				randomAccessFile.write(buffer, 0, length);
				completeSize += length;

				Utils.loge("completeSize="+completeSize);
				i++;

				if (i % 5 == 0) {//每5次更新一下进度
					Log.e("while", i + "");
					long kbSize = completeSize - startSize;
					long secondTime = System.currentTimeMillis()  - startTime;
					DecimalFormat df = new DecimalFormat("0.00");
					float floatKbSize = ((float) kbSize) / secondTime * 1000;
					Utils.loge("下载速度："+df.format(floatKbSize /1024)+"KB/s");
                    callback(MESSAGE_UPDATE_PROCESS, completeSize, floatKbSize);

//					Message message1 = handler.obtainMessage();
//					message1.what = MESSAGE_UPDATE_PROCESS;
//					message1.obj = completeSize;
//					sendEmptyMessage(message1);
				}

			}
			long kbSize = completeSize - startSize;
			long secondTime = System.currentTimeMillis()  - startTime;
			float floatKbSize = ((float) kbSize) / secondTime * 1000;
			
            callback(MESSAGE_UPDATE_PROCESS, completeSize, floatKbSize);
//			Message message1 = handler.obtainMessage();
//			message1.what = MESSAGE_UPDATE_PROCESS;
//			message1.obj = completeSize;
//			sendEmptyMessage(message1);

			if (length == -1 && completeSize == fileLength) {//网络切换的时候文件未下载完会返回-1
				completeSize = 0;
                callback(MESSAGE_DOWNLOAD_COMPLETE, 0, 0);

			}else if(isDownLoad &&length == -1 && completeSize < fileLength){//文件还没真正下载完就返回-1
				Utils.loge("download not really finish completeSize="+completeSize+"fileLength="+fileLength);
				callback(MESSAGE_CONNETCT_FAIL, 0, 0);
			}else if(!isDownLoad){// 已经停止下载
				callback(MESSAGE_HAS_STOPPED_DOWNLOAD, null, 0);
			}
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			isDownLoad = false;
            callback(MESSAGE_FILEWRITE_ERROR, 0, 0);

		} catch (IOException e) {
			e.printStackTrace();
			// TODO Auto-generated catch block
			connectCount++;
			if(connectCount<6){// 如果网络连接失败则自动重连5次
				download();
			}else{

				isDownLoad = false;
				connectCount =1;
				if ("java.net.SocketTimeoutException".equals(e.toString())) {
                    callback(MESSAGE_CONNETCT_TIMEOUT, 0, 0);
                } else {
                    callback(MESSAGE_CONNETCT_FAIL, 0, 0);

				}
			}
		}finally{
			try{
				randomAccessFile.close();
				inputStream.close();
			}catch(Exception e){
				e.printStackTrace();
			}
			
		}

		isDownLoad = false;

	}
//	class DownLoadThread extends Thread {

		@Override
		public void run() {
			download();
	}





}