package com.mxnavi.tspv2.driving;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.driving.model.CityPriceResult;
import com.mxnavi.tspv2.driving.model.DriverInfo;
import com.mxnavi.tspv2.driving.model.HistoryOrderDetail;
import com.mxnavi.tspv2.driving.model.HistoryOrderInfoItem;
import com.mxnavi.tspv2.driving.model.HistoryOrders;
import com.mxnavi.tspv2.driving.model.OrderInfo;
import com.mxnavi.tspv2.driving.model.OrderRequest;
import com.mxnavi.tspv2.driving.model.PartPrice;
import com.mxnavi.tspv2.driving.model.PollOrderRequest;
import com.mxnavi.tspv2.driving.model.PollOrderResult;
import com.mxnavi.tspv2.driving.model.PriceItem;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.model.TspDate;
import com.mxnavi.tspv2.model.TspDateTime;
import com.mxnavi.tspv2.model.TspTime;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2016/5/3.
 * 代驾服务
 */
public class DrivingService extends BaseTsp {

    // 请求服务类型为代驾服务
    final int thisServiceType = Config.MxNGTPServiceType.DRIVING.getServiceType();
    // 注册此对象为代驾服务信息接收类
    final int serviceDataVer = Config.MxNGTPServiceType.DRIVING.getServiceVer();

    private Map<Integer,Integer> listenertypeHashMap = new HashMap<Integer,Integer>();
    final int DS_GetDriversNearbyIdle =1;
    final int DS_GetPriceByCity =2;
    final int DS_GetValidNum =3;
    final int DS_Login =4;
    final int DS_CommitOrder =5;
    final int DS_PollOrder =6;
    final int DS_GetDriverInfo =7;
    final int DS_CancelOrder =8;
    final int DS_GetHistoryOrders =9;
    final int DS_GetHistoryOrderDetail =10;



    OnGetDriversNearbyIdleListener mOnGetDriversNearbyIdleListener;
    OnGetPriceByCityListener mOnGetPriceByCityListener;
    OnGetValidNumListener mOnGetValidNumListener;
    OnLoginListener mOnLoginListener;
    OnCommitOrderListener mOnCommitOrderListener;
    OnPollOrderListener mOnPollOrderListener;
    OnGetDriverInfoListener mOnGetDriverInfoListener;
    OnCancelOrderListener mOnCancelOrderListener;
    OnGetHistoryOrdersListener mOnGetHistoryOrdersListener;
    OnGeHistorytOrderDetailListener mOnGeHistorytOrderDetailListener;

    /**
     * 请求获取周边空闲司机信息
     * @param location 用户位置：经纬度
     * @return 请求ID
     */
    public int getDriversNearbyIdle(LatLng location){
        return getDriversNearbyIdle(location,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取周边空闲司机信息
     * @param location 用户位置：经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getDriversNearbyIdle(LatLng location,int timeOutSecs){
        return getDriversNearbyIdle(location,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取周边空闲司机信息
     * @param location 用户位置：经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getDriversNearbyIdle(LatLng location,int timeOutSecs,int priorityLevel){
        if(location == null){
            if (mOnGetDriversNearbyIdleListener != null) {
                mOnGetDriversNearbyIdleListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "driver/nearbyIdle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("lon",Utils.converDoubleCoordinate2MX(location.getLongitude()));
        params.put("lat",Utils.converDoubleCoordinate2MX(location.getLatitude()));
        params.put("udid",Config.VIN);

        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetDriversNearbyIdle);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取城市价格信息
     * @param location 用户位置：经纬度
     * @return 请求ID
     */
    public int getPriceByCity(LatLng location){
        return getPriceByCity(location, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取城市价格信息
     * @param location 用户位置：经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getPriceByCity(LatLng location,int timeOutSecs){
        return getPriceByCity(location, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取城市价格信息
     * @param location 用户位置：经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getPriceByCity(LatLng location,int timeOutSecs,int priorityLevel){
        if(location == null){
            if (mOnGetPriceByCityListener != null) {
                mOnGetPriceByCityListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "city/price";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("lon",Utils.converDoubleCoordinate2MX(location.getLongitude()));
        params.put("lat",Utils.converDoubleCoordinate2MX(location.getLatitude()));

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetPriceByCity);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求验证码获取信息
     * @param phone 手机号码
     * @return 请求ID
     */
    public int getValidNum(String phone){
        return getValidNum(phone, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求验证码获取信息
     * @param phone 手机号码
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getValidNum(String phone,int timeOutSecs){
        return getValidNum(phone, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求验证码获取信息
     * @param phone 手机号码
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getValidNum(String phone,int timeOutSecs,int priorityLevel){
        if(phone == null){
            if (mOnGetValidNumListener != null) {
                mOnGetValidNumListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "customer/loginpre";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("phone",phone);
        params.put("udid",Config.VIN);

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetValidNum);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求Token获取信息
     * @param phone 手机号码
     * @param validNum 验证码
     * @return 请求ID
     */
    public int login(String phone,String validNum){
        return login(phone, validNum, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求Token获取信息
     * @param phone 手机号码
     * @param validNum 验证码
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int login(String phone,String validNum,int timeOutSecs){
        return login(phone,validNum, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求Token获取信息
     * @param phone 手机号码
     * @param validNum 验证码
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int login(String phone,String validNum,int timeOutSecs,int priorityLevel){
        if(phone == null || validNum == null){
            if (mOnLoginListener != null) {
                mOnLoginListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "customer/login";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("udid",Config.VIN);
        params.put("phone",phone);
        params.put("vcode",validNum);

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_Login);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取下单接口信息
     * @param orderRequest 下单接口信息条件
     * @return 请求ID
     */
    public int commitOrder(OrderRequest orderRequest){
        return commitOrder(orderRequest, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取下单接口信息
     * @param orderRequest 下单接口信息条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int commitOrder(OrderRequest orderRequest,int timeOutSecs){
        return commitOrder(orderRequest, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取下单接口信息
     * @param orderRequest 下单接口信息条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public  int commitOrder(OrderRequest orderRequest,int timeOutSecs,int priorityLevel){
        if(orderRequest == null){
            if (mOnCommitOrderListener != null) {
                mOnCommitOrderListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "order/commit";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("phone",orderRequest.getPhone());
        params.put("address",orderRequest.getAddress());
        params.put("number",orderRequest.getNumber());
        params.put("lon", Utils.converDoubleCoordinate2MX(orderRequest.getLocation().getLongitude()));
        params.put("lat",Utils.converDoubleCoordinate2MX(orderRequest.getLocation().getLatitude()));
        params.put("token",orderRequest.getToken());
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_CommitOrder);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取轮询订单信息
     * @param pollOrderRequest 轮询订单信息条件
     * @return 请求ID
     */
    public int pollOrder(PollOrderRequest pollOrderRequest){
        return pollOrder(pollOrderRequest, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取轮询订单信息
     * @param pollOrderRequest 轮询订单信息条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int pollOrder(PollOrderRequest pollOrderRequest,int timeOutSecs){
        return pollOrder(pollOrderRequest, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取轮询订单信息
     * @param pollOrderRequest 轮询订单信息条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int pollOrder(PollOrderRequest pollOrderRequest,int timeOutSecs,int priorityLevel){
        if(pollOrderRequest == null){
            if (mOnPollOrderListener != null) {
                mOnPollOrderListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "order/poll";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("bookingType",pollOrderRequest.getBookingType());
        params.put("bookingID",pollOrderRequest.getOrderId());
        params.put("token",pollOrderRequest.getToken());
        params.put("pollCnt",pollOrderRequest.getPollCnt());

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_PollOrder);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取司机信息
     * @param driverId 司机ID
     * @return 请求ID
     */
    public int getDriverInfo(String driverId){
        return getDriverInfo(driverId, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取司机信息
     * @param driverId 司机ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getDriverInfo(String driverId,int timeOutSecs){
        return getDriverInfo(driverId, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取司机信息
     * @param driverId 司机ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getDriverInfo(String driverId,int timeOutSecs,int priorityLevel){
        if(driverId == null){
            if (mOnGetDriverInfoListener != null) {
                mOnGetDriverInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "driver/info";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("driverID",driverId);

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetDriverInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求订单取消信息
     * @param orderId 预约ID
     * @param token 用户凭证
     * @return 请求ID
     */
    public int cancelOrder(String orderId,String token){
        return cancelOrder(orderId, token,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求订单取消信息
     * @param orderId 预约ID
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int cancelOrder(String orderId,String token,int timeOutSecs){
        return cancelOrder(orderId, token,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求订单取消信息
     * @param orderId 预约ID
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int cancelOrder(String orderId,String token,int timeOutSecs,int priorityLevel){
        if(orderId == null || token == null){
            if (mOnCancelOrderListener != null) {
                mOnCancelOrderListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "order/cancel";
        HashMap<String, Object> params = new HashMap<String, Object>();

        params.put("token",token);
        params.put("bookingID",orderId);
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_CancelOrder);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求历史订单信息
     * @param pageNo 第几页
     * @param pageSize 每页显示条数
     * @param token 用户凭证
     * @return 请求ID
     */
    public int getHistoryOrders(int pageNo,int pageSize,String token){
        return getHistoryOrders(pageNo, pageSize, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求历史订单信息
     * @param pageNo 第几页
     * @param pageSize 每页显示条数
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getHistoryOrders(int pageNo,int pageSize,String token,int timeOutSecs){
        return getHistoryOrders(pageNo, pageSize, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *  请求历史订单信息
     * @param pageNo 第几页
     * @param pageSize 每页显示条数
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getHistoryOrders(int pageNo,int pageSize,String token,int timeOutSecs,int priorityLevel){
        if(token == null){
            if (mOnGetHistoryOrdersListener != null) {
                mOnGetHistoryOrdersListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "order/list";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("pageNo",pageNo);
        params.put("pageSize",pageSize);
        params.put("token",token);

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetHistoryOrders);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求订单详情信息
     * @param orderId 订单编号
     * @param token 用户凭证
     * @return 请求ID
     */
    public int getHistoryOrderDetail(String orderId,String token){
        return getHistoryOrderDetail(orderId,token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求订单详情信息
     * @param orderId 订单编号
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getHistoryOrderDetail(String orderId,String token,int timeOutSecs){
        return getHistoryOrderDetail(orderId,token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求订单详情信息
     * @param orderId 订单编号
     * @param token 用户凭证
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getHistoryOrderDetail(String orderId,String token,int timeOutSecs,int priorityLevel){
        if(orderId == null || token == null){
            if (mOnGeHistorytOrderDetailListener != null) {
                mOnGeHistorytOrderDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "order/detail";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("orderID",orderId);
        params.put("token",token);

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, DS_GetHistoryOrderDetail);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg){
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case DS_GetDriversNearbyIdle:
                if(mOnGetDriversNearbyIdleListener != null) {
                    if(hasDSPTError){
                        mOnGetDriversNearbyIdleListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONObject dataObject =  jsonObject.getJSONObject("data");
                            int driverCnt = dataObject.getInt("driverCnt");
                            float minDist = dataObject.getInt("minDist");
                            mOnGetDriversNearbyIdleListener.onGetDriversNearbyIdle(driverCnt,minDist);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetDriversNearbyIdleListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetDriversNearbyIdleListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_GetPriceByCity:
                if(mOnGetPriceByCityListener != null) {
                    if(hasDSPTError){
                        mOnGetPriceByCityListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    List<PriceItem> priceList;
                    List<PartPrice> partPriceList;
                    PriceItem priceItem;
                    PartPrice partPrice;
                    CityPriceResult result = new CityPriceResult();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String cityName = jsonObject.getString("city");

                            priceList = new ArrayList<PriceItem>();

                            priceItem = new PriceItem();
                            JSONObject priceObject = jsonObject.getJSONObject("priceList");

                            JSONArray partPriceDataArray = priceObject.getJSONArray("partPrice");
                            partPriceList = new ArrayList<PartPrice>();
                            for (int j = 0; j < partPriceDataArray.length(); j++) {

                                partPrice = new PartPrice();
                                JSONObject partPriceObject = partPriceDataArray.getJSONObject(j);

                                partPrice.setPrice(partPriceObject.getInt("price"));
                                partPrice.setStartTime(partPriceObject.getString("startTime"));
                                partPrice.setEndTime(partPriceObject.getString("endTime"));
                                partPriceList.add(partPrice);

                            }

                            priceItem.setPartPrice(partPriceList);
                            priceItem.setDistince(priceObject.getInt("distince"));
                            priceItem.setStepDistince(priceObject.getInt("nextDistince"));
                            priceItem.setStepPrice(priceObject.getInt("nextPrice"));
                            priceItem.setBeforeWaitingTime(priceObject.getInt("beforeWaitingTime"));
                            priceItem.setBeforeWaitingPrice(priceObject.getInt("beforeWaitingPrice"));
                            priceItem.setDesc(priceObject.getString("desc"));
                            priceList.add(priceItem);
                            result.setCityName(cityName);
                            result.setPriceList(priceList);
                            mOnGetPriceByCityListener.onGetPriceByCity(result);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetPriceByCityListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetPriceByCityListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_GetValidNum:
                if(mOnGetValidNumListener != null) {
                    if(hasDSPTError){
                        mOnGetValidNumListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnGetValidNumListener.onGetValidNumSuccess();
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetValidNumListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetValidNumListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_Login:
                if(mOnLoginListener != null) {
                    if(hasDSPTError){
                        mOnLoginListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnLoginListener.onLoginSuccess(jsonObject.getString("token"));
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnLoginListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnLoginListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_CommitOrder:
                if(mOnCommitOrderListener != null) {
                    if(hasDSPTError){
                        mOnCommitOrderListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONObject dataObiect = jsonObject.getJSONObject("data");
                            OrderInfo orderInfo = new OrderInfo();
                            orderInfo.setOrderId(dataObiect.getString("bookingId"));
                            long second = dataObiect.getLong("orderTime");
                            Date date = new Date(second*1000);
                            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            String dateStr = sdf.format(date);

                            String[] secondArray = dateStr.split(" ");
                            String StringDate = secondArray[0];
                            String StringTime = secondArray[1];

                            String[] dateArray = StringDate.split("-");
                            String[] timeArray = StringTime.split(":");

                            TspDateTime orderTime = new TspDateTime();
                            TspDate tspdate = new TspDate();
                            tspdate.setYear(Integer.parseInt(dateArray[0]));
                            tspdate.setMonth(Integer.parseInt(dateArray[1]));
                            tspdate.setDay(Integer.parseInt(dateArray[2]));

                            TspTime tspTime = new TspTime();
                            tspTime.setHour(Integer.parseInt(timeArray[0]));
                            tspTime.setMin(Integer.parseInt(timeArray[1]));
                            tspTime.setSec(Integer.parseInt(timeArray[2]));

                            orderTime.setDate(tspdate);
                            orderTime.setTime(tspTime);

                            orderInfo.setOrderTime(orderTime);
                            orderInfo.setBookingType(dataObiect.getInt("bookingType"));
                            orderInfo.setTimeout(dataObiect.getInt("timeout"));

                            mOnCommitOrderListener.onCommitOrder(orderInfo);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnCommitOrderListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnCommitOrderListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_PollOrder:
                if(mOnPollOrderListener != null) {
                    if(hasDSPTError){
                        mOnPollOrderListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);

                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            PollOrderResult pollOrderResult = new PollOrderResult();
                            JSONObject dataObject = jsonObject.getJSONObject("data");
                            pollOrderResult.setDriverId(dataObject.getString("driverID"));
                            pollOrderResult.setOrderId(dataObject.getString("orderID"));
                            pollOrderResult.setPollState(dataObject.getInt("pollStat"));
                            mOnPollOrderListener.OnPollOrder(pollOrderResult);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnPollOrderListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnPollOrderListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_GetDriverInfo:
                if(mOnGetDriverInfoListener != null) {
                    if(hasDSPTError){
                        mOnGetDriverInfoListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);

                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            DriverInfo driverInfo = new DriverInfo();
                            driverInfo.setName(jsonObject.getString("name"));
                            driverInfo.setAvatarUrl(jsonObject.getString("avatar"));
                            driverInfo.setPhone(jsonObject.getString("phone"));
                            mOnGetDriverInfoListener.OnGetDriverInfo(driverInfo);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetDriverInfoListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetDriverInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_CancelOrder:
                if(mOnCancelOrderListener != null) {
                    if(hasDSPTError){
                        mOnCancelOrderListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);

                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnCancelOrderListener.OnCancelOrderSuccess();
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnCancelOrderListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnCancelOrderListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_GetHistoryOrders:
                if(mOnGetHistoryOrdersListener != null) {
                    if(hasDSPTError){
                        mOnGetHistoryOrdersListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);

                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        List<HistoryOrderInfoItem> ordersList;
                        HistoryOrderInfoItem historyOrderInfoItem;
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            HistoryOrders historyOrders = new HistoryOrders();
                            historyOrders.setTotal(jsonObject.getInt("total"));
                            ordersList = new ArrayList<HistoryOrderInfoItem>();
                            JSONArray ordersDataArray = jsonObject.getJSONArray("orders");
                            for(int i =0; i < ordersDataArray.length(); i++){
                                historyOrderInfoItem = new HistoryOrderInfoItem();
                                JSONObject ordersObject = ordersDataArray.getJSONObject(i);
                                TspDateTime orderTime = new TspDateTime();
                                int startTime = ordersObject.getInt("startTime");

                                Date date = new Date(startTime);
                                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");

                                String dateStr = sdf.format(date);

                                String[] secondArray = dateStr.split(" ");
                                String StringDate = secondArray[0];
                                String StringTime = secondArray[1];

                                String[] dateArray = StringDate.split("-");
                                String[] timeArray = StringTime.split(":");

                                TspDate tspdate = new TspDate();
                                tspdate.setYear(Integer.parseInt(dateArray[0]));
                                tspdate.setMonth(Integer.parseInt(dateArray[1]));
                                tspdate.setDay(Integer.parseInt(dateArray[2]));

                                TspTime tspTime = new TspTime();
                                tspTime.setHour(Integer.parseInt(timeArray[0]));
                                tspTime.setMin(Integer.parseInt(timeArray[1]));
                                tspTime.setSec(Integer.parseInt(timeArray[2]));

                                orderTime.setDate(tspdate);
                                orderTime.setTime(tspTime);


                                historyOrderInfoItem.setOrderTime(orderTime);
                                historyOrderInfoItem.setStartPos(ordersObject.getString("startPos"));
                                historyOrderInfoItem.setEndPos(ordersObject.getString("endPos"));
                                historyOrderInfoItem.setOrderId(ordersObject.getString("orderID"));
                                ordersList.add(historyOrderInfoItem);
                            }
                            historyOrders.setOrders(ordersList);
                            mOnGetHistoryOrdersListener.OnGetHistoryOrderss(historyOrders);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetHistoryOrdersListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetHistoryOrdersListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DS_GetHistoryOrderDetail:
                if(mOnGeHistorytOrderDetailListener != null) {
                    if(hasDSPTError){
                        mOnGeHistorytOrderDetailListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);

                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            HistoryOrderDetail historyOrderDetail = new HistoryOrderDetail();
                            historyOrderDetail.setOrderId(jsonObject.getString("orderID"));
                            historyOrderDetail.setStartPos(jsonObject.getString("startPos"));
                            TspDateTime orderTime = new TspDateTime();
                            int year = 0;
                            int month = 0;
                            int day = 0;
                            int hour = 0;
                            int minute = 0;
                            int second = 0;

                            String dateStr = jsonObject.getString("orderTime");

                            if(Integer.parseInt(dateStr) != 0 && dateStr != null && dateStr != ""){
                                String[] secondArray = dateStr.split(" ");
                                String StringDate = secondArray[0];
                                String StringTime = secondArray[1];

                                String[] dateArray = StringDate.split("-");
                                String[] timeArray = StringTime.split(":");

                                 year = Integer.parseInt(dateArray[0]);
                                 month = Integer.parseInt(dateArray[1]);
                                 day = Integer.parseInt(dateArray[2]);
                                 hour = Integer.parseInt(timeArray[0]);
                                 minute = Integer.parseInt(timeArray[1]);
                                 second = Integer.parseInt(timeArray[2]);
                            }

                            TspDate tspdate = new TspDate();
                            tspdate.setYear(year);
                            tspdate.setMonth(month);
                            tspdate.setDay(day);

                            TspTime tspTime = new TspTime();
                            tspTime.setHour(hour);
                            tspTime.setMin(minute);
                            tspTime.setSec(second);

                            orderTime.setDate(tspdate);
                            orderTime.setTime(tspTime);
                            historyOrderDetail.setOrderTime(orderTime);
                            historyOrderDetail.setDriverName(jsonObject.getString("driverName"));
                            historyOrderDetail.setDriverPhone(jsonObject.getString("driverPhone"));
                            historyOrderDetail.setTotalPrice(jsonObject.getString("totalPrice"));
                            historyOrderDetail.setSubsidy(jsonObject.getString("subsidy"));
                            historyOrderDetail.setKiloFee(jsonObject.getString("kiloFee"));
                            historyOrderDetail.setWaitingFee(jsonObject.getString("waitingFee"));
                            mOnGeHistorytOrderDetailListener.onGetHistoryOrderDetail(historyOrderDetail);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGeHistorytOrderDetailListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGeHistorytOrderDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }

    public void setOnGetDriversNearbyIdleListener(DrivingService.OnGetDriversNearbyIdleListener onGetDriversNearbyIdleListener) {
        mOnGetDriversNearbyIdleListener = onGetDriversNearbyIdleListener;
    }

    public void setOnGetPriceByCityListener(DrivingService.OnGetPriceByCityListener onGetPriceByCityListener) {
        mOnGetPriceByCityListener = onGetPriceByCityListener;
    }

    public void setOnGetValidNumListener(DrivingService.OnGetValidNumListener onGetValidNumListener) {
        mOnGetValidNumListener = onGetValidNumListener;
    }

    public void setOnLoginListener(DrivingService.OnLoginListener onLoginListener) {
        mOnLoginListener = onLoginListener;
    }

    public void setOnCommitOrderListener(DrivingService.OnCommitOrderListener onCommitOrderListener) {
        mOnCommitOrderListener = onCommitOrderListener;
    }

    public void setOnPollOrderListener(DrivingService.OnPollOrderListener onPollOrderListener) {
        mOnPollOrderListener = onPollOrderListener;
    }

    public void setOnGetDriverInfoListener(DrivingService.OnGetDriverInfoListener onGetDriverInfoListener) {
        mOnGetDriverInfoListener = onGetDriverInfoListener;
    }

    public void setOnCancelOrderListener(DrivingService.OnCancelOrderListener onCancelOrderListener) {
        mOnCancelOrderListener = onCancelOrderListener;
    }

    public void setOnGetHistoryOrdersListener(DrivingService.OnGetHistoryOrdersListener onGetHistoryOrdersListener) {
       mOnGetHistoryOrdersListener = onGetHistoryOrdersListener;
    }

    public void setOnGeHistorytOrderDetailListener(DrivingService.OnGeHistorytOrderDetailListener onGeHistorytOrderDetailListener) {
        mOnGeHistorytOrderDetailListener = onGeHistorytOrderDetailListener;
    }

    public interface OnGetDriversNearbyIdleListener extends CallBackListener{
        /**
         * 获取周边空闲司机结果回调接口
         * @param driverCnt 周边司机个数
         * @param minDist 最近的司机距离（单位是公里）
         */
        public void  onGetDriversNearbyIdle(int driverCnt,float minDist);
    }
    public interface OnGetPriceByCityListener  extends CallBackListener{
        public void  onGetPriceByCity(CityPriceResult cityPriceResult);
    }
    public interface OnGetValidNumListener  extends CallBackListener{
        public void  onGetValidNumSuccess();
    }
    public interface OnLoginListener  extends CallBackListener{
        public void  onLoginSuccess(String token);
    }
    public interface OnCommitOrderListener  extends CallBackListener{
        public void  onCommitOrder(OrderInfo orderInfo);
    }
    public interface OnPollOrderListener  extends CallBackListener{
        public void  OnPollOrder(PollOrderResult pollOrderResult);
    }
    public interface OnGetDriverInfoListener  extends CallBackListener{
        public void  OnGetDriverInfo(DriverInfo driverInfo);
    }
    public interface OnCancelOrderListener  extends CallBackListener{
        public void  OnCancelOrderSuccess();
    }
    public interface OnGetHistoryOrdersListener  extends CallBackListener{
        public void  OnGetHistoryOrderss(HistoryOrders historyOrders);
    }
    public interface OnGeHistorytOrderDetailListener  extends CallBackListener{
        public void  onGetHistoryOrderDetail(HistoryOrderDetail historyOrderDetail);
    }

    private  void addCallbackMap(int eventId,int type){
        super.addCallbackMap(eventId, DrivingService.this);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId){
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {

        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case DS_GetDriversNearbyIdle:
                if (mOnGetDriversNearbyIdleListener != null) {
                    mOnGetDriversNearbyIdleListener.onTimeOut();
                }
                break;
            case DS_GetPriceByCity:
                if (mOnGetPriceByCityListener != null) {
                    mOnGetPriceByCityListener.onTimeOut();
                }
                break;
            case DS_GetValidNum:
                if (mOnGetValidNumListener != null) {
                    mOnGetValidNumListener.onTimeOut();
                }
                break;
            case DS_Login:
                if (mOnLoginListener != null) {
                    mOnLoginListener.onTimeOut();
                }
                break;
            case DS_CommitOrder:
                if (mOnCommitOrderListener != null) {
                    mOnCommitOrderListener.onTimeOut();
                }
                break;
            case DS_PollOrder:
                if (mOnPollOrderListener != null) {
                    mOnPollOrderListener.onTimeOut();
                }
                break;
            case DS_GetDriverInfo:
                if (mOnGetDriverInfoListener != null) {
                    mOnGetDriverInfoListener.onTimeOut();
                }
                break;
            case DS_CancelOrder:
                if (mOnCancelOrderListener != null) {
                    mOnCancelOrderListener.onTimeOut();
                }
                break;
            case DS_GetHistoryOrders:
                if (mOnGetHistoryOrdersListener != null) {
                    mOnGetHistoryOrdersListener.onTimeOut();
                }
                break;
            case DS_GetHistoryOrderDetail:
                if (mOnGeHistorytOrderDetailListener != null) {
                    mOnGeHistorytOrderDetailListener.onTimeOut();
                }
                break;
        }
    }

}
