package com.mxnavi.tspv2.drivingrecord;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.drivingrecord.model.DownloadMediaInfo;
import com.mxnavi.tspv2.drivingrecord.model.DownloadMediaRequestResult;
import com.mxnavi.tspv2.drivingrecord.model.FileInfo;
import com.mxnavi.tspv2.drivingrecord.model.MediaInfo;
import com.mxnavi.tspv2.drivingrecord.model.Param;
import com.mxnavi.tspv2.drivingrecord.model.UploadMediaRequestResult;
import com.mxnavi.tspv2.http.AsyncHttpResponseHandler;
import com.mxnavi.tspv2.http.HttpRequestUtil;
import com.mxnavi.tspv2.http.RequestParams;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.SocketTimeoutException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

//import org.apache.http.Header;

/**
 * Created by Administrator on 2016/4/1.
 * 行车记录仪
 */
public  class DrivingRecord extends BaseTsp {
    // 请求服务类型为行车记录
    final int thisServiceType = Config.MxNGTPServiceType.MX_DRIVING_RECORD.getServiceType();
    //  行车记录的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_DRIVING_RECORD.getServiceVer();

    public DrivingRecord() {
    }


    public interface OnDrivingRecordUploadListener extends CallBackListener {
        public void onUploadMediaSuccess();
        public void onGetUploadMediaParam(String url, List<Param> paramList,String filePath, FileInfo thumbnail, String thumbnailFilePath);
    }
    public interface OnUploadFileListener extends CallBackListener {
        public void onUploadFileSuccess(String noticeParam);
    }
    public interface OnDrivingRecordDownloadListener extends CallBackListener {
        public void onDownloadMediaRequest(List<DownloadMediaRequestResult> list);
    }
    public interface NotifyReceiveCameraListener extends CallBackListener {
        public void onNotifyReceiveCameraSuccess();
    }
    public interface NoticeUploadSuccessListener extends CallBackListener {
        public void onNoticeUploadSuccess();
    }
    public interface OnDrivingRecordUploadFileListener extends CallBackListener {
        public void onUploadFile(FileInfo fileInfo);
//        public void onGetUploadMediaParam(String url, List<Param> paramList,String filePath, FileInfo thumbnail, String thumbnailFilePath);
    }
    OnDrivingRecordUploadListener mDrivingRecordUploadListener;

    public void setOnDrivingRecordUploadListener(OnDrivingRecordUploadListener drivingRecordUploadListener) {
        mDrivingRecordUploadListener = drivingRecordUploadListener;
    }
    OnDrivingRecordUploadFileListener mDrivingRecordUploadFileListener;

    public void setOnDrivingRecordUploadFileListener(OnDrivingRecordUploadFileListener drivingRecordUploadFileListener) {
        mDrivingRecordUploadFileListener = drivingRecordUploadFileListener;
    }

    OnDrivingRecordDownloadListener mDrivingRecordDownloadListener;

    public void setOnDrivingRecordDownloadListener(OnDrivingRecordDownloadListener drivingRecordDownloadListener) {
        mDrivingRecordDownloadListener = drivingRecordDownloadListener;
    }
    NoticeUploadSuccessListener mNoticeUploadSuccessListener;

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    private Map<Integer, MediaInfo> mediaHashMap = new HashMap<Integer, MediaInfo>();
    final int DrivingRecordType_upload = 1;
    final int DrivingRecordType_download = 2;
    final int DrivingRecordType_noticeUploadSuccess = 3;
    final int DrivingRecordType_uploadFile = 4;
    /**
     * 请求上传视频文件
     * @param mediaInfo 视频文件
     * @return 请求ID
     */
    public int uploadMedia(MediaInfo mediaInfo) {
       return uploadMedia(mediaInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求上传视频文件
     * @param mediaInfo 视频文件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int uploadMedia(MediaInfo mediaInfo,int timeOutSecs) {
        return uploadMedia( mediaInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求上传视频文件
     * @param mediaInfo 视频文件
     * @param timeOutSecs 接口超时时间，单位是秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID 0:参数错误
     */
    public int uploadMedia(MediaInfo mediaInfo,int timeOutSecs,int priorityLevel) {
        if(mediaInfo == null||mediaInfo.getPosition()==null){
            if (mDrivingRecordUploadListener != null) {
                mDrivingRecordUploadListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        int eventId = RequestDispathUtil.createEventId();
        String url = "upload/uploadMedia";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY+"");
        params.put("device_no", Config.VIN);
        params.put("record_type", mediaInfo.getRecordType());
        params.put("lon",Utils.converDoubleCoordinate2MX(mediaInfo.getPosition().getLongitude()));
        params.put("lat", Utils.converDoubleCoordinate2MX(mediaInfo.getPosition().getLatitude()));
        params.put("time", (System.currentTimeMillis() / 1000));

        if(mediaInfo.getFilename()!=null){
            params.put("filename", mediaInfo.getFilename());
        }
//        params.put("filename", mediaInfo.getFilename());
        if(mediaInfo.getThumbnailName()!=null){
            params.put("thumbnail", mediaInfo.getThumbnailName());
        }

        String requestData = Utils.makeUrl(url, params);
        Utils.loge("缩率图路径"+mediaInfo.getThumbnailFilePath());
        Utils.loge("requestData="+requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, DrivingRecordType_upload);
        mediaHashMap.put(eventId, mediaInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }



    public int noticeUploadSuccess(String param,NoticeUploadSuccessListener noticeUploadSuccessListener){
        return noticeUploadSuccess(param,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY,noticeUploadSuccessListener);
    }
    public int noticeUploadSuccess(String param,int timeOutSecs,int priorityLevel,NoticeUploadSuccessListener noticeUploadSuccessListener){
        int eventId = RequestDispathUtil.createEventId();
        String url = "callback/notify?"+param;
        this.mNoticeUploadSuccessListener = noticeUploadSuccessListener;
        Utils.loge("noticeUploadSuccess URL="+url);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, url, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, DrivingRecordType_noticeUploadSuccess);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    /**
     * 请求批量下载视频文件
     * @param downloadMediaInfo 请求下载信息
     * @return 请求ID
     */
    public int downloadMediaRequest(DownloadMediaInfo downloadMediaInfo,String vin) {
        return downloadMediaRequest( downloadMediaInfo,vin,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求批量下载视频文件
     * @param downloadMediaInfo 请求下载信息
     * @param timeoutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int downloadMediaRequest(DownloadMediaInfo downloadMediaInfo,String vin,int timeoutSecs) {
        return downloadMediaRequest(downloadMediaInfo, vin,timeoutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求批量下载视频文件
     * @param downloadMediaInfo 请求下载信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int downloadMediaRequest(DownloadMediaInfo downloadMediaInfo,String vin,int timeOutSecs,int priorityLevel) {
        if(downloadMediaInfo == null || Utils.strIsEmpty(vin)){
            if (mDrivingRecordDownloadListener != null) {
                mDrivingRecordDownloadListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "search/getMedia";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY+"");
        params.put("device_no", vin);
        params.put("record_type", downloadMediaInfo.getRecordType());
        params.put("start_time", downloadMediaInfo.getStartTime());
        params.put("end_time", downloadMediaInfo.getEndTime());
        params.put("latest_cnt", downloadMediaInfo.getLatestCnt());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, DrivingRecordType_download);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, DrivingRecord.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);

        }
    }
    /**
     * 请求上传数据文件
     * @param mediaInfo 数据文件信息（thumbnail不用上传）
     * @return 请求ID
     */
    public int uploadFile(MediaInfo mediaInfo) {
        return uploadFile(mediaInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求上传数据文件
     * @param mediaInfo 数据文件信息（thumbnail不用上传）
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int uploadFile(MediaInfo mediaInfo,int timeOutSecs) {
        return uploadFile( mediaInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求上传数据文件
     * @param mediaInfo 数据文件信息（thumbnail不用上传）
     * @param timeOutSecs 接口超时时间，单位是秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID 0:参数错误
     */
    public int uploadFile(MediaInfo mediaInfo,int timeOutSecs,int priorityLevel) {
        if(mediaInfo == null
                ||mediaInfo.getPosition()==null
                ||mediaInfo.getFilename()==null){
            if (mDrivingRecordUploadFileListener != null) {
                mDrivingRecordUploadFileListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/upload/uploadFile";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY+"");
        params.put("device_no", Config.VIN);
        params.put("record_type", mediaInfo.getRecordType());
        params.put("lonlat_type", mediaInfo.getLonlatType());
        params.put("lon",Utils.converDoubleCoordinate2MX(mediaInfo.getPosition().getLongitude()));
        params.put("lat", Utils.converDoubleCoordinate2MX(mediaInfo.getPosition().getLatitude()));
        params.put("time", (System.currentTimeMillis() / 1000));
        params.put("filename", mediaInfo.getFilename());

        String requestData = Utils.makeUrl(url, params);
        Utils.loge("requestData="+requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, DrivingRecordType_uploadFile);
        mediaHashMap.put(eventId, mediaInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
            mediaHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {

        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case DrivingRecordType_upload:
                if (mDrivingRecordUploadListener != null) {
                    mDrivingRecordUploadListener.onTimeOut();
                }
                break;
            case DrivingRecordType_download:
                if (mDrivingRecordDownloadListener != null) {
                    mDrivingRecordDownloadListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        MediaInfo mdiaInfo =  mediaHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        Utils.loge(response);
        switch (type) {
            case DrivingRecordType_upload:
                if (mDrivingRecordUploadListener != null) {
                    if(hasDSPTError){
                        mDrivingRecordUploadListener.onFail(errorCode,errMsg);
                        return;
                    }
                    UploadMediaRequestResult result = new UploadMediaRequestResult();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            if(mdiaInfo.getMediaFilePath() == null){//表示不上传文件
                                mDrivingRecordUploadListener.onUploadMediaSuccess();
                                return;
                            }

                            JSONObject dataObject = jsonObject.getJSONObject("data");
                            JSONObject fileObject = dataObject.getJSONObject("filename");
                            FileInfo filename = new FileInfo();
                            filename.setUpload_url(fileObject.getString("uploadUrl"));
                            filename.setFullname(fileObject.getString("fullname"));
                            JSONObject paramsObject = fileObject.getJSONObject("params");
                            List<Param> list = new ArrayList<Param>();

                            Iterator<String> it = paramsObject.keys();
                            Param param;
                            while (it.hasNext()){
                                String key = it.next();
                                String value = paramsObject.getString(key);
                                param = new Param();
                                param.setName(key);
                                param.setValue(value);
                                list.add(param);
                            }
                            filename.setParams(list);

                            JSONObject fileObjectthu = dataObject.getJSONObject("thumbnail");
                            JSONObject paramsthuObject =fileObjectthu.getJSONObject("params");

                            List<Param> listthu = new ArrayList<Param>();
                            Iterator<String> itthu = paramsthuObject.keys();
                            Param paramsthu ;
                            while (itthu.hasNext()){
                                String key = itthu.next();
                                String value = paramsthuObject.getString(key);
                                paramsthu = new Param();
                                paramsthu.setName(key);
                                paramsthu.setValue(value);
                                listthu.add(paramsthu);
                            }
                            FileInfo thumbnail = new FileInfo();
                            thumbnail.setParams(listthu);
                            thumbnail.setUpload_url(fileObjectthu.getString("uploadUrl"));
                            thumbnail.setFullname(fileObjectthu.getString("fullname"));

                            result.setFileInfo(filename);
                            result.setThumbnailInfo(thumbnail);
                            if(mdiaInfo!=null){
                                Utils.loge("缩率图路径2="+mdiaInfo.getThumbnailFilePath());
                                mDrivingRecordUploadListener.onGetUploadMediaParam(filename.getUpload_url(),filename.getParams(),mdiaInfo.getMediaFilePath(),  thumbnail,mdiaInfo.getThumbnailFilePath());
//                                uploadFile(filename.getUpload_url(),filename.getParams(),mdiaInfo.getMediaFilePath(),  thumbnail,mdiaInfo.getThumbnailFilePath(),mDrivingRecordUploadListener);
                            }
//                            mDrivingRecordUploadListener.onUploadMediaRequest(result);
                        }else{
                            String errorMsg = jsonObject.getString("msg");
                            mDrivingRecordUploadListener.onFail(respCode,errorMsg);
                        }
                    }   catch (JSONException e) {
                        e.printStackTrace();
                        mDrivingRecordUploadListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
            case DrivingRecordType_download:
                if (mDrivingRecordDownloadListener != null) {
                    if(hasDSPTError){
                        mDrivingRecordDownloadListener.onFail(errorCode,errMsg);
                        return;
                    }
                    List<DownloadMediaRequestResult> list = new ArrayList<DownloadMediaRequestResult>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONArray dataarray = jsonObject.getJSONArray("lists");
                            DownloadMediaRequestResult result;
                            for (int i = 0; i < dataarray.length(); i++) {
                                dataarray.getJSONObject(i);
                                JSONObject object = dataarray.getJSONObject(i);
                                result = new DownloadMediaRequestResult();
                                result.setDeviceNo(object.getString("deviceNo"));
                                result.setFilePath(object.getString("filePath"));
                                result.setThumbnailPath(object.getString("thumbnailPath"));
                                result.setLonlatType(object.getInt("lonlatType"));
                                result.setRecordType(object.getInt("recordType"));
                                result.setUploadTime(object.getInt("time"));
                                result.getPosition().setLatitude(Utils.converMX2DoubleCoordinate(object.getInt("lat")));
                                result.getPosition().setLongitude(Utils.converMX2DoubleCoordinate(object.getInt("lon")));
                                list.add(result);

                            }
                            mDrivingRecordDownloadListener.onDownloadMediaRequest(list);
                        }else{
                            String errorMsg = jsonObject.getString("msg");
                            mDrivingRecordDownloadListener.onFail(respCode, errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mDrivingRecordDownloadListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
            case DrivingRecordType_noticeUploadSuccess:
                if (mNoticeUploadSuccessListener != null) {
                    if (hasDSPTError) {
                        mNoticeUploadSuccessListener.onFail(errorCode, errMsg);
                        return;
                    }
                }
                JSONTokener jsonParser = new JSONTokener(response);
                try {
                    JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                    int respCode = jsonObject.getInt("respCode");
                    if (respCode == Constants.RESPCODE_SUCCESS) {
                        mNoticeUploadSuccessListener.onNoticeUploadSuccess();
                    }else{
                        String msg = "";
                        if(jsonObject.has("msg")){
                            msg = jsonObject.getString("msg");
                        }
                        mNoticeUploadSuccessListener.onFail(respCode,msg);
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                    mNoticeUploadSuccessListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                }
                break;

            case DrivingRecordType_uploadFile:
                if (mDrivingRecordUploadFileListener != null) {
                    if(hasDSPTError){
                        mDrivingRecordUploadFileListener.onFail(errorCode,errMsg);
                        return;
                    }
                    FileInfo fileInfo = new FileInfo();
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            if (jsonObject.has("data")){
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                if (dataObject.has("filename")){
                                    JSONObject fileObject = jsonObject.getJSONObject("filename");
                                    if (fileObject.has("fullname")){
                                        fileInfo.setFullname(fileObject.getString("fullname"));
                                        JSONObject paramsObject = fileObject.getJSONObject("params");
                                        List<Param> list = new ArrayList<Param>();
                                        Iterator<String> it = paramsObject.keys();
                                        Param param;
                                        while (it.hasNext()){
                                            String key = it.next();
                                            String value = paramsObject.getString(key);
                                            param = new Param();
                                            param.setName(key);
                                            param.setValue(value);
                                            list.add(param);
                                        }
                                        fileInfo.setParams(list);
                                    }
                                    if (jsonObject.has("uploadUrl")){
                                        fileInfo.setUpload_url(fileObject.getString("uploadUrl"));
                                    }
                                }
                                mDrivingRecordUploadFileListener.onUploadFile(fileInfo);
                            }
                        }else{
                            String errorMsg = jsonObject.getString("msg");
                            mDrivingRecordUploadFileListener.onFail(respCode,errorMsg);
                        }
                    }   catch (JSONException e) {
                        e.printStackTrace();
                        mDrivingRecordUploadFileListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }

    }

    /**
     * 上传行车记录文件
     * @param url 服务器url
     * @param paramList 参数列表
     * @param filePath 文件绝对路径
     * @param onUploadFileListener 上传回调
     */
    public void uploadFile(final String url, List<Param> paramList,String filePath,final OnUploadFileListener onUploadFileListener){
        File file = new File(filePath);
        if (file.exists() && file.length() > 0) {
            RequestParams requestParams = new RequestParams();
            for(Param param: paramList){
                requestParams.put(param.getName(),param.getValue());
            }
            requestParams.put("file",file);

            HttpRequestUtil.doPost(url, requestParams, new AsyncHttpResponseHandler() {
                @Override
                public void onSuccess(int i, byte[] bytes) {
                    Utils.loge("uploadFileSuccess");
                    String response = new String(bytes);
                    Utils.loge("drivingrecord uploadfile response =  "+response);
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        String mimetype = jsonObject.getString("mimetype");
                        String url = jsonObject.getString("url");
                        try {
                            url = URLEncoder.encode(url,"UTF-8");
                        }catch (UnsupportedEncodingException e){
                            e.printStackTrace();
                        }
                        try {
                            mimetype = URLEncoder.encode(mimetype,"UTF-8");
                        }catch (UnsupportedEncodingException e){
                            e.printStackTrace();
                        }
                        if (onUploadFileListener != null) {
                            onUploadFileListener.onUploadFileSuccess("code=200&mimetype="+mimetype+"&url="+url);
                        }
//                        noticeUploadSuccess("code=200&mimetype="+mimetype+"&url="+url,20,Config.DEFAULT_PRIORITY);
                    }catch (JSONException e){
                        e.printStackTrace();
                        if(onUploadFileListener!=null){
                            onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,e.getMessage());
                        }
                    }
                }

                @Override
                public void onFailure(int i, byte[] bytes, Throwable throwable) {
                    // 上传失败后
                    Utils.loge("上传文件失败-code="+ i);

                    if(onUploadFileListener!=null){
                        onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"上传失败");
                    }
                }
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done){
                    Utils.loge("上传文件 Progress>>>>>"+ bytesWritten + " / " + contentLength+"done="+done);
                }
            },true);

        }else{
            if(onUploadFileListener!=null){
                onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL, "文件不存在");
            }
        }
    }
//    public void uploadFile(final String url, List<Param> paramList,String filePath,final FileInfo thumbnail,final String thumbnailFilePath, final OnUploadFileListener onUploadFileListener)  {
////        final String url="http://v0.api.upyun.com/driving-record-test";
//        File file = new File(filePath);
//        if (file.exists() && file.length() > 0) {
//            final AsyncHttpClient client ;
//           final boolean usePoolThread ;
//            if(Looper.myLooper() == null){
//                usePoolThread = true;
//                client = new SyncHttpClient();
//            }else{
//                usePoolThread = false;
//                client = new AsyncHttpClient();
//            }
//            RequestParams params = new RequestParams();
//            for(Param param: paramList){
//                params.put(param.getName(), param.getValue());
//
//            }
//            try {
//                params.put("file", file);
//            } catch (FileNotFoundException e) {
//                Utils.loge(e.getMessage());
//                if(onUploadFileListener!=null){
//                    onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"缩率图文件不存在");
//                }
//                return;
//            }
//            // 上传文件
//            RequestHandle requestHandle =  client.post(url, params, new AsyncHttpResponseHandler(usePoolThread) {
//                @Override
//                public void onSuccess(int statusCode, Header[] headers,
//                                      byte[] responseBody) {
//                    Utils.loge("onSuccessonSuccess");
////                    if(onUploadFileListener!=null){
////                        onUploadFileListener.onUploadFileSuccess();
////                    }
//                        mNoticeUploadSuccessListener = new NoticeUploadSuccessListener() {
//                            @Override
//                            public void onNoticeUploadSuccess() {
//                                File thumbnailFile = new File(thumbnailFilePath);
//                                if(thumbnailFile!=null&&thumbnail!=null){
//                                    RequestParams params = new RequestParams();
//                                    List<Param> thumbnaiParamList = thumbnail.getParams();
//                                    for(Param param: thumbnaiParamList){
//                                        params.put(param.getName(),param.getValue());
//                                    }
//                                    try {
//                                        params.put("file", thumbnailFile);
//                                    } catch (FileNotFoundException e) {
//                                        Utils.loge(e.getMessage());
//                                        if(onUploadFileListener!=null){
//                                            onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"缩率图文件不存在");
//                                        }
//                                        return;
//                                    }
//
//                                    client.post(thumbnail.getUpload_url(), params, new AsyncHttpResponseHandler(usePoolThread) {
//                                        @Override
//                                        public void onSuccess(int statusCode, Header[] headers,
//                                                              byte[] responseBody) {
//
//                                            if(onUploadFileListener!=null){
////                                                onUploadFileListener.onUploadThumbnailFileSuccess();
//                                            }
//                                            mNoticeUploadSuccessListener = new NoticeUploadSuccessListener() {
//                                                @Override
//                                                public void onNoticeUploadSuccess() {
//                                                    if(onUploadFileListener!=null){
////                                                        onUploadFileListener.onUploadFileSuccess();
//                                                    }
//                                                }
//
//                                                @Override
//                                                public void onTimeOut() {
//                                                    if(onUploadFileListener!=null){
//                                                        onUploadFileListener.onTimeOut();
//                                                    }
//                                                }
//
//                                                @Override
//                                                public void onFail(int errorCode, String errorMsg) {
//                                                    if(onUploadFileListener!=null){
//                                                        onUploadFileListener.onFail(errorCode,errorMsg);
//                                                    }
//                                                }
//                                            };
//
//                                            String response = new String(responseBody);
//                                            Utils.loge("drivingrecord uploadthumbnail "+response);
//                                            JSONTokener jsonParser = new JSONTokener(response);
//                                            try {
//                                                JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
//                                                String mimetype = jsonObject.getString("mimetype");
//                                                String url = jsonObject.getString("url");
//                                                try {
//                                                    url = URLEncoder.encode(url,"UTF-8");
//                                                }catch (UnsupportedEncodingException e){
//                                                    e.printStackTrace();
//                                                }
//                                                try {
//                                                    mimetype = URLEncoder.encode(mimetype,"UTF-8");
//                                                }catch (UnsupportedEncodingException e){
//                                                    e.printStackTrace();
//                                                }
//
////                                                noticeUploadSuccess("code=200&mimetype="+mimetype+"&url="+url,20,Config.DEFAULT_PRIORITY);
//                                            }catch (JSONException e){
//                                                e.printStackTrace();
//                                            }
//
//
//                                        }
//                                        @Override
//                                        public void onFailure(int statusCode, Header[] headers,
//                                                              byte[] responseBody, Throwable error) {
//                                            // 上传失败后要做到工作
//                                            Utils.loge("上传缩率图失败-statusCode=" + statusCode);
//                                            if(responseBody!=null&&responseBody.length>0){
//                                                Utils.loge("上传缩率图失败-responseBody=" + new String(responseBody));
//                                            }
//                                            if(error !=null ){
//                                                Utils.loge("上传缩率图失败-"+error.toString());
//                                            }
//                                            if(onUploadFileListener!=null){
//                                                onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"上传失败");
//                                            }
//                                        }
//                                        @Override
//                                        public void onRetry(int retryNo) {
//                                            // TODO Auto-generated method stub
//                                            super.onRetry(retryNo);
//                                            // 返回重试次数
//                                        }
//                                    });
//                                }else{
//                                    if(onUploadFileListener!=null){
////                                        onUploadFileListener.onUploadFileSuccess();
//                                    }
//                                }
//                            }
//
//                            @Override
//                            public void onTimeOut() {
//                                if(onUploadFileListener!=null){
//                                    onUploadFileListener.onTimeOut();
//                                }
//                            }
//
//                            @Override
//                            public void onFail(int errorCode, String errorMsg) {
//                                if(onUploadFileListener!=null){
//                                    onUploadFileListener.onFail(errorCode,errorMsg);
//                                }
//                            }
//                        };
////                    AsyncHttpClient client = new AsyncHttpClient();
//                    String response = new String(responseBody);
//                    Utils.loge("drivingrecord uploadfile "+response);
//                    JSONTokener jsonParser = new JSONTokener(response);
//                    try {
//                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
//                        String mimetype = jsonObject.getString("mimetype");
//                        String url = jsonObject.getString("url");
////                        if(thumbnailFile==null||thumbnail==null){
////                            url = makeUrl(url);
////                        }
//                        try {
//                            url = URLEncoder.encode(url,"UTF-8");
//                        }catch (UnsupportedEncodingException e){
//                            e.printStackTrace();
//                        }
//                        try {
//                            mimetype = URLEncoder.encode(mimetype,"UTF-8");
//                        }catch (UnsupportedEncodingException e){
//                            e.printStackTrace();
//                        }
////                        noticeUploadSuccess("code=200&mimetype="+mimetype+"&url="+url,20,Config.DEFAULT_PRIORITY);
//                    }catch (JSONException e){
//                        e.printStackTrace();
//                    }
//
////                    if(thumbnailFile!=null&&thumbnail!=null){
////
////                    }else{
////                        if(drivingRecordUploadListener!=null){
////                            drivingRecordUploadListener.onUploadMediaSuccess();
////                        }
////                    }
//
//                }
//
//                @Override
//                public void onFailure(int statusCode, Header[] headers,
//                                      byte[] responseBody, Throwable error) {
//                    // 上传失败后要做到工作
//                    Utils.loge("上传视频文件失败-statusCode=" + statusCode);
//                    if(responseBody!=null&&responseBody.length>0){
//                        Utils.loge("上传视频文件失败-responseBody=" + new String(responseBody));
//                    }
//
//                    if(error !=null ){
//                        Utils.loge("上传视频文件失败-"+error.toString());
//                    }
//                    if(onUploadFileListener!=null){
//                        onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"上传失败");
//                    }
//                }
//
//                @Override
//                public void onProgress(long bytesWritten, long totalSize) {
//                    // TODO Auto-generated method stub
//                    super.onProgress(bytesWritten, totalSize);
////                    int count = (int) ((bytesWritten * 1.0 / totalSize) * 100);
////                    Utils.loge("上传进度-bytesWritten="+bytesWritten+" totalSize="+totalSize);
//                    // 上传进度显示
//
//                    Log.e("上传视频文件 Progress>>>>>", bytesWritten + " / " + totalSize);
//                }
//
//                @Override
//                public void onRetry(int retryNo) {
//                    // TODO Auto-generated method stub
//                    super.onRetry(retryNo);
//                    // 返回重试次数
//                }
//
//            });
//
//        } else {
//            if(onUploadFileListener!=null){
//                onUploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL, "文件不存在");
//            }
////            Utils.loge("文件不存在");
//        }
//    }

    /**
     * 通知服务器拍照上传地址URL
     * @param notifyURL 服务器地址
     * @param id 服务器发送给车机的ID
     * @param fileUrl 拍照文件的URL
     * @param notifyReceiveCameraListener 通知结果回调接口
     */
    public void notifyReceiveCamera(String notifyURL,String id,String fileUrl,final NotifyReceiveCameraListener notifyReceiveCameraListener){
        RequestParams requestParams = new RequestParams();
        requestParams.put("id", id);
        requestParams.put("fileUrl",fileUrl);
        HttpRequestUtil.doGet(notifyURL, requestParams, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(int i, byte[] bytes) {
                String responseStr = new String(bytes);
                if(notifyReceiveCameraListener==null){
                    return;
                }
                int respCode = 0;
                try {
                    if(responseStr != null) {
                        responseStr = responseStr.trim();
                        if(responseStr.startsWith("{") || responseStr.startsWith("[")) {
                            JSONObject  result = (JSONObject)(new JSONTokener(responseStr)).nextValue();
                            respCode = result.getInt("respCode");
                        }
                    }

                } catch (JSONException e) {
                    e.printStackTrace();
                    notifyReceiveCameraListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    return;
                }

                if(respCode ==  Constants.RESPCODE_SUCCESS) {
                    notifyReceiveCameraListener.onNotifyReceiveCameraSuccess();
                }else{
                    if(responseStr == null||responseStr.length()==0){
                        notifyReceiveCameraListener.onFail(0, "此次通知已超时失效");
                    }else{
                        notifyReceiveCameraListener.onFail(respCode,"");
                    }

                }
            }

            @Override
            public void onFailure(int i, byte[] bytes, Throwable throwable) {
                if(notifyReceiveCameraListener==null){
                    return;
                }
                if(throwable.getCause().equals(SocketTimeoutException.class)){
                    // 连接超时
                    notifyReceiveCameraListener.onTimeOut();

                }else{
                    notifyReceiveCameraListener.onFail(Constants.ERROR_CODE_NETCONNECT_ERROR,Constants.ERROR_MESSAGE_NETCONNECT_ERROR);

                }
            }
        },true);

    }
//    private String makeUrl(String url){
//        int index = url.lastIndexOf("/");
//        String left = url.substring(0,index);
//        String right =  url.substring(index,url.length());
//        url = left+"-image"+right;
//        return url;
//    }

}

/*    public void test(){
        String response = "{\"resp_code\":100,\"msg\":\"查询成功！\",\"data\":null,\"lists\":[{\"id\":95,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-10-48-58/test.mp4?_upt=488fb0d61460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-10-48-58/aaa.png?_upt=87fddd061460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":2,\"lonlat_type\":0,\"lon\":50,\"lat\":120,\"time\":\"1459219726\",\"filename\":\"test.mp4\",\"thumbnail\":\"aaa.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":96,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-10-53-13/test.mp4?_upt=cf08d1d01460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-10-53-13/a.png?_upt=4c89ecff1460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459219985\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":97,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-11-21-27/test.mp4?_upt=083a2cb71460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-11-21-27/a.png?_upt=984a0ffc1460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459219985\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":99,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-13-04-17/test.mp4?_upt=061210a61460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-13-04-17/a.png?_upt=e2e6ca801460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459227836\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":100,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-13-05-36/test.mp4?_upt=c057f7641460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-13-05-36/a.png?_upt=b41aa9141460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459227933\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":103,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-16-58-37/test.mp4?_upt=be9927f41460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-16-58-37/a.png?_upt=fb85587c1460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459241885\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":104,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-17-18-35/test.mp4?_upt=318c48901460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-17-18-35/a.png?_upt=9aca29a31460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459243076\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":105,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-17-48-43/test.mp4?_upt=9fbadb4e1460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-17-48-44/a.png?_upt=f88a50111460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459244893\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":107,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-18-31-33/test.mp4?_upt=b8bf01301460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-29-18-31-33/a.png?_upt=49dda18c1460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459247471\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":109,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-09-40-25/test.mp4?_upt=386d58121460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-09-40-26/a.png?_upt=c71881051460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459302005\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":110,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-10-10-00/test.mp4?_upt=a61e9e9c1460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-10-10-00/a.png?_upt=67b78e921460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459303455\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":111,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-10-27-13/test.mp4?_upt=c072fb541460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-10-27-13/a.png?_upt=6bcc5e091460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459304827\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":113,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-11-47-06/test.mp4?_upt=b32c9bf81460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-11-47-07/a.png?_upt=aecf8a461460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459309611\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":0,\"thumbnail_success\":0},{\"id\":114,\"file_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-12-26-50/test.mp4?_upt=5fec33e21460093591\",\"thumbnail_path\":\"http://gmate-driving-record.mxnavi.com/aaa/100/2016-03-30-12-26-51/a.png?_upt=f1d75d871460093591\",\"user_id\":null,\"phone\":null,\"device_no\":\"100\",\"record_type\":1,\"lonlat_type\":0,\"lon\":1072865920,\"lat\":367979200,\"time\":\"1459312004\",\"filename\":\"test.mp4\",\"thumbnail\":\"a.png\",\"product_id\":\"aaa\",\"file_success\":1,\"thumbnail_success\":0}]}";
        Gson gson = new Gson();
        List<DownloadMediaRequestResult> list = new ArrayList<>();
                JSONTokener jsonParser = new JSONTokener(response);
                   try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode =  jsonObject.getInt("resp_code");
                        if(respCode == 100) {
                            JSONArray dataarray = jsonObject.getJSONArray("lists");
                            DownloadMediaRequestResult result;
                            for (int i = 0; i < dataarray.length(); i++) {
                                dataarray.getJSONObject(i);
                                JSONObject object = dataarray.getJSONObject(i);
                                result = new DownloadMediaRequestResult();
                                result.setDeviceNo(object.getString("device_no"));
                                result.setFilePath(object.getString("file_path"));
                                result.setThumbnailPath(object.getString("thumbnail_path"));
                                result.setLonlatType(object.getInt("lonlat_type"));
                                result.setRecordType(object.getInt("record_type"));
                                result.setUploadTime(object.getInt("time"));
                                result.getPosition().setLatitude(object.getInt("lat"));
                                result.getPosition().setLongitude(object.getInt("lon"));
                                list.add(result);

                            }
                        }
                    } catch (JSONException e) {
                       e.printStackTrace();
                   }
        mDrivingRecordDownloadListener.onDownloadMediaRequest(list);
                    }
    }*/