package com.mxnavi.tspv2.feichuan.client.core;


import com.mxnavi.tspv2.feichuan.client.core.listener.Connection;
import com.mxnavi.tspv2.feichuan.client.core.listener.ITcpFileSenderTask;
import com.mxnavi.tspv2.feichuan.client.core.listener.ITcpSender;
import com.mxnavi.tspv2.feichuan.client.listener.SendFileListener;
import com.mxnavi.tspv2.feichuan.client.model.FileInfo;
import com.mxnavi.tspv2.feichuan.common.Constance;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Created by wangq on 2018/1/4.
 */

public class TcpClient implements Connection,ITcpSender {
    ServerInfo mServerInfo;
    SendFileListener mSendFileListener;

    Socket tcpClientSocket = null;
    volatile boolean isExiteConnect = false;
    volatile boolean isTcpConnectSuccess  = false;
    long firstConnectTCPFailTime;
    int timeout = 30000;//30秒

    DataOutputStream dos = null;
    DataInputStream dis = null;

//    TcpWriter tcpWriter;
    ITcpFileSenderTask mFileSenderTask;
    TcpReader mTcpReader;
    TcpHeatbeatHandler mTcpHeatbeatHandler;
    public TcpClient(ServerInfo serverInfo, SendFileListener sendFileListener){
        mServerInfo = serverInfo;
        mSendFileListener = sendFileListener;
        mFileSenderTask = new TcpTcpFileSenderTaskTask(this,sendFileListener);
        mTcpReader = new TcpReader(this);
        mTcpHeatbeatHandler = new TcpHeatbeatHandler(this);
//        tcpWriter = new TcpWriter(this,dos);
    }


    private void connectTcpServer(){
        try {
            if(isExiteConnect){
                return;
            }
            tcpClientSocket = new Socket(mServerInfo.serverAddress.getHostAddress(), mServerInfo.serverTcpPort);
            if(isExiteConnect){
                tcpClientSocket.close();
                return;
            }
            tcpClientSocket.setKeepAlive(true);
            tcpClientSocket.setTcpNoDelay(true);
            tcpClientSocket.setSoTimeout(Constance.TCP_READ_TIME_OUT);
            isTcpConnectSuccess = true;
            if (mSendFileListener != null) {
                mSendFileListener.onConnectSuccess();
            }
            // 连接成功，启动发送和收取线程
//            synchronized (startTcpFinishLock){
//                startTcpFinishLock.notify();
//            }
            OutputStream outputStream = tcpClientSocket.getOutputStream();
            dos = new DataOutputStream(outputStream);
            dis = new DataInputStream(tcpClientSocket.getInputStream());
            onConnected();
        } catch (IOException e) {
            e.printStackTrace();
            if(isExiteConnect){
                return;
            }
            if(firstConnectTCPFailTime == 0){
                firstConnectTCPFailTime = System.currentTimeMillis();
            }else{
                // 连接超时,停止连接
                if((System.currentTimeMillis()-firstConnectTCPFailTime)>timeout){
                    if(mSendFileListener !=null) {
                        mSendFileListener.onConnectTimeOut();
                    }
                    isTcpConnectSuccess = false;
                    return;
                }
            }

            try{
                TimeUnit.SECONDS.sleep(5);
            }catch (InterruptedException f) {
                return;
            }
            connectTcpServer();
        }
    }
    @Override
    public void connect() {
        connectTcpServer();
    }

    @Override
    public void sendFile(List<FileInfo> list){
        mFileSenderTask.sendFiles(list);
    }
    @Override
    public void cancelSend(){
        mFileSenderTask.cancelSend();
    }
    @Override
    public void pauseSend(){
        mFileSenderTask.pauseSend();
    }
    @Override
    public void continueSend(){
        mFileSenderTask.continueSend();
    }

    @Override
    public void exitSend(){
        close();
    }
    private void onConnected(){
        mTcpReader.startRead(dis);
        mFileSenderTask.startSendTask(dos);
        mTcpHeatbeatHandler.startSend(dos);
    }


    @Override
    public void close() {
        if(isTcpConnectSuccess){
            mSendFileListener.onDisConnect();
            isTcpConnectSuccess = false;
            mFileSenderTask.stopSend();
            mTcpReader.stopRead();
            mTcpHeatbeatHandler.stopSend();
        }
        isExiteConnect = true;

        if(tcpClientSocket!=null){
            try {
                tcpClientSocket.close();
            }catch (IOException e){
                e.printStackTrace();
            }
            tcpClientSocket = null;
        }

    }

    @Override
    public boolean isConnected() {
        return isTcpConnectSuccess;
    }

    @Override
    public void onReceiveHeartBeat() {

    }
    @Override
    public void  onDisconnect(){

    }
//    @Override
//    public void send(Packet packet){
////        tcpWriter.write(packet);
//    }
    @Override
    public void onReceiveStrMsg(String msg){
        mFileSenderTask.onReceiveMsg(msg);
    }
}
