package com.mxnavi.tspv2.feichuan.client.core;

import android.util.Log;


import com.mxnavi.tspv2.feichuan.client.core.listener.Connection;
import com.mxnavi.tspv2.feichuan.client.core.listener.ITcpFileSenderTask;
import com.mxnavi.tspv2.feichuan.client.listener.SendFileListener;
import com.mxnavi.tspv2.feichuan.client.model.FileInfo;
import com.mxnavi.tspv2.feichuan.common.Constance;
import com.mxnavi.tspv2.feichuan.common.ExecutorManager;
import com.mxnavi.tspv2.feichuan.common.PacketEncoder;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * Created by Administrator on 2018/1/4.
 */

public class TcpTcpFileSenderTaskTask implements Runnable,ITcpFileSenderTask {

    String TAG = Constance.LOG_TAG;

    int FILE_READ_LENDTH = 1024 * 40;

    Connection mConnection;

    SendFileListener mSendFileListener ;

    int totalFileCount;

    int hasSendFileCount;

    long hasSendLength;
//    volatile boolean isExiteConnect = false;
    long totalFileLength = 0;

    long startTime;

    RandomAccessFile randomAccessFile;

    private BlockingQueue<String> recvMsgQueue = new ArrayBlockingQueue<String>(5);
    // 无界文件信息队列，可以无限制添加要传输的文件
    private LinkedBlockingQueue<FileInfo> fileinfoQueue = new LinkedBlockingQueue<FileInfo>(10);

    volatile boolean isStopSendFile = false;

    volatile boolean isPauseSendFile = false;

    volatile boolean isCancelSendFile = false;

    Object pauseLock = new Object();

    DataOutputStream dos;

    String SPEC_FLAG = "sendtotalinfo";
    public TcpTcpFileSenderTaskTask(Connection connection, SendFileListener sendFileListener){
        mConnection = connection;
        mSendFileListener = sendFileListener;
    }
    private void initData(){
         totalFileCount = 0;

         hasSendFileCount = 0;

         hasSendLength = 0;
//    volatile boolean isExiteConnect = false;
         totalFileLength = 0;

         startTime = 0;
    }
    @Override
    public void sendFiles(List<FileInfo> list){
        Log.e(TAG,"startSendTask sendFiles");
        isCancelSendFile = false;
        isPauseSendFile = false;
        isStopSendFile = false;
        Log.e(TAG,"startSendTask list.size()="+list.size());
        if(list!=null&&list.size()>0){
            totalFileCount = list.size();
            FileInfo specFileInfo = new FileInfo();
            specFileInfo.setFilePath(SPEC_FLAG);
            fileinfoQueue.offer(specFileInfo);
            for(FileInfo fileInfo:list){
                totalFileLength+=fileInfo.getFileSize();
            }
            for(FileInfo fileInfo:list){
                fileinfoQueue.offer(fileInfo);

            }
        }
    }
    @Override
    public void startSendTask(DataOutputStream dos){
        this.dos = dos;
        Log.e(TAG,"startSendTask");
        ExecutorManager.INSTANCE.getWriteThread().execute(this);
    }
    @Override
    public void pauseSend(){
        isPauseSendFile = true;
    }
    @Override
    public void continueSend(){
        isPauseSendFile = false;
        synchronized (pauseLock) {
            pauseLock.notify();
        }
    }
    @Override
    public void cancelSend(){
        isCancelSendFile = true;
        // 防止线程在recvMsgQueue.take时挂起
        recvMsgQueue.offer(makeCancelSendMsg());
        fileinfoQueue.clear();
        continueSend();
        recvMsgQueue.clear();
    }
    @Override
    public void stopSend(){
        isStopSendFile = true;
        fileinfoQueue.clear();
        recvMsgQueue.clear();
        ExecutorManager.INSTANCE.getWriteThread().shutdown();
    }
    @Override
    public void onReceiveMsg(String msg){
        recvMsgQueue.offer(msg);
    }
    @Override
    public void run() {
        Log.e(TAG,"startSendTask run");
        while(!isStopSendFile){
            Log.e(TAG,"startSendTask run loop");
            FileInfo fileInfo;
            try {
                fileInfo = fileinfoQueue.take();
            }catch (InterruptedException e) {
                e.printStackTrace();
                Log.e(TAG,Log.getStackTraceString(e));
                break;
            }

            // 发送单个文件的文件信息
            try {
                // 判断是否已暂停
                if(isPauseSendFile){
                    synchronized (pauseLock) {
                        pauseLock.wait();
                    }
                }
                if (SPEC_FLAG.equals(fileInfo.getFilePath())) {
                    PacketEncoder.sendTcpTextMsg(dos,makeTotalFileMsg(totalFileCount));

                }else{
                    sendSingleFileInfo(fileInfo);
                }

            }catch (IOException e){
                Log.e(TAG,Log.getStackTraceString(e));
                e.printStackTrace();
                mConnection.close();
                break;
            }catch (InterruptedException e){
                Log.e(TAG,Log.getStackTraceString(e));
                e.printStackTrace();
                break;
            }
           while (!isStopSendFile&&!isCancelSendFile){
               String revmsg;
               // 读取回应信息
               try {
                   revmsg = recvMsgQueue.take();
               } catch (InterruptedException e) {
                   e.printStackTrace();
                   Log.e(TAG,Log.getStackTraceString(e));
                   break;
               }
               JSONObject jsonMsg = parseTCPMsg(revmsg);

               String msgType = "";
               try {
                   msgType = jsonMsg.getString("msgType");
                   if("fileTotalInfo".equals(msgType)){
                       break;
                   }else if("fileInfo".equals(msgType)){
                       int result = jsonMsg.getInt("result");
                       if(result ==  Constance.RESULT_OK){
                           sendFile(fileInfo);
                       }else if(result == Constance.RESULT_NEED_MD5){
                           String md5 = getFileMD5(fileInfo.getFilePath());
                           PacketEncoder.sendTcpTextMsg(dos,makeSingleFileMd5Msg(fileInfo,md5));
                       }else if(result == Constance.RESULT_WIRTE_FILE_FAIL){
                           if(mSendFileListener != null){
                               mSendFileListener.onSendFileFail(fileInfo.getFilePath(),Constance.ERROR_CODE_SEND_FAIL_RECERVER_ERROR,"接收端出现异常");
                           }
                           break;
                       }else if(result == Constance.RESULT_RECEVER_NOT_EnoughMemory){
                           if(mSendFileListener != null){
                               mSendFileListener.onReceiverNotEnoughMemory();
                           }
                           // 停止传输
                           fileinfoQueue.clear();
                           break;
                       }
                   }else if("fileInfoMD5".equals(msgType)){
                       //200" 可以发送，"201"：已经发送过，不需要再发送
                       int result = jsonMsg.getInt("result");
                       if(result == Constance.RESULT_OK){
                           sendFile(fileInfo);
                       }else if(result == Constance.RESULT_RECEVER_NOT_EnoughMemory){
                           if(mSendFileListener != null){
                               mSendFileListener.onReceiverNotEnoughMemory();
                           }
                           // 停止传输
                           fileinfoQueue.clear();
                           break;
                       }else{
                           break;
                       }


                   }else if("fileReceive".equals(msgType)){
                       int result = jsonMsg.getInt("result");
                       if(result ==  Constance.RESULT_OK){
                           break;
                       }
                   }
               }catch (Exception e){
                   closeFile();
                   e.printStackTrace();
                   Log.e(TAG,Log.getStackTraceString(e));
                   break;
               }

           }
           if(isCancelSendFile){
               try{
                   PacketEncoder.sendTcpTextMsg(dos,makeCancelSendMsg());
               }catch (IOException e){
                   Log.e(TAG,Log.getStackTraceString(e));
               }

           }



        }
    }

    private JSONObject parseTCPMsg(String recvMsg){
        JSONTokener jsonTokener = new JSONTokener(recvMsg);
        JSONObject json = null;
        try {
             json = (JSONObject) jsonTokener.nextValue();
        }catch (JSONException e) {
            Log.e(TAG,Log.getStackTraceString(e));
            e.printStackTrace();
        }
        return json;
    }
    private void sendSingleFileInfo(FileInfo fileInfo)throws IOException{
        String str = makeSingleFileMsg(fileInfo);
        if(isCancelSendFile){
            return;
        }

        PacketEncoder.sendTcpTextMsg(dos,str);
    }
    private void closeFile(){
        try {
            if(randomAccessFile != null){
                randomAccessFile.close();
                randomAccessFile = null;
            }
        } catch (IOException e) {
            e.printStackTrace();
            randomAccessFile = null;
        }
    }
    private void sendFile(FileInfo fileInfo)throws InterruptedException{
        Log.e(TAG,"start send file"+fileInfo.getFilePath());
        if(isPauseSendFile){
            synchronized (pauseLock) {
                pauseLock.wait();
            }
        }
        byte[] filebytes = new byte[FILE_READ_LENDTH];
        long realSendFileLength = 0;
        try {
            randomAccessFile = new RandomAccessFile(fileInfo.getFilePath(), "rwd");
            int readSize = 0;
            long sTime = System.currentTimeMillis();
            long eTime = 0;
            long gapTime = 0;
            startTime = System.currentTimeMillis();
            Log.e(TAG,"isStop="+isStopSendFile+"andomAccessFile.length()"+  randomAccessFile.length());
            while (!isStopSendFile&&!isCancelSendFile&&(readSize = randomAccessFile.read(filebytes)) != -1) {
//                Log.e(TAG,"start send file readSize="+readSize);
                PacketEncoder.sendTcpByteMsg(dos,filebytes,readSize);
//                mConnection.send(PacketEncoder.makePackage(filebytes,readSize));
                hasSendLength += readSize;
                realSendFileLength+=readSize;
                eTime = System.currentTimeMillis();
                gapTime = eTime - sTime;
                if(gapTime > 500){ //大于500ms 才进行一次监听
                    sTime = eTime;
                    if(mSendFileListener != null){
                        mSendFileListener.updateProcess(getPercent(),((float) realSendFileLength) / (eTime-startTime) * 1000);
                    }
                }
                if(isPauseSendFile){
                    synchronized (pauseLock) {
                        pauseLock.wait();
                    }
                }

            }
            Log.e(TAG,"end send file readSize="+readSize);
            if(gapTime>0 && gapTime<=500){
                if(mSendFileListener != null){
                    mSendFileListener.updateProcess(getPercent(),((float) realSendFileLength) / (eTime-startTime) * 1000);
                }
            }

            if(readSize == -1){
//                mFileInfoList.remove(0);
                hasSendFileCount ++;
                if(mSendFileListener != null){
                    mSendFileListener.updateSendFileNum(totalFileCount,hasSendFileCount);
                }
            }
            if(!isStopSendFile&&fileinfoQueue.size()==0){
                initData();
//                PacketEncoder.sendTcpTextMsg(dos,makeFinishSendMsg());
                if(mSendFileListener != null){
                    mSendFileListener.onSendAllFileFinish();
                }
            }

        }catch (IOException e) {
            Log.e(TAG,"sendFile IOException"+Log.getStackTraceString(e));
            e.printStackTrace();
            if(mSendFileListener != null){
                mSendFileListener.onSendFileFail(fileInfo.getFilePath(),Constance.ERROR_CODE_READ_FILE_ERROR,"读取文件异常");
            }
            isStopSendFile = true;
        }
        closeFile();
    }
    private float getPercent(){
        int temp = (int)(((float)hasSendLength)/totalFileLength*10000);
        float percent = (float)temp/100;
        return percent;
    }

    private String makeSingleFileMd5Msg(FileInfo fileInfo,String md5){

        JSONObject object = new JSONObject();
        try{
            object.put("msgType","fileInfoMD5");
            object.put("fileType","music");
            object.put("fileName",fileInfo.getFileName());
            object.put("fileSize", new File(fileInfo.getFilePath()).length());
            object.put("MD5", md5);
            return object.toString();
        }catch (JSONException e){
            e.printStackTrace();
        }


        return null;
    }
    private String makeSingleFileMsg(FileInfo fileInfo){

        JSONObject object = new JSONObject();
        try{
            object.put("msgType","fileInfo");
            object.put("fileType","music");
            object.put("fileName",fileInfo.getFileName());
            object.put("fileSize", fileInfo.getFileSize());
            return object.toString();
        }catch (JSONException e){

        }


        return null;
    }
    private String makeTotalFileMsg(int totalFileCount){
        JSONObject object = new JSONObject();
        try{
            object.put("msgType","fileTotalInfo");
            object.put("totalFileCount",totalFileCount);
            object.put("deviceNo","");
            object.put("bindCarId","");
            return object.toString();
        }catch (JSONException e){

        }


        return null;
    }
    //    private String makeFinishSendMsg(){
//        return "{\"msgType\":\"finish\"}";
//    }
    private String makeCancelSendMsg(){
        return "{\"msgType\":\"cancel\"}";
    }
    private String getFileMD5(String path){
        try {
            MessageDigest msgDigest = MessageDigest.getInstance("MD5");

            String str = Utils.getFileMD5String(new File(path),msgDigest);
            msgDigest.reset();
            return str;
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalStateException(
                    "System doesn't support MD5 algorithm.");
        }catch (IOException e){
            e.printStackTrace();
        }
        return null;
    }


}
