package com.mxnavi.tspv2.feichuan.client.core;

import android.util.Log;

import com.mxnavi.tspv2.feichuan.client.FileSenderManager;
import com.mxnavi.tspv2.feichuan.client.listener.SendFileListener;
import com.mxnavi.tspv2.feichuan.common.Constance;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

/**
 * Created by Administrator on 2017/12/27.
 */

public class UdpSender {
    String TAG = Constance.LOG_TAG;
    final String currVer = "1.0";
    String[] supportVer = {"1.0", "1.1"};
    // 发广播
    final String broadcastMsg = "{\"msgType\":0, \"msgNo\":" + 1 + ",\"version\":\"1.0\"," +
            " \"ptt\":[\"1.0\",\"1.1\"] ," +
            "\"deviceName\":\"wangqian\"" +
            "}";

    boolean isPause = false;
    DatagramSocket clientsocket;
    int timeout = 30000;//30秒
    int socketTimeOut = 5000;
    int broadcastInterval = 5;//广播重发间隔为5秒
    int msgInterval = 5;
    int loopMaxNum = timeout / socketTimeOut;
    int sendBroadCastNum = 0;

    public void stopConnect() {
        isPause = true;
    }

    public void start() {
        isPause = false;
    }

    public ServerInfo startUdpScan(SendFileListener sendFileListener) {
        isPause = false;
        ServerInfo serverInfo = null;
        try {
            if (clientsocket == null || clientsocket.isClosed()) {
                clientsocket = new DatagramSocket();
                clientsocket.setSoTimeout(socketTimeOut);
                clientsocket.setReceiveBufferSize(1024 * 64);
                clientsocket.setSendBufferSize(1024 * 64);
            }
            clientsocket.setBroadcast(true);
        } catch (SocketException e) {
            e.printStackTrace();
        }
//            Utils.loge("socket.getLocalPort=" + clientPort);

        // 发送广播
        broadcastToServer(clientsocket);

        byte[] buf = new byte[1024];
        DatagramPacket packet = new DatagramPacket(buf, buf.length, null, Constance.SERVER_UDP_PORT);

        while (!isPause) {
            try {
                clientsocket.receive(packet);
            } catch (IOException e) {
                e.printStackTrace();
                if (sendBroadCastNum >= loopMaxNum+1) {
                    Log.e(TAG,"sendBroadCastNum >= loopMaxNum+1");
                    if (sendFileListener != null) {
                        Log.e(TAG,"onScannCarTimeOut");
                        sendFileListener.onScannCarTimeOut();
                    }else {
                        Log.e(TAG,"sendFileListener == nul");
                    }
                    isPause = true;
                    break;
                }else{
                    // 发送广播
                    broadcastToServer(clientsocket);
                    continue;
                }

            }

            String recvMsg = new String(buf, 0,
                    packet.getLength());
            recvMsg = Utils.decode(recvMsg);
            Log.e(TAG,"clientsocket.recvMsg="+recvMsg);
            try {

                 serverInfo = receiveRespOfBroadcast(packet, recvMsg);
                 if (serverInfo != null) {

                    if (sendFileListener != null) {
                        sendFileListener.onScannedCar(serverInfo.verMatchResult);
                    }

                    clientsocket.close();
                     break;
                } else {
                    broadcastToServer(clientsocket);
                }

            } catch (JSONException e) {
                e.printStackTrace();
                broadcastToServer(clientsocket);
                continue;
            }


        }
        if (isPause) {
            clientsocket.close();
        }
       return serverInfo;
    }

    private void broadcastToServer(DatagramSocket mBRDSocket) {
        DatagramPacket mBRDPacket = null;
        try {
//            String encodeStr = Utils.encode(getbroadcastMsg());
            String encodeStr = getbroadcastMsg();
            byte[] buf = encodeStr.getBytes();
            mBRDPacket = new DatagramPacket(buf, buf.length, null, Constance.SERVER_UDP_PORT);
            InetAddress broadcastAddress = Utils.getBroadcastAddress();
//                Utils.loge("Util.getBroadcastAddress()=" + broadcastAddress.toString());
            Log.e(TAG,"Util.getBroadcastAddress()=" + broadcastAddress.toString());
            mBRDPacket.setAddress(broadcastAddress);

            mBRDSocket.send(mBRDPacket);
        } catch (Exception e) {
            e.printStackTrace();
        }
        sendBroadCastNum++;


    }

    private ServerInfo receiveRespOfBroadcast(DatagramPacket packet, String msg) throws JSONException {
        JSONTokener jsonTokener = new JSONTokener(msg);
        Object obj = jsonTokener.nextValue();
        if (!(obj instanceof JSONObject)) {
            throw new JSONException(msg + "is not json ");
        }
        JSONObject json = (JSONObject) obj;
        int msgType = json.getInt("msgType");
        int msgNo = json.getInt("msgNo");
        if (msgType == 1 && msgNo == 1) {

            String carId = json.getString("carId");
//            String pttVer = json.getString("ptt");
            int verMatch = json.getInt("matchResult");
            int serverPort = json.getInt("serverPort");
            ServerInfo serverInfo = new ServerInfo();
            serverInfo.carId = carId;
            serverInfo.serverTcpPort = serverPort;
            serverInfo.verMatchResult = verMatch;
            serverInfo.serverAddress = packet.getAddress();
            return serverInfo;

        }
        return null;

    }

    private String getbroadcastMsg() {
        JSONTokener jsonTokener = new JSONTokener(broadcastMsg);

        JSONObject json = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            json.put("version", currVer);
            JSONArray pptArray = new JSONArray();
            for (int i = 0; i < supportVer.length; i++) {
                pptArray.put(supportVer[i]);
            }
            json.put("ptt", pptArray);
            json.put("deviceName", "");
            json.put("bindCarId", "");

            return Utils.encode(json.toString());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

}
