package com.mxnavi.tspv2.flowcontrol;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.flowcontrol.model.AppIdAndFlow;
import com.mxnavi.tspv2.flowcontrol.model.EffectiveTime;
import com.mxnavi.tspv2.flowcontrol.model.FlowDetail;
import com.mxnavi.tspv2.flowcontrol.model.FlowInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/10/17.
 */
public class FlowControl extends BaseTsp {
    // 请求服务类型为流量控制
    final int thisServiceType = Config.MxNGTPServiceType.MX_FLOW_CONTROL.getServiceType();
    // 流量控制服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_FLOW_CONTROL.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int FC_FIRSTACTIVESIMCARD = 0;//SIM卡首次激活
    final int FC_UPLOADUSEFLOWDETAIL = 1;//车机内SIM卡上传流量数据
    final int FC_DOWNLOADFLOWDETAIL = 2;//车机内SIM卡下载流量数据
    final int GETSIMCARDCOMBOTERM = 3;//查询sim卡套餐有效期(smart)
    final int GETUSEFLOW = 4;//下载流量(smart)
    final int GETSIMCARDSTATUS = 5;//获取sim卡激活状态(smart)

    public interface OnFirstActiveSIMCardListener extends CallBackListener {
        public void firstActiveSIMCard();
    }

    public interface OnUploadUseFlowDetailListener extends CallBackListener {
        public void uploadUseFlowDetail();
    }

    public interface OnDownloadFlowDetailListener extends CallBackListener {
        public void downloadFlowDetail(Map<String,FlowDetail> map);
    }

    public interface OnGetSimCardComboTermListener extends CallBackListener {
        public void getSimCardComboTerm(List<EffectiveTime> effectiveTimeList);
    }

    public interface OnGetUseFlowListener extends CallBackListener {
        public void getUseFlow(Map<String,FlowDetail> map);
    }

    public interface OnGetSIMCardStatusListener extends CallBackListener {
        public void getSIMCardStatus(String simCardStatus);
    }

    OnFirstActiveSIMCardListener mOnFirstActiveSIMCardListener;
    OnUploadUseFlowDetailListener mOnUploadUseFlowDetailListener;
    OnDownloadFlowDetailListener mOnDownloadFlowDetailListener;
    OnGetSimCardComboTermListener mOnGetSimCardComboTermListener;
    OnGetUseFlowListener mOnGetUseFlowListener;
    OnGetSIMCardStatusListener mOnGetSIMCardStatusListener;

    /**
     * SIM卡首次激活
     * @param simNo Sim卡号即ICCID
     * @param deviceNo 车机硬件序列号
     * @param onFirstActiveSIMCardListener 结果监听
     * @return 请求ID，0:参数错误
     */
    public int firstActiveSIMCard(String simNo,String deviceNo,OnFirstActiveSIMCardListener onFirstActiveSIMCardListener){
        return firstActiveSIMCard(simNo,deviceNo,onFirstActiveSIMCardListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * SIM卡首次激活
     * @param simNo Sim卡号即ICCID
     * @param deviceNo 车机硬件序列号
     * @param onFirstActiveSIMCardListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int firstActiveSIMCard(String simNo,String deviceNo,OnFirstActiveSIMCardListener onFirstActiveSIMCardListener,int timeOutSecs){
        return firstActiveSIMCard(simNo, deviceNo, onFirstActiveSIMCardListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * SIM卡首次激活
     * @param simNo Sim卡号即ICCID
     * @param deviceNo 车机硬件序列号
     * @param onFirstActiveSIMCardListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int firstActiveSIMCard(String simNo,String deviceNo,OnFirstActiveSIMCardListener onFirstActiveSIMCardListener
            ,int timeOutSecs, int priorityLevel){
        if (simNo==null||deviceNo==null){
            if (onFirstActiveSIMCardListener!=null){
                onFirstActiveSIMCardListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnFirstActiveSIMCardListener = onFirstActiveSIMCardListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "simCard/firstActiveSIMCard";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        params.put("device_no", deviceNo);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, FC_FIRSTACTIVESIMCARD);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 车机内SIM卡上传流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIdAndFlowList 应用ID和对应流量的集合
     * @param onUploadUseFlowDetailListener 结果监听
     * @return 请求ID，0:参数错误
     */
    public int uploadUseFlowDetail(String simNo,List<AppIdAndFlow> appIdAndFlowList,OnUploadUseFlowDetailListener onUploadUseFlowDetailListener){
        return uploadUseFlowDetail(simNo, appIdAndFlowList, onUploadUseFlowDetailListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机内SIM卡上传流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIdAndFlowList 应用ID和对应流量的集合
     * @param onUploadUseFlowDetailListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int uploadUseFlowDetail(String simNo,List<AppIdAndFlow> appIdAndFlowList,OnUploadUseFlowDetailListener onUploadUseFlowDetailListener
            ,int timeOutSecs ){
        return uploadUseFlowDetail(simNo, appIdAndFlowList, onUploadUseFlowDetailListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机内SIM卡上传流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIdAndFlowList 应用ID和对应流量的集合
     * @param onUploadUseFlowDetailListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int uploadUseFlowDetail(String simNo,List<AppIdAndFlow> appIdAndFlowList,OnUploadUseFlowDetailListener onUploadUseFlowDetailListener
            ,int timeOutSecs, int priorityLevel ){
        if (simNo==null||appIdAndFlowList==null){
            if (onUploadUseFlowDetailListener!=null){
                onUploadUseFlowDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnUploadUseFlowDetailListener = onUploadUseFlowDetailListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "useFlowDetail/uploadUseFlowDetail";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        JSONObject jsonObject = new JSONObject();
        for (int i = 0;i<appIdAndFlowList.size();i++){
            AppIdAndFlow appIdAndFlow = appIdAndFlowList.get(i);
            try {
                jsonObject.put(appIdAndFlow.getApplicationId(),appIdAndFlow.getUseFlow());
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        params.put("useFlowDeatilJSON", jsonObject);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, FC_UPLOADUSEFLOWDETAIL);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 车机内SIM卡下载流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIds 应用ID
     * @param onDownloadFlowDetailListener 结果监听
     * @return 请求ID，0:参数错误
     */
    public int downloadFlowDetail(String simNo,List<String> appIds,OnDownloadFlowDetailListener onDownloadFlowDetailListener){
        return downloadFlowDetail(simNo,appIds,onDownloadFlowDetailListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机内SIM卡下载流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIds 应用ID
     * @param onDownloadFlowDetailListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int downloadFlowDetail(String simNo,List<String> appIds,OnDownloadFlowDetailListener onDownloadFlowDetailListener
            ,int timeOutSecs){
        return downloadFlowDetail(simNo,appIds,onDownloadFlowDetailListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机内SIM卡下载流量数据
     * @param simNo Sim卡号即ICCID
     * @param appIds 应用ID
     * @param onDownloadFlowDetailListener 结果监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int downloadFlowDetail(String simNo,List<String> appIds,OnDownloadFlowDetailListener onDownloadFlowDetailListener
            ,int timeOutSecs, int priorityLevel ){
        if (simNo==null||appIds==null){
            if (onDownloadFlowDetailListener!=null){
                onDownloadFlowDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnDownloadFlowDetailListener = onDownloadFlowDetailListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "useFlowDetail/downloadFlowDetail";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        StringBuffer appIdSbf = new StringBuffer();
        for (int i = 0;i<appIds.size();i++){
            String appId = appIds.get(i);
            if (i==(appIds.size()-1)){
                appIdSbf.append(appId);
            }else {
                appIdSbf.append(appId).append(",");
            }
        }
        params.put("application_ids", appIdSbf);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, FC_DOWNLOADFLOWDETAIL);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 查询sim卡套餐有效期(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetSimCardComboTermListener
     * @return
     */
    public int getSimCardComboTerm(String simNo,String appId,OnGetSimCardComboTermListener onGetSimCardComboTermListener){
        return getSimCardComboTerm(simNo,appId,onGetSimCardComboTermListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询sim卡套餐有效期(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetSimCardComboTermListener
     * @param timeOutSecs
     * @return
     */
    public int getSimCardComboTerm(String simNo,String appId,OnGetSimCardComboTermListener onGetSimCardComboTermListener,int timeOutSecs){
        return getSimCardComboTerm(simNo,appId,onGetSimCardComboTermListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询sim卡套餐有效期(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetSimCardComboTermListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getSimCardComboTerm(String simNo,String appId,OnGetSimCardComboTermListener onGetSimCardComboTermListener,int timeOutSecs, int priorityLevel ){
        if (simNo==null){
            if (onGetSimCardComboTermListener!=null){
                onGetSimCardComboTermListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetSimCardComboTermListener = onGetSimCardComboTermListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "/simCardTermDetail/getSIMCardComboTermForSmart";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        if (appId!=null){
            params.put("application_id", appId);
        }
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETSIMCARDCOMBOTERM);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取用户流量(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetUseFlowListener
     * @return
     */
    public int getUseFlow(String simNo,String appId,OnGetUseFlowListener onGetUseFlowListener){
        return getUseFlow(simNo,appId,onGetUseFlowListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取用户流量(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetUseFlowListener
     * @param timeOutSecs
     * @return
     */
    public int getUseFlow(String simNo,String appId,OnGetUseFlowListener onGetUseFlowListener,int timeOutSecs){
        return getUseFlow(simNo,appId,onGetUseFlowListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取用户流量(smart)
     * @param simNo sim卡号
     * @param appId 应用id。例：“JICHU000”，“YINYUE001”
     * @param onGetUseFlowListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getUseFlow(String simNo,String appId,OnGetUseFlowListener onGetUseFlowListener,int timeOutSecs, int priorityLevel ){
        if (simNo==null){
            if (onGetUseFlowListener!=null){
                onGetUseFlowListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetUseFlowListener = onGetUseFlowListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "/useFlowDetail/getUseFlowForSmart";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        if (appId!=null){
            params.put("application_id", appId);
        }
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETUSEFLOW);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取sim卡激活状态(smart)
     * @param simNo
     * @param onGetSIMCardStatusListener
     * @return
     */
    public int getSIMCardStatus(String simNo,OnGetSIMCardStatusListener onGetSIMCardStatusListener){
        return getSIMCardStatus(simNo,onGetSIMCardStatusListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取sim卡激活状态(smart)
     * @param simNo
     * @param onGetSIMCardStatusListener
     * @param timeOutSecs
     * @return
     */
    public int getSIMCardStatus(String simNo,OnGetSIMCardStatusListener onGetSIMCardStatusListener,int timeOutSecs){
        return getSIMCardStatus(simNo,onGetSIMCardStatusListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取sim卡激活状态(smart)
     * @param simNo
     * @param onGetSIMCardStatusListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getSIMCardStatus(String simNo,OnGetSIMCardStatusListener onGetSIMCardStatusListener,int timeOutSecs, int priorityLevel ){
        if (simNo==null){
            if (onGetSIMCardStatusListener!=null){
                onGetSIMCardStatusListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetSIMCardStatusListener = onGetSIMCardStatusListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "/simCard/getSIMCardStatus";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("sim_no", simNo);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETSIMCARDSTATUS);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case FC_FIRSTACTIVESIMCARD://SIM卡首次激活
                if (mOnFirstActiveSIMCardListener != null) {
                    mOnFirstActiveSIMCardListener.onTimeOut();
                }
                break;
            case FC_UPLOADUSEFLOWDETAIL://车机内SIM卡上传流量数据
                if (mOnUploadUseFlowDetailListener != null) {
                    mOnUploadUseFlowDetailListener.onTimeOut();
                }
                break;
            case FC_DOWNLOADFLOWDETAIL://车机内SIM卡下载流量数据
                if (mOnDownloadFlowDetailListener != null) {
                    mOnDownloadFlowDetailListener.onTimeOut();
                }
                break;
            case GETSIMCARDCOMBOTERM://查询sim卡套餐有效期(smart)
                if (mOnGetSimCardComboTermListener != null) {
                    mOnGetSimCardComboTermListener.onTimeOut();
                }
                break;
            case GETUSEFLOW://获取用户流量(smart)
                if (mOnGetUseFlowListener != null) {
                    mOnGetUseFlowListener.onTimeOut();
                }
                break;
            case GETSIMCARDSTATUS://获取sim卡激活状态(smart)
                if (mOnGetSIMCardStatusListener != null) {
                    mOnGetSIMCardStatusListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case FC_FIRSTACTIVESIMCARD://SIM卡首次激活
                if (mOnFirstActiveSIMCardListener != null){
                    if(hasDSPTError){
                        mOnFirstActiveSIMCardListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            mOnFirstActiveSIMCardListener.firstActiveSIMCard();
                        } else{
                            mOnFirstActiveSIMCardListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnFirstActiveSIMCardListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FC_UPLOADUSEFLOWDETAIL://车机内SIM卡上传流量数据
                if (mOnUploadUseFlowDetailListener!=null){
                    if(hasDSPTError){
                        mOnUploadUseFlowDetailListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            mOnUploadUseFlowDetailListener.uploadUseFlowDetail();
                        }else {
                            mOnUploadUseFlowDetailListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnUploadUseFlowDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FC_DOWNLOADFLOWDETAIL://车机内SIM卡下载流量数据
                if (mOnDownloadFlowDetailListener!=null){
                    if(hasDSPTError){
                        mOnDownloadFlowDetailListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            Map<String,FlowDetail> map = new HashMap<String,FlowDetail>();
                            Iterator<String> keys = jsonObject.keys();
                            FlowDetail flowDetail = null;
                            while (keys.hasNext()){
                                String key = keys.next();
                                if (!"respCode".equals(key)){
                                    JSONObject object  = jsonObject.getJSONObject(key);
                                    flowDetail = new FlowDetail();
                                    flowDetail.setTotalFlow(object.getLong("totalFlow"));
                                    flowDetail.setUseFlow(object.getLong("useFlow"));
                                    flowDetail.setCanUseFlow(object.getLong("canUseFlow"));
                                    map.put(key,flowDetail);
                                }
                            }
                            mOnDownloadFlowDetailListener.downloadFlowDetail(map);
                        }else {
                            mOnDownloadFlowDetailListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDownloadFlowDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETSIMCARDCOMBOTERM://查询sim卡套餐有效期(smart)
                if (mOnGetSimCardComboTermListener!=null){
                    if(hasDSPTError){
                        mOnGetSimCardComboTermListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONArray jsonArray = jsonObject.getJSONArray("rows");

                            List<EffectiveTime> effectiveTimeList = new ArrayList<EffectiveTime>();
                            for(int i = 0; i<jsonArray.length(); i++) {
                                JSONObject object = jsonArray.getJSONObject(i);
                                EffectiveTime effectiveTime = new EffectiveTime();
                                effectiveTime.setSimNo(object.getString("sim_no"));
                                effectiveTime.setComboId(object.getString("combo_id"));
                                effectiveTime.setApplicationId(object.getString("application_id"));
                                effectiveTime.setApplicationName(object.getString("application_name"));
                                effectiveTime.setEffectiveDate(object.getString("effective_date"));
                                effectiveTime.setEffectiveLeftDate(object.getInt("effective_left_date"));
                                effectiveTime.setFreeEffectiveDate(object.getString("free_effective_date"));
                                effectiveTime.setFreeLeftDays(object.getInt("free_left_days"));
                                effectiveTimeList.add(effectiveTime);
                            }
                            mOnGetSimCardComboTermListener.getSimCardComboTerm(effectiveTimeList);
                        }else {
                            mOnGetSimCardComboTermListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetSimCardComboTermListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETUSEFLOW://获取用户流量(smart)
                if (mOnGetUseFlowListener!=null){
                    if(hasDSPTError){
                        mOnGetUseFlowListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONArray jsonArray = jsonObject.getJSONArray("rows");
                            Map<String,FlowDetail> map = new HashMap<String,FlowDetail>();
                            FlowDetail flowDetail = null;
//                            List<FlowInfo> flowInfoList = new ArrayList<FlowInfo>();
                            for(int i = 0; i<jsonArray.length(); i++) {
                                JSONObject object = jsonArray.getJSONObject(i);
                                flowDetail = new FlowDetail();
                                if (object.has("total_flow")){
                                    flowDetail.setTotalFlow(object.getLong("total_flow"));
                                }
                                if (object.has("used_flow")){
                                    flowDetail.setUseFlow(object.getLong("used_flow"));
                                }
                                if (object.has("total_flow")&&object.has("used_flow")){
                                    flowDetail.setCanUseFlow(object.getLong("total_flow")-object.getLong("used_flow"));
                                }
                                map.put(object.getString("application_id"),flowDetail);
//                                FlowInfo flowInfo = new FlowInfo();
//                                flowInfo.setApplicationId(object.getString("application_id"));
//                                flowInfo.setApplicationName(object.getString("application_name"));
//                                if (object.has("used_flow")){
//                                    flowInfo.setUsedFlow(object.getString("used_flow"));
//                                }
//                                if (object.has("total_flow")){
//                                    flowInfo.setTotalFlow(object.getString("total_flow"));
//                                }
//                                flowInfo.setEffectiveDate(object.getString("effective_date"));
//                                flowInfo.setEffectiveLeftDate(object.getString("effective_left_date"));
//                                flowInfo.setFreeEffectiveDate(object.getString("free_effective_date"));
//                                flowInfo.setFreeLeftDays(object.getString("free_left_days"));
//                                flowInfoList.add(flowInfo);
                            }//chika  for  smart 2018.4.3

                            mOnGetUseFlowListener.getUseFlow(map);
                        }else {
                            mOnGetUseFlowListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetUseFlowListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETSIMCARDSTATUS://获取sim卡激活状态(smart)
                if (mOnGetSIMCardStatusListener!=null){
                    if(hasDSPTError){
                        mOnGetSIMCardStatusListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            String simCardStatus = jsonObject.getString("simcard_status");
                            mOnGetSIMCardStatusListener.getSIMCardStatus(simCardStatus);
                        }else {
                            mOnGetSIMCardStatusListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetSIMCardStatusListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, FlowControl.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }
}
