package com.mxnavi.tspv2.http;

import android.os.Looper;

import com.mxnavi.tspv2.core.okhttp.ProgressRequestBody;
import com.mxnavi.tspv2.core.okhttp.ProgressRequestListener;
import com.mxnavi.tspv2.core.okhttp.RetryIntercepter;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.Headers;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * Created by Administrator on 2016/4/8.
 * desc
 */
public class HttpRequestUtil {
    public static OkHttpClient client;
    static {
        client = new OkHttpClient.Builder()
                .connectTimeout(10, TimeUnit.SECONDS)
                .readTimeout(45, TimeUnit.SECONDS)
                .writeTimeout(45, TimeUnit.SECONDS)
                .addInterceptor(new RetryIntercepter(3))
                .build();
    }
    static private RequestBody convertRequestParams(RequestParams params){
        if(params!=null&&params.fileParams.size()>0){
            MultipartBody.Builder builder =  new MultipartBody.Builder();
            builder.setType(MultipartBody.FORM);
            if(params!=null&&params.urlParams.size()>0){
                Iterator<Map.Entry<String,String>> urlParamIterator = params.urlParams.entrySet().iterator();
                while (urlParamIterator.hasNext()){
                    Map.Entry<String,String> entry = urlParamIterator.next();
                    builder.addFormDataPart(entry.getKey(), entry.getValue());
                }
            }



            Iterator<Map.Entry<String,File>> iterator =   params.fileParams.entrySet().iterator();
            MediaType type=MediaType.parse("application/octet-stream");
            while (iterator.hasNext()){
                Map.Entry<String,File> entry = iterator.next();
                RequestBody fileBody = RequestBody.create(type, entry.getValue());
                builder.addFormDataPart(entry.getKey(), entry.getValue().getName(), fileBody);
            }



            return  builder.build();
        }else{
            if(params!=null&&params.urlParams.size()>0){
                FormBody.Builder builder =  new FormBody.Builder();
                Iterator<Map.Entry<String,String>> urlParamIterator = params.urlParams.entrySet().iterator();
                while (urlParamIterator.hasNext()){
                    Map.Entry<String,String> entry = urlParamIterator.next();
                    builder.add(entry.getKey(), entry.getValue());
                }
                return  builder.build();
            }

        }
        return null;
    }

    /**
     * 发送post请求
     * @param url
     * @param params
     * @param responseHandler
     * @param isAsync 是否异步（目前只支持异步请求）
     * @return
     */
    public static RequestHandle doPost(String url,RequestParams params,final AsyncHttpResponseHandler responseHandler,boolean isAsync){
        final boolean isMainThread = isMainThread();
        if(responseHandler!=null){
            responseHandler.setCallBackMode(isMainThread);
        }
        RequestBody requestBody = convertRequestParams(params);
        if(params.fileParams.size()>0){
            requestBody = new ProgressRequestBody(requestBody, new ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    if(responseHandler!=null){
                        responseHandler.sendProgressMessage(bytesWritten,contentLength,done);
                    }
                }
            }) ;
        }

        Request requestPost = new Request.Builder()
                .url(url)
                .post(requestBody)
                .build();
        Call call = client.newCall(requestPost);
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                    callBackFailture(responseHandler,e,0);
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                if(response.isSuccessful()){
                    callBackSuccess(responseHandler,response);
                }else{

                    callBackFailture(responseHandler,null,response.code());
                }
            }
        } );
        return new RequestHandle().setRequest(call);



    }

    /**
     * 发送get请求
     * @param url
     * @param params
     * @param responseHandler
     * @param isAsync 是否异步（目前只支持异步请求）
     * @return
     */
    public static RequestHandle  doGet(String url,RequestParams params,final AsyncHttpResponseHandler responseHandler,boolean isAsync){
    	if(params != null){
    		url = makeUrl(url,params.urlParams);
    	}
        Request request = new Request.Builder()
                .get()
                .url(url)
                .build();
        Call call = client.newCall(request);
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                callBackFailture(responseHandler,e,0);
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                 if(response.isSuccessful()){
                     callBackSuccess(responseHandler,response);
                 }else{
                     callBackFailture(responseHandler,null,response.code());
                 }

            }
        });
        return new RequestHandle().setRequest(call);
    }
    /**
     * 取消请求
     * @param requestHandle 请求句柄
     * @return
     */
    public static void cancelRequest(RequestHandle requestHandle){
         requestHandle.cancel();
    }
    private static void callBackFailture(final AsyncHttpResponseHandler responseHandler,IOException e,int httpCode){
        if(responseHandler!=null){
            responseHandler.sendFailureMessage(httpCode,null,e);

        }
    }
    private static boolean isMainThread() {
        return Looper.getMainLooper() == Looper.myLooper();
    }
    private static void callBackSuccess(final AsyncHttpResponseHandler responseHandler,Response response){

        if(responseHandler!=null){
            try {
                responseHandler.sendSuccessMessage(response.code(),response.body().bytes());
            }catch (Exception e){
                e.printStackTrace();
                responseHandler.sendFailureMessage(response.code(),null, e);
            }

        }
    }
    public  static String makeUrl(String url,Map<String, String> params){
// 添加url参数
        if (params != null) {
            Iterator<String> it = params.keySet().iterator();
            StringBuffer a = null;
            while (it.hasNext()) {
                String key = it.next();
                String value = String.valueOf(params.get(key));
                if (a == null) {
                    a = new StringBuffer();
                    a.append("?");
                } else {
                    a.append("&");
                }
                a.append(key);
                a.append("=");
                a.append(value);
            }
            url += a.toString();
        }
        return url;

    }

}
