package com.mxnavi.tspv2.location;


import com.google.protobuf.ByteString;
import com.google.protobuf.InvalidProtocolBufferException;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxLocationMessage;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.core.message.MxResponseLocationMessageInfo;
import com.mxnavi.tspv2.location.model.CurrentLocationInfo;
import com.mxnavi.tspv2.location.model.LocationDataInfo;
import com.mxnavi.tspv2.location.model.LocationItem;
import com.mxnavi.tspv2.location.model.TrackInfo;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by wangqian on 2016/4/21.
 *位置上传
 */
public class Location extends BaseTsp {

    private int SUB_SERVICE_TYPE =1;
    // 请求服务类型为位置上传
    final int thisServiceType = Config.MxNGTPServiceType.MX_UPLOAD.getServiceType();
    // 位置上传的ServiceData协议版本
    final int serviceDataVer =  Config.MxNGTPServiceType.MX_UPLOAD.getServiceVer();
    // 请求服务类型为获取设备最新上传的位置
    final int locationServiceType = Config.MxNGTPServiceType.MX_LOCATION.getServiceType();
    //获取设备最新上传位置的ServiceData协议版本
    final int locationDataVer = Config.MxNGTPServiceType.MX_LOCATION.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int LOCATIONUPLOAD = 1;//位置上传
    final int CURRLOCATION = 2;//获取此设备最新上传的位置
    final int TRACKBYTIMERANGE = 3;//获取设备在某一时间段的轨迹


    OnSaveLoactionInfoListener mOnSaveLoactionInfoListener;
    OnGetCurrLocationListener mOnGetCurrLocationListener;
    OnGetTrackByTimeRangeListener mOnGetTrackByTimeRangeListener;

    /**
     * 位置信息上传
     * @param locationlist 位置信息列表
     * @param trakId 轨迹ID
     * @return
     */
    public int saveLoactionInfo(List<LocationItem> locationlist,int trakId){
        return saveLoactionInfo(locationlist,trakId,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 位置信息上传
     * @param locationlist 位置信息列表
     * @param trakId 轨迹ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int saveLoactionInfo(List<LocationItem> locationlist,int trakId,int timeOutSecs){
        return saveLoactionInfo( locationlist,trakId,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 位置信息上传
     * @param locationlist 位置信息列表
     * @param trakId 轨迹ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int saveLoactionInfo(List<LocationItem> locationlist,int trakId,int timeOutSecs,int priorityLevel){
        if(locationlist == null||locationlist.size()==0){
            if (mOnSaveLoactionInfoListener != null) {
                mOnSaveLoactionInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        LocationItem firstLoc = locationlist.get(0);
        MxLocationMessage.LocationDataInfo.Builder msgLocationDataInfo = MxLocationMessage.LocationDataInfo.newBuilder();
        msgLocationDataInfo.setFirstDir(firstLoc.getDirection());
        msgLocationDataInfo.setFirstLat(Utils.converDoubleCoordinate2MX(firstLoc.getLocation().getLatitude()));
        msgLocationDataInfo.setFirstLocationTime(firstLoc.getLocationTime());
        msgLocationDataInfo.setFirstSpeed(firstLoc.getSpeed());
        msgLocationDataInfo.setFirstLon(Utils.converDoubleCoordinate2MX(firstLoc.getLocation().getLongitude()));
        msgLocationDataInfo.setTrackId(trakId);
        MxLocationMessage.LocationData.Builder msgLocationItem ;

        int size = locationlist.size();
        msgLocationDataInfo.setSeqNum(size - 1);
        for (int i = 1; i < size; i++) {
            msgLocationItem  =  MxLocationMessage.LocationData.newBuilder();
            LocationItem locationItemnew = locationlist.get(i);
//            if(i == 0){
//                msgLocationItem.setOffsetLat(Utils.converDoubleCoordinate2MX(locationItemnew.getLocation().getLatitude()-locationDataInfo.getFirstLocation().getLatitude()));
//                msgLocationItem.setOffsetLon(Utils.converDoubleCoordinate2MX(locationItemnew.getLocation().getLongitude()-locationDataInfo.getFirstLocation().getLongitude()));
//                msgLocationItem.setOffsetLocationTime((int)(locationItemnew.getLocationTime()-locationDataInfo.getFirstLocationTime()));
//                msgLocationItem.setDir(locationItemnew.getDirection()-locationDataInfo.getFirstDirection());
//                msgLocationItem.setSpeed(locationItemnew.getSpeed()-locationDataInfo.getFirstSpeed());
//
//            }else{
            LocationItem locationItemold = locationlist.get(i-1);
            msgLocationItem.setOffsetLat(Utils.converDoubleCoordinate2MX(locationItemnew.getLocation().getLatitude()-locationItemold.getLocation().getLatitude()));
            msgLocationItem.setOffsetLon(Utils.converDoubleCoordinate2MX(locationItemnew.getLocation().getLongitude()-locationItemold.getLocation().getLongitude()));
            msgLocationItem.setOffsetLocationTime((int)(locationItemnew.getLocationTime()-locationItemold.getLocationTime()));
            msgLocationItem.setDir(locationItemnew.getDirection()-locationItemold.getDirection());
            msgLocationItem.setSpeed(locationItemnew.getSpeed()-locationItemold.getSpeed());
                
//            }
            msgLocationDataInfo.addLocationData(msgLocationItem);

        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessProtoBufMsg(eventId,msgLocationDataInfo.build(),thisServiceType, serviceDataVer,SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, LOCATIONUPLOAD);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取设备当前位置
     * @return
     */
    public int getCurrLocation(){
        return getCurrLocation(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取设备当前位置
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getCurrLocation(int timeOutSecs){
        return getCurrLocation(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取设备当前位置
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getCurrLocation(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_location/getCurrLocation";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("deviceNo", Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, locationServiceType, locationDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, CURRLOCATION);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }


    /**
     * 获取设备在某一时间段的轨迹
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return
     */
    public int getTrackByTimeRange(int startTime,int endTime){
        return getTrackByTimeRange(startTime,endTime,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取设备在某一时间段的轨迹
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getTrackByTimeRange(int startTime,int endTime,int timeOutSecs){
        return getTrackByTimeRange(startTime,endTime,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取设备在某一时间段的轨迹
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getTrackByTimeRange(int startTime,int endTime,int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "getTrackByTimeRange";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("deviceNo", Config.VIN);
        params.put("startTime", startTime);
        params.put("endTime", endTime);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, locationServiceType, locationDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        addCallbackMap(eventId, TRACKBYTIMERANGE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    public interface OnSaveLoactionInfoListener extends CallBackListener{
        /**
         * 位置上传成功回调
         * @param trackId 轨迹ID
         */
        public void onSaveLoactionInfo(int trackId);
    }
    public interface OnGetCurrLocationListener extends CallBackListener{
        public void onGetCurrLocation(CurrentLocationInfo currentLocationInfo);

    }
    public interface OnGetTrackByTimeRangeListener extends CallBackListener{
        public void onGetTrackByTimeRange(TrackInfo trackInfo);

    }


    public void setOnSaveLoactionInfoListener(OnSaveLoactionInfoListener mOnSaveLoactionInfoListener) {
        this.mOnSaveLoactionInfoListener = mOnSaveLoactionInfoListener;
    }

    public void setOnGetCurrLocationListener(OnGetCurrLocationListener mOnGetCurrLocationListener) {
        this.mOnGetCurrLocationListener = mOnGetCurrLocationListener;
    }

    public void setOnGetTrackByTimeRangeListener(OnGetTrackByTimeRangeListener mOnGetTrackByTimeRangeListener) {
        this.mOnGetTrackByTimeRangeListener = mOnGetTrackByTimeRangeListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, Location.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case LOCATIONUPLOAD:
                if (mOnSaveLoactionInfoListener != null) {
                    mOnSaveLoactionInfoListener.onTimeOut();
                }
                break;
            case CURRLOCATION:
                if (mOnGetCurrLocationListener != null) {
                    mOnGetCurrLocationListener.onTimeOut();
                }
                break;
            case TRACKBYTIMERANGE:
                if (mOnGetTrackByTimeRangeListener != null) {
                    mOnGetTrackByTimeRangeListener.onTimeOut();
                }
                break;

        }
    }
    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {

            case CURRLOCATION:
                if(mOnGetCurrLocationListener != null){
                    if(hasDSPTError){
                        mOnGetCurrLocationListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            CurrentLocationInfo currentLocationInfo = new CurrentLocationInfo();
                            currentLocationInfo.setUpdateTime(jsonObject.getInt("updateTime"));
                            LatLng location = new LatLng();
                            location.setLatitude(Utils.converMX2DoubleCoordinate(jsonObject.getInt("lat")));
                            location.setLongitude(Utils.converMX2DoubleCoordinate(jsonObject.getInt("lon")));
                            currentLocationInfo.setLocation(location);
                            mOnGetCurrLocationListener.onGetCurrLocation(currentLocationInfo);
                        }else{
                            mOnGetCurrLocationListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetCurrLocationListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case TRACKBYTIMERANGE:
                if(mOnGetTrackByTimeRangeListener != null){
                    if(hasDSPTError){
                        mOnGetTrackByTimeRangeListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        TrackInfo trackInfo = new TrackInfo();
                        List<CurrentLocationInfo> list = new ArrayList<CurrentLocationInfo>();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray result = jsonObject.getJSONArray("result");
                            for (int i = 0;i< result.length();i++) {
                                JSONObject track = result.getJSONObject(i);
                                trackInfo.setTrackId(track.getInt("trackId"));
                                trackInfo.setTrackStartTime(track.getInt("trackStartTime"));
                                trackInfo.setTrackEndTime(track.getInt("trackEndTime"));
                                JSONArray tracks = track.getJSONArray("track");
                                for (int j = 0; j < tracks.length(); j++) {
                                    JSONObject CurrentLocationInfo = tracks.getJSONObject(j);
                                    CurrentLocationInfo locationInfo = new CurrentLocationInfo();
                                    LatLng location = new LatLng();
                                    location.setLatitude(Utils.converMX2DoubleCoordinate(CurrentLocationInfo.getInt("lat")));
                                    location.setLongitude(Utils.converMX2DoubleCoordinate(CurrentLocationInfo.getInt("lon")));
                                    locationInfo.setLocation(location);
                                    locationInfo.setUpdateTime(CurrentLocationInfo.getInt("updateTime"));
                                    list.add(locationInfo);
                                }
                                trackInfo.setLocations(list);
                            }
                            mOnGetTrackByTimeRangeListener.onGetTrackByTimeRange(trackInfo);

                        }else{
                            mOnGetTrackByTimeRangeListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetTrackByTimeRangeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, ByteString response, boolean hasDSPTError, int errorCode, String errorMsg){
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case LOCATIONUPLOAD:
                if (mOnSaveLoactionInfoListener != null) {
                    if(hasDSPTError){
                        mOnSaveLoactionInfoListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    MxResponseLocationMessageInfo.ResponseLocationMessage responseLocationMessage = null;
                    try {
                        responseLocationMessage = MxResponseLocationMessageInfo.ResponseLocationMessage.parseFrom(response);
                    } catch (InvalidProtocolBufferException e) {
                        e.printStackTrace();
                    }
                    if(responseLocationMessage!=null){
                        int respCode = responseLocationMessage.getRespCode();
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int trackId = responseLocationMessage.getTrackId();
                            mOnSaveLoactionInfoListener.onSaveLoactionInfo(trackId);
                        }else{
                            mOnSaveLoactionInfoListener.onFail(respCode, "");
                        }
                    }else{
                        mOnSaveLoactionInfoListener.onFail(-100, "");
                    }
                }
                break;
        }
    }
}
