package com.mxnavi.tspv2.news;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.news.model.Channel;
import com.mxnavi.tspv2.news.model.News;
import com.mxnavi.tspv2.news.model.NewsContent;
import com.mxnavi.tspv2.news.model.NewsDetailQueryItem;
import com.mxnavi.tspv2.news.model.NewsImage;
import com.mxnavi.tspv2.news.model.NewsInfoItem;
import com.mxnavi.tspv2.news.model.NewsListQuery;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.RequestDispathUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.mxnavi.tspv2.news.model.Channels;
import com.mxnavi.tspv2.news.model.NewsListResult;
import com.mxnavi.tspv2.news.model.NewsRows;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

/**
 * Created by Administrator on 2016/4/27.
 * 新闻
 */
public class NewsQuery extends BaseTsp {

    // 请求服务类型为新闻
    final int thisServiceType = Config.MxNGTPServiceType.NEWS.getServiceType();
    // 注册此对象为新闻业务信息接收类
    final int serviceDataVer = Config.MxNGTPServiceType.NEWS.getServiceVer();

    public NewsQuery(){
    }


    private Map<Integer,Integer> listenertypeHashMap = new HashMap<Integer,Integer>();
    final int News_chanel =1;
    final int News_chanelList =2;
    final int News_detailInformation =3;
    final int News_newsList =4;

    OnQueryNewsChannelListener mOnQueryNewsChannelListener;
    OnQueryNewsListListener mOnQueryNewsListListener;
    OnQueryNewsDetailListener mOnQueryNewsDetailListener;
    OnGetNewsListListener mOnGetNewsListListener;

    /**
     * 请求获取新闻频道信息
     * @return 请求ID
     */
    public int queryNewsChannel(){
        return queryNewsChannel(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取新闻频道信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int queryNewsChannel(int timeOutSecs){
        return queryNewsChannel(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *  请求获取新闻频道信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int queryNewsChannel(int timeOutSecs,int priorityLevel){
        String url = "get_news_channel";
        HashMap<String, Object> params = new HashMap<String, Object>();
//        if (dataSourceId!=0){
//            params.put("data_source_id",dataSourceId);
//        }
//        params.put("data_source_id",2);
//        String requestData = Utils.makeUrl(url, params);
        String requestData = "get_news_channel";

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, News_chanel);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取新闻频道信息(与queryNewsChannel接口封装的为同一个接口，for T10)
     * @param dataSourceId 数据源id(1、光明网2、凤凰网)
     * @return
     */
    public int getNewsChannel(int dataSourceId){
        return getNewsChannel(dataSourceId,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取新闻频道信息(与queryNewsChannel接口封装的为同一个接口，for T10)
     * @param dataSourceId 数据源id(1、光明网2、凤凰网)
     * @param timeOutSecs
     * @return
     */
    public int getNewsChannel(int dataSourceId,int timeOutSecs){
        return getNewsChannel(dataSourceId,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取新闻频道信息(与queryNewsChannel接口封装的为同一个接口，for T10)
     * @param dataSourceId 数据源id(1、光明网2、凤凰网)
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getNewsChannel(int dataSourceId,int timeOutSecs,int priorityLevel){
        String url = "get_news_channel";
        HashMap<String, Object> params = new HashMap<String, Object>();

        params.put("data_source_id",dataSourceId);
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, News_chanel);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取某频道下的新闻列表信息
     * @param newsListQuery 新闻列表信息查询条件
     * @return 请求ID
     */
    public int queryNewsList(NewsListQuery newsListQuery){
        return queryNewsList(newsListQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }


    /**
     * 请求获取某频道下的新闻列表信息
     * @param newsListQuery 新闻列表信息查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int queryNewsList(NewsListQuery newsListQuery, int timeOutSecs){
        return queryNewsList(newsListQuery,timeOutSecs, Config.DEFAULT_PRIORITY);
    }


    /**
     * 请求获取某频道下的新闻列表信息
     * @param newsListQuery 新闻列表信息查询条件
     * @return 请求ID
     */
    public int queryNewsList(NewsListQuery newsListQuery,int timeOutSecs,int priorityLevel){
        if(newsListQuery == null){
            if (mOnQueryNewsListListener != null) {
                mOnQueryNewsListListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "get_news_info_list";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<Integer> channelIdList = newsListQuery.getChannelIds();
        StringBuffer channelString = new StringBuffer();
        if(channelIdList==null||channelIdList.size()==0){
            channelString.append("0");
        }else{
            int inte;
            int size = channelIdList.size();

            for(int i = 0;i<size;i++){
                inte =  channelIdList.get(i);
                if(i<size-1) {
                    channelString.append(inte + ",");
                }else{
                    channelString.append(inte);
                }
            }

        }


        params.put("channel_id",channelString.toString());
        params.put("info_kind", newsListQuery.getInfoKind());
        params.put("offset",newsListQuery.getOffset());
        params.put("limit",newsListQuery.getLimit());
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, News_chanelList);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求获取新闻详细信息
     * @param queryItems 新闻详细信息查询条件
     * @return 请求ID
     */
    public int queryNewsDetail(List<NewsDetailQueryItem> queryItems){
        return queryNewsDetail(queryItems, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }


    /**
     * 请求获取新闻详细信息
     * @param queryItems 新闻详细信息查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int queryNewsDetail(List<NewsDetailQueryItem> queryItems, int timeOutSecs){
        return queryNewsDetail(queryItems, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求获取新闻详细信息
     * @param queryItems 新闻详细信息查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int queryNewsDetail(List<NewsDetailQueryItem> queryItems, int timeOutSecs,int priorityLevel){
        if(queryItems == null){
            if (mOnQueryNewsDetailListener != null) {
                mOnQueryNewsDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "get_news_detail_info";
        HashMap<String, Object> params = new HashMap<String, Object>();

        StringBuffer channelString = new StringBuffer();
        StringBuffer newsString = new StringBuffer();

        NewsDetailQueryItem newsDetailQueryItem;
        int size = queryItems.size();
        for(int i =0;i<size;i++){
            newsDetailQueryItem = queryItems.get(i);
            if(i<size-1) {
                channelString.append(newsDetailQueryItem.getChannelId() + ",");
                newsString.append(newsDetailQueryItem.getNewsId() + ",");
            }else{
                channelString.append(newsDetailQueryItem.getChannelId());
                newsString.append(newsDetailQueryItem.getNewsId());
            }
        }
        params.put("channel_id",channelString.toString());
        params.put("news_id", newsString.toString());
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, News_detailInformation);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取某频道下的新闻列表
     * @param newsListQuery
     * @return 请求ID
     */
    public int getNewsList(NewsListQuery newsListQuery){
        return getNewsList(newsListQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某频道下的新闻列表
     * @param newsListQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int getNewsList(NewsListQuery newsListQuery, int timeOutSecs){
        return getNewsList(newsListQuery,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某频道下的新闻列表
     * @param newsListQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int getNewsList(NewsListQuery newsListQuery,int timeOutSecs,int priorityLevel){
        if(newsListQuery == null){
            if (mOnGetNewsListListener != null) {
                mOnGetNewsListListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "get_news_list";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<Integer> channelIdList = newsListQuery.getChannelIds();
        StringBuffer channelString = new StringBuffer();
        if(channelIdList==null||channelIdList.size()==0){
            channelString.append("0");
        }else{
            int inte;
            int size = channelIdList.size();

            for(int i = 0;i<size;i++){
                inte =  channelIdList.get(i);
                if(i<size-1) {
                    channelString.append(inte + ",");
                }else{
                    channelString.append(inte);
                }
            }

        }


        params.put("channel_id",channelString.toString());
        params.put("offset",newsListQuery.getOffset());
        params.put("limit",newsListQuery.getLimit());
        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, News_newsList);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    public interface OnQueryNewsChannelListener extends CallBackListener{
        public void onQueryNewsChannel(int requestId, List<Channel> channelList);
    }
    public interface OnQueryNewsListListener extends CallBackListener{
        public void onQueryNewsList(int requestId, List<NewsInfoItem> newsList);
    }

    public interface OnQueryNewsDetailListener extends CallBackListener{
        public void onQueryNewsDetail(int requestId, List<NewsInfoItem> newsList);
    }

    public interface OnGetNewsListListener extends CallBackListener{
        public void onGetNewsList(int requestId, NewsListResult newsListResult);
    }


    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg){
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case News_chanel:
                if(mOnQueryNewsChannelListener != null) {
                    if(hasDSPTError){
                        mOnQueryNewsChannelListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    List<Channel> chanelList = new ArrayList<Channel>();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Channel result;
                            JSONArray channelsDataArray = jsonObject.getJSONArray("channels");
                            for (int i = 0; i < channelsDataArray.length(); i++) {
                                result = new Channel();
                                JSONObject object = channelsDataArray.getJSONObject(i);
                                result.setId(object.getInt("id"));
                                result.setTitle(object.getString("title"));
                                chanelList.add(result);
                            }

                            mOnQueryNewsChannelListener.onQueryNewsChannel(requestHandleId, chanelList);
                        } else if(respCode == 101) {
                            mOnQueryNewsChannelListener.onQueryNewsChannel(requestHandleId, chanelList);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnQueryNewsChannelListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnQueryNewsChannelListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case News_chanelList:
                if(mOnQueryNewsListListener != null){
                    if(hasDSPTError){
                        mOnQueryNewsListListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<NewsInfoItem> newsInfoItemList = this.returnNewsInfoItemList(jsonObject);
                            mOnQueryNewsListListener.onQueryNewsList(requestHandleId, newsInfoItemList);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnQueryNewsListListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnQueryNewsListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case News_detailInformation:
                if(mOnQueryNewsDetailListener != null){
                    if(hasDSPTError){
                        mOnQueryNewsDetailListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<NewsInfoItem> newsInfoItemList = this.returnNewsInfoItemList(jsonObject);
                            mOnQueryNewsDetailListener.onQueryNewsDetail(requestHandleId, newsInfoItemList);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnQueryNewsDetailListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnQueryNewsDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case News_newsList:
                if(mOnGetNewsListListener != null){
                    if(hasDSPTError){
                        mOnGetNewsListListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    NewsListResult newsListResult = new NewsListResult();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<Channels> channelsList = new ArrayList<Channels>();
                            JSONArray jsonArray = jsonObject.getJSONArray("channels");
                            for (int i=0; i<jsonArray.length(); i++) {
                                JSONObject channelObj = jsonArray.getJSONObject(i);
                                Channels channels = new Channels();
                                channels.setChannelId(Integer.parseInt(channelObj.getString("channelId")));
                                channels.setChannelName(channelObj.getString("channelName"));

                                List<NewsRows> newsRowsList = new ArrayList<NewsRows>();
                                JSONArray newsRowsArr = channelObj.getJSONArray("newsRows");
                                for (int j=0; j< newsRowsArr.length(); j++) {
                                    JSONObject newsRowObj = newsRowsArr.getJSONObject(j);
                                    NewsRows newsRows = new NewsRows();
                                    newsRows.setNewsAbstract(newsRowObj.getString("newsAbstract"));
                                    newsRows.setNewsAuthor(newsRowObj.getString("newsAuthor"));
                                    newsRows.setNewsId(newsRowObj.getString("newsId"));
                                    newsRows.setNewsLink(newsRowObj.getString("newsLink"));
                                    newsRows.setNewsSource(newsRowObj.getString("newsSource"));
                                    newsRows.setNewsThumbnail(newsRowObj.getString("newsThumbnail"));
                                    newsRows.setNewsTitle(newsRowObj.getString("newsTitle"));
                                    newsRows.setPublicDate(newsRowObj.getString("publicDate"));
                                    newsRowsList.add(newsRows);
                                }
                                channels.setNewsRowsList(newsRowsList);
                                channelsList.add(channels);
                            }
                            newsListResult.setChannels(channelsList);
                            mOnGetNewsListListener.onGetNewsList(requestHandleId, newsListResult);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnGetNewsListListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnGetNewsListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            default:
                break;
        }
    }

    /**
     * 共通JSON数据格式
     * @param jsonObject
     * @return
     */
    public List<NewsInfoItem> returnNewsInfoItemList (JSONObject jsonObject)throws Exception{
        String id = "";
        String title = "";
        String pubData = "";
        String author = "";
        String source = "";
        String newsAbstract = "";
        String newsType = "";
        String info = "";
        String imageURL = "";
        String imageType = "";
        String imageTitle = "";
        int channelId;
        NewsInfoItem result;
        List<News> newsList;
        List<NewsImage> newsImageList;
        List<NewsInfoItem> newsInfoItemList = new ArrayList<NewsInfoItem>();
        JSONArray channelsDataArray = jsonObject.getJSONArray("channels");
        for (int i = 0; i < channelsDataArray.length(); i++) {
            result = new NewsInfoItem();
            JSONObject object = channelsDataArray.getJSONObject(i);
            if(object.has("channelID")){
                 channelId = object.getInt("channelID");
                 result.setChannelId(channelId);
            }
            News news ;
            if(object.has("news")){
                newsList = new ArrayList<News>();
                JSONArray newsDataArray = object.getJSONArray("news");
                    for (int j = 0; j < newsDataArray.length(); j++) {
                        news = new News();
                        JSONObject newsObject = newsDataArray.getJSONObject(j);
                        if(newsObject.has("id")){
                            id = newsObject.getString("id");
                        }
                        if(newsObject.has("title")){
                            title = newsObject.getString("title");
                        }

                        if(newsObject.has("pubData")){
                            pubData = newsObject.getString("pubData");
                        }
                        if(newsObject.has("author")){
                            author = newsObject.getString("author");
                        }
                        if(newsObject.has("source")) {
                            source = newsObject.getString("source");
                        }
                        if(newsObject.has("abstract")) {
                            newsAbstract = newsObject.getString("abstract");
                        }

                        NewsContent content = new NewsContent();
                        if(newsObject.has("content")){
                            JSONObject contentObj = newsObject.getJSONObject("content");
                            if(contentObj.has("type")) {
                                if (contentObj.getInt("type") == 1) {
                                    newsType = "url";
                                } else if (contentObj.getInt("type") == 2) {
                                    newsType = "text";
                                }
                                content.setType(newsType);
                            }
                            if(contentObj.has("info")) {
                                info = contentObj.getString("info");
                                content.setInfo(info);
                            }
                            if(contentObj.has("images")){
                                newsImageList = new ArrayList<NewsImage>();
                                NewsImage newsImage;
                                JSONArray imagesDataArray = contentObj.getJSONArray("images");
                                for (int z = 0; z < imagesDataArray.length(); z++) {
                                    newsImage = new NewsImage();
                                    JSONObject imagesObject = imagesDataArray.getJSONObject(z);
                                    if(imagesObject.has("imageURL")) {
                                        imageURL = imagesObject.getString("imageURL");
                                    }
                                    if(imagesObject.has("imageType")) {
                                        imageType = imagesObject.getString("imageType");
                                    }
                                    if(imagesObject.has("imageTitle")) {
                                        imageTitle = imagesObject.getString("imageTitle");
                                    }
                                    newsImage.setImageURL(imageURL);
                                    newsImage.setImageType(imageType);
                                    newsImage.setImageTitle(imageTitle);
                                    newsImageList.add(newsImage);
                                }
                                content.setImages(newsImageList);
                            }
                        }
                        news.setContent(content);
                        news.setId(id);
                        news.setTitle(title);
                        news.setPubData(pubData);
                        news.setAuthor(author);
                        news.setSource(source);
                        news.setNewsAbstract(newsAbstract);
                        newsList.add(news);
                    }
                result.setNews(newsList);
            }
            newsInfoItemList.add(result);
        }
        return newsInfoItemList;
    }

    private  void addCallbackMap(int eventId,int type){
        super.addCallbackMap(eventId, NewsQuery.this);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId){
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {

        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case News_chanel:
                if (mOnQueryNewsChannelListener != null) {
                    mOnQueryNewsChannelListener.onTimeOut();
                }
                break;
            case News_chanelList:
                if (mOnQueryNewsListListener != null) {
                    mOnQueryNewsListListener.onTimeOut();
                }
                break;
            case News_detailInformation:
                if (mOnQueryNewsDetailListener != null) {
                    mOnQueryNewsDetailListener.onTimeOut();
                }
                break;
            case News_newsList:
                if (mOnGetNewsListListener != null) {
                    mOnGetNewsListListener.onTimeOut();
                }
                break;
        }
    }

    public void setOnQueryNewsChannelListener(OnQueryNewsChannelListener onQueryNewsChannelListener) {
        this.mOnQueryNewsChannelListener = onQueryNewsChannelListener;
    }

    public void setOnQueryNewsListListener(OnQueryNewsListListener onQueryNewsListListener) {
        this.mOnQueryNewsListListener = onQueryNewsListListener;
    }

    public void setOnQueryNewsDetailListener(OnQueryNewsDetailListener onQueryNewsDetailListener) {
        this.mOnQueryNewsDetailListener = onQueryNewsDetailListener;
    }
    public void setOnGetNewsListListener(OnGetNewsListListener onGetNewsListListener) {
        this.mOnGetNewsListListener = onGetNewsListListener;
    }
}
