package com.mxnavi.tspv2.placeservice;

import com.mxnavi.tspv2.http.AsyncHttpResponseHandler;
import com.mxnavi.tspv2.http.HttpRequestUtil;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.placeservice.model.District;
import com.mxnavi.tspv2.placeservice.model.DistrictInfo;
import com.mxnavi.tspv2.placeservice.model.RangeData;
import com.mxnavi.tspv2.placeservice.model.RoadData;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/7/12.
 */
public class PlaceService extends BaseTsp{
    // 请求服务类型为位置服务(地址Code获取与转换等)
    final int thisServiceType = Config.MxNGTPServiceType.MX_PLACE.getServiceType();
    // 位置服务(地址Code获取与转换等)的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_PLACE.getServiceVer();

    private Map<Integer, String> pathMap = new HashMap<Integer, String>();
    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int GETNEWVERSIONADMINCODE = 1;
    final int GETDISTRICTSBYRANGE = 2;
    final int GETDISTRICTSBYROAD= 3;

    public interface OnGetLastestDistrictListener extends CallBackListener {
        /**
         * 行政区编码升级回调接口
         * @param hasNewVersion 是否有新版本，false：没有；true：有
         * @param version 最新版本号，如果没有最新版本则为空
         */
        public void onGetLastestDistrictSuccess( boolean hasNewVersion,String version);
    }
    public interface OnGetDistrictsByRangeListener extends CallBackListener {
        /**
         * 矩形框内代表经纬度行政区回调接口
         * @param districtList 区域信息
         */
        public void OnGetDistrictsByRangeSuccess(int requestHandleId, List<District> districtList);
    }
    public interface OnGetDistrictsByRoadListener extends CallBackListener {
        /**
         * 获取沿路途径的市级行政区
         * @param districtInfo 区域信息
         */
        public void OnGetDistrictsByRoadSuccess(List<DistrictInfo> districtInfo);
    }
    OnGetLastestDistrictListener mOnGetLastestDistrictListener;
    OnGetDistrictsByRangeListener mOnGetDistrictsByRangeListener;
    OnGetDistrictsByRoadListener mOnGetDistrictsByRoadListener;
    /**
     * 行政区编码升级
     * @param currentver 版本号,如果本地没有版本号的时候可以传空串
     * @param filePath 行政区划编码文件存储全路径，包含文件名，必传
     * @param onGetLastestDistrictListener 回调接口
     * @return 请求ID，0:参数错误
     */
    public int getLastestDistrict(String currentver,String filePath,OnGetLastestDistrictListener onGetLastestDistrictListener){
        return getLastestDistrict(currentver, filePath, onGetLastestDistrictListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *行政区编码升级
     * @param currentver 版本号,如果本地没有版本号的时候可以传空串
     * @param filePath 行政区划编码文件存储全路径，包含文件名，必传
     * @param onGetLastestDistrictListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getLastestDistrict(String currentver,String filePath,OnGetLastestDistrictListener onGetLastestDistrictListener,int timeOutSecs){
        return getLastestDistrict(currentver, filePath, onGetLastestDistrictListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *行政区编码升级
     * @param currentver 版本号,如果本地没有版本号的时候可以传空串
     * @param filePath 行政区划编码文件存储全路径，包含文件名，必传
     * @param onGetLastestDistrictListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getLastestDistrict(String currentver,String filePath,OnGetLastestDistrictListener onGetLastestDistrictListener,int timeOutSecs, int priorityLevel){
        if( Utils.strIsEmpty(filePath)){
            if (onGetLastestDistrictListener != null) {
                onGetLastestDistrictListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetLastestDistrictListener = onGetLastestDistrictListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetNewVersionAdminCode";
        JSONObject param = new JSONObject();
        try {
            param.put("version",Utils.converNullToEmpty(currentver));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param);
        params.put("return_type",0);
        params.put("compress", 0);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETNEWVERSIONADMINCODE);
        pathMap.put(eventId,filePath);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     *
     * @param rangeData
     * @param onGetDistrictsByRangeListener
     * @return
     */
    public int getDistrictsByRange(RangeData rangeData, OnGetDistrictsByRangeListener onGetDistrictsByRangeListener){
        return getDistrictsByRange(rangeData, onGetDistrictsByRangeListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *
     * @param rangeData
     * @param onGetDistrictsByRangeListener
     * @param timeOutSecs
     * @return
     */
    public int getDistrictsByRange(RangeData rangeData, OnGetDistrictsByRangeListener onGetDistrictsByRangeListener, int timeOutSecs){
        return getDistrictsByRange(rangeData, onGetDistrictsByRangeListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *
     * @param rangeData
     * @param onGetDistrictsByRangeListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getDistrictsByRange(RangeData rangeData, OnGetDistrictsByRangeListener onGetDistrictsByRangeListener, int timeOutSecs, int priorityLevel){
        if( rangeData == null
                || rangeData.getDistType() == null
                || rangeData.getSource() ==null
                || rangeData.getLeftDown() == null
                || rangeData.getRightUp() == null){
            if (onGetDistrictsByRangeListener != null) {
                onGetDistrictsByRangeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetDistrictsByRangeListener = onGetDistrictsByRangeListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetDistrictsByRange";
        JSONObject param = new JSONObject();
        try {
//            param.put("source",rangeDate.getSource());
            param.put("distType", rangeData.getDistType());
            param.put("leftDownLon",Utils.converDoubleCoordinate2MX(rangeData.getLeftDown().getLongitude()));
            param.put("leftDownLat",Utils.converDoubleCoordinate2MX(rangeData.getLeftDown().getLatitude()));
            param.put("rightUpLon",Utils.converDoubleCoordinate2MX(rangeData.getRightUp().getLongitude()));
            param.put("rightUpLat",Utils.converDoubleCoordinate2MX(rangeData.getRightUp().getLatitude()));

        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param.toString());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETDISTRICTSBYRANGE);
//        pathMap.put(eventId,filePath);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     *
     * @param roadData
     * @param userId
     * @param token
     * @param onGetDistrictsByRoadListener
     * @return
     */
    public int getDistrictsByRoad(RoadData roadData, String userId, String token, OnGetDistrictsByRoadListener onGetDistrictsByRoadListener){
        return getDistrictsByRoad(roadData, userId, token, onGetDistrictsByRoadListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *
     * @param roadData
     * @param userId
     * @param token
     * @param onGetDistrictsByRoadListener
     * @param rangeData
     * @param timeOutSecs
     * @return
     */
    public int getDistrictsByRoad(RoadData roadData, String userId, String token, OnGetDistrictsByRoadListener onGetDistrictsByRoadListener, RangeData rangeData, int timeOutSecs){
        return getDistrictsByRoad(roadData, userId, token, onGetDistrictsByRoadListener,  timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *
     * @param roadData
     * @param userId
     * @param token
     * @param onGetDistrictsByRoadListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getDistrictsByRoad(RoadData roadData, String userId, String token, OnGetDistrictsByRoadListener onGetDistrictsByRoadListener, int timeOutSecs, int priorityLevel){
        if( roadData == null
                || roadData.getDistType() == null
                || roadData.getSource() ==null
                || roadData.getPointList() == null){
            if (onGetDistrictsByRoadListener != null) {
                onGetDistrictsByRoadListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnGetDistrictsByRoadListener = onGetDistrictsByRoadListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetDistrictsOnRoad";
//        List<LatLng> pointList = new ArrayList<LatLng>();

        int lof = Utils.converDoubleCoordinate2MX(roadData.getPointList().get(0).getLongitude());
        int laf = Utils.converDoubleCoordinate2MX(roadData.getPointList().get(0).getLatitude());
        String pointList = String.valueOf(lof)+" "+String.valueOf(laf);
//        pointList.add(roadDate.getPointList().get(0));

        Utils.loge(pointList.toString()+"1111111111111111111111111111111111");
        int size = roadData.getPointList().size();
        for (int i = 1; i < size; i++) {
            LatLng pointItemNew = roadData.getPointList().get(i);
            LatLng pointItemOld = roadData.getPointList().get(i-1);
//            LatLng pointOff = new LatLng();
            int lo = Utils.converDoubleCoordinate2MX(pointItemNew.getLongitude()-pointItemOld.getLongitude());
//            pointOff.setLongitude(pointItemNew.getLongitude()-pointItemOld.getLongitude());
            int la = Utils.converDoubleCoordinate2MX(pointItemNew.getLatitude()-pointItemOld.getLatitude());
//            pointOff.setLatitude(pointItemNew.getLatitude()-pointItemOld.getLatitude());
            String mix = ","+String.valueOf(lo) +" "+String.valueOf(la);
//            pointList.add(pointOff);
            pointList += mix;
            Utils.loge(pointList.toString()+"222222222222222222222222222222");
        }

        JSONObject param = new JSONObject();
        try {
//            param.put("source",roadDate.getSource());
            param.put("distType", roadData.getDistType());
            param.put("radius", roadData.getRadius());
            param.put("pointList",pointList);

        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param.toString());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge("requestData------------------------------------------"+requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
//        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
//                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
//        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
//                userId, token, requestData, thisServiceType, serviceDataVer,
//                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GETDISTRICTSBYROAD);
//        pathMap.put(eventId,filePath);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }



    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, PlaceService.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
            pathMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GETNEWVERSIONADMINCODE://1
                if (mOnGetLastestDistrictListener != null) {
                    mOnGetLastestDistrictListener.onTimeOut();
                }
                break;
            case GETDISTRICTSBYRANGE://2
                if (mOnGetDistrictsByRangeListener != null) {
                    mOnGetDistrictsByRangeListener.onTimeOut();
                }
                break;
            case GETDISTRICTSBYROAD://3
                if (mOnGetDistrictsByRoadListener != null) {
                    mOnGetDistrictsByRoadListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        String tempPath =  pathMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GETNEWVERSIONADMINCODE:
                if (mOnGetLastestDistrictListener != null) {
                    if(hasDSPTError){
                        mOnGetLastestDistrictListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    try {
                        JSONTokener jsonParser = new JSONTokener(response);
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONObject dataObj = jsonObject.getJSONObject("data");
                            boolean hasNewVersion;
                            if (dataObj.getInt("hasNewVersion")==0){
                                hasNewVersion = false;
                            }else {
                                hasNewVersion = true;
                            }

                            if(hasNewVersion){
                                downloadAdminCode(dataObj.getString("url"),tempPath,dataObj.getString("version"), mOnGetLastestDistrictListener);
                            }else{
                                if (mOnGetLastestDistrictListener!=null){
                                    mOnGetLastestDistrictListener.onGetLastestDistrictSuccess(hasNewVersion, dataObj.getString("version"));
                                }
                            }

                        } else {
                            mOnGetLastestDistrictListener.onFail(respCode,"出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetLastestDistrictListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETDISTRICTSBYRANGE:
                if (mOnGetDistrictsByRangeListener != null) {
                    if(hasDSPTError){
                        mOnGetDistrictsByRangeListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    try {
                        JSONTokener jsonParser = new JSONTokener(response);
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONArray districtObj = jsonObject.getJSONArray("listDistrict");
                            List<District> districtList = new ArrayList<District>();
                            for(int i = 0; i<districtObj.length(); i++) {
                                JSONObject object = districtObj.getJSONObject(i);
                                District district = new District();
                                district.setCode(object.getInt("code"));
                                district.setDistName(object.getString("distName"));
                                LatLng latLng = new LatLng();
                                latLng.setLatitude(Utils.converMX2DoubleCoordinate(object.getInt("lat")));
                                latLng.setLongitude(Utils.converMX2DoubleCoordinate(object.getInt("lon")));
                                district.setLatLng(latLng);
                                district.setDistType(object.getInt("distType"));
                                districtList.add(district);
                            }
                            mOnGetDistrictsByRangeListener.OnGetDistrictsByRangeSuccess(requestHandleId, districtList);

                        } else {
                            mOnGetDistrictsByRangeListener.onFail(respCode,"出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetDistrictsByRangeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETDISTRICTSBYROAD:
                if (mOnGetDistrictsByRoadListener != null) {
                    if(hasDSPTError){
                        mOnGetDistrictsByRoadListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    try {
                        JSONTokener jsonParser = new JSONTokener(response);
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONArray districtObj = jsonObject.getJSONArray("listDistrict");
                            List<DistrictInfo> districtList = new ArrayList<DistrictInfo>();
                            for(int i = 0; i<districtObj.length(); i++) {
                                JSONObject object = districtObj.getJSONObject(i);
                                DistrictInfo districtInfo = new DistrictInfo();
                                districtInfo.setCode(object.getInt("code"));
                                districtInfo.setCityName(object.getString("cityName"));
                                LatLng latLng = new LatLng();
                                latLng.setLatitude(Utils.converMX2DoubleCoordinate(object.getInt("lat")));
                                latLng.setLongitude(Utils.converMX2DoubleCoordinate(object.getInt("lon")));
                                districtInfo.setLatLng(latLng);
                                districtList.add(districtInfo);
                            }
                            mOnGetDistrictsByRoadListener.OnGetDistrictsByRoadSuccess(districtList);

                        } else {
                            mOnGetDistrictsByRoadListener.onFail(respCode,"出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetDistrictsByRoadListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }

    public void downloadAdminCode(String url,final String filePath,final String version, final OnGetLastestDistrictListener mOnGetLastestDistrictListener){
        final String URL = "http://"+ url;
            HttpRequestUtil.doGet(URL, null, new AsyncHttpResponseHandler() {
                @Override
                public void onSuccess(int i,  byte[] bytes) {
                    try{
                        File file = new File(filePath);
                        if (!file.getParentFile().exists()) {
                            file.getParentFile().mkdirs();
                        }
                        file.createNewFile();
                        FileOutputStream in = new FileOutputStream(file);
                        in.write(bytes);
                        in.close();
                        if (mOnGetLastestDistrictListener!=null){
                            mOnGetLastestDistrictListener.onGetLastestDistrictSuccess(true, version);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }

                @Override
                public void onFailure(int i, byte[] bytes, Throwable throwable) {
                    if(mOnGetLastestDistrictListener!=null){
                        mOnGetLastestDistrictListener.onFail(Constants.ERROR_CODE_GETADMINCODE_FAIL,"请求失败");
                    }
                }
            }, true);

    }

}
