package com.mxnavi.tspv2.remotecontrol;

import com.google.gson.Gson;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAirBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAirMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAirTemperatureBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAirTemperatureMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAllStateBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlAllStateMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlChairBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlChairMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlChargeBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlChargeMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlCommonParamBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlCommonParamTboxBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlDiagnosisBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlDiagnosisBean.DiagnosisType;
import com.mxnavi.tspv2.remotecontrol.message.CtrlDiagnosisMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlDoorsBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlDoorsMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlEkeyBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlEkeyMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlElectromotorBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlElectromotorMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlEngineBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlEngineMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlLightHornBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlLightHornMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlPrivateInformationBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlPrivateInformationMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlTopWindowsBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlTopWindowsMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlTrunkDoorsBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlTrunkDoorsMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlWakeupMobileBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlWindowsBean;
import com.mxnavi.tspv2.remotecontrol.message.CtrlWindowsMobileBean;
import com.mxnavi.tspv2.remotecontrol.model.CarState;
import com.mxnavi.tspv2.remotecontrol.model.CtrlAirMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlAirTemperatureMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlAirTemperatureMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlChairMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlChairMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlChargeMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlCommonParamMobile;
import com.mxnavi.tspv2.remotecontrol.model.CtrlAirMessage;
import com.mxnavi.tspv2.remotecontrol.model.CtrlChargeMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlCommonParamTbox;
import com.mxnavi.tspv2.remotecontrol.model.CtrlDoorsMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlEkeyMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlElectromotorMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlElectromotorMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlEngineMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlLightHornMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlPrivateInformationMessage;
import com.mxnavi.tspv2.remotecontrol.model.CtrlDoorsMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlEkeyMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlEngineMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlLightHornMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlPrivateInformationMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlTopWindowsMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlTopWindowsMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlTrunkDoorsMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlTrunkDoorsMsg;
import com.mxnavi.tspv2.remotecontrol.model.CtrlWindowsMobileMsg;
import com.mxnavi.tspv2.remotecontrol.model.ReqBody;
import com.mxnavi.tspv2.remotecontrol.model.ReqHeader;
import com.mxnavi.tspv2.remotecontrol.model.CtrlWindowsMsg;
import com.mxnavi.tspv2.remotecontrol.model.WindowsCtrlInfo;
import com.mxnavi.tspv2.remotecontrol.model.WindowsStateInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2017/9/4.
 */

public class RemoteControlService extends BaseTsp {
    // 请求服务类型为远程控制
    final int thisServiceType = Config.MxNGTPServiceType.REMOTE_CONTROL.getServiceType();
    //  用户管理的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.REMOTE_CONTROL.getServiceVer();
    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int SEND_TO_CAR = 1;//发送指令到车机
    final int SEND_STATE_TO_APP = 2;//发送指令到车机

    final int CTRL_WINDOWS = 3;//远程控制车窗
    final int CTRL_WINDOWS_GET = 4;//获取车窗状态
    final int CTRL_LIGHTHORN = 5;// 寻车控制
    final int CTRL_LIGHTHORN_GET = 6;//
    final int CTRL_ENGINE = 7;//
    final int CTRL_ENGINE_GET = 8;//
    final int CTRL_Electromotor = 9;//
    final int CTRL_Electromotor_GET = 10;//
    final int CTRL_Ekey = 11;
    final int CTRL_Ekey_GET = 12;
    final int CTRL_Doors = 13;
    final int CTRL_Doors_GET = 14;
    final int CTRL_Ac = 15;
    final int CTRL_Ac_GET = 16;
    final int CTRL_Chair = 17;
    final int CTRL_Chair_GET = 18;
    final int CTRL_Charge = 19;
    final int CTRL_Charge_GET = 20;
    final int CTRL_AllState_GET = 21;
    final int CTRL_WAKEUPMOBILE = 22;
    final int CTRL_DIAGNOSIS = 23;
    final int CTRL_PRIVATEINFORMATION_GET = 24;
    final int CTRL_PRIVATEINFORMATION = 25;
    final int CTRL_TOPWINDOWS = 26; // 天窗控制
    final int CTRL_TOPWINDOWS_GET = 27; // 天窗控制状态
    final int CTRL_AIRTEMPERATURE = 28; // 远程空调温度设定
    final int CTRL_AIRTEMPERATURE_GET = 29; // 远程空调温度设定状态
    final int CTRL_TRUNKDOORS = 30;  // 后备箱控制
    final int CTRL_TRUNKDOORS_GET = 31; // 后备箱控制状态


    public interface SendCommandToCarListener extends CallBackListener {
        public void onSendCommandToCarSuccess();
    }

    public interface SendStateToAppListener extends CallBackListener {
        public void onSendStateToAppSuccess();
    }

    public interface SendCtrlCarWindowsCommandListener extends CallBackListener {
        /**
         * 发送控制车窗指令成功
         */
        public void onSendCtrlCarWindowsCommandSuccess();

    }

    public interface SendGetCarWindowsStateCommandListener extends CallBackListener {
        /**
         * 发送获取车窗状态指令成功
         */
        public void onSendGetCarWindowsStateCommandSuccess();
    }

    public interface SendCtrlLightHornCommandListener extends CallBackListener {
        /**
         * 发送控制寻车灯指令成功
         */
        public void onSendCtrlLightHornCommandSuccess();
    }

    public interface SendGetCarLightHornStateCommandListener extends CallBackListener {
        /**
         * 发送获取寻车灯指令成功
         */
        public void onSendGetCarLightHornStateCommandSuccess();
    }

    public interface SendCtrlEngineCommandListener extends CallBackListener {
        /**
         * 发送控制发动机指令成功
         */
        public void onSendCtrlEngineCommandSuccess();
    }

    public interface SendGetCarEngineStateCommandListener extends CallBackListener {
        /**
         * 发送获取发动机状态指令成功
         */
        public void onSendGetCarEngineStateCommandSuccess();
    }

    public interface SendCtrlElectromotorCommandListener extends CallBackListener {
        /**
         * 发送控制驱动电机指令成功
         */
        public void onSendCtrlElectromotorCommandSuccess();
    }

    public interface SendGetCarElectromotorStateCommandListener extends CallBackListener {
        /**
         * 发送获取驱动电机状态指令成功
         */
        public void onSendGetCarElectromotorStateCommandSuccess();
    }

    public interface SendCtrlEkeyCommandListener extends CallBackListener {
        /**
         * 发送控制电子钥匙状态指令成功
         */
        public void onSendCtrlEkeyCommandSuccess();
    }

    public interface SendGetCarEkeyStateCommandListener extends CallBackListener {
        /**
         * 发送获取电子钥匙状态指令成功
         */
        public void onSendGetCarEkeyStateCommandSuccess();
    }

    public interface SendCtrlDoorsCommandListener extends CallBackListener {
        /**
         * 发送控制车门指令成功
         */
        public void onSendCtrlDoorsCommandSuccess();
    }

    public interface SendGetCarDoorsStateCommandListener extends CallBackListener {

        /**
         * 发送获取车门状态指令成功
         */
        public void onSendGetCarDoorsStateCommandSuccess();
    }

    public interface SendCtrlAcCommandListener extends CallBackListener {
        /**
         * 发送空调控制指令成功
         */
        public void onSendCtrlAcCommandSuccess();
    }

    public interface SendGetCarAcStateCommandListener extends CallBackListener {
        /**
         * 发送获取空调状态指令成功
         */
        public void onSendGetCarAcStateCommandSuccess();
    }

    public interface SendCtrlChargeCommandListener extends CallBackListener {
        /**
         * 发送控制充电指令称
         */
        public void onSendCtrlChargeCommandSuccess();
    }

    public interface SendGetCarChargeStateCommandListener extends CallBackListener {
        /**
         * 发送获取充电状态指令成功
         */
        public void onSendGetCarChargeStateCommandSuccess();
    }

    public interface SendCtrlChairCommandListener extends CallBackListener {
        /**
         * 发送座椅控制指令成功
         */
        public void onSendCtrlChairCommandSuccess();
    }

    public interface SendGetCarChairStateCommandListener extends CallBackListener {
        /**
         * 发送获取座椅状态指令成功
         */
        public void onSendGetCarChairStateCommandSuccess();
    }

    public interface SendGetAllStateCommandListener extends CallBackListener {
        /**
         * 发送获取全部状态指令成功
         */
        public void onSendGetAllStateCommandSuccess();
    }

    public interface SendCtrlWakeupMobileListener extends CallBackListener {
        /**
         * 发送获取全部状态指令成功
         */
        public void onSendCtrlWakeupMobileSuccess();
    }

    public interface SendCtrlDiagnosisListener extends CallBackListener {
        /**
         * 发送获取诊断车辆指令成功
         */
        public void onSendCtrlDiagnosisSuccess();
    }

    public interface SendGetPrivateInformationListener extends CallBackListener {
        /**
         * 发送获取远程中控保护指令成功
         */
        public void onSendGetPrivateInformationSuccess();
    }

    public interface SendCtrlPrivateInformationListener extends CallBackListener {
        /**
         * 发送远程中控保护指令成功
         */
        public void onSendCtrlPrivateInformationSuccess();
    }

    public interface SendCtrlTopWindowsListener extends CallBackListener {
        /**
         * 发送远程天窗控制指令成功
         */
        public void onSendCtrlTopWindowsSuccess();
    }

    public interface SendGetTopWindowsStateListener extends CallBackListener {
        /**
         * 发送获取天窗控制指令成功
         */
        public void onSendGetTopWindowsStateSuccess();
    }

    public interface SendCtrlAirTemperatureListener extends CallBackListener {
        /**
         * 发送 控制指令成功
         */
        public void onSendCtrlAirTemperatureSuccess();
    }

    public interface SendGetAirTemperatureStateListener extends CallBackListener {
        /**
         * 发送获取 指令成功
         */
        public void onSendGetAirTemperatureStateSuccess();
    }

    public interface SendCtrlTrunkDoorsListener extends CallBackListener {
        /**
         * 发送远程  指令成功
         */
        public void onSendCtrlTrunkDoorsSuccess();
    }

    public interface SendGetTrunkDoorsStateListener extends CallBackListener {
        /**
         * 发送获取  指令成功
         */
        public void onSendGetTrunkDoorsStateSuccess();
    }

    SendCommandToCarListener mSendCommandToCarListener;
    SendStateToAppListener mSendStateToAppListener;
    SendCtrlCarWindowsCommandListener mSendCtrlCarWindowsCommandListener;

    SendGetCarWindowsStateCommandListener mSendGetCarWindowsStateCommandListener;
    SendCtrlLightHornCommandListener mSendCtrlLightHornCommandListener;
    SendGetCarLightHornStateCommandListener mSendGetCarLightHornStateCommandListener;
    SendCtrlEngineCommandListener mSendCtrlEngineCommandListener;
    SendGetCarEngineStateCommandListener mSendGetCarEngineStateCommandListener;

    SendCtrlElectromotorCommandListener mSendCtrlElectromotorCommandListener;
    SendGetCarElectromotorStateCommandListener mSendGetCarElectromotorStateCommandListener;
    SendCtrlEkeyCommandListener mSendCtrlEkeyCommandListener;
    SendGetCarEkeyStateCommandListener mSendGetCarEkeyStateCommandListener;
    SendCtrlDoorsCommandListener mSendCtrlDoorsCommandListener;
    SendGetCarDoorsStateCommandListener mSendGetCarDoorsStateCommandListener;
    SendCtrlAcCommandListener mSendCtrlAcCommandListener;
    SendGetCarAcStateCommandListener mSendGetCarAcStateCommandListener;
    SendCtrlChargeCommandListener mSendCtrlChargeCommandListener;
    SendGetCarChargeStateCommandListener mSendGetCarChargeStateCommandListener;
    SendCtrlChairCommandListener mSendCtrlChairCommandListener;
    SendGetCarChairStateCommandListener mSendGetCarChairStateCommandListener;
    SendGetAllStateCommandListener mSendGetAllStateCommandListener;
    SendCtrlWakeupMobileListener mSendCtrlWakeupMobileListener;

    SendCtrlDiagnosisListener mSendCtrlDiagnosisListener;
    SendCtrlPrivateInformationListener mSendCtrlPrivateInformationListener;
    SendGetPrivateInformationListener mSendGetPrivateInformationListener;
    SendCtrlTopWindowsListener mSendCtrlTopWindowsListener;
    SendGetTopWindowsStateListener mSendGetTopWindowsStateListener;
    SendCtrlAirTemperatureListener mSendCtrlAirTemperatureListener;
    SendGetAirTemperatureStateListener mSendGetAirTemperatureStateListener;
    SendCtrlTrunkDoorsListener mSendCtrlTrunkDoorsListener;
    SendGetTrunkDoorsStateListener mSendGetTrunkDoorsStateListener;

    public void setSendCommandToCarListener(SendCommandToCarListener sendCommandToCarListener) {
        mSendCommandToCarListener = sendCommandToCarListener;
    }

    public void setSendStateToAppListener(SendStateToAppListener sendStateToAppListener) {
        mSendStateToAppListener = sendStateToAppListener;
    }

    public void setSendCtrlCarWindowsCommandListener(SendCtrlCarWindowsCommandListener sendCtrlCarWindowsCommandListener) {
        mSendCtrlCarWindowsCommandListener = sendCtrlCarWindowsCommandListener;
    }

    public void setSendGetCarWindowsStateCommandListener(SendGetCarWindowsStateCommandListener sendGetCarWindowsStateCommandListener) {
        mSendGetCarWindowsStateCommandListener = sendGetCarWindowsStateCommandListener;
    }

    public void setSendCtrlLightHornCommandListener(SendCtrlLightHornCommandListener sendCtrlLightHornCommandListener) {
        mSendCtrlLightHornCommandListener = sendCtrlLightHornCommandListener;
    }

    public void setSendGetCarLightHornStateCommandListener(SendGetCarLightHornStateCommandListener sendGetCarLightHornStateCommandListener) {
        mSendGetCarLightHornStateCommandListener = sendGetCarLightHornStateCommandListener;
    }

    public void setSendCtrlEngineCommandListener(SendCtrlEngineCommandListener sendCtrlEngineCommandListener) {
        mSendCtrlEngineCommandListener = sendCtrlEngineCommandListener;
    }

    public void setSendGetCarEngineStateCommandListener(SendGetCarEngineStateCommandListener sendGetCarEngineStateCommandListener) {
        mSendGetCarEngineStateCommandListener = sendGetCarEngineStateCommandListener;
    }

    public void setSendCtrlElectromotorCommandListener(SendCtrlElectromotorCommandListener sendCtrlElectromotorCommandListener) {
        mSendCtrlElectromotorCommandListener = sendCtrlElectromotorCommandListener;
    }

    public void setSendGetCarElectromotorStateCommandListener(SendGetCarElectromotorStateCommandListener sendGetCarElectromotorStateCommandListener) {
        mSendGetCarElectromotorStateCommandListener = sendGetCarElectromotorStateCommandListener;
    }

    public void setSendCtrlEkeyCommandListener(SendCtrlEkeyCommandListener sendCtrlEkeyCommandListener) {
        mSendCtrlEkeyCommandListener = sendCtrlEkeyCommandListener;
    }

    public void setSendGetCarEkeyStateCommandListener(SendGetCarEkeyStateCommandListener sendGetCarEkeyStateCommandListener) {
        mSendGetCarEkeyStateCommandListener = sendGetCarEkeyStateCommandListener;
    }

    public void setSendCtrlDoorsCommandListener(SendCtrlDoorsCommandListener sendCtrlDoorsCommandListener) {
        mSendCtrlDoorsCommandListener = sendCtrlDoorsCommandListener;
    }

    public void setSendGetCarDoorsStateCommandListener(SendGetCarDoorsStateCommandListener sendGetCarDoorsStateCommandListener) {
        mSendGetCarDoorsStateCommandListener = sendGetCarDoorsStateCommandListener;
    }

    public void setSendCtrlAcCommandListener(SendCtrlAcCommandListener sendCtrlAcCommandListener) {
        mSendCtrlAcCommandListener = sendCtrlAcCommandListener;
    }

    public void setSendGetCarAcStateCommandListener(SendGetCarAcStateCommandListener sendGetCarAcStateCommandListener) {
        mSendGetCarAcStateCommandListener = sendGetCarAcStateCommandListener;
    }

    public void setSendCtrlChargeCommandListener(SendCtrlChargeCommandListener sendCtrlChargeCommandListener) {
        mSendCtrlChargeCommandListener = sendCtrlChargeCommandListener;
    }

    public void setSendGetCarChargeStateCommandListener(SendGetCarChargeStateCommandListener sendGetCarChargeStateCommandListener) {
        mSendGetCarChargeStateCommandListener = sendGetCarChargeStateCommandListener;
    }

    public void setSendCtrlChairCommandListener(SendCtrlChairCommandListener sendCtrlChairCommandListener) {
        mSendCtrlChairCommandListener = sendCtrlChairCommandListener;
    }

    public void setSendGetCarChairStateCommandListener(SendGetCarChairStateCommandListener sendGetCarChairStateCommandListener) {
        mSendGetCarChairStateCommandListener = sendGetCarChairStateCommandListener;
    }

    public void setSendGetAllStateCommandListener(SendGetAllStateCommandListener sendGetAllStateCommandListener) {
        mSendGetAllStateCommandListener = sendGetAllStateCommandListener;
    }

    public void setSendCtrlWakeupMobileListener(SendCtrlWakeupMobileListener sendCtrlWakeupMobileListener) {
        mSendCtrlWakeupMobileListener = sendCtrlWakeupMobileListener;
    }

    public void setSendCtrlDiagnosisListener(SendCtrlDiagnosisListener sendCtrlDiagnosisListener) {
        mSendCtrlDiagnosisListener = sendCtrlDiagnosisListener;
    }

    public void setSendCtrlPrivateInformationListener(SendCtrlPrivateInformationListener sendCtrlPrivateInformationListener) {
        mSendCtrlPrivateInformationListener = sendCtrlPrivateInformationListener;
    }

    public void setSendGetPrivateInformationListener(SendGetPrivateInformationListener sendGetPrivateInformationListener) {
        mSendGetPrivateInformationListener = sendGetPrivateInformationListener;
    }

    public void setSendCtrlTopWindowsListener(SendCtrlTopWindowsListener sendCtrlTopWindowsListener) {
        mSendCtrlTopWindowsListener = sendCtrlTopWindowsListener;
    }

    public void setSendGetTopWindowsStateListener(SendGetTopWindowsStateListener sendGetTopWindowsStateListener) {
        mSendGetTopWindowsStateListener = sendGetTopWindowsStateListener;
    }

    public void setSendCtrlAirTemperatureListener (SendCtrlAirTemperatureListener sendCtrlAirTemperatureListener) {
        mSendCtrlAirTemperatureListener = sendCtrlAirTemperatureListener;
    }

    public void setSendGetAirTemperatureStateListener(SendGetAirTemperatureStateListener sendGetAirTemperatureStateListener) {
        mSendGetAirTemperatureStateListener = sendGetAirTemperatureStateListener;
    }

    public void setSendCtrlTrunkDoorsListener (SendCtrlTrunkDoorsListener sendCtrlTrunkDoorsListener) {
        mSendCtrlTrunkDoorsListener = sendCtrlTrunkDoorsListener;
    }

    public void setSendGetTrunkDoorsStateListener (SendGetTrunkDoorsStateListener sendGetTrunkDoorsStateListener) {
        mSendGetTrunkDoorsStateListener = sendGetTrunkDoorsStateListener;
    }

    /**
     * APP发送远程控制信息到车机
     *
     * @param header 信息头
     * @param body   信息体
     * @return 请求ID
     */
    public int sendCommandToCar(String userId, String token, ReqHeader header, ReqBody body) {
        return sendCommandToCar(userId, token, header, body, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP发送远程控制信息到车机
     *
     * @param header      信息头
     * @param body        信息体
     * @param timeOutSecs 请求超时时间
     * @return
     */
    public int sendCommandToCar(String userId, String token, ReqHeader header, ReqBody body, int timeOutSecs) {
        return sendCommandToCar(userId, token, header, body, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    public int sendCommandToCar(String userId, String token, ReqHeader header, ReqBody body, int timeOutSecs, int priorityLevel) {
        if (header == null || body == null) {
            if (mSendCommandToCarListener != null) {
                mSendCommandToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        int eventId = RequestDispathUtil.createEventId();
        String url = "instruction/recv";

        if (header.getBusinessId() == null) {
            header.setBusinessId("buss");
        }
        if (header.getCarType() == null) {
            header.setCarType("car");
        }
        if (header.getCarConfigure() == null) {
            header.setCarConfigure("1");
        }
        JSONObject allObj = new JSONObject();
        try {
            Gson gson = new Gson();
            String headerStr = gson.toJson(header);


            JSONTokener headerjsonParser = new JSONTokener(headerStr);
            JSONObject headerContentJson = (JSONObject) headerjsonParser.nextValue();
            allObj.put("reqHeader", headerContentJson);

            String bodyStr = gson.toJson(body);
            JSONTokener bodyjsonParser = new JSONTokener(bodyStr);
            JSONObject bodyContentJson = (JSONObject) bodyjsonParser.nextValue();
            allObj.put("reqBody", bodyContentJson);


        } catch (JSONException e) {
            e.printStackTrace();
            if (mSendCommandToCarListener != null) {
                mSendCommandToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String requestData = url + "?json=" + allObj.toString();
        Utils.loge("sendCommandToCar=" + requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId, userId, token, requestData, thisServiceType, serviceDataVer, MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEND_TO_CAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 车辆状态通知
     *
     * @param carState    车辆状态信息
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int sendCarStateToApp(CarState carState, int timeOutSecs) {
        return sendCarStateToApp(carState, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车辆状态通知
     *
     * @param carState 车辆状态信息
     * @return 请求ID
     */
    public int sendCarStateToApp(CarState carState) {
        return sendCarStateToApp(carState, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 车辆状态通知
     *
     * @param carState      车辆状态信息
     * @param timeOutSecs
     * @param priorityLevel
     * @return 请求ID
     */
    public int sendCarStateToApp(CarState carState, int timeOutSecs, int priorityLevel) {
        if (carState == null) {
            if (mSendStateToAppListener != null) {
                mSendStateToAppListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        int eventId = RequestDispathUtil.createEventId();
        String url = "instruction/stateNotice";
        Gson gson = new Gson();

        String carStateStr = gson.toJson(carState);
        JSONTokener jsonParser = new JSONTokener(carStateStr);

        if (carState.getStateDataJson() != null) {
            JSONObject jsonObject = null;
            try {
                jsonObject = (JSONObject) jsonParser.nextValue();

                String stateDataStr = jsonObject.getString("stateDataJson");
                JSONTokener stateDataParser = new JSONTokener(stateDataStr);
                JSONObject carstatejsonObject = (JSONObject) stateDataParser.nextValue();
                jsonObject.put("stateData", carstatejsonObject);
                carStateStr = jsonObject.toString();
            } catch (Exception e) {
                e.printStackTrace();
                if (mSendStateToAppListener != null) {
                    mSendStateToAppListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                }
                return 0;
            }
        }

        String requestData = url + "?json=" + carStateStr;
        Utils.loge("sendCarStateToApp requestData=" + requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEND_STATE_TO_APP);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送车控控制指令
     *
     * @param ctrlWindowsMobileMsg
     * @param windowCtrlMsg
     * @return
     */
    public int sendCtrlCarWindowsCommand(CtrlWindowsMobileMsg ctrlWindowsMobileMsg, CtrlWindowsMsg windowCtrlMsg) {
        return sendCtrlCarWindowsCommand(ctrlWindowsMobileMsg, windowCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlCarWindowsCommand(CtrlWindowsMobileMsg ctrlWindowsMobileMsg, CtrlWindowsMsg windowCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlWindowsMobileMsg == null || Utils.strIsEmpty(ctrlWindowsMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlWindowsMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlWindowsMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlWindowsMobileMsg.getCommonParam().getUserId())
                || windowCtrlMsg == null) {
            if (mSendGetCarWindowsStateCommandListener != null) {
                mSendGetCarWindowsStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlWindowsMobileMsg.getCommonParam());

        CtrlWindowsMobileBean.CtrlWindowsMobileMessage.Builder ctrlWindowsMobile =
                CtrlWindowsMobileBean.CtrlWindowsMobileMessage.newBuilder();
        CtrlWindowsBean.CtrlWindowsMessage.Builder ctrlWindowsMessage = CtrlWindowsBean.CtrlWindowsMessage.newBuilder();
        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(windowCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(windowCtrlMsg.getCommonParam().getDelayTime());
        ctrlWindowsMessage.setCommonParam(ctrlCommonParamTbox);

        List<WindowsCtrlInfo> windowsCtrlInfoList = windowCtrlMsg.getWindowsCtrlInfo();
        //List<WindowsStateInfo> windowsStateInfoList = windowCtrlMsg.getWindowsStateInfo();
        if (windowsCtrlInfoList != null && windowsCtrlInfoList.size() > 0) {
            for (WindowsCtrlInfo windowsCtrlInfo : windowsCtrlInfoList) {
                ctrlWindowsMessage.addWindowsCtrlInfo(transformWindowInfo(windowsCtrlInfo));
            }
        }
        /*
        if (windowsStateInfoList != null && windowsStateInfoList.size() > 0) {
            for (WindowsStateInfo windowsStateInfo : windowsStateInfoList) {
                ctrlWindowsMessage.addWindowsStateInfo(transformWindowState(windowsStateInfo));
            }
        }
        */
        ctrlWindowsMobile.setWindowsMessage(ctrlWindowsMessage);
        ctrlWindowsMobile.setCommonParam(ctrlCommonParamMessage);
        if (ctrlWindowsMobileMsg.getSafePassword() != null) {
            ctrlWindowsMobile.setSafePassword(ctrlWindowsMobileMsg.getSafePassword());
        }

        byte[] bytes = ctrlWindowsMobile.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);

        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlWindows?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlWindows");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlWindowsMobileMsg.getCommonParam().getUserId(), ctrlWindowsMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_WINDOWS);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送驱动电机控制命令
     *
     * @param ctrlElectromotorMobileMsg
     * @param ctrlElectromotorMsg
     * @return
     */
    public int sendCtrlElectromotorCommand(CtrlElectromotorMobileMsg ctrlElectromotorMobileMsg, CtrlElectromotorMsg ctrlElectromotorMsg) {
        return sendCtrlElectromotorCommand(ctrlElectromotorMobileMsg, ctrlElectromotorMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlElectromotorCommand(CtrlElectromotorMobileMsg ctrlElectromotorMobileMsg, CtrlElectromotorMsg ctrlElectromotorMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlElectromotorMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlElectromotorMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlElectromotorMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlElectromotorMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlElectromotorMobileMsg.getCommonParam().getUserId())
                || ctrlElectromotorMsg == null) {
            if (mSendCtrlElectromotorCommandListener != null) {
                mSendCtrlElectromotorCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlElectromotorMobileMsg.getCommonParam());

        CtrlElectromotorBean.CtrlElectromotorMessage.Builder ctrlElectromotor =
                CtrlElectromotorBean.CtrlElectromotorMessage.newBuilder();
        CtrlElectromotorMobileBean.CtrlElectromotorMobileMessage.Builder ctrlElectromotorMobileMessage =
                CtrlElectromotorMobileBean.CtrlElectromotorMobileMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlElectromotorMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlElectromotorMsg.getCommonParam().getDelayTime());
        ctrlElectromotor.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        //ctrlCommonInfo.setResult(ctrlElectromotorMsg.getElectromotorCtrlInfo().getResult());
        ctrlCommonInfo.setState(ctrlElectromotorMsg.getElectromotorCtrlInfo().getState());
        ctrlCommonInfo.setValid(ctrlElectromotorMsg.getElectromotorCtrlInfo().getValid());
        ctrlElectromotor.setElectromotorCtrlInfo(ctrlCommonInfo);

//        CtrlElectromotorBean.ElectromotorStateInfo.Builder electromotorStateInfo = CtrlElectromotorBean.ElectromotorStateInfo.newBuilder();
//        electromotorStateInfo.setElectromotorOnOff(ctrlElectromotorMsg.getElectromotorStateInfo().getElectromotorOnOff());
//        ctrlElectromotor.setElectromotorStateInfo(electromotorStateInfo);

        ctrlElectromotorMobileMessage.setElectromotorMessage(ctrlElectromotor);
        ctrlElectromotorMobileMessage.setCommonParam(ctrlCommonParamMessage);
        if (ctrlElectromotorMobileMsg.getSafePassword() != null) {
            ctrlElectromotorMobileMessage.setSafePassword(ctrlElectromotorMobileMsg.getSafePassword());
        }

        byte[] bytes = ctrlElectromotorMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);

        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlElectromotor?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlElectromotor");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlElectromotorMobileMsg.getCommonParam().getUserId(), ctrlElectromotorMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Electromotor);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送电子钥匙控制指令
     *
     * @param ctrlEkeyMobileMsg
     * @param ekeyCtrlMsg
     * @return
     */
    public int sendCtrlEkeyCommand(CtrlEkeyMobileMsg ctrlEkeyMobileMsg, CtrlEkeyMsg ekeyCtrlMsg) {
        return sendCtrlEkeyCommand(ctrlEkeyMobileMsg, ekeyCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlEkeyCommand(CtrlEkeyMobileMsg ctrlEkeyMobileMsg, CtrlEkeyMsg ekeyCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlEkeyMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlEkeyMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlEkeyMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlEkeyMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlEkeyMobileMsg.getCommonParam().getUserId())
                || ekeyCtrlMsg == null) {
            if (mSendCtrlEkeyCommandListener != null) {
                mSendCtrlEkeyCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlEkeyMobileMsg.getCommonParam());

        CtrlEkeyBean.CtrlEkeyMessage.Builder ctrlEkey = CtrlEkeyBean.CtrlEkeyMessage.newBuilder();
        CtrlEkeyMobileBean.CtrlEkeyMobileMessage.Builder ctrlEkeyMobileMessage =
                CtrlEkeyMobileBean.CtrlEkeyMobileMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ekeyCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ekeyCtrlMsg.getCommonParam().getDelayTime());
        ctrlEkey.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(ekeyCtrlMsg.getEkeyCtrlInfo().getValid());
        ctrlCommonInfo.setState(ekeyCtrlMsg.getEkeyCtrlInfo().getState());
        //ctrlCommonInfo.setResult(ekeyCtrlMsg.getEkeyCtrlInfo().getResult());
        ctrlEkey.setEkeyCtrlInfo(ctrlCommonInfo);

/*
        CtrlEkeyBean.EkeyStateInfo.Builder ekeyStateInfo = CtrlEkeyBean.EkeyStateInfo.newBuilder();
        ekeyStateInfo.setRemoteEnable(ekeyCtrlMsg.getEkeyStateInfo().getRemoteEnable());
        ctrlEkey.setEkeyStateInfo(ekeyStateInfo);
*/

        ctrlEkeyMobileMessage.setEkeyMessage(ctrlEkey);
        ctrlEkeyMobileMessage.setCommonParam(ctrlCommonParamMessage);
        if (ctrlEkeyMobileMsg.getSafePassword() != null) {
            ctrlEkeyMobileMessage.setSafePassword(ctrlEkeyMobileMsg.getSafePassword() );
        }

        byte[] bytes = ctrlEkeyMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);

        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlEkey?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlEkey");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlEkeyMobileMsg.getCommonParam().getUserId(), ctrlEkeyMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Ekey);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送寻车控制指令
     *
     * @param ctrlLightHornMobileMsg
     * @param lightHornCtrlMsg
     * @return
     */
    public int sendCtrlLightHornCommand(CtrlLightHornMobileMsg ctrlLightHornMobileMsg, CtrlLightHornMsg lightHornCtrlMsg) {
        return sendCtrlLightHornCommand(ctrlLightHornMobileMsg, lightHornCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlLightHornCommand(CtrlLightHornMobileMsg ctrlLightHornMobileMsg, CtrlLightHornMsg lightHornCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlLightHornMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlLightHornMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlLightHornMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlLightHornMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlLightHornMobileMsg.getCommonParam().getUserId())
                || lightHornCtrlMsg == null) {
            if (mSendCtrlLightHornCommandListener != null) {
                mSendCtrlLightHornCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlLightHornMobileMsg.getCommonParam());
        CtrlLightHornBean.CtrlLightHornMessage.Builder ctrlLightHornMsg =
                CtrlLightHornBean.CtrlLightHornMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(lightHornCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(lightHornCtrlMsg.getCommonParam().getDelayTime());
        ctrlLightHornMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(lightHornCtrlMsg.getLighthornCtrlInfo().getValid());
        ctrlCommonInfo.setState(lightHornCtrlMsg.getLighthornCtrlInfo().getState());
        //ctrlCommonInfo.setResult(lightHornCtrlMsg.getLighthornCtrlInfo().getResult());
        ctrlLightHornMsg.setLighthornCtrlInfo(ctrlCommonInfo);

        CtrlLightHornMobileBean.CtrlLightHornMobileMessage.Builder lightHornCtrlMobileMsg
                = CtrlLightHornMobileBean.CtrlLightHornMobileMessage.newBuilder();
        if (lightHornCtrlMobileMsg.getSafePassword()  != null) {
            lightHornCtrlMobileMsg.setSafePassword(ctrlLightHornMobileMsg.getSafePassword() );
        }

        lightHornCtrlMobileMsg.setCommonParam(ctrlCommonParamMessage);
        lightHornCtrlMobileMsg.setLightHornMessage(ctrlLightHornMsg);

        byte[] bytes = lightHornCtrlMobileMsg.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);

        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlLightHorn?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlLightHorn");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlLightHornMobileMsg.getCommonParam().getUserId(), ctrlLightHornMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_LIGHTHORN);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;


    }

    /**
     * 发送车门控制指令
     *
     * @param ctrlDoorsMobileMsg
     * @param doorsCtrlMsg
     * @return
     */
    public int sendCtrlDoorsCommand(CtrlDoorsMobileMsg ctrlDoorsMobileMsg, CtrlDoorsMsg doorsCtrlMsg) {
        return sendCtrlDoorsCommand(ctrlDoorsMobileMsg, doorsCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlDoorsCommand(CtrlDoorsMobileMsg ctrlDoorsMobileMsg, CtrlDoorsMsg doorsCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlDoorsMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlDoorsMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlDoorsMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlDoorsMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlDoorsMobileMsg.getCommonParam().getUserId())
                || doorsCtrlMsg == null) {
            if (mSendCtrlDoorsCommandListener != null) {
                mSendCtrlDoorsCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlDoorsMobileMsg.getCommonParam());

        CtrlDoorsBean.CtrlDoorsMessage.Builder ctrlDoorMsg = CtrlDoorsBean.CtrlDoorsMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(doorsCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(doorsCtrlMsg.getCommonParam().getDelayTime());
        ctrlDoorMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(doorsCtrlMsg.getDoorsCtrlInfo().getValid());
        ctrlCommonInfo.setState(doorsCtrlMsg.getDoorsCtrlInfo().getState());
        //ctrlCommonInfo.setResult(doorsCtrlMsg.getDoorsCtrlInfo().getResult());
        ctrlDoorMsg.setDoorsCtrlInfo(ctrlCommonInfo);
/*

        CtrlDoorsBean.DoorsStateInfo.Builder doorsStateInfo = CtrlDoorsBean.DoorsStateInfo.newBuilder();
        doorsStateInfo.setDoorOnOff(doorsCtrlMsg.getDoorsStateInfo().getDoorOnOff());
        doorsStateInfo.setLockOnOff(doorsCtrlMsg.getDoorsStateInfo().getLockOnOff());
        doorsStateInfo.setDoorStateException(doorsCtrlMsg.getDoorsStateInfo().getDoorStateException());
        doorsStateInfo.setLockStateException(doorsCtrlMsg.getDoorsStateInfo().getLockStateException());
        ctrlDoorMsg.setDoorsStateInfo(doorsStateInfo);
*/

        CtrlDoorsMobileBean.CtrlDoorsMobileMessage.Builder ctrlDoorsMobileMessage = CtrlDoorsMobileBean.CtrlDoorsMobileMessage.newBuilder();
        ctrlDoorsMobileMessage.setCommonParam(ctrlCommonParamMessage);
        ctrlDoorsMobileMessage.setDoorsMessage(ctrlDoorMsg);
        if (ctrlDoorsMobileMsg.getSafePassword()  != null) {
            ctrlDoorsMobileMessage.setSafePassword(ctrlDoorsMobileMsg.getSafePassword() );
        }

        byte[] bytes = ctrlDoorsMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlDoors?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlDoors");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlDoorsMobileMsg.getCommonParam().getUserId(), ctrlDoorsMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Doors);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送充电控制指令
     *
     * @param ctrlChargeMobileMsg
     * @param chargeCtrlMsg
     * @return
     */
    public int sendCtrlChargeCommand(CtrlChargeMobileMsg ctrlChargeMobileMsg, CtrlChargeMsg chargeCtrlMsg) {
        return sendCtrlChargeCommand(ctrlChargeMobileMsg, chargeCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlChargeCommand(CtrlChargeMobileMsg ctrlChargeMobileMsg, CtrlChargeMsg chargeCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlChargeMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlChargeMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlChargeMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlChargeMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlChargeMobileMsg.getCommonParam().getUserId())
                || chargeCtrlMsg == null) {
            if (mSendCtrlChargeCommandListener != null) {
                mSendCtrlChargeCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlChargeMobileMsg.getCommonParam());

        CtrlChargeBean.CtrlChargeMessage.Builder ctrlChargeMsg = CtrlChargeBean.CtrlChargeMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(chargeCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(chargeCtrlMsg.getCommonParam().getDelayTime());
        ctrlChargeMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(chargeCtrlMsg.getChargeCtrlInfo().getValid());
        ctrlCommonInfo.setState(chargeCtrlMsg.getChargeCtrlInfo().getState());
        //ctrlCommonInfo.setResult(chargeCtrlMsg.getChargeCtrlInfo().getResult());
        ctrlChargeMsg.setChargeCtrlInfo(ctrlCommonInfo);

/*
        CtrlChargeBean.ChargeStateInfo.Builder chargeStateInfo = CtrlChargeBean.ChargeStateInfo.newBuilder();
        chargeStateInfo.setChargeOnOff(chargeCtrlMsg.getChargeStateInfo().getChargeOnOff());
        ctrlChargeMsg.setChargeStateInfo(chargeStateInfo);
*/

        CtrlChargeMobileBean.CtrlChargeMobileMessage.Builder ctrlChargeMobileMessage = CtrlChargeMobileBean.CtrlChargeMobileMessage.newBuilder();
        ctrlChargeMobileMessage.setCommonParam(ctrlCommonParamMessage);
        ctrlChargeMobileMessage.setChargeMessage(ctrlChargeMsg);

        if (ctrlChargeMobileMsg.getSafePassword()  != null) {
            ctrlChargeMobileMessage.setSafePassword(ctrlChargeMobileMsg.getSafePassword() );
        }

        byte[] bytes = ctrlChargeMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlCharge?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlCharge");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlChargeMobileMsg.getCommonParam().getUserId(), ctrlChargeMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Charge);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送座椅控制指令
     *
     * @param ctrlChairMobileMsg
     * @param chairCtrlMsg
     * @return
     */
    public int sendCtrlChairCommand(CtrlChairMobileMsg ctrlChairMobileMsg, CtrlChairMsg chairCtrlMsg) {
        return sendCtrlChairCommand(ctrlChairMobileMsg, chairCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);

    }

    public int sendCtrlChairCommand(CtrlChairMobileMsg ctrlChairMobileMsg, CtrlChairMsg chairCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlChairMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlChairMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlChairMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlChairMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlChairMobileMsg.getCommonParam().getUserId())
                || chairCtrlMsg == null) {
            if (mSendCtrlChairCommandListener != null) {
                mSendCtrlChairCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlChairMobileMsg.getCommonParam());

        CtrlChairBean.CtrlChairMessage.Builder ctrlChairMsg = CtrlChairBean.CtrlChairMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(chairCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(chairCtrlMsg.getCommonParam().getDelayTime());
        ctrlChairMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo  = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(chairCtrlMsg.getChairCtrlInfo().getValid());
        ctrlCommonInfo.setState(chairCtrlMsg.getChairCtrlInfo().getState());
        //ctrlCommonInfo.setResult(chairCtrlMsg.getChairCtrlInfo().getResult());
        ctrlChairMsg.setChairCtrlInfo(ctrlCommonInfo);

        CtrlChairMobileBean.CtrlChairMobileMessage.Builder ctrlChairMobileMessage = CtrlChairMobileBean.CtrlChairMobileMessage.newBuilder();
        ctrlChairMobileMessage.setCommonParam(ctrlCommonParamMessage);
        ctrlChairMobileMessage.setChairMessage(ctrlChairMsg);
        if (ctrlChairMobileMsg.getSafePassword()  != null) {
            ctrlChairMobileMessage.setSafePassword(ctrlChairMobileMsg.getSafePassword() );
        }
        
        byte[] bytes = ctrlChairMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlChair?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlChair");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlChairMobileMsg.getCommonParam().getUserId(), ctrlChairMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Chair);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送空调控制指令
     *
     * @param ctrlAirMobileMsg
     * @param ctrlAirMessage
     * @return
     */
    public int sendCtrlAcCommand(CtrlAirMobileMsg ctrlAirMobileMsg, CtrlAirMessage ctrlAirMessage) {
        return sendCtrlAcCommand(ctrlAirMobileMsg, ctrlAirMessage, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlAcCommand(CtrlAirMobileMsg ctrlAirMobileMsg, CtrlAirMessage ctrlAirMessage, int timeOutSecs, int priorityLevel) {
        if (ctrlAirMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlAirMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlAirMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlAirMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlAirMobileMsg.getCommonParam().getUserId())
                || ctrlAirMessage == null) {
            if (mSendCtrlAcCommandListener != null) {
                mSendCtrlAcCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlAirMobileMsg.getCommonParam());

        CtrlAirBean.CtrlAirMessage.Builder ctrlAirMsg = CtrlAirBean.CtrlAirMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlAirMessage.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlAirMessage.getCommonParam().getDelayTime());
        ctrlAirMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo  = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(ctrlAirMessage.getAirCtrlInfo().getValid());
        ctrlCommonInfo.setState(ctrlAirMessage.getAirCtrlInfo().getState());
        //ctrlCommonInfo.setResult(ctrlAirMessage.getAirCtrlInfo().getResult());
        ctrlAirMsg.setAirCtrlInfo(ctrlCommonInfo);

/*
        CtrlAirBean.AirStateInfo.Builder airStateInfo = CtrlAirBean.AirStateInfo.newBuilder();
        airStateInfo.setAc(ctrlAirMessage.getAirStateInfo().getAc());
        airStateInfo.setAfterDefrosting(ctrlAirMessage.getAirStateInfo().getAfterDefrosting());
        airStateInfo.setBlowerMode(ctrlAirMessage.getAirStateInfo().getBlowerMode());
        airStateInfo.setCycleMode(ctrlAirMessage.getAirStateInfo().getCycleMode());
        airStateInfo.setPreDefrosting(ctrlAirMessage.getAirStateInfo().getPreDefrosting());
        airStateInfo.setPtc(ctrlAirMessage.getAirStateInfo().getPtc());
        airStateInfo.setWindLevel(ctrlAirMessage.getAirStateInfo().getWindLevel());
        ctrlAirMsg.setAirStateInfo(airStateInfo);
*/

        CtrlAirMobileBean.CtrlAirMobileMessage.Builder ctrlAcMobileMessage = CtrlAirMobileBean.CtrlAirMobileMessage.newBuilder();
        ctrlAcMobileMessage.setCommonParam(ctrlCommonParamMessage);
        if (ctrlAirMobileMsg.getSafePassword()  != null) {
            ctrlAcMobileMessage.setSafePassword(ctrlAirMobileMsg.getSafePassword() );
        }

        ctrlAcMobileMessage.setAirMessage(ctrlAirMsg);

        byte[] bytes = ctrlAcMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlAir?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlAir");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlAirMobileMsg.getCommonParam().getUserId(), ctrlAirMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_Ac);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送引擎控制指令
     *
     * @param ctrlEngineMobileMsg
     * @param engineCtrlMsg
     * @return
     */
    public int sendCtrlEngineCommand(CtrlEngineMobileMsg ctrlEngineMobileMsg, CtrlEngineMsg engineCtrlMsg) {
        return sendCtrlEngineCommand(ctrlEngineMobileMsg, engineCtrlMsg, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送控制引擎指令
     *
     * @param ctrlEngineMobileMsg
     * @param engineCtrlMsg
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int sendCtrlEngineCommand(CtrlEngineMobileMsg ctrlEngineMobileMsg, CtrlEngineMsg engineCtrlMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlEngineMobileMsg.getCommonParam() == null || Utils.strIsEmpty(ctrlEngineMobileMsg.getCommonParam().getDeviceNo())
                || Utils.strIsEmpty(ctrlEngineMobileMsg.getCommonParam().getProductId())
                || Utils.strIsEmpty(ctrlEngineMobileMsg.getCommonParam().getToken()) || Utils.strIsEmpty(ctrlEngineMobileMsg.getCommonParam().getUserId())
                || engineCtrlMsg == null) {
            if (mSendCtrlEngineCommandListener != null) {
                mSendCtrlEngineCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlEngineMobileMsg.getCommonParam());
        CtrlEngineBean.CtrlEngineMessage.Builder ctrlEngineMsg = CtrlEngineBean.CtrlEngineMessage.newBuilder();

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(engineCtrlMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(engineCtrlMsg.getCommonParam().getDelayTime());
        ctrlEngineMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo  = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(engineCtrlMsg.getEngineCtrlInfo().getValid());
        ctrlCommonInfo.setState(engineCtrlMsg.getEngineCtrlInfo().getState());
        //ctrlCommonInfo.setResult(engineCtrlMsg.getEngineCtrlInfo().getResult());
        ctrlEngineMsg.setEngineCtrlInfo(ctrlCommonInfo);
/*

        CtrlEngineBean.EngineStateInfo.Builder engineStateInfo = CtrlEngineBean.EngineStateInfo.newBuilder();
        engineStateInfo.setEngineStartState(engineCtrlMsg.getEngineStateInfo().getEngineStartState());
        ctrlEngineMsg.setEngineStateInfo(engineStateInfo);
*/

        CtrlEngineMobileBean.CtrlEngineMobileMessage.Builder ctrlEngineMobileMessage = CtrlEngineMobileBean.CtrlEngineMobileMessage.newBuilder();
        ctrlEngineMobileMessage.setCommonParam(ctrlCommonParamMessage);
        ctrlEngineMobileMessage.setEngineMessage(ctrlEngineMsg);
        if (ctrlEngineMobileMsg.getSafePassword()  != null) {
            ctrlEngineMobileMessage.setSafePassword(ctrlEngineMobileMsg.getSafePassword() );
        }

        byte[] bytes = ctrlEngineMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlEngine?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlEngine");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlEngineMobileMsg.getCommonParam().getUserId(), ctrlEngineMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_ENGINE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

   /* */
    public int sendCtrlWakeupMobile(CtrlCommonParamMobile ctrlCommonParamMobile) {
        return sendCtrlWakeupMobile(ctrlCommonParamMobile, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlWakeupMobile(CtrlCommonParamMobile ctrlCommonParamMobile, int timeOutSecs, int priorityLevel) {
        if (ctrlCommonParamMobile == null
                || Utils.strIsEmpty(ctrlCommonParamMobile.getDeviceNo())
                || Utils.strIsEmpty(ctrlCommonParamMobile.getProductId())
                || Utils.strIsEmpty(ctrlCommonParamMobile.getToken())
                || Utils.strIsEmpty(ctrlCommonParamMobile.getUserId())) {
            if (mSendCtrlWakeupMobileListener != null) {
                mSendCtrlWakeupMobileListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlCommonParamMobile);

        CtrlWakeupMobileBean.CtrlWakeupMobileMessage.Builder ctrlWakeupMobile =
                CtrlWakeupMobileBean.CtrlWakeupMobileMessage.newBuilder();

        ctrlWakeupMobile.setCommonParam(ctrlCommonParamMessage);

        byte[] bytes = ctrlWakeupMobile.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "mx_remote_control/instruction/v2/ctrlWakeup?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: ctrlWakeup");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlCommonParamMobile.getUserId(), ctrlCommonParamMobile.getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_WAKEUPMOBILE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }
    /* */
    public int sendCtrlDiagnosis(CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox ) {
        return sendCtrlDiagnosis(ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlDiagnosis(CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        if (ctrlCommonParamMobile == null) {
            if (mSendCtrlDiagnosisListener != null) {
                mSendCtrlDiagnosisListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlCommonParamMobile);

        CtrlDiagnosisMobileBean.CtrlDiagnosisMobileMessage.Builder ctrlDiagnosisMobileMsg = CtrlDiagnosisMobileBean.CtrlDiagnosisMobileMessage.newBuilder();
        CtrlDiagnosisBean.CtrlDiagnosisMessage.Builder ctrlDiagnosisMessage = CtrlDiagnosisBean.CtrlDiagnosisMessage.newBuilder();
        CtrlDiagnosisBean.DiagnosisTypeInfo.Builder diagnosisTypeInfo = CtrlDiagnosisBean.DiagnosisTypeInfo.newBuilder();

//        ctrlDiagnosisMobileMsg.setSafePassword(safePassword);
        ctrlDiagnosisMobileMsg.setCommonParam(ctrlCommonParamMessage);

//        Long requestId =System.currentTimeMillis();
        ctrlDiagnosisMessage.setRequestId(ctrlCommonParamTbox.getRequestId());

        diagnosisTypeInfo.setDiagnosisType(DiagnosisType.DIAG_TYPE_ALL);
        ctrlDiagnosisMessage.addDiagnosisTypeInfo(diagnosisTypeInfo);

        ctrlDiagnosisMobileMsg.setDagnosisMessage(ctrlDiagnosisMessage);


        byte[] bytes = ctrlDiagnosisMobileMsg.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/queryDiagnosis?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: queryDiagnosis");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlCommonParamMobile.getUserId(), ctrlCommonParamMobile.getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_DIAGNOSIS);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }
    /* */

    public int sendCtrlPrivateInformation(CtrlPrivateInformationMobileMsg ctrlPrivateInformationMobileMsg, CtrlPrivateInformationMessage ctrlPrivateInformationMessage) {
        return sendCtrlPrivateInformation(ctrlPrivateInformationMobileMsg, ctrlPrivateInformationMessage, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendCtrlPrivateInformation(CtrlPrivateInformationMobileMsg ctrlPrivateInformationMobileMsg, CtrlPrivateInformationMessage ctrlPrivateInformationMessage, int timeOutSecs, int priorityLevel) {

        if (ctrlPrivateInformationMobileMsg.getCommonParam() == null
                || ctrlPrivateInformationMessage.getPrivateInfoStateInfo().getLockState() == null) {
            if (mSendCtrlPrivateInformationListener != null) {
                mSendCtrlPrivateInformationListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlPrivateInformationMobileMsg.getCommonParam());

        CtrlPrivateInformationBean.CtrlPrivateInformationMessage.Builder ctrlPrivateInformationMsg = CtrlPrivateInformationBean.CtrlPrivateInformationMessage.newBuilder();
        CtrlPrivateInformationMobileBean.CtrlPrivateInformationMobileMessage.Builder privateCtrlInformationMobileMsg = CtrlPrivateInformationMobileBean.CtrlPrivateInformationMobileMessage.newBuilder();

        if ( ctrlPrivateInformationMobileMsg.getSafePassword()!= null) {
            privateCtrlInformationMobileMsg.setSafePassword(ctrlPrivateInformationMobileMsg.getSafePassword());
        }
        privateCtrlInformationMobileMsg.setCommonParam(ctrlCommonParamMessage);

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlPrivateInformationMessage.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlPrivateInformationMessage.getCommonParam().getDelayTime());
        ctrlPrivateInformationMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(ctrlPrivateInformationMessage.getPrivateInfoCtrlInfo().getValid());
        ctrlCommonInfo.setState(ctrlPrivateInformationMessage.getPrivateInfoCtrlInfo().getState());
        //ctrlCommonInfo.setResult(ctrlPrivateInformationMessage.getPrivateInfoCtrlInfo().getResult());
        ctrlPrivateInformationMsg.setPrivateInfoCtrlInfo(ctrlCommonInfo);

/*
        CtrlPrivateInformationBean.PrivateInfoStateInfo.Builder privateInfoStateInfo = CtrlPrivateInformationBean.PrivateInfoStateInfo.newBuilder();
        privateInfoStateInfo.setLockState(ctrlPrivateInformationMessage.getPrivateInfoStateInfo().getLockState());
        ctrlPrivateInformationMsg.setPrivateInfoStateInfo(privateInfoStateInfo);
*/


        privateCtrlInformationMobileMsg.setPrivateInformationMessage(ctrlPrivateInformationMsg);

        byte[] bytes = privateCtrlInformationMobileMsg.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlPrivateInformation?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: queryDiagnosis");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlPrivateInformationMobileMsg.getCommonParam().getUserId(), ctrlPrivateInformationMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_PRIVATEINFORMATION);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 天窗控制消息
     * @param ctrlTopWindowsMobileMsg
     * @param ctrlTopWindowsMsg
     * @return
     */
    public int sendCtrlTopWindows(CtrlTopWindowsMobileMsg ctrlTopWindowsMobileMsg, CtrlTopWindowsMsg ctrlTopWindowsMsg) {
        return sendCtrlTopWindows(ctrlTopWindowsMobileMsg,ctrlTopWindowsMsg,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY );
    }

    /**
     * 天窗控制消息
     * @param ctrlTopWindowsMobileMsg
     * @param ctrlTopWindowsMsg
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int sendCtrlTopWindows(CtrlTopWindowsMobileMsg ctrlTopWindowsMobileMsg, CtrlTopWindowsMsg ctrlTopWindowsMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlTopWindowsMobileMsg.getCommonParam() == null
                || ctrlTopWindowsMobileMsg.getCommonParam().getDeviceNo() == null
                || ctrlTopWindowsMobileMsg.getCommonParam().getUserId() == null
                || ctrlTopWindowsMobileMsg.getCommonParam().getProductId() == null
                || ctrlTopWindowsMobileMsg.getCommonParam().getToken() == null ) {
            if (mSendCtrlTopWindowsListener != null) {
                mSendCtrlTopWindowsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlTopWindowsMobileMsg.getCommonParam());

        CtrlTopWindowsBean.CtrlTopWindowsMessage.Builder topWindowsCtrlMsg = CtrlTopWindowsBean.CtrlTopWindowsMessage.newBuilder();
        CtrlTopWindowsMobileBean.CtrlTopWindowsMobileMessage.Builder ctrlTopWindowsMobileMessage = CtrlTopWindowsMobileBean.CtrlTopWindowsMobileMessage.newBuilder();

        if ( ctrlTopWindowsMobileMsg.getSafePassword()!= null) {
            ctrlTopWindowsMobileMessage.setSafePassword(ctrlTopWindowsMobileMsg.getSafePassword());
        }
        ctrlTopWindowsMobileMessage.setCommonParam(ctrlCommonParamMessage);

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlTopWindowsMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlTopWindowsMsg.getCommonParam().getDelayTime());
        topWindowsCtrlMsg.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(ctrlTopWindowsMsg.getTopWindowsCtrlInfo().getValid());
        ctrlCommonInfo.setState(ctrlTopWindowsMsg.getTopWindowsCtrlInfo().getState());
        //ctrlCommonInfo.setResult(ctrlTopWindowsMsg.getTopWindowsCtrlInfo().getResult());
        topWindowsCtrlMsg.setTopWindowsCtrlInfo(ctrlCommonInfo);

/*
        CtrlTopWindowsBean.TopWindowsStateInfo.Builder  topWindowsStateInfo = CtrlTopWindowsBean.TopWindowsStateInfo.newBuilder();
        topWindowsStateInfo.setSunroofCurtain(ctrlTopWindowsMsg.getTopWindowsStateInfo().getSunroofCurtain());
        topWindowsStateInfo.setSunroofGlass(ctrlTopWindowsMsg.getTopWindowsStateInfo().getSunroofGlass());
        topWindowsStateInfo.setSunroofPossition(ctrlTopWindowsMsg.getTopWindowsStateInfo().getSunroofPossition());
        topWindowsCtrlMsg.setTopWindowsStateInfo(topWindowsStateInfo);
*/

        ctrlTopWindowsMobileMessage.setTopWindowsMessage(topWindowsCtrlMsg);

        byte[] bytes = ctrlTopWindowsMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlTopWindows?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: queryDiagnosis");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlTopWindowsMobileMsg.getCommonParam().getUserId(), ctrlTopWindowsMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_TOPWINDOWS);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 远程空调温度设定消息
     * @param ctrlAirTemperatureMobileMsg
     * @param ctrlAirTemperatureMsg
     * @return
     */
    public int sendCtrlAirTemperature(CtrlAirTemperatureMobileMsg ctrlAirTemperatureMobileMsg, CtrlAirTemperatureMsg ctrlAirTemperatureMsg) {
        return sendCtrlAirTemperature(ctrlAirTemperatureMobileMsg,ctrlAirTemperatureMsg,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY );
    }

    /**
     * 远程空调温度设定消息
     * @param ctrlAirTemperatureMobileMsg
     * @param ctrlAirTemperatureMsg
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int sendCtrlAirTemperature(CtrlAirTemperatureMobileMsg ctrlAirTemperatureMobileMsg, CtrlAirTemperatureMsg ctrlAirTemperatureMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlAirTemperatureMobileMsg.getCommonParam() == null
                || ctrlAirTemperatureMobileMsg.getCommonParam().getUserId() == null
                || ctrlAirTemperatureMobileMsg.getCommonParam().getDeviceNo() == null
                || ctrlAirTemperatureMobileMsg.getCommonParam().getUserId() == null
                || ctrlAirTemperatureMobileMsg.getCommonParam().getToken() == null ){
            if (mSendCtrlAirTemperatureListener != null ) {
                mSendCtrlAirTemperatureListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT );
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlAirTemperatureMobileMsg.getCommonParam());
        CtrlAirTemperatureBean.CtrlAirTemperatureMessage.Builder ctrlAirTemperatureMessage = CtrlAirTemperatureBean.CtrlAirTemperatureMessage.newBuilder();
        CtrlAirTemperatureMobileBean.CtrlAirTemperatureMobileMessage.Builder ctrlAirTemperatureMobileMessage =
                CtrlAirTemperatureMobileBean.CtrlAirTemperatureMobileMessage.newBuilder();
        if (ctrlAirTemperatureMobileMsg.getSafePassword() != null ){
            ctrlAirTemperatureMobileMessage.setSafePassword(ctrlAirTemperatureMobileMsg.getSafePassword());
        }
        ctrlAirTemperatureMobileMessage.setCommonParam(ctrlCommonParamMessage);

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlAirTemperatureMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlAirTemperatureMsg.getCommonParam().getDelayTime());
        ctrlAirTemperatureMessage.setCommonParam(ctrlCommonParamTbox);

        CtrlAirTemperatureBean.AirTemperatureCtrlInfo.Builder airTemperatureCtrlInfo = CtrlAirTemperatureBean.AirTemperatureCtrlInfo.newBuilder();
        // airTemperatureCtrlInfo.setResult(ctrlAirTemperatureMsg.getAirTemperatureCtrlInfo().getResult());
        airTemperatureCtrlInfo.setSetTemperature(ctrlAirTemperatureMsg.getAirTemperatureCtrlInfo().getSetTemperature());
        ctrlAirTemperatureMessage.setAirTemperatureCtrlInfo(airTemperatureCtrlInfo);
/*

        CtrlAirTemperatureBean.AirTemperatureStateInfo.Builder airTemperatureStateInfo = CtrlAirTemperatureBean.AirTemperatureStateInfo.newBuilder();
        airTemperatureStateInfo.setRealTemperature(ctrlAirTemperatureMsg.getAirTemperatureStateInfo().getRealTemperature());
        airTemperatureStateInfo.setSetTemperature(ctrlAirTemperatureMsg.getAirTemperatureStateInfo().getSetTemperature());
        ctrlAirTemperatureMessage.setAirTemperatureStateInfo(airTemperatureStateInfo);
*/

        ctrlAirTemperatureMobileMessage.setAirTemperatureMessage(ctrlAirTemperatureMessage);

        byte[] bytes = ctrlAirTemperatureMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlAirTemperature?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: queryDiagnosis");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlAirTemperatureMobileMsg.getCommonParam().getUserId(), ctrlAirTemperatureMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_AIRTEMPERATURE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 后备箱控制消息
     * @param ctrlTrunkDoorsMobileMsg
     * @param ctrlTrunkDoorsMsg
     * @return
     */
    public int sendCtrlTrunkDoors(CtrlTrunkDoorsMobileMsg ctrlTrunkDoorsMobileMsg, CtrlTrunkDoorsMsg ctrlTrunkDoorsMsg) {
        return sendCtrlTrunkDoors(ctrlTrunkDoorsMobileMsg,ctrlTrunkDoorsMsg,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY );
    }

    /**
     * 后备箱控制消息
     * @param ctrlTrunkDoorsMobileMsg
     * @param ctrlTrunkDoorsMsg
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int sendCtrlTrunkDoors(CtrlTrunkDoorsMobileMsg ctrlTrunkDoorsMobileMsg, CtrlTrunkDoorsMsg ctrlTrunkDoorsMsg, int timeOutSecs, int priorityLevel) {
        if (ctrlTrunkDoorsMobileMsg.getCommonParam() == null
                || ctrlTrunkDoorsMobileMsg.getCommonParam().getUserId() == null
                || ctrlTrunkDoorsMobileMsg.getCommonParam().getDeviceNo() == null
                || ctrlTrunkDoorsMobileMsg.getCommonParam().getUserId() == null
                || ctrlTrunkDoorsMobileMsg.getCommonParam().getToken() == null ){
            if (mSendCtrlAirTemperatureListener != null ) {
                mSendCtrlAirTemperatureListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT );
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlTrunkDoorsMobileMsg.getCommonParam());
        CtrlTrunkDoorsBean.CtrlTrunkDoorsMessage.Builder ctrlTrunkDoorsMessage = CtrlTrunkDoorsBean.CtrlTrunkDoorsMessage.newBuilder();
        CtrlTrunkDoorsMobileBean.CtrlTrunkDoorsMobileMessage.Builder ctrlTrunkDoorsMobileMessage =
                CtrlTrunkDoorsMobileBean.CtrlTrunkDoorsMobileMessage.newBuilder();
        if (ctrlTrunkDoorsMobileMsg.getSafePassword() != null ){
            ctrlTrunkDoorsMobileMessage.setSafePassword(ctrlTrunkDoorsMobileMsg.getSafePassword());
        }
        ctrlTrunkDoorsMobileMessage.setCommonParam(ctrlCommonParamMessage);

        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTbox = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParamTbox.setRequestId(ctrlTrunkDoorsMsg.getCommonParam().getRequestId());
        ctrlCommonParamTbox.setDelayTime(ctrlTrunkDoorsMsg.getCommonParam().getDelayTime());
        ctrlTrunkDoorsMessage.setCommonParam(ctrlCommonParamTbox);

        CtrlCommonParamTboxBean.CtrlCommonInfo.Builder ctrlCommonInfo = CtrlCommonParamTboxBean.CtrlCommonInfo.newBuilder();
        ctrlCommonInfo.setValid(ctrlTrunkDoorsMsg.getTrunkDoorsCtrlInfo().getValid());
        ctrlCommonInfo.setState(ctrlTrunkDoorsMsg.getTrunkDoorsCtrlInfo().getState());
        //ctrlCommonInfo.setResult(ctrlTrunkDoorsMsg.getTrunkDoorsCtrlInfo().getResult());
        ctrlTrunkDoorsMessage.setTrunkDoorsCtrlInfo(ctrlCommonInfo);

/*
        CtrlTrunkDoorsBean.TrunkDoorsStateInfo.Builder trunkDoorsStateInfo = CtrlTrunkDoorsBean.TrunkDoorsStateInfo.newBuilder();
        trunkDoorsStateInfo.setLockOnOff(ctrlTrunkDoorsMsg.getTrunkDoorsStateInfo().getLockOnOff());
        trunkDoorsStateInfo.setLockStateException(ctrlTrunkDoorsMsg.getTrunkDoorsStateInfo().getLockStateException());
        trunkDoorsStateInfo.setTrunkOnOff(ctrlTrunkDoorsMsg.getTrunkDoorsStateInfo().getTrunkOnOff());
        trunkDoorsStateInfo.setTrunkStateException(ctrlTrunkDoorsMsg.getTrunkDoorsStateInfo().getTrunkStateException());
        ctrlTrunkDoorsMessage.setTrunkDoorsStateInfo(trunkDoorsStateInfo);
*/

        ctrlTrunkDoorsMobileMessage.setTrunkDoorsMessage(ctrlTrunkDoorsMessage);

        byte[] bytes = ctrlTrunkDoorsMobileMessage.build().toByteArray();
        String bufString = Utils.encodeBase64(bytes);
        String requestData = "";
        try {
            requestData = "instruction/v2/ctrlTrunkDoors?proto=" + URLEncoder.encode(bufString, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            Utils.loge("url encode error: queryDiagnosis");
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlTrunkDoorsMobileMsg.getCommonParam().getUserId(), ctrlTrunkDoorsMobileMsg.getCommonParam().getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, CTRL_TRUNKDOORS);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }


    private int getCarCommonState(int type, String safePassword, CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        if (ctrlCommonParamMobile == null || Utils.strIsEmpty(ctrlCommonParamMobile.getDeviceNo())
                || Utils.strIsEmpty(ctrlCommonParamMobile.getProductId())
                || Utils.strIsEmpty(ctrlCommonParamMobile.getToken()) || Utils.strIsEmpty(ctrlCommonParamMobile.getUserId())
                ) {
            switch (type) {
                case CTRL_ENGINE_GET:
                    if (mSendGetCarEngineStateCommandListener != null) {
                        mSendGetCarEngineStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_WINDOWS_GET:
                    if (mSendGetCarWindowsStateCommandListener != null) {
                        mSendGetCarWindowsStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_LIGHTHORN_GET:
                    if (mSendGetCarLightHornStateCommandListener != null) {
                        mSendGetCarLightHornStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Electromotor_GET:
                    if (mSendCtrlElectromotorCommandListener != null) {
                        mSendCtrlElectromotorCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Ekey_GET:
                    if (mSendGetCarEkeyStateCommandListener != null) {
                        mSendGetCarEkeyStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Doors_GET:
                    if (mSendGetCarDoorsStateCommandListener != null) {
                        mSendGetCarDoorsStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Charge_GET:
                    if (mSendGetCarChargeStateCommandListener != null) {
                        mSendGetCarChargeStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Chair_GET:
                    if (mSendGetCarChairStateCommandListener != null) {
                        mSendGetCarChairStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_Ac_GET:
                    if (mSendGetCarAcStateCommandListener != null) {
                        mSendGetCarAcStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_AllState_GET:
                    if (mSendGetAllStateCommandListener != null) {
                        mSendGetAllStateCommandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                case CTRL_PRIVATEINFORMATION_GET:
                    if (mSendGetPrivateInformationListener != null) {
                        mSendGetPrivateInformationListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                    break;
                case CTRL_TOPWINDOWS_GET:
                    if (mSendGetTopWindowsStateListener != null) {
                        mSendGetTopWindowsStateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT );
                    }
                    break;
                case CTRL_AIRTEMPERATURE_GET:
                    if (mSendGetAirTemperatureStateListener != null ){
                        mSendGetAirTemperatureStateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT );
                    }
                    break;
                case CTRL_TRUNKDOORS_GET:
                    if (mSendGetTrunkDoorsStateListener != null ){
                        mSendGetTrunkDoorsStateListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT );
                    }
                    break;
                default:
                    break;
            }
            return 0;
        }
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParamMessage = buildCtrlCommonParamProto(ctrlCommonParamMobile);
        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParamTboxMsg = buildCtrlCommonParamProto(ctrlCommonParamTbox);
        String requestData = "";
        byte[] bytes;
        String bufString;
        switch (type) {
            case CTRL_ENGINE_GET:
                CtrlEngineBean.CtrlEngineMessage.Builder ctrlEngineMsg = CtrlEngineBean.CtrlEngineMessage.newBuilder();
                ctrlEngineMsg.setCommonParam(ctrlCommonParamTboxMsg);

                CtrlEngineMobileBean.CtrlEngineMobileMessage.Builder ctrlEngineMobileMessage = CtrlEngineMobileBean.CtrlEngineMobileMessage.newBuilder();
                ctrlEngineMobileMessage.setCommonParam(ctrlCommonParamMessage);
                ctrlEngineMobileMessage.setEngineMessage(ctrlEngineMsg);
                if (safePassword != null) {
                    ctrlEngineMobileMessage.setSafePassword(safePassword);
                }
                bytes = ctrlEngineMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlEngine?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlEngine");
                }
                break;
            case CTRL_WINDOWS_GET:
                CtrlWindowsMobileBean.CtrlWindowsMobileMessage.Builder ctrlWindowsMobile =
                        CtrlWindowsMobileBean.CtrlWindowsMobileMessage.newBuilder();
                CtrlWindowsBean.CtrlWindowsMessage.Builder ctrlWindowsMessage = CtrlWindowsBean.CtrlWindowsMessage.newBuilder();

                ctrlWindowsMessage.setCommonParam(ctrlCommonParamTboxMsg);
                ctrlWindowsMobile.setWindowsMessage(ctrlWindowsMessage);
                ctrlWindowsMobile.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlWindowsMobile.setSafePassword(safePassword);
                }
                bytes = ctrlWindowsMobile.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlWindows?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlWindows");
                }
                break;
            case CTRL_LIGHTHORN_GET:
                CtrlLightHornBean.CtrlLightHornMessage.Builder ctrlLightHornMsg =
                        CtrlLightHornBean.CtrlLightHornMessage.newBuilder();
                ctrlLightHornMsg.setCommonParam(ctrlCommonParamTboxMsg);

                CtrlLightHornMobileBean.CtrlLightHornMobileMessage.Builder ctrlLightHornMobileMsg
                        = CtrlLightHornMobileBean.CtrlLightHornMobileMessage.newBuilder();
                if (safePassword != null) {
                    ctrlLightHornMobileMsg.setSafePassword(safePassword);
                }

                ctrlLightHornMobileMsg.setCommonParam(ctrlCommonParamMessage);
                ctrlLightHornMobileMsg.setLightHornMessage(ctrlLightHornMsg);

                bytes = ctrlLightHornMobileMsg.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlLightHorn?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlLightHorn");
                }
                break;
            case CTRL_Electromotor_GET:
                CtrlElectromotorBean.CtrlElectromotorMessage.Builder ctrlElectromotor =
                        CtrlElectromotorBean.CtrlElectromotorMessage.newBuilder();
                CtrlElectromotorMobileBean.CtrlElectromotorMobileMessage.Builder ctrlElectromotorMobileMessage =
                        CtrlElectromotorMobileBean.CtrlElectromotorMobileMessage.newBuilder();

                ctrlElectromotor.setCommonParam(ctrlCommonParamTboxMsg);

                ctrlElectromotorMobileMessage.setElectromotorMessage(ctrlElectromotor);
                ctrlElectromotorMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlElectromotorMobileMessage.setSafePassword(safePassword);
                }

                bytes = ctrlElectromotorMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlElectromotor?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlElectromotor");
                }
                break;
            case CTRL_Ekey_GET:

                CtrlEkeyBean.CtrlEkeyMessage.Builder ctrlEkey =
                        CtrlEkeyBean.CtrlEkeyMessage.newBuilder();
                CtrlEkeyMobileBean.CtrlEkeyMobileMessage.Builder ctrlEkeyMobileMessage =
                        CtrlEkeyMobileBean.CtrlEkeyMobileMessage.newBuilder();

                ctrlEkey.setCommonParam(ctrlCommonParamTboxMsg);

                ctrlEkeyMobileMessage.setEkeyMessage(ctrlEkey);
                ctrlEkeyMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlEkeyMobileMessage.setSafePassword(safePassword);
                }

                bytes = ctrlEkeyMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlEkey?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlEkey");
                }
                break;
            case CTRL_Doors_GET:
                CtrlDoorsBean.CtrlDoorsMessage.Builder ctrlDoorMsg = CtrlDoorsBean.CtrlDoorsMessage.newBuilder();
                ctrlDoorMsg.setCommonParam(ctrlCommonParamTboxMsg);

                CtrlDoorsMobileBean.CtrlDoorsMobileMessage.Builder ctrlDoorsMobileMessage = CtrlDoorsMobileBean.CtrlDoorsMobileMessage.newBuilder();
                ctrlDoorsMobileMessage.setCommonParam(ctrlCommonParamMessage);
                ctrlDoorsMobileMessage.setDoorsMessage(ctrlDoorMsg);
                if (safePassword != null) {
                    ctrlDoorsMobileMessage.setSafePassword(safePassword);
                }
                bytes = ctrlDoorsMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlDoors?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlDoors");
                }
                break;
            case CTRL_Charge_GET:
                CtrlChargeBean.CtrlChargeMessage.Builder ctrlChargeMsg = CtrlChargeBean.CtrlChargeMessage.newBuilder();
                ctrlChargeMsg.setCommonParam(ctrlCommonParamTboxMsg);

                CtrlChargeMobileBean.CtrlChargeMobileMessage.Builder ctrlChargeMobileMessage = CtrlChargeMobileBean.CtrlChargeMobileMessage.newBuilder();
                ctrlChargeMobileMessage.setCommonParam(ctrlCommonParamMessage);
                ctrlChargeMobileMessage.setChargeMessage(ctrlChargeMsg);
                if (safePassword != null) {
                    ctrlChargeMobileMessage.setSafePassword(safePassword);
                }
                bytes = ctrlChargeMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlCharge?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlCharge");
                }
                break;
            case CTRL_Chair_GET:
                CtrlChairBean.CtrlChairMessage.Builder ctrlChairMsg = CtrlChairBean.CtrlChairMessage.newBuilder();
                ctrlChairMsg.setCommonParam(ctrlCommonParamTboxMsg);

                CtrlChairMobileBean.CtrlChairMobileMessage.Builder ctrlChairMobileMessage = CtrlChairMobileBean.CtrlChairMobileMessage.newBuilder();
                ctrlChairMobileMessage.setCommonParam(ctrlCommonParamMessage);
                ctrlChairMobileMessage.setChairMessage(ctrlChairMsg);
                if (safePassword != null) {
                    ctrlChairMobileMessage.setSafePassword(safePassword);
                }

                bytes = ctrlChairMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlChair?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlChair");
                }
                break;
            case CTRL_Ac_GET:
                CtrlAirBean.CtrlAirMessage.Builder ctrlAcMsg = CtrlAirBean.CtrlAirMessage.newBuilder();

                ctrlAcMsg.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlAirMobileBean.CtrlAirMobileMessage.Builder ctrlAcMobileMessage = CtrlAirMobileBean.CtrlAirMobileMessage.newBuilder();
                ctrlAcMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlAcMobileMessage.setSafePassword(safePassword);
                }

                ctrlAcMobileMessage.setAirMessage(ctrlAcMsg);

                bytes = ctrlAcMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlAir?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlAir");
                }
                break;
            case CTRL_AllState_GET:
                CtrlAllStateBean.CtrlAllStateMessage.Builder ctrlAllStateMsg = CtrlAllStateBean.CtrlAllStateMessage.newBuilder();
                ctrlAllStateMsg.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlAllStateMobileBean.CtrlAllStateMobileMessage.Builder ctrlAllStateMobileMessage
                        = CtrlAllStateMobileBean.CtrlAllStateMobileMessage.newBuilder();
                ctrlAllStateMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlAllStateMobileMessage.setSafePassword(safePassword);
                }

                ctrlAllStateMobileMessage.setAllStateMessage(ctrlAllStateMsg);
                bytes = ctrlAllStateMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlAllState?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlAllState");
                }
                break;
            case CTRL_PRIVATEINFORMATION_GET:
                CtrlPrivateInformationBean.CtrlPrivateInformationMessage.Builder ctrlPrivateInformationMsg = CtrlPrivateInformationBean.CtrlPrivateInformationMessage.newBuilder();
                ctrlPrivateInformationMsg.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlPrivateInformationMobileBean.CtrlPrivateInformationMobileMessage.Builder ctrlPrivateInformationMobileMessage
                        = CtrlPrivateInformationMobileBean.CtrlPrivateInformationMobileMessage.newBuilder();
                ctrlPrivateInformationMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlPrivateInformationMobileMessage.setSafePassword(safePassword);
                }

                ctrlPrivateInformationMobileMessage.setPrivateInformationMessage(ctrlPrivateInformationMsg);

                bytes = ctrlPrivateInformationMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlPrivateInformation?proto=" + URLEncoder.encode(bufString, "utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                    Utils.loge("url encode error: ctrlAllState");
                }
                break;
            case CTRL_TOPWINDOWS_GET:
                CtrlTopWindowsBean.CtrlTopWindowsMessage.Builder ctrlTopWindowsMessage = CtrlTopWindowsBean.CtrlTopWindowsMessage.newBuilder();
                ctrlTopWindowsMessage.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlTopWindowsMobileBean.CtrlTopWindowsMobileMessage.Builder ctrlTopWindowsMobileMessage = CtrlTopWindowsMobileBean.CtrlTopWindowsMobileMessage.newBuilder();
                ctrlTopWindowsMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null) {
                    ctrlTopWindowsMobileMessage.setSafePassword(safePassword);
                }
                ctrlTopWindowsMobileMessage.setTopWindowsMessage(ctrlTopWindowsMessage);

                bytes = ctrlTopWindowsMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlTopWindows?proto=" + URLEncoder.encode(bufString,"utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
                break;
            case CTRL_AIRTEMPERATURE_GET:
                CtrlAirTemperatureBean.CtrlAirTemperatureMessage.Builder ctrlAirTemperatureMessage = CtrlAirTemperatureBean.CtrlAirTemperatureMessage.newBuilder();
                ctrlAirTemperatureMessage.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlAirTemperatureMobileBean.CtrlAirTemperatureMobileMessage.Builder ctrlAirTemperatureMobileMessage =
                        CtrlAirTemperatureMobileBean.CtrlAirTemperatureMobileMessage.newBuilder();
                ctrlAirTemperatureMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null){
                    ctrlAirTemperatureMobileMessage.setSafePassword(safePassword);
                }
                ctrlAirTemperatureMobileMessage.setAirTemperatureMessage(ctrlAirTemperatureMessage);

                bytes = ctrlAirTemperatureMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlAirTemperature?proto=" + URLEncoder.encode(bufString,"utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
                break;
            case CTRL_TRUNKDOORS_GET:
                CtrlTrunkDoorsBean.CtrlTrunkDoorsMessage.Builder ctrlTrunkDoorsMessage = CtrlTrunkDoorsBean.CtrlTrunkDoorsMessage.newBuilder();
                ctrlTrunkDoorsMessage.setCommonParam(ctrlCommonParamTboxMsg);
                CtrlTrunkDoorsMobileBean.CtrlTrunkDoorsMobileMessage.Builder ctrlTrunkDoorsMobileMessage =
                        CtrlTrunkDoorsMobileBean.CtrlTrunkDoorsMobileMessage.newBuilder();
                ctrlTrunkDoorsMobileMessage.setCommonParam(ctrlCommonParamMessage);
                if (safePassword != null){
                    ctrlTrunkDoorsMobileMessage.setSafePassword(safePassword);
                }
                ctrlTrunkDoorsMobileMessage.setTrunkDoorsMessage(ctrlTrunkDoorsMessage);

                bytes = ctrlTrunkDoorsMobileMessage.build().toByteArray();
                bufString = Utils.encodeBase64(bytes);
                try {
                    requestData = "instruction/v2/ctrlTrunkDoors?proto=" + URLEncoder.encode(bufString,"utf-8");
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
                break;
            default:
                break;
        }
        if (Utils.strIsEmpty(requestData)) {
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                ctrlCommonParamMobile.getUserId(), ctrlCommonParamMobile.getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, 1);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, type);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;

    }

    private CtrlCommonParamBean.CtrlCommonParamMobile.Builder buildCtrlCommonParamProto(CtrlCommonParamMobile ctrlCommonParamMobile) {
        CtrlCommonParamBean.CtrlCommonParamMobile.Builder ctrlCommonParam = CtrlCommonParamBean.CtrlCommonParamMobile.newBuilder();
        ctrlCommonParam.setDeviceNo(ctrlCommonParamMobile.getDeviceNo());
        ctrlCommonParam.setProductId(ctrlCommonParamMobile.getProductId());
        ctrlCommonParam.setToken(ctrlCommonParamMobile.getToken());
        ctrlCommonParam.setUserId(ctrlCommonParamMobile.getUserId());
        return ctrlCommonParam;
    }

    private CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder buildCtrlCommonParamProto(CtrlCommonParamTbox ctrlCommonParamTbox) {
        CtrlCommonParamTboxBean.CtrlCommonParamTbox.Builder ctrlCommonParam = CtrlCommonParamTboxBean.CtrlCommonParamTbox.newBuilder();
        ctrlCommonParam.setRequestId(ctrlCommonParamTbox.getRequestId());
        ctrlCommonParam.setDelayTime(ctrlCommonParamTbox.getDelayTime());
        return ctrlCommonParam;
    }

    /**
     * 发送获取车窗状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarWindowsStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarWindowsStateCommand(safePassword,ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取寻车设置状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarLightHornStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarLightHornStateCommand(safePassword, ctrlCommonParamMobile,ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车辆引擎设置状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarEngineStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarEngineStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取驱动电机设置状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarElectromotorStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarElectromotorStateCommand(safePassword,ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车辆电子钥匙状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarEkeyStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarEkeyStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车门状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarDoorsStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarDoorsStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车辆充电状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarChargeStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarChargeStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车辆座椅状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarChairStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarChairStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取车辆空调状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetCarAcStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetCarAcStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送获取全部状态指令
     *
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetAllStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetAllStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 天窗控制
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetTopWindows(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetAllStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 远程空调温度设定
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetAirTemperature(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetAllStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY );
    }

    /**
     * 后备箱控制
     * @param safePassword
     * @param ctrlCommonParamMobile
     * @param ctrlCommonParamTbox
     * @return
     */
    public int sendGetTrunkDoors(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetAllStateCommand(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY );
    }

    /**
     * 发送获取车窗状态指令
     *
     * @param safePassword 安防密码
     * @param ctrlCommonParamMobile   通用参数
     * @param ctrlCommonParamTbox     通用参数
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int sendGetCarWindowsStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_WINDOWS_GET, safePassword,ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetPrivateInformation(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox) {
        return this.sendGetPrivateInformation(safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int sendGetCarLightHornStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_LIGHTHORN_GET, safePassword, ctrlCommonParamMobile,ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarEngineStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_ENGINE_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarElectromotorStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Electromotor_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarEkeyStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Ekey_GET, safePassword, ctrlCommonParamMobile,ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarDoorsStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Doors_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarChargeStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Charge_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarChairStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Chair_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetCarAcStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_Ac_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetAllStateCommand(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_AllState_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetPrivateInformation(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_PRIVATEINFORMATION_GET, safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetTopWindows(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_TOPWINDOWS_GET,safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetAirTemperature(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_AIRTEMPERATURE_GET,safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    public int sendGetTrunkDoors(String safePassword,CtrlCommonParamMobile ctrlCommonParamMobile, CtrlCommonParamTbox ctrlCommonParamTbox, int timeOutSecs, int priorityLevel) {
        return this.getCarCommonState(CTRL_TRUNKDOORS_GET,safePassword, ctrlCommonParamMobile, ctrlCommonParamTbox, timeOutSecs, priorityLevel);
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, RemoteControlService.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEND_TO_CAR://1
                if (mSendCommandToCarListener != null) {
                    mSendCommandToCarListener.onTimeOut();
                }
                break;
            case SEND_STATE_TO_APP:
                if (mSendStateToAppListener != null) {
                    mSendStateToAppListener.onTimeOut();
                }
                break;
            case CTRL_WINDOWS:
                if (mSendCtrlCarWindowsCommandListener != null) {
                    mSendCtrlCarWindowsCommandListener.onTimeOut();
                }
                break;
            case CTRL_WINDOWS_GET:
                if (mSendGetCarWindowsStateCommandListener != null) {
                    mSendGetCarWindowsStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_LIGHTHORN:
                if (mSendCtrlLightHornCommandListener != null) {
                    mSendCtrlLightHornCommandListener.onTimeOut();
                }
                break;
            case CTRL_LIGHTHORN_GET:
                if (mSendGetCarLightHornStateCommandListener != null) {
                    mSendGetCarLightHornStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_ENGINE:
                if (mSendCtrlEngineCommandListener != null) {
                    mSendCtrlEngineCommandListener.onTimeOut();
                }
                break;
            case CTRL_ENGINE_GET:
                if (mSendGetCarEngineStateCommandListener != null) {
                    mSendGetCarEngineStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Electromotor:
                if (mSendCtrlElectromotorCommandListener != null) {
                    mSendCtrlElectromotorCommandListener.onTimeOut();
                }
                break;
            case CTRL_Electromotor_GET:
                if (mSendGetCarElectromotorStateCommandListener != null) {
                    mSendGetCarElectromotorStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Ekey:
                if (mSendCtrlEkeyCommandListener != null) {
                    mSendCtrlEkeyCommandListener.onTimeOut();
                }
                break;
            case CTRL_Ekey_GET:
                if (mSendGetCarEkeyStateCommandListener != null) {
                    mSendGetCarEkeyStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Doors:
                if (mSendCtrlDoorsCommandListener != null) {
                    mSendCtrlDoorsCommandListener.onTimeOut();
                }
                break;
            case CTRL_Doors_GET:
                if (mSendGetCarDoorsStateCommandListener != null) {
                    mSendGetCarDoorsStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Ac:
                if (mSendCtrlAcCommandListener != null) {
                    mSendCtrlAcCommandListener.onTimeOut();
                }
                break;
            case CTRL_Ac_GET:
                if (mSendGetCarAcStateCommandListener != null) {
                    mSendGetCarAcStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Chair:
                if (mSendCtrlChairCommandListener != null) {
                    mSendCtrlChairCommandListener.onTimeOut();
                }
                break;
            case CTRL_Chair_GET:
                if (mSendGetCarChairStateCommandListener != null) {
                    mSendGetCarChairStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_Charge:
                if (mSendCtrlChargeCommandListener != null) {
                    mSendCtrlChargeCommandListener.onTimeOut();
                }
                break;
            case CTRL_Charge_GET:
                if (mSendGetCarChargeStateCommandListener != null) {
                    mSendGetCarChargeStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_AllState_GET:
                if (mSendGetAllStateCommandListener != null) {
                    mSendGetAllStateCommandListener.onTimeOut();
                }
                break;
            case CTRL_WAKEUPMOBILE:
                if (mSendCtrlWakeupMobileListener != null) {
                    mSendCtrlWakeupMobileListener.onTimeOut();
                }
                break;
            case CTRL_DIAGNOSIS:
                if (mSendCtrlDiagnosisListener != null) {
                    mSendCtrlDiagnosisListener.onTimeOut();
                }
                break;
            case CTRL_PRIVATEINFORMATION:
                if (mSendCtrlPrivateInformationListener != null) {
                    mSendCtrlPrivateInformationListener.onTimeOut();
                }
                break;
            case CTRL_PRIVATEINFORMATION_GET:
                if (mSendGetPrivateInformationListener != null) {
                    mSendGetPrivateInformationListener.onTimeOut();
                }
                break;
            case CTRL_TOPWINDOWS:
                if (mSendCtrlTopWindowsListener != null) {
                    mSendCtrlTopWindowsListener.onTimeOut();
                }
                break;
            case CTRL_TOPWINDOWS_GET:
                if (mSendGetTopWindowsStateListener != null) {
                    mSendGetTopWindowsStateListener.onTimeOut();
                }
                break;
            case CTRL_AIRTEMPERATURE:
                if (mSendCtrlAirTemperatureListener != null ) {
                    mSendCtrlAirTemperatureListener.onTimeOut();
                }
                break;
            case CTRL_AIRTEMPERATURE_GET:
                if (mSendGetAirTemperatureStateListener != null ) {
                    mSendGetAirTemperatureStateListener.onTimeOut();
                }
                break;
            case CTRL_TRUNKDOORS:
                if (mSendCtrlTrunkDoorsListener != null ){
                    mSendCtrlTrunkDoorsListener.onTimeOut();
                }
                break;
            case CTRL_TRUNKDOORS_GET:
                if (mSendGetTrunkDoorsStateListener != null ){
                    mSendGetTrunkDoorsStateListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEND_TO_CAR://1
                if (mSendCommandToCarListener != null) {
                    if (hasDSPTError) {
                        mSendCommandToCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCommandToCarListener.onSendCommandToCarSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCommandToCarListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCommandToCarListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }


                }
                break;
            case SEND_STATE_TO_APP:
                if (mSendStateToAppListener != null) {
                    if (hasDSPTError) {
                        mSendStateToAppListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendStateToAppListener.onSendStateToAppSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendStateToAppListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendStateToAppListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }


                }
                break;
            case CTRL_WINDOWS:
                if (mSendCtrlCarWindowsCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlCarWindowsCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlCarWindowsCommandListener.onSendCtrlCarWindowsCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlCarWindowsCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlCarWindowsCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_WINDOWS_GET:
                if (mSendGetCarWindowsStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarWindowsStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarWindowsStateCommandListener.onSendGetCarWindowsStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarWindowsStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarWindowsStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_LIGHTHORN:
                if (mSendCtrlLightHornCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlLightHornCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlLightHornCommandListener.onSendCtrlLightHornCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlLightHornCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlLightHornCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;

            case CTRL_LIGHTHORN_GET:
                if (mSendGetCarLightHornStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarLightHornStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarLightHornStateCommandListener.onSendGetCarLightHornStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarLightHornStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarLightHornStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_ENGINE:
                if (mSendCtrlEngineCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlEngineCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlEngineCommandListener.onSendCtrlEngineCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlEngineCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlEngineCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_ENGINE_GET:
                if (mSendGetCarEngineStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarEngineStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarEngineStateCommandListener.onSendGetCarEngineStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarEngineStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarEngineStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Electromotor:
                if (mSendCtrlElectromotorCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlElectromotorCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlElectromotorCommandListener.onSendCtrlElectromotorCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlElectromotorCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlElectromotorCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Electromotor_GET:
                if (mSendGetCarElectromotorStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarElectromotorStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarElectromotorStateCommandListener.onSendGetCarElectromotorStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarElectromotorStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarElectromotorStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Ekey:
                if (mSendCtrlEkeyCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlEkeyCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlEkeyCommandListener.onSendCtrlEkeyCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlEkeyCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlEkeyCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Ekey_GET:
                if (mSendGetCarEkeyStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarEkeyStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarEkeyStateCommandListener.onSendGetCarEkeyStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarEkeyStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarEkeyStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Doors:
                if (mSendCtrlDoorsCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlDoorsCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlDoorsCommandListener.onSendCtrlDoorsCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlDoorsCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlDoorsCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Doors_GET:
                if (mSendGetCarDoorsStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarDoorsStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarDoorsStateCommandListener.onSendGetCarDoorsStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarDoorsStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarDoorsStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Charge:
                if (mSendCtrlChargeCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlChargeCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlChargeCommandListener.onSendCtrlChargeCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlChargeCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlChargeCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Charge_GET:
                if (mSendGetCarChargeStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarChargeStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarChargeStateCommandListener.onSendGetCarChargeStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarChargeStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarChargeStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Chair:
                if (mSendCtrlChairCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlChairCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlChairCommandListener.onSendCtrlChairCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlChairCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlChairCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Chair_GET:
                if (mSendGetCarChairStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarChairStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarChairStateCommandListener.onSendGetCarChairStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarChairStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarChairStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Ac:
                if (mSendCtrlAcCommandListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlAcCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlAcCommandListener.onSendCtrlAcCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlAcCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlAcCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_Ac_GET:
                if (mSendGetCarAcStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetCarAcStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetCarAcStateCommandListener.onSendGetCarAcStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetCarAcStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetCarAcStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );

                    }
                }
                break;
            case CTRL_AllState_GET:
                if (mSendGetAllStateCommandListener != null) {
                    if (hasDSPTError) {
                        mSendGetAllStateCommandListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetAllStateCommandListener.onSendGetAllStateCommandSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendGetAllStateCommandListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetAllStateCommandListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );

                    }
                }
                break;
            case CTRL_WAKEUPMOBILE:
                if (mSendCtrlWakeupMobileListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlWakeupMobileListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlWakeupMobileListener.onSendCtrlWakeupMobileSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlWakeupMobileListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlWakeupMobileListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );

                    }
                }
                break;
            case CTRL_DIAGNOSIS:
                if (mSendCtrlDiagnosisListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlDiagnosisListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlDiagnosisListener.onSendCtrlDiagnosisSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlDiagnosisListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlDiagnosisListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );

                    }
                }
                break;
            case CTRL_PRIVATEINFORMATION:
                if (mSendCtrlPrivateInformationListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlPrivateInformationListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlPrivateInformationListener.onSendCtrlPrivateInformationSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }

                            mSendCtrlPrivateInformationListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlPrivateInformationListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_PRIVATEINFORMATION_GET:
                if (mSendGetPrivateInformationListener != null) {
                    if (hasDSPTError) {
                        mSendGetPrivateInformationListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetPrivateInformationListener.onSendGetPrivateInformationSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendGetPrivateInformationListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetPrivateInformationListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case CTRL_TOPWINDOWS:
                if (mSendCtrlTopWindowsListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlTopWindowsListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlTopWindowsListener.onSendCtrlTopWindowsSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendCtrlTopWindowsListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlTopWindowsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case CTRL_TOPWINDOWS_GET:
                if (mSendGetTopWindowsStateListener != null) {
                    if (hasDSPTError) {
                        mSendGetTopWindowsStateListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetTopWindowsStateListener.onSendGetTopWindowsStateSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendGetTopWindowsStateListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetTopWindowsStateListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case CTRL_AIRTEMPERATURE:
                if (mSendCtrlAirTemperatureListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlAirTemperatureListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlAirTemperatureListener.onSendCtrlAirTemperatureSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendCtrlAirTemperatureListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlAirTemperatureListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case CTRL_AIRTEMPERATURE_GET:
                if (mSendGetAirTemperatureStateListener != null) {
                    if (hasDSPTError) {
                        mSendGetAirTemperatureStateListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetAirTemperatureStateListener.onSendGetAirTemperatureStateSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendGetAirTemperatureStateListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetAirTemperatureStateListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case CTRL_TRUNKDOORS:
                if (mSendCtrlTrunkDoorsListener != null) {
                    if (hasDSPTError) {
                        mSendCtrlTrunkDoorsListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendCtrlTrunkDoorsListener.onSendCtrlTrunkDoorsSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendCtrlTrunkDoorsListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendCtrlTrunkDoorsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case CTRL_TRUNKDOORS_GET:
                if (mSendGetTrunkDoorsStateListener != null) {
                    if (hasDSPTError) {
                        mSendGetTrunkDoorsStateListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSendGetTrunkDoorsStateListener.onSendGetTrunkDoorsStateSuccess();
                        } else {
                            String msg = "";
                            if (jsonObject.has("msg")) {
                                msg = jsonObject.getString("msg");
                            }
                            mSendGetTrunkDoorsStateListener.onFail(respCode, msg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSendGetTrunkDoorsStateListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            default:
                break;
        }
    }

    private CtrlWindowsBean.WindowsCtrlInfo transformWindowInfo(WindowsCtrlInfo windowsCtrlInfo) {
        CtrlWindowsBean.WindowsCtrlInfo.Builder ctrlWindowsInfo = CtrlWindowsBean.WindowsCtrlInfo.newBuilder();
        ctrlWindowsInfo.setWindowOpenedValue(windowsCtrlInfo.getWindowOpenedValue());

        switch (windowsCtrlInfo.getWindowPos().getValue()) {
            case 0x01:
                ctrlWindowsInfo.setWindowPos(CtrlWindowsBean.WindowPos.LEFT_FRONT_WINDOW);
                break;
            case 0x02:
                ctrlWindowsInfo.setWindowPos(CtrlWindowsBean.WindowPos.RIGHT_FRONT_WINDOW);
                break;
            case 0x03:
                ctrlWindowsInfo.setWindowPos(CtrlWindowsBean.WindowPos.RIGHT_BACK_WINDOW);
                break;
            case 0x04:
                ctrlWindowsInfo.setWindowPos(CtrlWindowsBean.WindowPos.LEFT_BACK_WINDOW);
                break;
            default:
                break;
        }
        return ctrlWindowsInfo.build();
    }

    private CtrlWindowsBean.WindowsStateInfo transformWindowState(WindowsStateInfo windowsStateInfo) {
        CtrlWindowsBean.WindowsStateInfo.Builder stateWindowInfo = CtrlWindowsBean.WindowsStateInfo.newBuilder();
        stateWindowInfo.setWindowOpenedValue(windowsStateInfo.getWindowOpenedValue());

        switch (windowsStateInfo.getWindowPos().getValue()) {
            case 0x01:
                stateWindowInfo.setWindowPos(CtrlWindowsBean.WindowPos.LEFT_FRONT_WINDOW);
                break;
            case 0x02:
                stateWindowInfo.setWindowPos(CtrlWindowsBean.WindowPos.RIGHT_FRONT_WINDOW);
                break;
            case 0x03:
                stateWindowInfo.setWindowPos(CtrlWindowsBean.WindowPos.RIGHT_BACK_WINDOW);
                break;
            case 0x04:
                stateWindowInfo.setWindowPos(CtrlWindowsBean.WindowPos.LEFT_BACK_WINDOW);
                break;
            default:
                break;
        }

        switch (windowsStateInfo.getWindowStatus().getValue()) {
            case 0x0:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.WINDOW_CLOSED);
                break;
            case 0x1:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.WINDOW_OPENED);
                break;
            case 0x2:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.STOPPED);
                break;
            case 0x3:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.AUTO_UP_MOVING);
                break;
            case 0x4:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.MANUAL_UP_MOVING);
                break;
            case 0x5:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.AUTO_DOWN_MOVING);
                break;
            case 0x6:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.MANUAL_DOWN_MOVING);
                break;
            case 0xFF:
                stateWindowInfo.setWindowStatus(CtrlWindowsBean.WindowStatus.STATE_EXCEPTION_STATUS);
                break;
            default:
                break;
        }
        return stateWindowInfo.build();
    }
}
