package com.mxnavi.tspv2.utils;

import android.content.Context;
import android.os.Environment;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public class Logcat2File {

    private static Logcat2File INSTANCE = null;
    private static String PATH_LOGCAT;
    private LogDumper mLogDumper = null;

    public static Logcat2File getInstance(Context context) {
        if (null == INSTANCE) {
            INSTANCE = new Logcat2File(context);
        }
        return INSTANCE;
    }

    private Logcat2File(Context context) {
        init(context);
    }

    private void init(Context context) {
        if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
            PATH_LOGCAT = Environment.getExternalStorageDirectory().getAbsolutePath() + File.separator
                    + context.getPackageName();
        } else {
            PATH_LOGCAT = context.getFilesDir().getAbsolutePath() + File.separator
                    + context.getPackageName();
        }
        File file = new File(PATH_LOGCAT);
        if (!file.exists()) {
            file.mkdirs();
        }
    }

    public void start() {
        if (null == mLogDumper) {
            mLogDumper = new LogDumper(PATH_LOGCAT);
        }
        mLogDumper.start();
    }

    public void stop() {
        if (mLogDumper != null) {
            mLogDumper.stopLogs();
            mLogDumper = null;
        }
    }

    private class LogDumper extends Thread {

        private Process logcatProcess = null;
        private BufferedReader mReader = null;
        private boolean mRunning = true;
        private String program = null;
        private String mPID = null;
        private FileOutputStream out = null;

        public LogDumper(String dir) {
            mPID = String.valueOf(android.os.Process.myPid());
            try {
                out = new FileOutputStream(new File(dir, "Log_" + getFileName() + ".log"));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
            /**
             * 日志等级：*:v , *:d , *:w , *:e , *:f , *:s
             */
//            program = "logcat *:e *:w | grep \"(" + mPID + ")\"";
            program = "logcat -v time | grep \"(" + mPID + ")\"";//打印所有日志信息
//            program = "logcat -s way";//打印标签过滤信息
        }

        public void stopLogs() {
            mRunning = false;
        }

        @Override
        public void run() {
            try {
                logcatProcess = Runtime.getRuntime().exec(program);
                mReader = new BufferedReader(new InputStreamReader(logcatProcess.getInputStream()), 1024);
                String line;
                while (mRunning && ((line = mReader.readLine()) != null)) {
                    if (!mRunning) {
                        break;
                    }
                    if (line.isEmpty()) {
                        continue;
                    }
                    if ((out != null) && line.contains(mPID)) {
                        out.write((line + "\n").getBytes());
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (logcatProcess != null) {
                    logcatProcess.destroy();
                    logcatProcess = null;
                }
                if (mReader != null) {
                    try {
                        mReader.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    mReader = null;
                }
                if (out != null) {
                    try {
                        out.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    out = null;
                }
            }
        }

        private String getFileName() {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd_HH-mm-ss", Locale.US);
            // 2015-12-17_13-49-51
            return format.format(new Date(System.currentTimeMillis()));
        }
    }
}
