package com.mxnavi.tspv2.weather;

import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.model.TspCity;
import com.mxnavi.tspv2.model.TspDate;
import com.mxnavi.tspv2.model.TspTime;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;
import com.mxnavi.tspv2.weather.model.AlarmInfo;
import com.mxnavi.tspv2.weather.model.CityInfo;
import com.mxnavi.tspv2.weather.model.CurrentWeather;
import com.mxnavi.tspv2.weather.model.DistInfo;
import com.mxnavi.tspv2.weather.model.FutureWeather;
import com.mxnavi.tspv2.weather.model.LifeIndexResult;
import com.mxnavi.tspv2.weather.model.NormalWeatherQuery;
import com.mxnavi.tspv2.weather.model.NormalWeatherResult;
import com.mxnavi.tspv2.weather.model.ProvinceInfo;
import com.mxnavi.tspv2.weather.model.QueryAlarmResult;
import com.mxnavi.tspv2.weather.model.QueryCityResult;
import com.mxnavi.tspv2.weather.model.QueryProvinceResult;
import com.mxnavi.tspv2.weather.model.SimpleFutureWeather;
import com.mxnavi.tspv2.weather.model.SimpleNormalWeatherQuery;
import com.mxnavi.tspv2.weather.model.SimpleNormalWeatherResult;
import com.mxnavi.tspv2.weather.model.Weather3hInfo;
import com.mxnavi.tspv2.weather.model.Weather3hResult;
import com.mxnavi.tspv2.weather.model.WeatherIndex;
import com.mxnavi.tspv2.weather.model.WeatherQuery;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

/**
 * Created by Administrator on 2016/3/25.
 */
public class WeatherSearch extends BaseTsp  {
    // 请求服务类型为天气
    final int thisServiceType = Config.MxNGTPServiceType.WEATHER.getServiceType();
    final int serviceDataVer = Config.MxNGTPServiceType.WEATHER.getServiceVer();

    public static final String UNKNOWN = "未知";
    public static final String EXCELLENT = "优";
    public static final String WELL =  "良好";
    public static final String MILD_CONTAMINATION = "轻度污染";
    public static final String MODERATELY_POLLUTED = "中度污染";
    public static final String SERIOUS_POLLUTION = "严重污染";

    private Map<Integer,Integer> listenertypeHashMap = new HashMap<Integer,Integer>();
    final int WeatherType_3day =1;
    final int WeatherType_3h =2;
    final int WeatherType_alarm =3;
    final int WeatherType_simplenormal =4;
    final int QueryLifeIndex =5;// 查询生活指数
    final int GETPROVINCEINFO = 6; //获取省份信息
    final int GETCITYINFO = 7; // 获取城市信息

    OnNormalWeatherQueryListener mOnNormalWeatherQueryListener;
    On3hWeatherQueryListener mOn3hWeatherQueryListener;
    OnAlarmQueryListener mOnAlarmQueryListener;
    OnSimpleNormalWeatherQueryListener mOnSimpleNormalWeatherQueryListener;
    OnLifeIndexQueryListener mOnLifeIndexQueryListener;
    OnGetProvinceInfoListener mOnGetProvinceInfoListener;
    OnGetCityInfoListener mOnGetCityInfoListener;

    public interface  OnNormalWeatherQueryListener extends CallBackListener{
        public void onNormalWeatherQuery(int requestId, List<NormalWeatherResult> list);
    }
    public interface  On3hWeatherQueryListener extends CallBackListener{
        public void on3hWeatherQuery(int requestId, List<Weather3hResult> list);
    }
    public interface  OnAlarmQueryListener extends CallBackListener{
        public void onAlarmQuery(int requestId, List<QueryAlarmResult> list);
    }
    public interface  OnSimpleNormalWeatherQueryListener extends CallBackListener{
        public void onSimpleNormalWeatherQuery(int requestId, List<SimpleNormalWeatherResult> list);
    }
    public interface  OnLifeIndexQueryListener extends CallBackListener{
        public void  onLifeIndexQuery(int requestId, List<LifeIndexResult> list);
    }
    public interface OnGetProvinceInfoListener extends CallBackListener {
        public void getProvinceInfo(int requestId, QueryProvinceResult queryProvinceResult);
    }

    public interface OnGetCityInfoListener extends CallBackListener {
        public void getCityInfo(int requestId, QueryCityResult queryCityResult);
    }

    /**
     * 转化空气指数
     * @param aqi
     * @return
     */
    private String getAirLevel(int aqi){
        //-1 <= 50  50 <api <=100 100 <api <=200 200<api <=300 >300
        if(aqi<=-1){
            return UNKNOWN;
        }else if(aqi>-1&&aqi<=50){
            return EXCELLENT;
        }else if(aqi>50&&aqi<=100){
            return WELL;
        }else if(aqi>100&&aqi<=200){
            return MILD_CONTAMINATION;
        }else if(aqi>200&&aqi<=300){
            return MODERATELY_POLLUTED;
        }else{
            return SERIOUS_POLLUTION;
        }


    }

    // 注册此对象为天气业务信息接收类
    public WeatherSearch(){
    }

    /**
     * 请求查询常规天气预报信息
     * @param normalWeatherQuery 常规天气预报查询条件
     * @return 请求ID
     */
    public int queryNormalForecast(NormalWeatherQuery normalWeatherQuery){
        return queryNormalForecast(normalWeatherQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询常规天气预报信息
     * @param normalWeatherQuery 常规天气预报查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int queryNormalForecast(NormalWeatherQuery normalWeatherQuery,int timeOutSecs){
        return queryNormalForecast(normalWeatherQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询常规天气预报信息
     * @param normalWeatherQuery 常规天气预报查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int queryNormalForecast(NormalWeatherQuery normalWeatherQuery,int timeOutSecs,int priorityLevel){
        if(normalWeatherQuery == null){
            if (mOnNormalWeatherQueryListener != null) {
                mOnNormalWeatherQueryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "QueryForecast";
        HashMap<String, Object> params = new HashMap<String, Object>();
        StringBuffer location = new StringBuffer();
        List<LatLng> latLngList;
        List<String> cityCodesList;
        if(normalWeatherQuery.getLocationType() == 1){
            latLngList =  normalWeatherQuery.getLocations();

            int latLngSize = latLngList.size();
            LatLng latLng;
            for(int i = 0; i < latLngSize; i++){
                latLng = latLngList.get(i);
                if(i < latLngSize-1) {
                    location.append(latLng.getLongitude() + "," + latLng.getLatitude() + "|");
                } else{
                    location.append(latLng.getLongitude() + "," + latLng.getLatitude());
                }
            }
        }else{
            cityCodesList = normalWeatherQuery.getCityCodes();

            int cityCodesListSize = cityCodesList.size();
            String strCityCode;
            for(int j = 0; j < cityCodesListSize; j++){
                strCityCode = cityCodesList.get(j);
                if(j < cityCodesListSize-1) {
                    location.append( strCityCode+ "|");
                } else{
                    location.append(strCityCode);
                }
            }
        }

        params.put("location",location.toString());
        params.put("future",normalWeatherQuery.getFutureDays());
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, WeatherType_3day);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求查询三小时精细化预报信息
     * @param weatherQuery 天气查询条件
     * @return 请求ID
     */
    public int query3hForecast(WeatherQuery weatherQuery){
        return query3hForecast(weatherQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询三小时精细化预报信息
     * @param weatherQuery 天气查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int query3hForecast(WeatherQuery weatherQuery,int timeOutSecs){
        return query3hForecast(weatherQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询三小时精细化预报信息
     * @param weatherQuery 天气查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public  int query3hForecast(WeatherQuery weatherQuery,int timeOutSecs,int priorityLevel){
        if(weatherQuery == null){
            if (mOn3hWeatherQueryListener != null) {
                mOn3hWeatherQueryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "QueryForecast3h";
        HashMap<String, Object> params = new HashMap<String, Object>();

        StringBuffer stringLocation = new StringBuffer();
        List<LatLng> locationsList;
        List<String> cityCodesList;
        if(weatherQuery.getLocationType() == 1){
            locationsList = weatherQuery.getLocations();
            int latLngSize = locationsList.size();
            LatLng latLng;
            for(int i = 0; i < latLngSize; i++){
                latLng = locationsList.get(i);
                if(i < latLngSize-1) {
                    stringLocation.append(latLng.getLongitude() + "," + latLng.getLatitude() + "|");
                } else{
                    stringLocation.append(latLng.getLongitude() + "," + latLng.getLatitude());
                }
            }
        }else{
            cityCodesList = weatherQuery.getCityCodes();
            int cityCodeSize = cityCodesList.size();
            String cityCode;
            for(int i = 0; i < cityCodeSize; i++){
                cityCode = cityCodesList.get(i);
                if(i < cityCodeSize-1) {
                    stringLocation.append(cityCode + "|");
                } else{
                    stringLocation.append(cityCode);
                }
            }
        }

        params.put("location",stringLocation.toString());
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType,serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, WeatherType_3h);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求查询预警信息信息
     * @param weatherQuery 天气查询条件
     * @return 请求ID
     */
    public int queryAlarm(WeatherQuery weatherQuery){
        return queryAlarm(weatherQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询预警信息信息
     * @param weatherQuery 天气查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int queryAlarm(WeatherQuery weatherQuery,int timeOutSecs){
        return queryAlarm(weatherQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }
    /**
     * 请求查询预警信息信息
     * @param weatherQuery 天气查询条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public  int queryAlarm(WeatherQuery weatherQuery,int timeOutSecs,int priorityLevel){
        if(weatherQuery == null){
            if (mOnAlarmQueryListener != null) {
                mOnAlarmQueryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        String url = "QueryAlarm";
        HashMap<String, Object> params = new HashMap<String, Object>();

        StringBuffer stringLocation = new StringBuffer();
        List<LatLng> locationsList;
        List<String> cityCodesList;
        if(weatherQuery.getLocationType() == 1){
            locationsList = weatherQuery.getLocations();
            int latLngSize = weatherQuery.getLocations().size();
            LatLng latLng;
            for(int i = 0; i < latLngSize; i++){
                latLng = locationsList.get(i);
                if(i < latLngSize-1) {
                    stringLocation.append(latLng.getLongitude() + "," + latLng.getLatitude() + "|");
                } else{
                    stringLocation.append(latLng.getLongitude() + "," + latLng.getLatitude());
                }
            }
        }else{
            cityCodesList = weatherQuery.getCityCodes();
            int cityCodeSize = cityCodesList.size();
            String cityCode;
            for(int i = 0; i < cityCodeSize; i++){
                cityCode = cityCodesList.get(i);
                if(i < cityCodeSize-1) {
                    stringLocation.append(cityCode + "|");
                } else{
                    stringLocation.append(cityCode);
                }
            }
        }

        params.put("location",stringLocation.toString());
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, WeatherType_alarm);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 查询常规天气（只有常规天气预报信息）
     * @param query
     * @return
     */
    public int querySimpleNormalWeather(SimpleNormalWeatherQuery query){
        return querySimpleNormalWeather(query, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询常规天气（只有常规天气预报信息）
     * @param query
     * @param timeOutSecs
     * @return
     */
    public int querySimpleNormalWeather(SimpleNormalWeatherQuery query,int timeOutSecs){
        return querySimpleNormalWeather(query, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询常规天气（只有常规天气预报信息）
     * @param query
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int querySimpleNormalWeather(SimpleNormalWeatherQuery query,int timeOutSecs,int priorityLevel){
        if(query == null){
            if (mOnSimpleNormalWeatherQueryListener != null) {
                mOnSimpleNormalWeatherQueryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "QueryFuture";
        HashMap<String, Object> params = new HashMap<String, Object>();
        StringBuffer location = new StringBuffer();
        List<LatLng> latLngList;
        List<String> cityCodesList;

        latLngList =  query.getLocations();

        int latLngSize = latLngList.size();
        LatLng latLng;
        for(int i = 0; i < latLngSize; i++){
            latLng = latLngList.get(i);
            if(i < latLngSize-1) {
                location.append(latLng.getLongitude() + "," + latLng.getLatitude() + "|");
            } else{
                location.append(latLng.getLongitude() + "," + latLng.getLatitude());
            }
        }


        params.put("location",location.toString());
        params.put("future",query.getFutureDays());
        params.put("cityType",query.getCityType());
        params.put("queryTypes",query.getQueryTypes());
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();

        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, WeatherType_simplenormal);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 查询生活指数
     * @param locations
     * @param onLifeIndexQueryListener
     * @return
     */
    public int queryLifeIndex(List<LatLng> locations,OnLifeIndexQueryListener onLifeIndexQueryListener){
        return  queryLifeIndex(locations,onLifeIndexQueryListener,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 查询生活指数
     * @param locations
     * @param onLifeIndexQueryListener
     * @param timeOutSecs
     * @return
     */
    public int queryLifeIndex(List<LatLng> locations,OnLifeIndexQueryListener onLifeIndexQueryListener,int timeOutSecs){
        return queryLifeIndex(locations,onLifeIndexQueryListener,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 查询生活指数
     * @param locations 位置
     * @param onLifeIndexQueryListener
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int queryLifeIndex(List<LatLng> locations,OnLifeIndexQueryListener onLifeIndexQueryListener,int timeOutSecs,int priorityLevel){
        mOnLifeIndexQueryListener = onLifeIndexQueryListener;
        if(locations == null||locations.size()==0){
            if (mOnLifeIndexQueryListener != null) {
                mOnLifeIndexQueryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "QueryIndex";
        HashMap<String, Object> params = new HashMap<String, Object>();
        StringBuffer location = new StringBuffer();


        int latLngSize = locations.size();
        LatLng latLng;
        for(int i = 0; i < latLngSize; i++){
            latLng = locations.get(i);
            if(i < latLngSize-1) {
                location.append(latLng.getLongitude() + "," + latLng.getLatitude() + "|");
            } else{
                location.append(latLng.getLongitude() + "," + latLng.getLatitude());
            }
        }


        params.put("location",location.toString());
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, QueryLifeIndex);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取 省份信息
     * @return
     */
    public int getProvinceInfo(){
        return getProvinceInfo(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取 省份信息
     * @param timeOutSecs
     * @return
     */
    public int getProvinceInfo(int timeOutSecs){
        return getProvinceInfo(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取 省份信息
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getProvinceInfo(int timeOutSecs,int priorityLevel){
        String url = "provinces";
        String requestData = Utils.makeUrl(url, null);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance()
                .makeRequestBusinessMsg(eventId,requestData,thisServiceType,serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId,GETPROVINCEINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取 城市信息
     * @param code
     * @return
     */
    public int getCityInfo(String code){
        return getCityInfo(code,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取 城市信息
     * @param code
     * @param timeOutSecs
     * @return
     */
    public int getCityInfo(String code,int timeOutSecs) {
        return getCityInfo(code,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取 城市信息
     * @param code
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getCityInfo(String code,int timeOutSecs,int priorityLevel) {
        if (code == null){
            if (mOnGetCityInfoListener != null){
                mOnGetCityInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "cities";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("code",code);
        String requestData = Utils.makeUrl(url,params);
        Utils.loge("zhao", requestData);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance()
                .makeRequestBusinessMsg(eventId,requestData,thisServiceType,serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId,GETCITYINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    private  void addCallbackMap(int eventId,int type){
        super.addCallbackMap(eventId, WeatherSearch.this);

        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }
    @Override
    protected void removeCallbackMap(int eventId){
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg){
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case WeatherType_3day:
                if(mOnNormalWeatherQueryListener != null) {
                    if(hasDSPTError){
                        mOnNormalWeatherQueryListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    List<NormalWeatherResult> list = new ArrayList<NormalWeatherResult>();
                    List<WeatherIndex> weatherIndexList = new ArrayList<WeatherIndex>();;
                    List<FutureWeather> futureWeatherList = new ArrayList<FutureWeather>();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            NormalWeatherResult result ;
                            JSONArray resultsDataArray = jsonObject.getJSONArray("results");
                            for (int i = 0; i < resultsDataArray.length(); i++) {
                                result = new NormalWeatherResult();
                                JSONObject object = resultsDataArray.getJSONObject(i);
                                String currentDate = object.getString("date");
                                String[] strArr =  currentDate.split("-");
                                TspDate resultsTspDate = new TspDate();
                                resultsTspDate.setYear(Integer.parseInt(strArr[0]));
                                resultsTspDate.setMonth(Integer.parseInt(strArr[1]));
                                resultsTspDate.setDay(Integer.parseInt(strArr[2]));
                                JSONObject distInfo = object.getJSONObject("distInfo");
                                TspCity tspCity = new TspCity();
                                tspCity.setCurrentCityCode(distInfo.getString("mxCode"));
                                tspCity.setProvName(distInfo.getString("prov"));
                                tspCity.setCityName(distInfo.getString("city"));
                                tspCity.setDistName(distInfo.getString("dist"));
                                int pm25 = object.getInt("pm25");
                                int aqi = object.getInt("aqi");
                                JSONObject currentWeatherObj = object.getJSONObject("observe");
                                int damp = currentWeatherObj.getInt("damp");
                                int temp = currentWeatherObj.getInt("temp");
                                int weatherId = currentWeatherObj.getInt("weatherId");
                                String weatherName = currentWeatherObj.getString("weatherName");
                                //int windLevelId = currentWeatherObj.getInt("windLevelId");
                                String windLevelName = currentWeatherObj.getString("windLevelName");
                                int windOrientationId = currentWeatherObj.getInt("windOrientationId");
                                String windOrientationName = currentWeatherObj.getString("windOrientationName");
                                String pubTime = currentWeatherObj.getString("pubTime");
                                String[] strArra =  pubTime.split(":");
                                CurrentWeather currentWeather = new CurrentWeather();
                                currentWeather.setDamp(damp);
                                currentWeather.setCurrentTmp(temp);
                                currentWeather.setWeatherId(weatherId);
                                currentWeather.setWeatherName(weatherName);
                                //currentWeather.setWindLevelId(windLevelId);
                                currentWeather.setWindLevelName(windLevelName);
                                currentWeather.setWindOrientationId(windOrientationId);
                                currentWeather.setWindOrientaionName(windOrientationName);
                                TspTime tspTime = new TspTime();
                                tspTime.setHour(Integer.parseInt(strArra[0]));
                                tspTime.setMin(Integer.parseInt(strArra[1]));
                                tspTime.setSec(0);
                                currentWeather.setPubTime(tspTime);
                                JSONArray indexDataArray = object.getJSONArray("index");
                                for (int j = 0; j < indexDataArray.length(); j++) {
                                    JSONObject objectIndex = indexDataArray.getJSONObject(j);
                                    int idxId = objectIndex.getInt("idxId");
                                    String name = objectIndex.getString("name");
                                    String level = objectIndex.getString("level");
                                    String info = objectIndex.getString("info");
                                    WeatherIndex weatherIndex = new WeatherIndex();
                                    weatherIndex.setIdxId(idxId);
                                    weatherIndex.setIndexName(name);
                                    weatherIndex.setIndexLevel(level);
                                    weatherIndex.setIndexInfo(info);
                                    weatherIndexList.add(weatherIndex);
                                }
                                JSONArray futureDataArray = object.getJSONArray("future");
                                for (int z = 0; z < futureDataArray.length(); z++) {
                                    JSONObject objectFuture = futureDataArray.getJSONObject(z);
                                    String futureDate = objectFuture.getString("date");
                                    String[] strArray =  futureDate.split("-");
                                    TspDate futureTspDate = new TspDate();
                                    futureTspDate.setYear(Integer.parseInt(strArray[0]));
                                    futureTspDate.setMonth(Integer.parseInt(strArray[1]));
                                    futureTspDate.setDay(Integer.parseInt(strArray[2]));
                                    int tmpDay = objectFuture.getInt("tempDay");
                                    int tmpNight = objectFuture.getInt("tempNight");
                                    int wphDayId = objectFuture.getInt("wphDayId");
                                    String wphDayName = objectFuture.getString("wphDayName");
                                    int wphNightId = objectFuture.getInt("wphNightId");
                                    String wphNightName = objectFuture.getString("wphNightName");
                                    int windDayLevelId = objectFuture.getInt("windDayLevelId");
                                    String windDayLevelName = objectFuture.getString("windDayLevelName");
                                    int windDayOrientationId = objectFuture.getInt("windDayOrientationId");
                                    String windDayOrientationName = objectFuture.getString("windDayOrientationName");
                                    int windNightLevelId = objectFuture.getInt("windNightLevelId");
                                    String windNightLevelName = objectFuture.getString("windNightLevelName");
                                    int windNightOrientationId = objectFuture.getInt("windNightOrientationId");
                                    String windNightOrientationName = objectFuture.getString("windNightOrientationName");
                                    FutureWeather futureWeather = new FutureWeather();
                                    futureWeather.setDate(futureTspDate);
                                    futureWeather.setTmpDay(tmpDay);
                                    futureWeather.setTmpNight(tmpNight);
                                    futureWeather.setWphDayId(wphDayId);
                                    futureWeather.setWphDayName(wphDayName);
                                    futureWeather.setWphNightId(wphNightId);
                                    futureWeather.setWphNightName(wphNightName);
                                    futureWeather.setWindDayLevelId(windDayLevelId);
                                    futureWeather.setWindDayLevelName(windDayLevelName);
                                    futureWeather.setWindDayOrientationId(windDayOrientationId);
                                    futureWeather.setWindDayOrientationName(windDayOrientationName);
                                    futureWeather.setWindNightLevelId(windNightLevelId);
                                    futureWeather.setWindNightLevelName(windNightLevelName);
                                    futureWeather.setWindNightOrientationId(windNightOrientationId);
                                    futureWeather.setWindNightOrientationName(windNightOrientationName);
                                    futureWeatherList.add(futureWeather);
                                }
                                result.setPm25(pm25);
                                result.setAqi(aqi);
                                result.setAriLevel(getAirLevel(aqi));
                                result.setCurrentDate(resultsTspDate);
                                result.setDistInfo(tspCity);
                                result.setCurrentWeather(currentWeather);
                                result.setWeatherIndexList(weatherIndexList);
                                result.setFutureWeatherList(futureWeatherList);
                                list.add(result);
                            }
                            mOnNormalWeatherQueryListener.onNormalWeatherQuery(requestHandleId, list);
                        } else if(respCode == 101) {
                            mOnNormalWeatherQueryListener.onNormalWeatherQuery(requestHandleId,list);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnNormalWeatherQueryListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnNormalWeatherQueryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case WeatherType_3h:
                if(mOn3hWeatherQueryListener != null) {
                    if(hasDSPTError){
                        mOn3hWeatherQueryListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    List<Weather3hResult> list = new ArrayList<Weather3hResult>();
                    List<Weather3hInfo> weather3hInfoList = new ArrayList<Weather3hInfo>();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Weather3hResult result ;
                            JSONArray resultsDataArray = jsonObject.getJSONArray("results");
                            for (int i = 0; i < resultsDataArray.length(); i++) {
                                result = new Weather3hResult();
                                JSONObject object = resultsDataArray.getJSONObject(i);
                                JSONObject distInfo = object.getJSONObject("distInfo");
                                TspCity tspCity = new TspCity();
                                tspCity.setCurrentCityCode(distInfo.getString("mxCode"));
                                tspCity.setProvName(distInfo.getString("prov"));
                                tspCity.setCityName(distInfo.getString("city"));
                                tspCity.setDistName(distInfo.getString("dist"));
                                JSONArray f3hDataArray = object.getJSONArray("f3h");
                                for (int j = 0; j < f3hDataArray.length(); j++) {
                                    JSONObject objectF3h = f3hDataArray.getJSONObject(j);
                                    String begin = objectF3h.getString("begin");
                                    String end = objectF3h.getString("end");
                                    int high = objectF3h.getInt("high");
                                    int low = objectF3h.getInt("low");
                                    int weatherId = objectF3h.getInt("weatherId");
                                    String weatherName = objectF3h.getString("weatherName");
                                    Weather3hInfo weather3hInfo = new Weather3hInfo();
                                    weather3hInfo.setBeginTime(begin);
                                    weather3hInfo.setBeginTime(end);
                                    weather3hInfo.setHighTemp(high);
                                    weather3hInfo.setLowTemp(low);
                                    weather3hInfo.setWeatherId(weatherId);
                                    weather3hInfo.setWeatherName(weatherName);
                                    weather3hInfoList.add(weather3hInfo);
                                }
                                result.setDistInfo(tspCity);
                                result.setWeather3hInfos(weather3hInfoList);
                                list.add(result);
                            }
                            mOn3hWeatherQueryListener.on3hWeatherQuery(requestHandleId, list);
                        } else if(respCode == 101) {
                            mOn3hWeatherQueryListener.on3hWeatherQuery(requestHandleId, list);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOn3hWeatherQueryListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOn3hWeatherQueryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case WeatherType_alarm:
                if(mOnAlarmQueryListener != null) {
                    if(hasDSPTError){
                        mOnAlarmQueryListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        List<QueryAlarmResult> queryAlarmResultList = new ArrayList<QueryAlarmResult>();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray resultsDataArray = jsonObject.getJSONArray("results");
                            QueryAlarmResult queryAlarmResult;
                            for (int i = 0; i < resultsDataArray.length(); i++) {
                                queryAlarmResult = new QueryAlarmResult();
                                JSONObject object = resultsDataArray.getJSONObject(i);
                                JSONObject distInfo = object.getJSONObject("distInfo");
                                TspCity tspCity = new TspCity();
                                tspCity.setCurrentCityCode(distInfo.getString("mxCode"));
                                tspCity.setProvName(distInfo.getString("prov"));
                                tspCity.setCityName(distInfo.getString("city"));
                                tspCity.setDistName(distInfo.getString("dist"));
                                queryAlarmResult.setTspCity(tspCity);
                                List<AlarmInfo> alarmInfoList = new ArrayList<AlarmInfo>();
                                JSONArray alarmArray = object.getJSONArray("alarm");
                                Utils.loge(alarmArray.toString());
                                for (int j = 0; j < alarmArray.length(); j++) {
                                    JSONObject alarmObject = alarmArray.getJSONObject(j);
                                    Utils.loge(alarmObject.toString());
                                    AlarmInfo alarmInfo = new AlarmInfo();
                                    alarmInfo.setLevelId(alarmObject.getInt("levelId"));
                                    alarmInfo.setLevelName(alarmObject.getString("levelName"));
                                    alarmInfo.setRelease_time(alarmObject.getString("releaseTime"));
                                    alarmInfo.setTypeId(alarmObject.getInt("typeId"));
                                    alarmInfo.setTypeName(alarmObject.getString("typeName"));
                                    alarmInfo.setInfo(alarmObject.getString("info"));
                                    alarmInfoList.add(alarmInfo);
                                }
                                queryAlarmResult.setAlarmInfoList(alarmInfoList);
                                queryAlarmResultList.add(queryAlarmResult);
                            }
                            mOnAlarmQueryListener.onAlarmQuery(requestHandleId, queryAlarmResultList);
                        } else if(respCode == 101) {
                            mOnAlarmQueryListener.onAlarmQuery(requestHandleId, queryAlarmResultList);
                        }
                        else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnAlarmQueryListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnAlarmQueryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case WeatherType_simplenormal:
                if(mOnSimpleNormalWeatherQueryListener != null) {
                    if (hasDSPTError) {
                        mOnSimpleNormalWeatherQueryListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    List<SimpleNormalWeatherResult> resultList = new ArrayList<SimpleNormalWeatherResult>();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {

                            JSONArray resultsDataArray = jsonObject.getJSONArray("results");
                            for (int i = 0; i < resultsDataArray.length(); i++) {
                                SimpleNormalWeatherResult result = new SimpleNormalWeatherResult();
                                JSONObject object = resultsDataArray.getJSONObject(i);
                                DistInfo distInfo = new DistInfo();
                                JSONObject distInfoObj = object.getJSONObject("distInfo");
                                distInfo.setCity(distInfoObj.getString("city"));
                                distInfo.setDist(distInfoObj.getString("dist"));
                                distInfo.setProv(distInfoObj.getString("prov"));
                                if (distInfoObj.has("lon")) {                                	
                                	distInfo.setLon(Utils.converMX2DoubleCoordinate(distInfoObj.getInt("lon")));
                                }
                                if (distInfoObj.has("lat")) {
                                	distInfo.setLat(Utils.converMX2DoubleCoordinate(distInfoObj.getInt("lat")));
                                }
                                distInfo.setMxCode(String.valueOf(distInfoObj.getInt("mxCode")));

                                List<SimpleFutureWeather> futureList = new ArrayList<SimpleFutureWeather>();
                                JSONArray futureArray = object.getJSONArray("future");
                                for (int j = 0; j < futureArray.length(); j++) {
                                    JSONObject futureObj = futureArray.getJSONObject(j);
                                    SimpleFutureWeather weather = new SimpleFutureWeather();
                                    weather.setDate(futureObj.getString("date"));
                                    weather.setTempDay(futureObj.getInt("tempDay"));
                                    weather.setTempNight(futureObj.getInt("tempNight"));
                                    weather.setWindDayLevelId(futureObj.getInt("windDayLevelId"));
                                    weather.setWindDayLevelName(futureObj.getString("windDayLevelName"));
                                    weather.setWindDayOrientationId(futureObj.getInt("windDayOrientationId"));
                                    weather.setWindDayOrientationName(futureObj.getString("windDayOrientationName"));
                                    weather.setWindNightLevelId(futureObj.getInt("windNightLevelId"));
                                    weather.setWindNightLevelName(futureObj.getString("windNightLevelName"));
                                    weather.setWindNightOrientationId(futureObj.getInt("windNightOrientationId"));
                                    weather.setWindNightOrientationName(futureObj.getString("windNightOrientationName"));
                                    weather.setWphDayId(futureObj.getInt("wphDayId"));
                                    weather.setWphDayName(futureObj.getString("wphDayName"));
                                    weather.setWphNightId(futureObj.getInt("wphNightId"));
                                    weather.setWphNightName(futureObj.getString("wphNightName"));;

                                    futureList.add(weather);
                                }

                                result.setDistInfo(distInfo);
                                result.setFuture(futureList);
                                resultList.add(result);
                            }
                            mOnSimpleNormalWeatherQueryListener.onSimpleNormalWeatherQuery(requestHandleId, resultList);
                        } else {
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnSimpleNormalWeatherQueryListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSimpleNormalWeatherQueryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case QueryLifeIndex:
                if(mOnLifeIndexQueryListener != null) {
                    if (hasDSPTError) {
                        mOnLifeIndexQueryListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray resultArr = jsonObject.getJSONArray("results");
                            int size = resultArr.length();
                            List<LifeIndexResult> list = new ArrayList<LifeIndexResult>(size);
                            LifeIndexResult lifeIndexResult;
                            for(int i=0;i<size;i++){
                                lifeIndexResult = new LifeIndexResult();
                                JSONObject result = resultArr.getJSONObject(i);
                                JSONObject distInfoObj = result.getJSONObject("distInfo");
                                TspCity tspCity = new TspCity();
                                tspCity.setCurrentCityCode(distInfoObj.getString("mxCode"));
                                tspCity.setProvName(distInfoObj.getString("prov"));
                                tspCity.setCityName(distInfoObj.getString("city"));
                                tspCity.setDistName(distInfoObj.getString("dist"));
                                lifeIndexResult.setDistInfo(tspCity);
                                JSONArray indexArr = result.getJSONArray("index");
                                int indexSize = indexArr.length();
                                List<WeatherIndex> lifeIndexList = new ArrayList<WeatherIndex>(indexSize);
                                WeatherIndex weatherIndex;
                                for(int j = 0;j<indexSize;j++){
                                    JSONObject indexJson = indexArr.getJSONObject(j);
                                    weatherIndex = new WeatherIndex();
                                    int idxId = indexJson.getInt("idxId");
                                    String name = indexJson.getString("name");
                                    String level = indexJson.getString("level");
                                    String info = indexJson.getString("info");
                                    weatherIndex.setIdxId(idxId);
                                    weatherIndex.setIndexName(name);
                                    weatherIndex.setIndexLevel(level);
                                    weatherIndex.setIndexInfo(info);
                                    lifeIndexList.add(weatherIndex);
                                }
                                lifeIndexResult.setLifeIndexList(lifeIndexList);
                                list.add(lifeIndexResult);
                            }
                            mOnLifeIndexQueryListener.onLifeIndexQuery(requestHandleId, list);
                        } else {
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnLifeIndexQueryListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnLifeIndexQueryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETPROVINCEINFO:
                if (mOnGetProvinceInfoListener != null){
                    if (hasDSPTError){
                        mOnGetProvinceInfoListener.onFail(errorCode,errMsg);
                        return;
                    }
                    QueryProvinceResult queryProvinceResult = new QueryProvinceResult();
                    List<ProvinceInfo> list = new ArrayList<ProvinceInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            JSONArray jsonArray = jsonObject.getJSONArray("provinces");
                            ProvinceInfo provinceInfo;
                            for (int i = 0; i< jsonArray.length();i++){
                                provinceInfo = new ProvinceInfo();
                                JSONObject object = jsonArray.getJSONObject(i);
                                provinceInfo.setCode(object.getString("code"));
                                provinceInfo.setName(object.getString("name"));
                                list.add(provinceInfo);
                            }
                            queryProvinceResult.setProvinceInfoList(list);
                            mOnGetProvinceInfoListener.getProvinceInfo(requestHandleId, queryProvinceResult);
                        }else {
                            mOnGetProvinceInfoListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetProvinceInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            case GETCITYINFO:
                if (mOnGetCityInfoListener != null){
                    if (hasDSPTError){
                        mOnGetCityInfoListener.onFail(errorCode,errMsg);
                        return;
                    }
                    QueryCityResult queryCityResult = new QueryCityResult();
                    List<CityInfo> list = new ArrayList<CityInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            JSONArray jsonArray = jsonObject.getJSONArray("cities");
                            CityInfo cityInfo;
                            for (int i= 0 ; i< jsonArray.length(); i++){
                                JSONObject object = jsonArray.getJSONObject(i);
                                cityInfo = new CityInfo();
                                cityInfo.setCode(object.getString("code"));
                                cityInfo.setName(object.getString("name"));
                                cityInfo.setLongitude(object.getDouble("longitude"));
                                cityInfo.setLatitude(object.getDouble("latitude"));
                                list.add(cityInfo);
                            }
                            queryCityResult.setCityInfoList(list);
                            mOnGetCityInfoListener.getCityInfo(requestHandleId, queryCityResult);
                        }else {
                            mOnGetCityInfoListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetCityInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case WeatherType_3day:
                if (mOnNormalWeatherQueryListener != null) {
                    mOnNormalWeatherQueryListener.onTimeOut();
                }
                break;
            case WeatherType_3h:
                if (mOn3hWeatherQueryListener != null) {
                    mOn3hWeatherQueryListener.onTimeOut();
                }
                break;
            case WeatherType_alarm:
                if (mOnAlarmQueryListener != null) {
                    mOnAlarmQueryListener.onTimeOut();
                }
                break;
            case WeatherType_simplenormal:
                if(mOnSimpleNormalWeatherQueryListener != null) {
                    mOnSimpleNormalWeatherQueryListener.onTimeOut();
                }
                break;
            case QueryLifeIndex:
                if(mOnLifeIndexQueryListener != null) {
                    mOnLifeIndexQueryListener.onTimeOut();
                }
                break;
            case GETPROVINCEINFO:
                if (mOnGetProvinceInfoListener != null){
                    mOnGetProvinceInfoListener.onTimeOut();
                }
                break;
            case GETCITYINFO:
                if (mOnGetCityInfoListener != null){
                    mOnGetCityInfoListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    public void setOnNormalWeatherQueryListener(OnNormalWeatherQueryListener onNormalWeatherQueryListener) {
        this.mOnNormalWeatherQueryListener = onNormalWeatherQueryListener;
    }

    public void setOn3hWeatherQueryListener(On3hWeatherQueryListener on3hWeatherQueryListener) {
        this.mOn3hWeatherQueryListener = on3hWeatherQueryListener;
    }

    public void setOnAlarmQueryListener(OnAlarmQueryListener onAlarmQueryListener) {
        this.mOnAlarmQueryListener = onAlarmQueryListener;
    }
    public void setOnSimpleNormalWeatherQueryListener(OnSimpleNormalWeatherQueryListener onSimpleNormalWeatherQueryListener) {
        this.mOnSimpleNormalWeatherQueryListener = onSimpleNormalWeatherQueryListener;
    }

    public void setOnWeatherQueryListener(OnNormalWeatherQueryListener onNormalWeatherQueryListener){
        mOnNormalWeatherQueryListener = onNormalWeatherQueryListener;
    }

    public void setOnGetProvinceInfoListener(OnGetProvinceInfoListener onGetProvinceInfoListener){
        mOnGetProvinceInfoListener = onGetProvinceInfoListener;
    }

    public void setOnGetCityInfoListener (OnGetCityInfoListener onGetCityInfoListener){
        mOnGetCityInfoListener = onGetCityInfoListener;
    }
}
